// Version: 0.0.1-beta.1
#![allow(dead_code)]

use super::{lang_id_consts, HashMap, LangID};

pub(crate) type L10nMap = ::phf::Map<&'static str, &'static str>;
pub(crate) type SubLocaleMap = ::phf::Map<&'static str, fn() -> L10nMap>;
pub(crate) type LocaleMap = ::phf::Map<&'static str, fn() -> SubLocaleMap>;
pub(crate) type LocaleHashMap = HashMap<LangID, SubLocaleMap>;

/// Language ID: af;
/// Map name: "log-core";
/// Description: Afrikaans, Latyn, Suid-Afrika;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ontfout");
/// ```
pub(super) const fn get_af_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ontfout"##),
        ("warn", r##"Waarsku"##),
        ("info", r##"Inligting"##),
        ("error", r##"Fout"##),
        ("trace", r##"Spoor"##),
        ("init-logger", r##"Inisialisering van logger ..."##),
    ],
}
}

/// Language ID: af;
/// Map name: "theme";
/// Description: Afrikaans, Latyn, Suid-Afrika;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ongeldige tema naam");
/// ```
pub(super) const fn get_af_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ongeldige tema naam"##),
    ],
}
}

/// af: Afrikaans, Latyn, Suid-Afrika
pub(super) const fn get_af_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_af_map_log_core),
        ("theme", get_af_map_theme),
    ],
}
}

/// Language ID: am;
/// Map name: "log-core";
/// Description: አማርኛ, ኢትዮፒክ, ኢትዮጵያ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "error");
///
/// assert_eq!(msg, "ስህተት");
/// ```
pub(super) const fn get_am_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 10121458955350035957,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("trace", r##"ዱካ"##),
        ("info", r##"መረጃ"##),
        ("init-logger", r##"የመመዝገቢያ ማስጀመር ..."##),
        ("error", r##"ስህተት"##),
        ("warn", r##"አስጠንቅቀዋል"##),
    ],
}
}

/// Language ID: am;
/// Map name: "theme";
/// Description: አማርኛ, ኢትዮፒክ, ኢትዮጵያ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "ልክ ያልሆነ የፊደል ስም");
/// ```
pub(super) const fn get_am_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"ልክ ያልሆነ የፊደል ስም"##),
    ],
}
}

/// am: አማርኛ, ኢትዮፒክ, ኢትዮጵያ
pub(super) const fn get_am_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_am_map_log_core),
        ("theme", get_am_map_theme),
    ],
}
}

/// Language ID: ar;
/// Map name: "log-core";
/// Description: العربية, العربية, مصر;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "تصحيح");
/// ```
pub(super) const fn get_ar_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"تصحيح"##),
        ("warn", r##"تحذير"##),
        ("info", r##"المعلومات"##),
        ("error", r##"خطأ"##),
        ("trace", r##"تتبع"##),
        ("init-logger", r##"تهيئة المسجل ..."##),
    ],
}
}

/// Language ID: ar;
/// Map name: "args";
/// Description: العربية, العربية, مصر;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "يعطل خلفية الموضوع (مفع\u{651}لة افتراضي\u{64b}ا)، يتم تعطيلها باستخدام `--dis-theme-bg`.");
/// ```
pub(super) const fn get_ar_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"يحدد ملف تجميع لموضوعات التلوين المخصصة. يجب تحديد اسم الموضوع باستخدام خيار `--theme`."##),
        ("shell-completion", r##"يولد نص استكمال الأوامر لل shell."##),
        ("theme-file-help", r##"يتم دعم صيغة الملفات التي يتم إنشاؤها بواسطة أداة `syntect` في Rust والتي تعرف بـ dump format. فعلى الأغلب، هذه الملفات تحتوي على مجموعة من الموضوعات التي كانت مستخدمة في Sublime Text القديمة. إذا لم يتم تحديد خيار `--tf`، فسيتم قراءة مسار الملف من البيئة `$TOMLYRE_THEME_FILE`. "##),
        ("from-format", r##"يحدد صيغة الملف الذي يتم قراءته من stdin (سيتم التعرف على الصيغة تلقائيًا افتراضيًا)."##),
        ("disable-theme-background", r##"يعطل خلفية الموضوع (مفعّلة افتراضيًا)، يتم تعطيلها باستخدام `--dis-theme-bg`."##),
        ("theme-uncompressed-file-help", r##"> يتم منح الأولوية للإصدار غير المضغوط على نسخة المضغوطة.

إذا لم يتم تحديد خيار `--tuf`، فسيتم قراءة مسار الملف من البيئة `$TOMLYRE_THEME_UNCOMPRESSED_FILE`."##),
        ("disable-theme-background-help", r##"إذا لم يتم تحديد `--dis-theme-bg`، فسيتم قراءة القيمة من المتغير `$TOMLYRE_THEME_BG`.
إذا تم تعيين هذه القيمة على false ، سيتم تعطيل خلفية الموضوع.

مثال للاستخدام:

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"يحدد ملف تجميع لموضوعات التلوين المخصصة، ولكن يجب أن يكون غير مضغوط (uncompressed). يجب تحديد اسم الموضوع باستخدام خيار `--theme`."##),
        ("shell-completion-help", r##"يجب تحديد اسم ال Shell المستخدم لإنشاء نص استكمال الأوامر.
على سبيل المثال، إذا كنت تستخدم PowerShell على Windows، يمكنك استخدام: `--sh-comp pwsh`

> Q: ما هو استكمال الأوامر في ال Shell؟

> A: عند كتابة أوامر في سطر الأوامر، يمكن الاستفادة من ميزة تعبئة الأوامر (Tab Completion) لتصحيح وإكمال الأوامر كما يجب. لاستخدام هذه الميزة بشكل فعال، يتم إنشاء نص استكمال الأوامر.

تختلف الموقع الذي يتم حفظ نص استكمال الأوامر فيه بين أنواع مختلفة من الـ Shells. على سبيل المثال ، في Zsh، يجب حفظ النص في مسارات `$fpath` المعروفة.
يمكنك الحصول على محتوى هذه المتغير باستخدام الأمر `print -l $fpath`.
فرضًا أن المسار `/usr/local/share/zsh/site-functions` موجود في مسار `$fpath` ، يمكنك استخدام الأمر التالي لحفظ نص استكمال الأوامر في الملف `_tomlyre` داخل هذا المسار:

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

لتجنب استخدام صلاحيات root في هذا المثال ، يمكن وضع النص في الدليل الخاص بالمستخدم.
على سبيل المثال، يمكنك استخدام الأمر التالي لإضافة دليل تابع لمسار `$fpath`:

```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"عند قراءة البيانات من stdin ، على سبيل المثال: `cat a | tomlyre conv -`، سيتم محاولة التعرف على صيغة الملف تلقائيًا في البداية. ومع ذلك، قد يفشل التعرف التلقائي، وفي هذه الحالة، يجب تحديد الصيغة بشكل يدوي.
مثال: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"باستخدام خيار `--ts` فقط، سيتم إدراج مجموعة الأنماط المضمنة.
لا يمكن استخدام الأسماء المختصرة للأنماط المختلفة باستثناء markdown والتي يمكن اختصارها إلى md.
قم بتحديد الأسماء الكاملة للأنماط الأخرى.
مثال: `--ts ascii`

ملاحظات:
- حالياً، لا يمكن قراءة مجموعات **أنماط الجداول** من ملف. إذا كنت بحاجة إلى هذه الميزة أو ميزات أخرى، يرجى تقديم issue لإبلاغنا بها.
"##),
        ("table-style", r##"يحدد نمط الجدول."##),
        ("theme-help", r##"عند استخدام الخيار `-t` فقط، يتم إدراج قائمة بموضوعات التلوين المتوفرة في الوقت الحالي. إذا لم يتم تحديد ملف موضوع، فسيتم استخدام مجموعة الموضوعات المضمّنة في البرنامج. على سبيل المثال، إذا كان لديك موضوع بالاسم **ayu-dark** ، فيمكنك استخدام `-t ayu-dark` لاستخدام هذا الموضوع. إذا كان اسم الموضوع يحتوي على مسافات، فيجب وضعه بين علامات اقتباس مزدوجة أو علامات اقتباس مفردة.

> ملاحظة: يوجد موضوع خاص بعدم تلوين النصوص تماماً يحمل الاسم `None`.

إذا لم يتم تحديد خيار `--theme` ، فسيتم البحث عن القيمة في المتغير `$TOMLYRE_THEME` في بيئة العمل. موضوع تلوين هذه المساعدة أيضًا يتم الحصول عليه من المتغير `$TOMLYRE_THEME`.

> تتم إدراج مساعدة الافتراضية بشكل مصفّح، وتحمّل بشكل سريع جدًا.
> يتطلب استخدام النص المصفّح والنص المُلوّن نفس الوقت لتحميلهما بنفس السرعة، بينما سيكون التحميل الديناميكي أبطأ بمقدار رتبة الإشارة.
> قد يؤدي تعيين متغير `$TOMLYRE_THEME` في المستوى العالمي إلى إعادة تحميل المظهر والتأثير بشكل كبير على سرعة التشغيل.
> يرجى استخدام `-t` بدلاً من تعيين متغير `$TOMLYRE_THEME` ما لم تكن ترغب في تغيير موضوع مساعدة البرنامج.

مثال للاستخدام باستخدام متغير البيئة:

```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

مثال للاستخدام باستخدام خيار السطر الأمر:

```sh
tomlyre -t "Monokai Extended" get test.toml
```

بالنسبة للإخراج الذي يتم إنشاؤه يدويًا مثل "conv/get/set"، يحصل موضوع الخطأ المحدد بواسطة `-t` على الأولوية على `$TOMLYRE_THEME`. إذا لم يتم استخدام `-t` ، فسيتم قراءة `$TOMLYRE_THEME` مباشرة. إذا كانت `$TOMLYRE_THEME` فارغة ، فيتم استخدام موضوع "Monokai Extended"."##),
        ("theme", r##"يحدد اسم موضوع تلوين الصفحة"##),
    ],
}
}

/// Language ID: ar;
/// Map name: "args_md";
/// Description: العربية, العربية, مصر;
pub(super) const fn get_ar_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mيحدد ملف تجميع لموضوعات التلوين المخصصة. يجب تحديد اسم الموضوع باستخدام خيار [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mيولد نص استكمال الأوامر لل shell.[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mيتم دعم صيغة الملفات التي يتم إنشاؤها بواسطة أداة [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m في Rust والتي تعرف بـ dump format. فعلى الأغلب، هذه الملفات تحتوي على مجموعة من الموضوعات التي كانت مستخدمة في Sublime Text القديمة. إذا لم يتم تحديد خيار [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m، فسيتم قراءة مسار الملف من البيئة [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. [0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mيحدد صيغة الملف الذي يتم قراءته من stdin (سيتم التعرف على الصيغة تلقائيًا افتراضيًا).[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mيعطل خلفية الموضوع (مفعّلة افتراضيًا)، يتم تعطيلها باستخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m يتم منح الأولوية للإصدار غير المضغوط على نسخة المضغوطة.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mإذا لم يتم تحديد خيار [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m، فسيتم قراءة مسار الملف من البيئة [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mإذا لم يتم تحديد [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m، فسيتم قراءة القيمة من المتغير [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mإذا تم تعيين هذه القيمة على false ، سيتم تعطيل خلفية الموضوع.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mمثال للاستخدام:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mيحدد ملف تجميع لموضوعات التلوين المخصصة، ولكن يجب أن يكون غير مضغوط (uncompressed). يجب تحديد اسم الموضوع باستخدام خيار [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mيجب تحديد اسم ال Shell المستخدم لإنشاء نص استكمال الأوامر.
[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال، إذا كنت تستخدم PowerShell على Windows، يمكنك استخدام: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q: ما هو استكمال الأوامر في ال Shell؟
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: عند كتابة أوامر في سطر الأوامر، يمكن الاستفادة من ميزة تعبئة الأوامر (Tab Completion) لتصحيح وإكمال الأوامر كما يجب. لاستخدام هذه الميزة بشكل فعال، يتم إنشاء نص استكمال الأوامر.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mتختلف الموقع الذي يتم حفظ نص استكمال الأوامر فيه بين أنواع مختلفة من الـ Shells. على سبيل المثال ، في Zsh، يجب حفظ النص في مسارات [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m المعروفة.
[48;2;34;34;34m[38;2;255;255;255mيمكنك الحصول على محتوى هذه المتغير باستخدام الأمر [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mفرضًا أن المسار [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m موجود في مسار [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ، يمكنك استخدام الأمر التالي لحفظ نص استكمال الأوامر في الملف [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m داخل هذا المسار:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mلتجنب استخدام صلاحيات root في هذا المثال ، يمكن وضع النص في الدليل الخاص بالمستخدم.
[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال، يمكنك استخدام الأمر التالي لإضافة دليل تابع لمسار [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mعند قراءة البيانات من stdin ، على سبيل المثال: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m، سيتم محاولة التعرف على صيغة الملف تلقائيًا في البداية. ومع ذلك، قد يفشل التعرف التلقائي، وفي هذه الحالة، يجب تحديد الصيغة بشكل يدوي.
[48;2;34;34;34m[38;2;255;255;255mمثال: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mباستخدام خيار [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m فقط، سيتم إدراج مجموعة الأنماط المضمنة.
[48;2;34;34;34m[38;2;255;255;255mلا يمكن استخدام الأسماء المختصرة للأنماط المختلفة باستثناء markdown والتي يمكن اختصارها إلى md.
[48;2;34;34;34m[38;2;255;255;255mقم بتحديد الأسماء الكاملة للأنماط الأخرى.
[48;2;34;34;34m[38;2;255;255;255mمثال: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mملاحظات:
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mحالياً، لا يمكن قراءة مجموعات [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mأنماط الجداول[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m من ملف. إذا كنت بحاجة إلى هذه الميزة أو ميزات أخرى، يرجى تقديم issue لإبلاغنا بها.
[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mيحدد نمط الجدول.[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mعند استخدام الخيار [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m فقط، يتم إدراج قائمة بموضوعات التلوين المتوفرة في الوقت الحالي. إذا لم يتم تحديد ملف موضوع، فسيتم استخدام مجموعة الموضوعات المضمّنة في البرنامج. على سبيل المثال، إذا كان لديك موضوع بالاسم [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m ، فيمكنك استخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m لاستخدام هذا الموضوع. إذا كان اسم الموضوع يحتوي على مسافات، فيجب وضعه بين علامات اقتباس مزدوجة أو علامات اقتباس مفردة.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m ملاحظة: يوجد موضوع خاص بعدم تلوين النصوص تماماً يحمل الاسم [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mإذا لم يتم تحديد خيار [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ، فسيتم البحث عن القيمة في المتغير [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m في بيئة العمل. موضوع تلوين هذه المساعدة أيضًا يتم الحصول عليه من المتغير [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m تتم إدراج مساعدة الافتراضية بشكل مصفّح، وتحمّل بشكل سريع جدًا.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m يتطلب استخدام النص المصفّح والنص المُلوّن نفس الوقت لتحميلهما بنفس السرعة، بينما سيكون التحميل الديناميكي أبطأ بمقدار رتبة الإشارة.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m قد يؤدي تعيين متغير [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m في المستوى العالمي إلى إعادة تحميل المظهر والتأثير بشكل كبير على سرعة التشغيل.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m يرجى استخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m بدلاً من تعيين متغير [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m ما لم تكن ترغب في تغيير موضوع مساعدة البرنامج.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mمثال للاستخدام باستخدام متغير البيئة:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mمثال للاستخدام باستخدام خيار السطر الأمر:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mبالنسبة للإخراج الذي يتم إنشاؤه يدويًا مثل "conv/get/set"، يحصل موضوع الخطأ المحدد بواسطة [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m على الأولوية على [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. إذا لم يتم استخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ، فسيتم قراءة [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m مباشرة. إذا كانت [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m فارغة ، فيتم استخدام موضوع "Monokai Extended".[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mيحدد اسم موضوع تلوين الصفحة[0m"##),
    ],
}
}

/// Language ID: ar;
/// Map name: "conv";
/// Description: العربية, العربية, مصر;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "يتيح تحويل الملفات بين تنسيقات toml و yaml و json و ron");
/// ```
pub(super) const fn get_ar_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"يتيح تحويل الملفات بين تنسيقات toml و yaml و json و ron"##),
        ("from", r##"يحدد ملف الإدخال."##),
        ("conv-to", r##"يحدد تنسيق / ملف الإخراج، يمكن استخدامه عدة مرات."##),
        ("save", r##"عند استدعاء هذا الخيار، سيتم حفظ الملف الذي حُدِّدَ باستخدام `--to`."##),
        ("conv-to-help", r##"إذا كان الملف **test.json** موجودًا في الدليل الحالي، يمكنك استخدام `conv test.json -t yml` لتحويله إلى yml (yaml)،
أو يمكنك تحديد مسار الملف يدويًا: `conv test.json -t /tmp/test.yml`

التنسيقات المدعومة تعتمد على الميزات التي تم تمكينها في هذه الحزمة ، وليست جميع الميزات تم تمكينها بشكل افتراضي.

بالإضافة إلى ذلك، يجب أن تعرف قبل استخدام هذه الأمر الآتي:
- 1. سيتم إعادة تسمية المسارات غير المحددة المحددة بشكل تلقائي.
     - 1.1 فقط الملفات ذات المستوى الواحد سيتم تشخيصها بأنها صيغة ملف، وليس مسار
       عند استخدام `conv test.toml -t ron` ، سيتم تغيير اسم الملف المخرج إلى `test.ron` بدلاً من `ron`.
       إذا كنت لا تريد هذه الميزة، فيرجى استخدام `-t ./ron` أو `-t $PWD/ron`..
     - 1.2 يشير إلى الدليل وليس الملف.
       على سبيل المثال:`-t /tmp/`، يمكننا وضع `test.toml` فقط في الدليل الفرعي لـ `/tmp/`، وليس الغطاء `/tmp` نفسه.
       إذا كنت لا تريد هذه الميزة، فيرجى استخدام مسار الملف الكامل (و ليس المسار الفرعي)، على سبيل المثال، `-t /tmp/xx.toml` بدلاً من `-t /tmp/`.
- 2. عندما لا تُستدعى `--save`، لن يتم حفظ الملفات التي تم تحويلها بشكل تلقائي.
- 3. عند استدعاء `--save`، لا يجوز أن يكون الملف الناتج هو نفس الملف الذي تم دخوله.
- 4. لا يتم الاحتفاظ بالتعليقات في الملفات التي تم تحويلها."##),
    ],
}
}

/// Language ID: ar;
/// Map name: "conv_md";
/// Description: العربية, العربية, مصر;
pub(super) const fn get_ar_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mيتيح تحويل الملفات بين تنسيقات toml و yaml و json و ron[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mيحدد ملف الإدخال.[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mيحدد تنسيق / ملف الإخراج، يمكن استخدامه عدة مرات.[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mعند استدعاء هذا الخيار، سيتم حفظ الملف الذي حُدِّدَ باستخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mإذا كان الملف [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m موجودًا في الدليل الحالي، يمكنك استخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m لتحويله إلى yml (yaml)،
[48;2;34;34;34m[38;2;255;255;255mأو يمكنك تحديد مسار الملف يدويًا: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mالتنسيقات المدعومة تعتمد على الميزات التي تم تمكينها في هذه الحزمة ، وليست جميع الميزات تم تمكينها بشكل افتراضي.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mبالإضافة إلى ذلك، يجب أن تعرف قبل استخدام هذه الأمر الآتي:
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m سيتم إعادة تسمية المسارات غير المحددة المحددة بشكل تلقائي.
[48;2;34;34;34m[38;2;255;255;255m     [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1 فقط الملفات ذات المستوى الواحد سيتم تشخيصها بأنها صيغة ملف، وليس مسار
[48;2;34;34;34m[38;2;255;255;255m       [48;2;34;34;34m[38;2;255;255;255mعند استخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ، سيتم تغيير اسم الملف المخرج إلى [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m بدلاً من [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m       [48;2;34;34;34m[38;2;255;255;255mإذا كنت لا تريد هذه الميزة، فيرجى استخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m أو [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m..
[48;2;34;34;34m[38;2;255;255;255m     [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2 يشير إلى الدليل وليس الملف.
[48;2;34;34;34m[38;2;255;255;255m       [48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال:[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m، يمكننا وضع [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m فقط في الدليل الفرعي لـ [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m، وليس الغطاء [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m نفسه.
[48;2;34;34;34m[38;2;255;255;255m       [48;2;34;34;34m[38;2;255;255;255mإذا كنت لا تريد هذه الميزة، فيرجى استخدام مسار الملف الكامل (و ليس المسار الفرعي)، على سبيل المثال، [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m بدلاً من [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. عندما لا تُستدعى [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m، لن يتم حفظ الملفات التي تم تحويلها بشكل تلقائي.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. عند استدعاء [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m، لا يجوز أن يكون الملف الناتج هو نفس الملف الذي تم دخوله.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. لا يتم الاحتفاظ بالتعليقات في الملفات التي تم تحويلها.[0m"##),
    ],
}
}

/// Language ID: ar;
/// Map name: "opt";
/// Description: العربية, العربية, مصر;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "مصفوفة جداول (الاستخدام: `--aot 'k1 ، v1 ، k2 ، v2' --aot 'k1، v1'`)");
/// ```
pub(super) const fn get_ar_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"يختلف `--ck` عن `-k` في أن `--ck` لا يقسم السلسلة باستخدام ".". 

بالنسبة لـ [a.b.c] ، يتم التعامل مع `-k a.b.c` و `--ck a --ck b --ck c` بنفس الطريقة. 

إذا احتوى اسم المفتاح على '.' ، فيرجى استخدام `--ck` بدلاً من `-k`.
على سبيل المثال:

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

في هذا الهيكل ، "editor.defaultFormatter" هو سلسلة كاملة ، و defaultFormatter ليس فرعًا لـ editor.

عند استخدام `-k '[typescript]'.'editor.defaultFormatter'` ، يتم قسم المفتاح إلى:

- "[typescript]"
- "editor"
- "defaultFormatter"

بينما عند استخدام `--ck '[typescript]' --ck editor.defaultFormatter` ، يتم التعامل مع المفتاح كما يلي:

- "[typescript]"
- "editor.defaultFormatter"

الأخير هو الهيكل الصحيح.
"##),
        ("num-arr", r##"مصفوفة أعداد صحيحة بـ 64 بت"##),
        ("bool", r##"قيم مثل boolean (true او false)"##),
        ("num", r##"i64: الأعداد الصحيحة مع 64 بت. ومثال على ذلك الرقم 1145."##),
        ("datetime-help", r##"على سبيل المثال:

```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

> يدعم هذا الخيار فقط `toml`. إذا تم إضافة `-t json` في نهاية الأمر، فستظهر النتيجة كما يلي:

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"حفظ بتنسيق / اسم محدد ، ويمكن حفظ ملف واحد في كل مرة"##),
        ("get-opts-help", r##"مثال الاستخدام1: `get Cargo.toml -k package.name`
مثال الاستخدام2: `get cfg.yaml -k .`
"##),
        ("none-help", r##"> ملاحظة: يمتلك TOML نوعًا حقيقيًا من `Option<T>` ، والذي يمكن أن يكون قيمته إما `Some(T)` أو `None`.
> يقوم هذا الخيار `--none` بحذف مفتاح محدد، دون تغيير نوع القيمة إلى `Option<T>`.

على سبيل المثال، إذا كان لدينا جدول مصفوفة على النحو التالي في ملف `test.yml`:

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

يمكننا حذف مفتاح `name: test` من الجدول الثاني عبر تنفيذ الأمر التالي:

```sh
set test.yml -k bin.1.name --none
```

> ملاحظة: لا تتطابق السلسلة الفارغة مع None.
> يمكنك استخدام الأمر `--kv bin.1.name ""` لتعيين قيمة فارغة، بينما يقوم `--none` بحذف المفتاح بالكامل.

يمكننا أيضًا حذف الجدول الثاني كله (ذو المؤشر 1) باستخدام الأمر التالي:
```sh
set test.yml -k bin.1 --none
```

أو باستخدام `--rm`:
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"الأرقام العشرية مزدوجة الدقة (IEEE 754-2008)، على سبيل المثال 3.14159265359."##),
        ("saot-help", r##"> ملاحظة: بالرغم من أن `--saot` يسمح بإنشاء زوج key-value واحد فقط لكل جدول، إلا أنه يمكنك تحديد مؤشر لمصفوفة الجدول، ثم استخدام خيارات أخرى لتوسيعه.

على سبيل المثال:

```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"تعيين الزوج (k-v) ، `--kv x.y.z value` يعادل `-k x.y.z -s value`."##),
        ("set-save", r##"في حالة عدم استخدام `--to` ، سيتم الكتابة فوق الملف الحالي إذا تم استدعاؤه."##),
        ("array", r##"مصفوفة نصية"##),
        ("aot-help", r##"على سبيل المثال:
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84, version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

يتم اختيار المفتاح أولاً بتحديد مفتاح محدد، ثم يتم تغييره بقيمة محددة من نوع البيانات المحدد.

> عندما يتم توفير المفتاح فقط ولم يتم توفير القيمة ، سيتم عرض نوع البيانات المقابل والقيمة.
> على سبيل المثال، يتم عرض قيمة package وجداولها الفرعية بشكل أكثر تفصيلاً من `get`.

## Examples

### مثال 1:
```sh
--ts md set test.toml --key package.edition --str 2021
```

نظرًا لأن `--str` تم تحديده بدلاً من `--num` ، فإن "2021" هو نوع بيانات السلسلة ، وليس نوع بيانات الصحيح.

| package | new value |
|---------|-----------|
| edition | "2021"    |

```toml
[package]
edition = "2021"
```

### مثال 2:
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### مثال 3:
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: This is the 1st index*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> ملاحظة: تبدأ جميع الأمثلة بمؤشر(argument)/sub-command، ويتم حذف اسم الأمر الجذري(root command) ، يجب أن تضيفه يدويًا في استخدام الأمر الفعلي.
"##),
        ("num-arr-help", r##"على سبيل المثال:

```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```
"##),
        ("inline-table-help", r##"إذا كنت غير متأكد مما إذا كان يجب عليك استخدام الجدول القياسي أو الجدول المضمن ، فاستخدم الجدول المضمن!

على سبيل المثال:

```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> باستثناء `toml` ، فإن جميع التنسيقات الأخرى ستستخدم الجدول القياسي بدلاً من الجدول المضمن.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"استخدام المفتاح (key) للاستعلام عن القيمة(value)."##),
        ("f64-arr-help", r##"على سبيل المثال:

```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

>لا يدعم JSON 1.0 NaN و Inf.
سيتم إخراج "null" بدلاً من "NaN".
```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```"##),
        ("none", r##"حذف مفتاح محدد"##),
        ("set-save-to-help", r##"يشبه تقريبًا `-t` في الأمر الفرعي `conv`، لكن الفرق الرئيسي هو أن `-t` في `conv` يمكن استخدامه متعدد مرات متتالية لتحويل إلى صيغ/ملفات متعددة.
و `-t` في أمر الفرعي `set` يمكن استخدامه مرة واحدة فقط وستكون النتيجة في ملف واحد.

> ملاحظة: عند تشغيل الأمر الفرعي `conv` ، لا يتم الاحتفاظ بالتعليقات في أي من صيغ الملفات.
> وعلى الجانب الآخر، الأمر الفرعي `set` سوف يحفظ التعليقات في ملفات TOML فقط. كما أنه لن يحفظ التعليقات في أي شكل آخر.

- إذا تم استدعاء `--save` دون استدعاء `-t` ، فسيتم الكتابة فوق الملف الأصلي.
- إذا تم استدعاء `-t` مع `--save` ، فسيتم حفظ الملف المعدل في المسار الذي حددته
- بشرط أن الملف المصدر والملف الذي يشير إليه `-t` لا يتطابقان ، سيتم تطبيق سلوك `-t` في أمر الفرعي `set` على نحو متسق مع `-t` في أمر الفرعي `conv`. ويتضمن هذا تغيير اسم الملفات التي ليست من نوعها تلقائيًا.
  - على سبيل المثال، إذا كان الملف المصدر هو **test.yml** والملف المستهدف هو **json** ، فسيتم إنشاء ملف باسم **test.json** وليس باسم **json**.
  - إذا لم يتم الحاجة إلى هذه الخاصية ، يرجى استخدام التعبيرات **./json** أو **$PWD/json** بدلاً من اسم الملف.
"##),
        ("preview", r##"عرض ملف كامل قبل تطبيق التغييرات"##),
        ("set-key", r##"تحديد المفتاح المحدد، واستخدام "." كفاصل"##),
        ("get-opts", r##"يسترد قيمة الإعداد المحددة."##),
        ("saot", r##"جدول يحتوي على زوج key-value واحد فقط (الاستخدام: `--saot k1 v1 --saot k2 v2 ...`)"##),
        ("key-help", r##"يتم استخدام تنسيق "a.b.c" للبحث عن القيمة، حيث يتم فصل أسماء الجداول الفرعية أو مؤشرات المصفوفة باستخدام ".".
عندما يكون هناك "." واحد فقط ، سيتم عرض جميع الجداول.

"##),
        ("set-opts", r##"يتيح إنشاء / تحديث / حذف هيكل الملفات التهيئة."##),
        ("bool-arr-help", r##"على سبيل المثال:

```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: العنصر الأول هو `true`.
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"مصفوفة من القيم البولية"##),
        ("table", r##"جدول قياسي (standard table)، يستخدم لتخزين عدة أزواج من القيم key-value"##),
        ("datetime", r##"نوع البيانات التاريخ والوقت"##),
        ("table-help", r##"على سبيل المثال:

```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"مصفوفة من الأرقام العشرية المزدوجة الدقة"##),
        ("string", r##"سلسلة"##),
        ("array-help", r##"يتم تعيين عناصر المصفوفة عن طريق استدعاء `-a` مرات متعددة.
على سبيل المثال:

```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> يتم تعيين العنصر الأول للمصفوفة عند أول استدعاء، والثاني عند الاستدعاء الثاني، وهكذا.

```toml
[test]
array = ["hello", "wonderful", "world"]
```
"##),
        ("aot", r##"مصفوفة جداول (الاستخدام: `--aot 'k1 ، v1 ، k2 ، v2' --aot 'k1، v1'`)"##),
        ("inline-table", r##"جدول مضمن ، مماثل للجدول القياسي ولكنه أكثر كفاءة في التخزين"##),
        ("concat-key", r##"إدخال مفاتيح متعددة وترك GPT-3 تجميعها بشكل صحيح"##),
    ],
}
}

/// Language ID: ar;
/// Map name: "opt_md";
/// Description: العربية, العربية, مصر;
pub(super) const fn get_ar_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255mيختلف [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m عن [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m في أن [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m لا يقسم السلسلة باستخدام ".". 
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mبالنسبة لـ [a.b.c] ، يتم التعامل مع [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m و [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m بنفس الطريقة. 
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mإذا احتوى اسم المفتاح على '.' ، فيرجى استخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m بدلاً من [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mفي هذا الهيكل ، "editor.defaultFormatter" هو سلسلة كاملة ، و defaultFormatter ليس فرعًا لـ editor.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mعند استخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ، يتم قسم المفتاح إلى:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mبينما عند استخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ، يتم التعامل مع المفتاح كما يلي:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mالأخير هو الهيكل الصحيح.
[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mمصفوفة أعداد صحيحة بـ 64 بت[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mقيم مثل boolean (true او false)[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: الأعداد الصحيحة مع 64 بت. ومثال على ذلك الرقم 1145.[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m يدعم هذا الخيار فقط [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m. إذا تم إضافة [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m في نهاية الأمر، فستظهر النتيجة كما يلي:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255mحفظ بتنسيق / اسم محدد ، ويمكن حفظ ملف واحد في كل مرة[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255mمثال الاستخدام1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mمثال الاستخدام2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m ملاحظة: يمتلك TOML نوعًا حقيقيًا من [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m ، والذي يمكن أن يكون قيمته إما [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m أو [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m يقوم هذا الخيار [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m بحذف مفتاح محدد، دون تغيير نوع القيمة إلى [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال، إذا كان لدينا جدول مصفوفة على النحو التالي في ملف [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mيمكننا حذف مفتاح [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mname: test[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m من الجدول الثاني عبر تنفيذ الأمر التالي:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m ملاحظة: لا تتطابق السلسلة الفارغة مع None.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m يمكنك استخدام الأمر [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m لتعيين قيمة فارغة، بينما يقوم [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m بحذف المفتاح بالكامل.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mيمكننا أيضًا حذف الجدول الثاني كله (ذو المؤشر 1) باستخدام الأمر التالي:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mأو باستخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255mالأرقام العشرية مزدوجة الدقة (IEEE 754-2008)، على سبيل المثال 3.14159265359.[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m ملاحظة: بالرغم من أن [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m يسمح بإنشاء زوج key-value واحد فقط لكل جدول، إلا أنه يمكنك تحديد مؤشر لمصفوفة الجدول، ثم استخدام خيارات أخرى لتوسيعه.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mتعيين الزوج (k-v) ، [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m يعادل [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255mفي حالة عدم استخدام [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ، سيتم الكتابة فوق الملف الحالي إذا تم استدعاؤه.[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255mمصفوفة نصية[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84, version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mيتم اختيار المفتاح أولاً بتحديد مفتاح محدد، ثم يتم تغييره بقيمة محددة من نوع البيانات المحدد.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m عندما يتم توفير المفتاح فقط ولم يتم توفير القيمة ، سيتم عرض نوع البيانات المقابل والقيمة.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m على سبيل المثال، يتم عرض قيمة package وجداولها الفرعية بشكل أكثر تفصيلاً من [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExamples[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mمثال 1:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mنظرًا لأن [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m تم تحديده بدلاً من [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ، فإن "2021" هو نوع بيانات السلسلة ، وليس نوع بيانات الصحيح.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m new value [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mمثال 2:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mمثال 3:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: This is the 1st index*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m ملاحظة: تبدأ جميع الأمثلة بمؤشر(argument)/sub-command، ويتم حذف اسم الأمر الجذري(root command) ، يجب أن تضيفه يدويًا في استخدام الأمر الفعلي.
[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255mإذا كنت غير متأكد مما إذا كان يجب عليك استخدام الجدول القياسي أو الجدول المضمن ، فاستخدم الجدول المضمن!
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m باستثناء [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m ، فإن جميع التنسيقات الأخرى ستستخدم الجدول القياسي بدلاً من الجدول المضمن.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mاستخدام المفتاح (key) للاستعلام عن القيمة(value).[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239mلا يدعم JSON 1.0 NaN و Inf.
[48;2;34;34;34m[38;2;102;217;239mسيتم إخراج "null" بدلاً من "NaN".
[48;2;34;34;34m[38;2;102;217;239m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m```[48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255mحذف مفتاح محدد[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mيشبه تقريبًا [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m في الأمر الفرعي [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m، لكن الفرق الرئيسي هو أن [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m في [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m يمكن استخدامه متعدد مرات متتالية لتحويل إلى صيغ/ملفات متعددة.
[48;2;34;34;34m[38;2;255;255;255mو [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m في أمر الفرعي [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m يمكن استخدامه مرة واحدة فقط وستكون النتيجة في ملف واحد.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m ملاحظة: عند تشغيل الأمر الفرعي [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m ، لا يتم الاحتفاظ بالتعليقات في أي من صيغ الملفات.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m وعلى الجانب الآخر، الأمر الفرعي [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m سوف يحفظ التعليقات في ملفات TOML فقط. كما أنه لن يحفظ التعليقات في أي شكل آخر.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mإذا تم استدعاء [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m دون استدعاء [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ، فسيتم الكتابة فوق الملف الأصلي.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m إذا تم استدعاء [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m مع [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ، فسيتم حفظ الملف المعدل في المسار الذي حددته
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m بشرط أن الملف المصدر والملف الذي يشير إليه [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m لا يتطابقان ، سيتم تطبيق سلوك [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m في أمر الفرعي [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m على نحو متسق مع [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m في أمر الفرعي [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. ويتضمن هذا تغيير اسم الملفات التي ليست من نوعها تلقائيًا.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m على سبيل المثال، إذا كان الملف المصدر هو [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m والملف المستهدف هو [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m ، فسيتم إنشاء ملف باسم [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m وليس باسم [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m إذا لم يتم الحاجة إلى هذه الخاصية ، يرجى استخدام التعبيرات [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m./json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m أو [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m$PWD/json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m بدلاً من اسم الملف.
[0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255mعرض ملف كامل قبل تطبيق التغييرات[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255mتحديد المفتاح المحدد، واستخدام "." كفاصل[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255mيسترد قيمة الإعداد المحددة.[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255mجدول يحتوي على زوج key-value واحد فقط (الاستخدام: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2 ...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mيتم استخدام تنسيق "a.b.c" للبحث عن القيمة، حيث يتم فصل أسماء الجداول الفرعية أو مؤشرات المصفوفة باستخدام ".".
[48;2;34;34;34m[38;2;255;255;255mعندما يكون هناك "." واحد فقط ، سيتم عرض جميع الجداول.
[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255mيتيح إنشاء / تحديث / حذف هيكل الملفات التهيئة.[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: العنصر الأول هو `true`.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mمصفوفة من القيم البولية[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255mجدول قياسي (standard table)، يستخدم لتخزين عدة أزواج من القيم key-value[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255mنوع البيانات التاريخ والوقت[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mمصفوفة من الأرقام العشرية المزدوجة الدقة[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mسلسلة[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255mيتم تعيين عناصر المصفوفة عن طريق استدعاء [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m مرات متعددة.
[48;2;34;34;34m[38;2;255;255;255mعلى سبيل المثال:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m يتم تعيين العنصر الأول للمصفوفة عند أول استدعاء، والثاني عند الاستدعاء الثاني، وهكذا.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255mمصفوفة جداول (الاستخدام: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1 ، v1 ، k2 ، v2' --aot 'k1، v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mجدول مضمن ، مماثل للجدول القياسي ولكنه أكثر كفاءة في التخزين[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mإدخال مفاتيح متعددة وترك GPT-3 تجميعها بشكل صحيح[0m"##),
    ],
}
}

/// Language ID: ar;
/// Map name: "parser";
/// Description: العربية, العربية, مصر;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "يشير هذا السطر في الكود إلى خطأ الإخراج (stderr) ويوضح الفرق بين استخدام `>` و`2>` عند توجيه الإخراج إلى ملف. يفضل استخدام `>` لتضمين كلا من الإخراج القياسي والإخراج غير القياسي (stderr).");
/// ```
pub(super) const fn get_ar_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"يشير هذا السطر في الكود إلى خطأ الإخراج (stderr) ويوضح الفرق بين استخدام `>` و`2>` عند توجيه الإخراج إلى ملف. يفضل استخدام `>` لتضمين كلا من الإخراج القياسي والإخراج غير القياسي (stderr)."##),
    ],
}
}

/// Language ID: ar;
/// Map name: "theme";
/// Description: العربية, العربية, مصر;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "اسم السمة غير صالح");
/// ```
pub(super) const fn get_ar_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"اسم السمة غير صالح"##),
    ],
}
}

/// ar: العربية, العربية, مصر
pub(super) const fn get_ar_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_ar_map_args),
        ("conv_md", get_ar_map_conv_md),
        ("parser", get_ar_map_parser),
        ("opt_md", get_ar_map_opt_md),
        ("args_md", get_ar_map_args_md),
        ("opt", get_ar_map_opt),
        ("log-core", get_ar_map_log_core),
        ("conv", get_ar_map_conv),
        ("theme", get_ar_map_theme),
    ],
}
}

/// Language ID: az;
/// Map name: "log-core";
/// Description: azərbaycan, latın, Azərbaycan;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_az_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"xəbərdarlıq"##),
        ("info", r##"Məlumat"##),
        ("error", r##"səhv"##),
        ("trace", r##"iz"##),
        ("init-logger", r##"Çıxışçılığı başlatmaq ..."##),
    ],
}
}

/// Language ID: az;
/// Map name: "theme";
/// Description: azərbaycan, latın, Azərbaycan;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Yanlış mövzu adı");
/// ```
pub(super) const fn get_az_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Yanlış mövzu adı"##),
    ],
}
}

/// az: azərbaycan, latın, Azərbaycan
pub(super) const fn get_az_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_az_map_log_core),
        ("theme", get_az_map_theme),
    ],
}
}

/// Language ID: be;
/// Map name: "log-core";
/// Description: беларуская, кірыліца, Беларусь;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "адладка");
/// ```
pub(super) const fn get_be_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"адладка"##),
        ("warn", r##"папярэджвайце"##),
        ("info", r##"Інфармацыя"##),
        ("error", r##"Памылка"##),
        ("trace", r##"след"##),
        ("init-logger", r##"Ініцыялізацыя рэгістратара ..."##),
    ],
}
}

/// Language ID: be;
/// Map name: "theme";
/// Description: беларуская, кірыліца, Беларусь;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Несапраўдная назва тэмы");
/// ```
pub(super) const fn get_be_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Несапраўдная назва тэмы"##),
    ],
}
}

/// be: беларуская, кірыліца, Беларусь
pub(super) const fn get_be_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_be_map_log_core),
        ("theme", get_be_map_theme),
    ],
}
}

/// Language ID: bg;
/// Map name: "log-core";
/// Description: български, кирилица, България;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "отстраняване на грешки");
/// ```
pub(super) const fn get_bg_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"отстраняване на грешки"##),
        ("warn", r##"Предупреждение"##),
        ("info", r##"информация"##),
        ("error", r##"Грешка"##),
        ("trace", r##"следа"##),
        ("init-logger", r##"Инициализиране на дърводобив ..."##),
    ],
}
}

/// Language ID: bg;
/// Map name: "theme";
/// Description: български, кирилица, България;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Невалидно име на тема");
/// ```
pub(super) const fn get_bg_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Невалидно име на тема"##),
    ],
}
}

/// bg: български, кирилица, България
pub(super) const fn get_bg_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_bg_map_log_core),
        ("theme", get_bg_map_theme),
    ],
}
}

/// Language ID: bn;
/// Map name: "log-core";
/// Description: বাংলা, বাংলা, বাংলাদেশ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ডিব\u{9be}গ");
/// ```
pub(super) const fn get_bn_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ডিবাগ"##),
        ("warn", r##"সতর্কতা"##),
        ("info", r##"তথ্য"##),
        ("error", r##"ত্রুটি"##),
        ("trace", r##"ট্রেস"##),
        ("init-logger", r##"লগার আরম্ভ করা হচ্ছে..."##),
    ],
}
}

/// Language ID: bn;
/// Map name: "theme";
/// Description: বাংলা, বাংলা, বাংলাদেশ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "অবৈধ থিমের ন\u{9be}ম");
/// ```
pub(super) const fn get_bn_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"অবৈধ থিমের নাম"##),
    ],
}
}

/// Language ID: bn;
/// Map name: "args";
/// Description: বাংলা, বাংলা, বাংলাদেশ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "থিম ব\u{9cd}য\u{9be}কগ\u{9cd}র\u{9be}উন\u{9cd}ড নিষ\u{9cd}ক\u{9cd}রিয\u{9bc} কর\u{9c1}ন (ডিফল\u{9cd}ট সক\u{9cd}ষম হয\u{9bc}ে থ\u{9be}কে, `--dis-theme-bg` ইউজ করে অক\u{9cd}ষম কর\u{9c1}ন)");
/// ```
pub(super) const fn get_bn_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"আপনি অবশ্যই থিম নাম উল্লেখ করতে `--theme` দিতে হবেন, স্বয়ংক্রিয়ভাবে এটি লোড হয় না।"##),
        ("shell-completion", r##"শেলের জন্য পূর্ণাঙ্গ সম্পূর্ণতা স্ক্রিপ্ট তৈরি করুন"##),
        ("theme-file-help", r##"এটি সমর্থিত ফাইল ফরম্যাট rust crate `syntect` দ্বারা স্থাপিত করা হয়েছে।  
মূলতঃ এগুলি পুরানো সাবলাইম থিমের মতো। 
`--tf` উল্লেখ না করলে, `$TOMLYRE_THEME_FILE` এনভায়রনমেন্ট ভেরিয়েবলের মান পড়া হবে।"##),
        ("from-format", r##"সোর্স ফাইল ফরম্যাট ম্যানুয়ালি নির্দিষ্ট করুন (ডিফল্টভাবে সমানুপাতিকভাবে পরীক্ষা করা হয়)"##),
        ("disable-theme-background", r##"থিম ব্যাকগ্রাউন্ড নিষ্ক্রিয় করুন (ডিফল্ট সক্ষম হয়ে থাকে, `--dis-theme-bg` ইউজ করে অক্ষম করুন)"##),
        ("theme-uncompressed-file-help", r##"> আনকম্প্রেসড ভার্সন কম্প্রেসড ভার্সনের চেয়ে উচ্চ প্রাথমিকতা দেয়।  

`--tuf` উল্লেখ না করলে, `$TOMLYRE_THEME_UNCOMPRESSED_FILE` এনভায়রনমেন্ট ভেরিয়েবলের মান পড়া হবে।"##),
        ("disable-theme-background-help", r##"`--dis-theme-bg` উল্লেখ না করলে, `$TOMLYRE_THEME_BG` এনভায়রনমেন্ট ভেরিয়েবলের মান পড়া হবে।

যদি মান false হয়, তবে থিম ব্যাকগ্রাউন্ড নিষ্ক্রিয় হবে। 

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"theme-file এর মতোই, তবে অবশ্যই অকম্প্রেসড ডাম্প ডেটা (ফাইল) ব্যবহার করতে হবে"##),
        ("shell-completion-help", r##"আপনাকে স্ক্রিপ্ট তৈরি করতে শেলের নাম নির্দিষ্ট করতে হবে।
উদাহরণঃ উইন্ডোজের জন্য powershell এর জন্য, আপনি ব্যবহার করতে পারেন : `--sh-comp pwsh`

> প্রশ্ন: শেলের কিছু ফাংশনালিটি কি?

> উত্তর: আমরা কমান্ড লাইনে কমান্ড ইনপুট করার সময় ট্যাব কী চাপা হলে, অটোমেটিক কমপ্লিট চালু হবে এবং সম্ভবত কমপ্লিট পছন্দগুলি খুঁজে পাওয়া শুরু করবে। এই ফাংশনালিটি কমান্ড লাইন ইনপুট এর দক্ষতা এবং নির্ভুলতা উন্নয়নে সাহায্য করে।

পার্থক্যপূর্ণ শেলের জন্য কমপ্লিট স্ক্রিপ্টের জায়গাগুলি ভিন্ন হতে পারে। zsh এর জন্য, আপনাকে `$fpath` এর উল্লেখ করতে হবে।
আপনি `print -l $fpath` ব্যবহার করে এই অ্যারে এর কন্টেন্ট জানতে পারেন। 
ধরা যাক, এটি `/usr/local/share/zsh/site-functions` ডিরেক্টরিকে ধারণ করে। তাহলে, নিম্নোক্ত কমান্ড ব্যবহার করে সেখানে `_tomlyre` নামক ফাইলটি লিখতে পারেন।

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

সুষ্ঠ কাজটির জন্য root হওয়ার প্রয়োজন নেই। ব্যবহারকারীর সাবডিরেক্টরিতে কমপ্লিশনগুলি সংরক্ষণ করা যেতে পারে। উদাহরণঃ আপনি এই কমান্ড ব্যবহার করে এই অ্যারের শুরুতে একটি উপাদান সংযুক্ত করতে পারেন

```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"ডেটা stdin থেকে পড়া হয়, যেমন: `cat a | tomlyre conv -`, এটি প্রথমে ফাইল ফরম্যাট অটোমেটিকভাবে চেষ্টা করবে।
তবে, অটোমেটিকভাবে চেষ্টা করার সময় সফলতা অসম্ভব হলে, ম্যানুয়ালি ফরম্যাট নির্দিষ্ট করা দরকার হতে পারে।

উদাহরণ: `cat test.bson | tomlyre --src-fmt bson get -` "##),
        ("table-style-help", r##"কেবলমাত্র `--ts` প্যারামিটার ব্যবহার করে আপনি উন্নয়নকৃত স্টাইল সেটের তালিকা জানতে পারেন। 
md এর ছাড়াও, অন্যান্য স্টাইলের জন্য পূর্ণ নাম ব্যবহার করতে হবে।

উদাহরণ 1: `--ts md`
উদাহরণ 2: `--table-style ascii`

লক্ষ্য করুন: **টেবিল স্টাইল** সেটের কালেকশন ফাইল থেকে পড়া সমর্থিত নয়, আপনি যদি এই বৈশিষ্ট্য ব্যবহার করতে চান বা অন্যান্য বৈশিষ্ট্যের জন্য যে কোনও পরামর্শ দিতে চান তবে সমস্যা রিপোর্ট করতে আপনার স্বাগতম।"##),
        ("table-style", r##"টেবিল স্টাইল নির্দিষ্ট করুন"##),
        ("theme-help", r##"শুধুমাত্র `-t` প্যারামিটার ব্যবহার করা হলে, বর্তমান থিম এর তালিকা দেখানো হবে। 
যদি থিম ফাইল উল্লেখ করা না হয়, তখন প্রস্তুতিভুক্ত থিম সেট ব্যবহৃত হবে।
যেমন যদি একটি থিম `ayu-dark` উপস্থিত হয়, তবে আপনি `-t ayu-dark` ব্যবহার করে সেটি ব্যবহার করতে পারেন।
থিম নামে যদি স্পেস থাকে, তবে এটি ডাবল কোটেশন অথবা সিঙ্গেল কোটেশন দিয়ে আবদ্ধ করুন।

> নোট: `None` একটি বিশেষ থিম যা কোন সিনট্যাক্স হাইলাইট না করে মৌলিক টেক্সট স্টাইল প্রদর্শিত করে। 

যখন `-t` ব্যবহার না করা হলে, তখন ডিফল্ট থিম `$TOMLYRE_THEME` এর মধ্যে থাকা থিম ব্যবহৃত হবে।
এই হেল্প সেকশনের **সিনট্যাক্স হাইলাইট** থিম ও এক্সট্রাক্ট করা `$TOMLYRE_THEME` এর মাধ্যমে প্রদর্শিত করা হয়েছে।

> ডিফল্ট হেল্প টেক্সট হার্ডকোড করা আছে, এটি খুব দ্রুত লোড করা যায়।
> হার্ডকোড টেক্সট এবং `None` থিম লোড করা সমস্ত লোড এর স্পীড একই হবে, যেখানে ডাইনামিক লোড এর স্পীড দুইটি অর্ডার বা তার চেয়ে বেশি ধীর। 
> `$TOMLYRE_THEME` এনভায়রনমেন্ট ভেরিয়েবল সেট করা মূলত থিমটি আবার লোড করতে পারে এবং এটি শুরুর স্পীডকে অত্যন্তভাবে **ধীর** করতে পারে।  
> সহায়তা তথ্য এর থিম পরিবর্তন করার পরিকল্পনা না করে কিংবা সিনট্যাক্স হাইলাইট নিষ্ক্রিয় করার পরিকল্পনা না করলে, আপনাকে `-t` প্যারামিটারটি ব্যবহার করতে হবে এবং এনভায়রনমেন্ট ভেরিয়েবল পরিবর্তন করতে হবে না।  

এনভায়রনমেন্ট ভেরিয়েবল ব্যবহারের উদাহরণ:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

কমান্ড লাইন প্যারামিটার ব্যবহারের উদাহরণ:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

"conv/get/set" এবং সম্পর্কিত অপারেশন সম্পাদনা করার জন্য ম্যানুয়ালি উত্তরগুলি, `-t` দ্রষ্টব্যতার করা থিমের প্রাথমিকতা > এনভায়রনমেন্ট ভেরিয়েবল।  
যদি `-t` কল করা না হয় তবে এনভায়রনমেন্ট ভেরিয়েবল পড়া হবে; যদি এনভায়রনমেন্ট ভেরিয়েবল ফাঁকা হয়, তবে **Monokai Extended** ব্যবহৃত হবে।"##),
        ("theme", r##"সিনট্যাক্স হাইলাইট এর প্রধান থিম নাম"##),
    ],
}
}

/// Language ID: bn;
/// Map name: "args_md";
/// Description: বাংলা, বাংলা, বাংলাদেশ;
pub(super) const fn get_bn_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mআপনি অবশ্যই থিম নাম উল্লেখ করতে [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m দিতে হবেন, স্বয়ংক্রিয়ভাবে এটি লোড হয় না।[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mশেলের জন্য পূর্ণাঙ্গ সম্পূর্ণতা স্ক্রিপ্ট তৈরি করুন[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mএটি সমর্থিত ফাইল ফরম্যাট rust crate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m দ্বারা স্থাপিত করা হয়েছে।[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mমূলতঃ এগুলি পুরানো সাবলাইম থিমের মতো। 
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m উল্লেখ না করলে, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m এনভায়রনমেন্ট ভেরিয়েবলের মান পড়া হবে।[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mসোর্স ফাইল ফরম্যাট ম্যানুয়ালি নির্দিষ্ট করুন (ডিফল্টভাবে সমানুপাতিকভাবে পরীক্ষা করা হয়)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mথিম ব্যাকগ্রাউন্ড নিষ্ক্রিয় করুন (ডিফল্ট সক্ষম হয়ে থাকে, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ইউজ করে অক্ষম করুন)[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m আনকম্প্রেসড ভার্সন কম্প্রেসড ভার্সনের চেয়ে উচ্চ প্রাথমিকতা দেয়।[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m উল্লেখ না করলে, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m এনভায়রনমেন্ট ভেরিয়েবলের মান পড়া হবে।[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m উল্লেখ না করলে, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m এনভায়রনমেন্ট ভেরিয়েবলের মান পড়া হবে।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mযদি মান false হয়, তবে থিম ব্যাকগ্রাউন্ড নিষ্ক্রিয় হবে। 
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mtheme-file এর মতোই, তবে অবশ্যই অকম্প্রেসড ডাম্প ডেটা (ফাইল) ব্যবহার করতে হবে[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mআপনাকে স্ক্রিপ্ট তৈরি করতে শেলের নাম নির্দিষ্ট করতে হবে।
[48;2;34;34;34m[38;2;255;255;255mউদাহরণঃ উইন্ডোজের জন্য powershell এর জন্য, আপনি ব্যবহার করতে পারেন : [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m প্রশ্ন: শেলের কিছু ফাংশনালিটি কি?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m উত্তর: আমরা কমান্ড লাইনে কমান্ড ইনপুট করার সময় ট্যাব কী চাপা হলে, অটোমেটিক কমপ্লিট চালু হবে এবং সম্ভবত কমপ্লিট পছন্দগুলি খুঁজে পাওয়া শুরু করবে। এই ফাংশনালিটি কমান্ড লাইন ইনপুট এর দক্ষতা এবং নির্ভুলতা উন্নয়নে সাহায্য করে।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mপার্থক্যপূর্ণ শেলের জন্য কমপ্লিট স্ক্রিপ্টের জায়গাগুলি ভিন্ন হতে পারে। zsh এর জন্য, আপনাকে [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m এর উল্লেখ করতে হবে।
[48;2;34;34;34m[38;2;255;255;255mআপনি [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ব্যবহার করে এই অ্যারে এর কন্টেন্ট জানতে পারেন। 
[48;2;34;34;34m[38;2;255;255;255mধরা যাক, এটি [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ডিরেক্টরিকে ধারণ করে। তাহলে, নিম্নোক্ত কমান্ড ব্যবহার করে সেখানে [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m নামক ফাইলটি লিখতে পারেন।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mসুষ্ঠ কাজটির জন্য root হওয়ার প্রয়োজন নেই। ব্যবহারকারীর সাবডিরেক্টরিতে কমপ্লিশনগুলি সংরক্ষণ করা যেতে পারে। উদাহরণঃ আপনি এই কমান্ড ব্যবহার করে এই অ্যারের শুরুতে একটি উপাদান সংযুক্ত করতে পারেন
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mডেটা stdin থেকে পড়া হয়, যেমন: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, এটি প্রথমে ফাইল ফরম্যাট অটোমেটিকভাবে চেষ্টা করবে।
[48;2;34;34;34m[38;2;255;255;255mতবে, অটোমেটিকভাবে চেষ্টা করার সময় সফলতা অসম্ভব হলে, ম্যানুয়ালি ফরম্যাট নির্দিষ্ট করা দরকার হতে পারে।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mউদাহরণ: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m [0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mকেবলমাত্র [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m প্যারামিটার ব্যবহার করে আপনি উন্নয়নকৃত স্টাইল সেটের তালিকা জানতে পারেন। 
[48;2;34;34;34m[38;2;255;255;255mmd এর ছাড়াও, অন্যান্য স্টাইলের জন্য পূর্ণ নাম ব্যবহার করতে হবে।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mউদাহরণ 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mউদাহরণ 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mলক্ষ্য করুন: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mটেবিল স্টাইল[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m সেটের কালেকশন ফাইল থেকে পড়া সমর্থিত নয়, আপনি যদি এই বৈশিষ্ট্য ব্যবহার করতে চান বা অন্যান্য বৈশিষ্ট্যের জন্য যে কোনও পরামর্শ দিতে চান তবে সমস্যা রিপোর্ট করতে আপনার স্বাগতম।[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mটেবিল স্টাইল নির্দিষ্ট করুন[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mশুধুমাত্র [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m প্যারামিটার ব্যবহার করা হলে, বর্তমান থিম এর তালিকা দেখানো হবে। 
[48;2;34;34;34m[38;2;255;255;255mযদি থিম ফাইল উল্লেখ করা না হয়, তখন প্রস্তুতিভুক্ত থিম সেট ব্যবহৃত হবে।
[48;2;34;34;34m[38;2;255;255;255mযেমন যদি একটি থিম [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m উপস্থিত হয়, তবে আপনি [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ব্যবহার করে সেটি ব্যবহার করতে পারেন।
[48;2;34;34;34m[38;2;255;255;255mথিম নামে যদি স্পেস থাকে, তবে এটি ডাবল কোটেশন অথবা সিঙ্গেল কোটেশন দিয়ে আবদ্ধ করুন।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m নোট: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m একটি বিশেষ থিম যা কোন সিনট্যাক্স হাইলাইট না করে মৌলিক টেক্সট স্টাইল প্রদর্শিত করে। 
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mযখন [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ব্যবহার না করা হলে, তখন ডিফল্ট থিম [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m এর মধ্যে থাকা থিম ব্যবহৃত হবে।
[48;2;34;34;34m[38;2;255;255;255mএই হেল্প সেকশনের [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mসিনট্যাক্স হাইলাইট[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m থিম ও এক্সট্রাক্ট করা [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m এর মাধ্যমে প্রদর্শিত করা হয়েছে।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m ডিফল্ট হেল্প টেক্সট হার্ডকোড করা আছে, এটি খুব দ্রুত লোড করা যায়।
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m হার্ডকোড টেক্সট এবং [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m থিম লোড করা সমস্ত লোড এর স্পীড একই হবে, যেখানে ডাইনামিক লোড এর স্পীড দুইটি অর্ডার বা তার চেয়ে বেশি ধীর। 
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m এনভায়রনমেন্ট ভেরিয়েবল সেট করা মূলত থিমটি আবার লোড করতে পারে এবং এটি শুরুর স্পীডকে অত্যন্তভাবে [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mধীর[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m করতে পারে।[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m সহায়তা তথ্য এর থিম পরিবর্তন করার পরিকল্পনা না করে কিংবা সিনট্যাক্স হাইলাইট নিষ্ক্রিয় করার পরিকল্পনা না করলে, আপনাকে [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m প্যারামিটারটি ব্যবহার করতে হবে এবং এনভায়রনমেন্ট ভেরিয়েবল পরিবর্তন করতে হবে না।[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mএনভায়রনমেন্ট ভেরিয়েবল ব্যবহারের উদাহরণ:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mকমান্ড লাইন প্যারামিটার ব্যবহারের উদাহরণ:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m"conv/get/set" এবং সম্পর্কিত অপারেশন সম্পাদনা করার জন্য ম্যানুয়ালি উত্তরগুলি, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m দ্রষ্টব্যতার করা থিমের প্রাথমিকতা > এনভায়রনমেন্ট ভেরিয়েবল।[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mযদি [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m কল করা না হয় তবে এনভায়রনমেন্ট ভেরিয়েবল পড়া হবে; যদি এনভায়রনমেন্ট ভেরিয়েবল ফাঁকা হয়, তবে [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m ব্যবহৃত হবে।[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mসিনট্যাক্স হাইলাইট এর প্রধান থিম নাম[0m"##),
    ],
}
}

/// Language ID: bn;
/// Map name: "conv";
/// Description: বাংলা, বাংলা, বাংলাদেশ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "tomlএবং yaml, json এবং ron ফরম\u{9cd}য\u{9be}টে র\u{9c1}প\u{9be}ন\u{9cd}তর কর\u{9be} হয়");
/// ```
pub(super) const fn get_bn_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"tomlএবং yaml, json এবং ron ফরম্যাটে রুপান্তর করা হয়"##),
        ("from", r##"ইনপুট ফাইল নির্দিষ্ট করুন"##),
        ("conv-to", r##"আউটপুট ফরম্যাট / ফাইল নির্দিষ্ট করুন, এটি একাধিকবার কল করা সমর্থিত"##),
        ("save", r##"`--save` কল করলে, `--to` দেওয়া ফাইলটি সংরক্ষিত হবে"##),
        ("conv-to-help", r##"যদি বর্তমান ডিরেক্টরিতে **test.json** ফাইল থাকে তবে আপনি `conv test.json -t yml` কল করে এটি একটি yml (yaml) ফরম্যাটে রুপান্তর করতে পারেন,
আপনি ম্যানুয়ালি ফাইল পথ নির্দিষ্ট করতে পারেন: `conv test.json -t /tmp/test.yml`

এটি সকল বৈশিষ্ট্য সক্ষম হলে সমর্থিত ফর্ম্যাটের উপর নির্ভর করে এবং সমস্ত বৈশিষ্ট্য ডিফল্ট ভাবে সক্রিয় নয়।

উপরোক্ত এর সাথে, আপনাকে জানা দরকারি -
- 1. এটি অস্পষ্ট ফাইল পথের জন্য স্বয়ংক্রিয়ভাবে নাম পরিবর্তন করবে।
  - 1.1: শুধুমাত্র একটি স্তরের ফাইলগুলি ফাইল ফরম্যাট হিসাবে চিহ্নিত হবে এবং পথ হিসাবে চিহ্নিত হবে না।
    `conv test.toml -t ron` কল করলে, পথ নির্দিষ্ট না করলেও, আউটপুট ফাইলটি স্বয়ংক্রিয়ভাবে `test.ron` হবে।
    যদি আপনি এই বৈশিষ্ট্যটি ব্যবহার করতে না চান তবে দয়া করে `-t ./ron` বা `-t $PWD/ron` ব্যবহার করুন।
  - 1.2: ফাইলের স্থানান্তর করা হচ্ছে ডিরেক্টরিতে বর্ধিত না
    `-t /tmp/` এর মতো হলে, আমরা শুধুমাত্র `/tmp/` এর সাবডিরে `test.toml` রাখতে পারি এবং `/tmp` ফাইলটি স্বয়ংক্রিয়ভাবে নয়।
    যদি আপনি এই বৈশিষ্ট্যটি ব্যবহার করতে না চান, তবে পূর্ণ ফাইল পথটি ব্যবহার করুন, উদাহরণস্বরূপ `-t /tmp/xx.toml`, বর্তমানে `-t /tmp/` হওয়া উচিত নয়।
- 2. যখন `--save` কল করা না হয় তখন নির্দিষ্ট ফাইলটি স্বয়ংক্রিয়ভাবে সংরক্ষিত হবে না।
- 3. `--save` কল করার সময়, conv উপ-কমান্ডের ইনপুট এবং আউটপুট একই ফাইল নিষ্ক্রিয় করা যাবে না।
- 4. রুপান্তরিত ফাইলে মন্তব্য সংরক্ষিত থাকবে না।"##),
    ],
}
}

/// Language ID: bn;
/// Map name: "conv_md";
/// Description: বাংলা, বাংলা, বাংলাদেশ;
pub(super) const fn get_bn_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mtomlএবং yaml, json এবং ron ফরম্যাটে রুপান্তর করা হয়[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mইনপুট ফাইল নির্দিষ্ট করুন[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mআউটপুট ফরম্যাট / ফাইল নির্দিষ্ট করুন, এটি একাধিকবার কল করা সমর্থিত[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m কল করলে, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m দেওয়া ফাইলটি সংরক্ষিত হবে[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mযদি বর্তমান ডিরেক্টরিতে [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m ফাইল থাকে তবে আপনি [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m কল করে এটি একটি yml (yaml) ফরম্যাটে রুপান্তর করতে পারেন,
[48;2;34;34;34m[38;2;255;255;255mআপনি ম্যানুয়ালি ফাইল পথ নির্দিষ্ট করতে পারেন: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mএটি সকল বৈশিষ্ট্য সক্ষম হলে সমর্থিত ফর্ম্যাটের উপর নির্ভর করে এবং সমস্ত বৈশিষ্ট্য ডিফল্ট ভাবে সক্রিয় নয়।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mউপরোক্ত এর সাথে, আপনাকে জানা দরকারি -
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m এটি অস্পষ্ট ফাইল পথের জন্য স্বয়ংক্রিয়ভাবে নাম পরিবর্তন করবে।
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: শুধুমাত্র একটি স্তরের ফাইলগুলি ফাইল ফরম্যাট হিসাবে চিহ্নিত হবে এবং পথ হিসাবে চিহ্নিত হবে না।
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m কল করলে, পথ নির্দিষ্ট না করলেও, আউটপুট ফাইলটি স্বয়ংক্রিয়ভাবে [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m হবে।
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mযদি আপনি এই বৈশিষ্ট্যটি ব্যবহার করতে না চান তবে দয়া করে [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m বা [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ব্যবহার করুন।
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: ফাইলের স্থানান্তর করা হচ্ছে ডিরেক্টরিতে বর্ধিত না
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m এর মতো হলে, আমরা শুধুমাত্র [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m এর সাবডিরে [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m রাখতে পারি এবং [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ফাইলটি স্বয়ংক্রিয়ভাবে নয়।
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mযদি আপনি এই বৈশিষ্ট্যটি ব্যবহার করতে না চান, তবে পূর্ণ ফাইল পথটি ব্যবহার করুন, উদাহরণস্বরূপ [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, বর্তমানে [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m হওয়া উচিত নয়।
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. যখন [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m কল করা না হয় তখন নির্দিষ্ট ফাইলটি স্বয়ংক্রিয়ভাবে সংরক্ষিত হবে না।
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m কল করার সময়, conv উপ-কমান্ডের ইনপুট এবং আউটপুট একই ফাইল নিষ্ক্রিয় করা যাবে না।
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. রুপান্তরিত ফাইলে মন্তব্য সংরক্ষিত থাকবে না।[0m"##),
    ],
}
}

/// Language ID: bn;
/// Map name: "parser";
/// Description: বাংলা, বাংলা, বাংলাদেশ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "এখ\u{9be}নে stderr আউটপ\u{9c1}ট দেওয\u{9bc}\u{9be} হয\u{9bc}েছে, কমপ\u{9cd}লিশন স\u{9cd}ক\u{9cd}রিপ\u{9cd}ট আউটপ\u{9c1}ট প\u{9c1}নর\u{9cd}নির\u{9cd}দেশ কর\u{9be}র জন\u{9cd}য আপন\u{9be}কে `>/path/to/file` ব\u{9cd}যবহ\u{9be}র কর\u{9be} উচিত এবং `2>/path/to/file` নয\u{9bc}।");
/// ```
pub(super) const fn get_bn_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"এখানে stderr আউটপুট দেওয়া হয়েছে, কমপ্লিশন স্ক্রিপ্ট আউটপুট পুনর্নির্দেশ করার জন্য আপনাকে `>/path/to/file` ব্যবহার করা উচিত এবং `2>/path/to/file` নয়।"##),
    ],
}
}

/// bn: বাংলা, বাংলা, বাংলাদেশ
pub(super) const fn get_bn_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (3, 4),
        (0, 0),
    ],
    entries: &[
        ("parser", get_bn_map_parser),
        ("conv_md", get_bn_map_conv_md),
        ("args", get_bn_map_args),
        ("args_md", get_bn_map_args_md),
        ("theme", get_bn_map_theme),
        ("conv", get_bn_map_conv),
        ("log-core", get_bn_map_log_core),
    ],
}
}

/// Language ID: bs;
/// Map name: "log-core";
/// Description: bosanski, latinica, Bosna i Hercegovina;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Ispravljanje pogrešaka");
/// ```
pub(super) const fn get_bs_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Ispravljanje pogrešaka"##),
        ("warn", r##"upozori"##),
        ("info", r##"Info"##),
        ("error", r##"greška"##),
        ("trace", r##"trag"##),
        ("init-logger", r##"Inicijaliziranje drvosječa ..."##),
    ],
}
}

/// Language ID: bs;
/// Map name: "theme";
/// Description: bosanski, latinica, Bosna i Hercegovina;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nevažeće ime tema");
/// ```
pub(super) const fn get_bs_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nevažeće ime tema"##),
    ],
}
}

/// bs: bosanski, latinica, Bosna i Hercegovina
pub(super) const fn get_bs_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_bs_map_log_core),
        ("theme", get_bs_map_theme),
    ],
}
}

/// Language ID: ca;
/// Map name: "log-core";
/// Description: català, llatí, Espanya;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "depuració");
/// ```
pub(super) const fn get_ca_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"depuració"##),
        ("warn", r##"adverteix"##),
        ("info", r##"Info"##),
        ("error", r##"Error"##),
        ("trace", r##"TRACE"##),
        ("init-logger", r##"Inicialització del registre ..."##),
    ],
}
}

/// Language ID: ca;
/// Map name: "theme";
/// Description: català, llatí, Espanya;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nom del tema no vàlid");
/// ```
pub(super) const fn get_ca_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nom del tema no vàlid"##),
    ],
}
}

/// ca: català, llatí, Espanya
pub(super) const fn get_ca_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ca_map_log_core),
        ("theme", get_ca_map_theme),
    ],
}
}

/// Language ID: ceb;
/// Map name: "log-core";
/// Description: Cebuano, Latin, Pilipinas;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_ceb_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"Pasidaan"##),
        ("info", r##"impormasyon"##),
        ("error", r##"sayup"##),
        ("trace", r##"pagsubay"##),
        ("init-logger", r##"Pag-una sa logger ..."##),
    ],
}
}

/// Language ID: ceb;
/// Map name: "theme";
/// Description: Cebuano, Latin, Pilipinas;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Dili husto nga ngalan sa tema");
/// ```
pub(super) const fn get_ceb_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Dili husto nga ngalan sa tema"##),
    ],
}
}

/// ceb: Cebuano, Latin, Pilipinas
pub(super) const fn get_ceb_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ceb_map_log_core),
        ("theme", get_ceb_map_theme),
    ],
}
}

/// Language ID: co;
/// Map name: "log-core";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_co_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"avisa"##),
        ("info", r##"info"##),
        ("error", r##"errore"##),
        ("trace", r##"traccia"##),
        ("init-logger", r##"Inizializà Logger ..."##),
    ],
}
}

/// Language ID: co;
/// Map name: "theme";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "U nome di tema micca validu");
/// ```
pub(super) const fn get_co_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"U nome di tema micca validu"##),
    ],
}
}

/// co: co-Latn-FR
pub(super) const fn get_co_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_co_map_log_core),
        ("theme", get_co_map_theme),
    ],
}
}

/// Language ID: cs;
/// Map name: "log-core";
/// Description: čeština, latinka, Česko;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "DEBUG");
/// ```
pub(super) const fn get_cs_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"DEBUG"##),
        ("warn", r##"VAROVÁNÍ"##),
        ("info", r##"INFORMACE"##),
        ("error", r##"CHYBA"##),
        ("trace", r##"STOPA"##),
        ("init-logger", r##"Inicializuji logger..."##),
    ],
}
}

/// Language ID: cs;
/// Map name: "theme";
/// Description: čeština, latinka, Česko;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Neplatný název motivu");
/// ```
pub(super) const fn get_cs_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Neplatný název motivu"##),
    ],
}
}

/// Language ID: cs;
/// Map name: "args";
/// Description: čeština, latinka, Česko;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Zakázat pozadí tématu (výchozí stav je povolený, přepínačem `--dis-theme-bg` ho můžete zakázat)");
/// ```
pub(super) const fn get_cs_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"Vlastní soubor s tématy, spolu s parametrem `--theme`, který specifikuje název tématu"##),
        ("shell-completion", r##"Vygenerování skriptu pro doplňování příkazů v shellu"##),
        ("theme-file-help", r##"Podporovaný formát souboru jsou binární data dump ze starší verze Sublime Text.
Pokud nezadáte volbu `--tf`, program načte hodnotu z proměnné prostředí `$TOMLYRE_THEME_FILE`."##),
        ("from-format", r##"Ruční specifikace formátu zdrojového souboru (výchozí je automatické rozpoznání)"##),
        ("disable-theme-background", r##"Zakázat pozadí tématu (výchozí stav je povolený, přepínačem `--dis-theme-bg` ho můžete zakázat)"##),
        ("theme-uncompressed-file-help", r##"> Nepoužívejte tuto volbu, pokud není nutné použít nekomprimovaná dump data.  

Pokud nezadáte volbu `--tuf`, program načte hodnotu z proměnné prostředí `$TOMLYRE_THEME_UNCOMPRESSED_FILE`."##),
        ("disable-theme-background-help", r##"Pokud není zadán parametr `-dis-theme-bg`, bude načtena hodnota z proměnné prostředí `$TOMLYRE_THEME_BG`.

Pokud je nastaveno na `false`, bude vypnuto.

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"Podobný jako `theme-file`, ale vyžaduje nekomprimovaná dump data (soubor)"##),
        ("shell-completion-help", r##"Musíte ručně specifikovat jméno shellu, pro který chcete vygenerovat skript pro doplňování příkazů.
Například pro powershell na Windows můžete použít: `--sh-comp pwsh`

> Otázka: Co znamená funkce doplňování příkazů (tab completion)?

> Odpověď: Když na příkazovém řádku začnete psát příkaz a stisknete klávesu Tab, spustí se skript pro doplňování příkazů, který začne hledat možné varianty doplnění a nabídne je k výběru. Tato funkce výrazně zvyšuje efektivitu a přesnost zadávání příkazů v příkazovém řádku.

Různé shelly mají různá místa pro ukládání skriptů pro doplňování příkazů. Pro zsh musí být skript uložen v cestách, které jsou obsaženy v poli `$fpath`.
Pro získání obsahu tohoto pole můžete použít příkaz `print -l $fpath`.
Pokud například obsahuje adresář `/usr/local/share/zsh/site-functions`, můžete pomocí následujícího příkazu zapsat skript pro doplňování příkazů do souboru `_tomlyre` v tomto adresáři.

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

Abychom se vyhnuli používání oprávnění roota, můžeme skript uložit do podadresáře uživatelské složky.
Například můžete použít následující příkazy k vložení nové položky na začátek pole a následné uložení skriptu do adresáře pro skripty pro doplňování příkazů.

```zsh
fpath=(~/.local/share/zsh/completions $fpath)
tomlyre --sh-comp zsh > ~/.local/share/zsh/completions/_tomlyre
```"##),
        ("from-format-help", r##"Při čtení dat ze stdin, například takto: `cat a | tomlyre conv -`, bude nejprve pokus o automatické rozpoznání formátu souboru.
Nicméně, automatické rozpoznání může selhat a v takových případech je třeba formát specifikovat ručně.
Příklad: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Pokud použijete pouze parametr `--ts`, vypíše se seznam dostupných stylů.
Kromě markdown, který může být zkrácen na md, musí být názvy ostatních stylů úplné.
Příklady použití: `--ts md`
Příklady použití: `--table-style ascii`

> Upozornění: Aktuálně není možné načíst sadu stylů pro tabulky ze souboru. Pokud potřebujete tuto funkci nebo jinou další, neváhejte nám poslat zpětnou vazbu prostřednictvím issue."##),
        ("table-style", r##"Specifikuje styl tabulek"##),
        ("theme-help", r##"Pokud použijete pouze parametr `-t`, vypíše se seznam dostupných témat.
Pokud nezadáte soubor s tématy, budou použity výchozí téma balíčku.
Předpokládejme, že existuje téma s názvem **ayu-dark**, pak můžete použít `-t ayu-dark` k použití tohoto tématu.
Pokud je název tématu obsahující mezery, musí být uzavřen do uvozovek nebo apostrofů.

> Pozn.: Existuje speciální téma `None`, které reprezentuje čistý textový styl bez zvýrazňování syntaxe.

Pokud nezadáte volbu `--theme`, program načte hodnotu z proměnné prostředí `$TOMLYRE_THEME`.
Témata syntaxe pro tuto nápovědu jsou také načtena z této proměnné prostředí.

> Výchozí nápověda je předem načtena, což zajišťuje velmi rychlé načtení.  
> Načítání textu ze zabudovaného kódu a textu bez zvýraznění syntaxe je stejně rychlé jako načítání textu se specifickým tématem.  
> Nastavení proměnné prostředí `$TOMLYRE_THEME` může způsobit opakované načtení témat, což výrazně zpomalí start programu.  
> Pokud nemáte v úmyslu měnit téma pro tuto nápovědu nebo zakázat zvýrazňování syntaxe, použijte volbu `-t` místo změny proměnné prostředí.

Příklady použití s proměnnou prostředí:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Příklady použití s přepínačem:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Pro obsah manuálně vygenerovaných výstupů jako `conv/get/set` atd. má nastavení tématu pomocí `-t` prioritu před proměnnou prostředí.
Pokud není použita volba `-t`, hodnota se načte z proměnné prostředí; pokud je proměnná prázdná, použije se téma **Monokai Extended**."##),
        ("theme", r##"Specifikuje název tématu pro zvýraznění syntaxe"##),
    ],
}
}

/// Language ID: cs;
/// Map name: "args_md";
/// Description: čeština, latinka, Česko;
pub(super) const fn get_cs_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mVlastní soubor s tématy, spolu s parametrem [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, který specifikuje název tématu[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mVygenerování skriptu pro doplňování příkazů v shellu[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mPodporovaný formát souboru jsou binární data dump ze starší verze Sublime Text.
[48;2;34;34;34m[38;2;255;255;255mPokud nezadáte volbu [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, program načte hodnotu z proměnné prostředí [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mRuční specifikace formátu zdrojového souboru (výchozí je automatické rozpoznání)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mZakázat pozadí tématu (výchozí stav je povolený, přepínačem [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ho můžete zakázat)[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nepoužívejte tuto volbu, pokud není nutné použít nekomprimovaná dump data.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPokud nezadáte volbu [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, program načte hodnotu z proměnné prostředí [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mPokud není zadán parametr [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, bude načtena hodnota z proměnné prostředí [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPokud je nastaveno na [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, bude vypnuto.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mPodobný jako [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ale vyžaduje nekomprimovaná dump data (soubor)[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mMusíte ručně specifikovat jméno shellu, pro který chcete vygenerovat skript pro doplňování příkazů.
[48;2;34;34;34m[38;2;255;255;255mNapříklad pro powershell na Windows můžete použít: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Otázka: Co znamená funkce doplňování příkazů (tab completion)?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Odpověď: Když na příkazovém řádku začnete psát příkaz a stisknete klávesu Tab, spustí se skript pro doplňování příkazů, který začne hledat možné varianty doplnění a nabídne je k výběru. Tato funkce výrazně zvyšuje efektivitu a přesnost zadávání příkazů v příkazovém řádku.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mRůzné shelly mají různá místa pro ukládání skriptů pro doplňování příkazů. Pro zsh musí být skript uložen v cestách, které jsou obsaženy v poli [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mPro získání obsahu tohoto pole můžete použít příkaz [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mPokud například obsahuje adresář [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, můžete pomocí následujícího příkazu zapsat skript pro doplňování příkazů do souboru [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m v tomto adresáři.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAbychom se vyhnuli používání oprávnění roota, můžeme skript uložit do podadresáře uživatelské složky.
[48;2;34;34;34m[38;2;255;255;255mNapříklad můžete použít následující příkazy k vložení nové položky na začátek pole a následné uložení skriptu do adresáře pro skripty pro doplňování příkazů.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh [48;2;34;34;34m[38;2;249;38;114m>[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mPři čtení dat ze stdin, například takto: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, bude nejprve pokus o automatické rozpoznání formátu souboru.
[48;2;34;34;34m[38;2;255;255;255mNicméně, automatické rozpoznání může selhat a v takových případech je třeba formát specifikovat ručně.
[48;2;34;34;34m[38;2;255;255;255mPříklad: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mPokud použijete pouze parametr [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, vypíše se seznam dostupných stylů.
[48;2;34;34;34m[38;2;255;255;255mKromě markdown, který může být zkrácen na md, musí být názvy ostatních stylů úplné.
[48;2;34;34;34m[38;2;255;255;255mPříklady použití: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPříklady použití: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Upozornění: Aktuálně není možné načíst sadu stylů pro tabulky ze souboru. Pokud potřebujete tuto funkci nebo jinou další, neváhejte nám poslat zpětnou vazbu prostřednictvím issue.[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mSpecifikuje styl tabulek[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mPokud použijete pouze parametr [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, vypíše se seznam dostupných témat.
[48;2;34;34;34m[38;2;255;255;255mPokud nezadáte soubor s tématy, budou použity výchozí téma balíčku.
[48;2;34;34;34m[38;2;255;255;255mPředpokládejme, že existuje téma s názvem [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, pak můžete použít [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m k použití tohoto tématu.
[48;2;34;34;34m[38;2;255;255;255mPokud je název tématu obsahující mezery, musí být uzavřen do uvozovek nebo apostrofů.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Pozn.: Existuje speciální téma [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, které reprezentuje čistý textový styl bez zvýrazňování syntaxe.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPokud nezadáte volbu [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, program načte hodnotu z proměnné prostředí [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mTémata syntaxe pro tuto nápovědu jsou také načtena z této proměnné prostředí.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Výchozí nápověda je předem načtena, což zajišťuje velmi rychlé načtení.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Načítání textu ze zabudovaného kódu a textu bez zvýraznění syntaxe je stejně rychlé jako načítání textu se specifickým tématem.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nastavení proměnné prostředí [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m může způsobit opakované načtení témat, což výrazně zpomalí start programu.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Pokud nemáte v úmyslu měnit téma pro tuto nápovědu nebo zakázat zvýrazňování syntaxe, použijte volbu [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m místo změny proměnné prostředí.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPříklady použití s proměnnou prostředí:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPříklady použití s přepínačem:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPro obsah manuálně vygenerovaných výstupů jako [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv/get/set[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m atd. má nastavení tématu pomocí [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m prioritu před proměnnou prostředí.
[48;2;34;34;34m[38;2;255;255;255mPokud není použita volba [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, hodnota se načte z proměnné prostředí; pokud je proměnná prázdná, použije se téma [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mSpecifikuje název tématu pro zvýraznění syntaxe[0m"##),
    ],
}
}

/// Language ID: cs;
/// Map name: "conv";
/// Description: čeština, latinka, Česko;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Převádění mezi formáty jako toml, yaml, json a ron");
/// ```
pub(super) const fn get_cs_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Převádění mezi formáty jako toml, yaml, json a ron"##),
        ("from", r##"Specifikuje vstupní soubor"##),
        ("conv-to", r##"Specifikuje výstupní formát/soubor, lze použít vícekrát"##),
        ("save", r##"Při použití této volby budou soubory specifikované pomocí `--to` uloženy"##),
        ("conv-to-help", r##"Pokud existuje soubor **test.json** v aktuálním adresáři, můžete jej převést na yml (yaml) pomocí `conv test.json -t yml`
nebo ručně specifikovat cestu k souboru: `conv test.json -t /tmp/test.yml`

Podporované formáty závisí na funkcích povolených v této balíčku a ne všechny funkce jsou implicitně povoleny.

Kromě toho musíte vědět následující věci před použitím:
- 1. Automaticky přejmenuje nekonkrétní cesty k souborům.
  - 1.1: Pouze soubory s jediným vrstvou se berou jako souborový formát místo cesty k souboru
    Pokud zavoláte `conv test.toml -t ron`, název výstupního souboru se změní na `test.ron` namísto `ron`.
    Pokud tuto funkci nepotřebujete, použijte `-t ./ron` nebo `-t $PWD/ron`.
  - 1.2: Cesta ukazuje na adresář místo souboru
    Například `-t /tmp/`, můžeme umístit `test.toml` pouze do podadresářů `/tmp/`, nikoli do adresáře `/tmp`.
    Pokud tuto funkci nepotřebujete, uveďte úplnou cestu k souboru (nikoli adresáři) jako `-t /tmp/xx.toml` namísto `-t /tmp/`.
- 2. Pokud není volána volba `--save`, specifikované soubory se automaticky neuloží.
- 3. Pokud je volba `--save` použita, vstup a výstup příkazu conv nesmí být stejný soubor.
- 4. Převzaté soubory nezachovávají komentáře. "##),
    ],
}
}

/// Language ID: cs;
/// Map name: "conv_md";
/// Description: čeština, latinka, Česko;
pub(super) const fn get_cs_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mPřevádění mezi formáty jako toml, yaml, json a ron[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mSpecifikuje vstupní soubor[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mSpecifikuje výstupní formát/soubor, lze použít vícekrát[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mPři použití této volby budou soubory specifikované pomocí [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m uloženy[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mPokud existuje soubor [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m v aktuálním adresáři, můžete jej převést na yml (yaml) pomocí [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mnebo ručně specifikovat cestu k souboru: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPodporované formáty závisí na funkcích povolených v této balíčku a ne všechny funkce jsou implicitně povoleny.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mKromě toho musíte vědět následující věci před použitím:
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Automaticky přejmenuje nekonkrétní cesty k souborům.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Pouze soubory s jediným vrstvou se berou jako souborový formát místo cesty k souboru
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mPokud zavoláte [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, název výstupního souboru se změní na [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m namísto [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mPokud tuto funkci nepotřebujete, použijte [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m nebo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: Cesta ukazuje na adresář místo souboru
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mNapříklad [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, můžeme umístit [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m pouze do podadresářů [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, nikoli do adresáře [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mPokud tuto funkci nepotřebujete, uveďte úplnou cestu k souboru (nikoli adresáři) jako [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m namísto [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. Pokud není volána volba [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, specifikované soubory se automaticky neuloží.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. Pokud je volba [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m použita, vstup a výstup příkazu conv nesmí být stejný soubor.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. Převzaté soubory nezachovávají komentáře. [0m"##),
    ],
}
}

/// Language ID: cs;
/// Map name: "parser";
/// Description: čeština, latinka, Česko;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "Níže je výstup na stderr. Pokud chcete přesměrovat výstup dokončovacího skriptu, měli byste použít `>/cesta/k/souboru` namísto `2>/cesta/k/souboru`.");
/// ```
pub(super) const fn get_cs_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"Níže je výstup na stderr. Pokud chcete přesměrovat výstup dokončovacího skriptu, měli byste použít `>/cesta/k/souboru` namísto `2>/cesta/k/souboru`."##),
    ],
}
}

/// cs: čeština, latinka, Česko
pub(super) const fn get_cs_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (3, 4),
        (0, 0),
    ],
    entries: &[
        ("parser", get_cs_map_parser),
        ("conv_md", get_cs_map_conv_md),
        ("args", get_cs_map_args),
        ("args_md", get_cs_map_args_md),
        ("theme", get_cs_map_theme),
        ("conv", get_cs_map_conv),
        ("log-core", get_cs_map_log_core),
    ],
}
}

/// Language ID: cy;
/// Map name: "log-core";
/// Description: Cymraeg, Lladin, Y Deyrnas Unedig;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Dadfygio");
/// ```
pub(super) const fn get_cy_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Dadfygio"##),
        ("warn", r##"Rhybudd"##),
        ("info", r##"Gwybodaeth"##),
        ("error", r##"Wallau"##),
        ("trace", r##"olrhain"##),
        ("init-logger", r##"Cychwyn Logger ..."##),
    ],
}
}

/// Language ID: cy;
/// Map name: "theme";
/// Description: Cymraeg, Lladin, Y Deyrnas Unedig;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Enw Thema Annilys");
/// ```
pub(super) const fn get_cy_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Enw Thema Annilys"##),
    ],
}
}

/// cy: Cymraeg, Lladin, Y Deyrnas Unedig
pub(super) const fn get_cy_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_cy_map_log_core),
        ("theme", get_cy_map_theme),
    ],
}
}

/// Language ID: da;
/// Map name: "log-core";
/// Description: dansk, latinsk, Danmark;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_da_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"advare"##),
        ("info", r##"info"##),
        ("error", r##"fejl"##),
        ("trace", r##"Spor"##),
        ("init-logger", r##"Initialisering af logger ..."##),
    ],
}
}

/// Language ID: da;
/// Map name: "theme";
/// Description: dansk, latinsk, Danmark;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ugyldigt temanavn");
/// ```
pub(super) const fn get_da_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ugyldigt temanavn"##),
    ],
}
}

/// da: dansk, latinsk, Danmark
pub(super) const fn get_da_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_da_map_log_core),
        ("theme", get_da_map_theme),
    ],
}
}

/// Language ID: de;
/// Map name: "log-core";
/// Description: Deutsch, Lateinisch, Deutschland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "DEBUG");
/// ```
pub(super) const fn get_de_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"DEBUG"##),
        ("warn", r##"WARNUNG"##),
        ("info", r##"INFO"##),
        ("error", r##"FEHLER"##),
        ("trace", r##"SPUR"##),
        ("init-logger", r##"Logger wird initialisiert ..."##),
    ],
}
}

/// Language ID: de;
/// Map name: "args";
/// Description: Deutsch, Lateinisch, Deutschland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Deaktiviert das Hintergrundbild des Themas (standardmäßig aktiviert). Wenn `--dis-theme-bg` aufgerufen wird, wird es deaktiviert.");
/// ```
pub(super) const fn get_de_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"Gibt die Datei mit den benutzerdefinierten Themen an. Sie müssen auch `--theme` verwenden, um den Namen des Themas anzugeben."##),
        ("shell-completion", r##"Erzeugt eine Shell-Autovervollständigung"##),
        ("theme-file-help", r##"Das unterstützte Dateiformat ist binäre Daten, die von einem rust crate namens `syntect` erstellt wurden.
Prinzipiell handelt es sich dabei um Themen für eine ältere Version von Sublime.
Wenn `--tf` nicht angegeben wird, wird versucht, den Wert aus der Umgebungsvariable `$TOMLYRE_THEME_FILE` zu lesen. "##),
        ("from-format", r##"Gibt das Dateiformat des Quelldokuments manuell an (standardmäßig wird es automatisch erkannt)"##),
        ("disable-theme-background", r##"Deaktiviert das Hintergrundbild des Themas (standardmäßig aktiviert). Wenn `--dis-theme-bg` aufgerufen wird, wird es deaktiviert."##),
        ("theme-uncompressed-file-help", r##"> Die unkomprimierte Version hat Vorrang vor der komprimierten.  

Wenn `--tuf` nicht angegeben wird, wird versucht, den Wert aus der Umgebungsvariable `$TOMLYRE_THEME_UNCOMPRESSED_FILE` zu lesen. "##),
        ("disable-theme-background-help", r##"Wenn `--dis-theme-bg` nicht angegeben wird, wird versucht, den Wert aus der Umgebungsvariable `$TOMLYRE_THEME_BG` zu lesen.

Wenn dieser Wert auf `false` gesetzt ist, wird er deaktiviert

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"Gibt die Datei mit den unkomprimierten binären Themen an. Auch hier müssen Sie `--theme` verwenden, um den Namen des Themas anzugeben."##),
        ("shell-completion-help", r##"Sie müssen den Namen der Shell manuell angeben, für die Sie Autovervollständigungsskripte generieren möchten.
Zum Beispiel können Sie für PowerShell unter Windows Folgendes verwenden: `--sh-comp pwsh`

> Q: Was ist die Autovervollständigungsfunktion für eine Shell?

> A: Wenn wir Befehle auf der Kommandozeile eingeben und die Tab-Taste drücken, wird das Autovervollständigungsskript ausgelöst, um mögliche Vervollständigungsoptionen zu durchsuchen und diese Optionen zur Auswahl bereitzustellen. Diese Funktion kann die Effizienz und Genauigkeit der Eingabe auf der Kommandozeile erheblich verbessern.

Die Speicherorte für Autovervollständigungsskripte sind je nach Shell unterschiedlich. Für zsh müssen Sie sie beispielsweise in einem Verzeichnis speichern, das im `$fpath`-Array enthalten ist.
Sie können `print -l $fpath` verwenden, um den Inhalt dieses Arrays abzurufen.
Angenommen, es enthält das Verzeichnis `/usr/local/share/zsh/site-functions`. Dann können Sie den folgenden Befehl verwenden, um das Autovervollständigungsskript in die Datei `_tomlyre` in diesem Verzeichnis zu schreiben.

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

Um die Verwendung von Root-Rechten zu vermeiden, können Sie das Skript in einem Unterverzeichnis des Benutzerordners ablegen. Beispielsweise können Sie mit dem folgenden Befehl am Anfang des Arrays ein Element einfügen und ein entsprechendes Verzeichnis verwenden.
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"Wenn Daten aus stdin gelesen werden, z.B. `cat a | tomlyre conv -`, wird zuerst versucht, das Dateiformat automatisch zu erkennen.
Das automatische Erkennen kann jedoch fehlschlagen, in diesem Fall muss das Format manuell angegeben werden.
Beispiel: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Wenn Sie nur `--ts` verwenden, wird eine Liste der verfügbaren integrierten Stile angezeigt.
Mit Ausnahme von Markdown, das als `md` abgekürzt werden kann, müssen für andere Stile vollständige Namen verwendet werden.
Beispiel 1: `--ts md`
Beispiel 2: `--table-style ascii`

Hinweis: Derzeit wird das Sammeln von **Tabellenstilen** aus einer Datei nicht unterstützt. Wenn Sie diese Funktion oder weitere Funktionen benötigen, können Sie gerne ein Issue erstellen, um Feedback zu geben. "##),
        ("table-style", r##"Gibt den Stil für Tabellen an"##),
        ("theme-help", r##"Wenn Sie nur `-t` verwenden, wird eine Liste der verfügbaren Themen angezeigt.
Wenn kein Thema angegeben ist, werden die integrierten Themen verwendet.
Angenommen, es gibt ein Thema namens **ayu-dark**, dann können Sie es mit `-t ayu-dark` verwenden.
Wenn der Themenname Leerzeichen enthält, sollten Sie ihn in Anführungszeichen setzen.

> Hinweis: Es gibt ein spezielles Thema `None`, das für einen reinen Textstil ohne Syntaxhervorhebung steht.

Wenn `--theme` nicht angegeben wird, wird versucht, den Wert aus der Umgebungsvariable `$TOMLYRE_THEME` zu lesen.
Das Thema für die Syntaxhervorhebung in dieser Hilfe stammt ebenfalls aus dieser **Umgebungsvariable**.

> Die Standardhilfe wird hartcodiert geladen und ist sehr schnell.  
> Der Ladevorgang für hartcodierten Text und für Text mit `None`-Thema ist in derselben Größenordnung, aber der dynamische Ladevorgang ist zwei oder mehr Größenordnungen langsamer.  
> Das Setzen der `$TOMLYRE_THEME`-Umgebungsvariable kann dazu führen, dass das Programm das Thema neu lädt, was die Startzeit erheblich verlangsamen kann.  
> Wenn Sie das Thema für die Hilfe nicht ändern oder die Syntaxhervorhebung deaktivieren möchten, verwenden Sie bitte die `-t`-Option anstelle der Änderung von Umgebungsvariablen.

Beispiel für die Verwendung von Umgebungsvariablen:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Beispiel für die Verwendung von Befehlszeilenoptionen:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Für manuelle Aktionen wie "conv/get/set" usw. hat das Thema, das mit `-t` angegeben wurde, Vorrang vor dem Thema, das aus der Umgebungsvariable gelesen wird.
Wenn `-t` nicht aufgerufen wird, wird stattdessen die Umgebungsvariable verwendet; wenn die Umgebungsvariable leer ist, wird **Monokai Extended** verwendet."##),
        ("theme", r##"Gibt den Namen des Themas für die Syntaxhervorhebung an"##),
    ],
}
}

/// Language ID: de;
/// Map name: "args_md";
/// Description: Deutsch, Lateinisch, Deutschland;
pub(super) const fn get_de_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mGibt die Datei mit den benutzerdefinierten Themen an. Sie müssen auch [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verwenden, um den Namen des Themas anzugeben.[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mErzeugt eine Shell-Autovervollständigung[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mDas unterstützte Dateiformat ist binäre Daten, die von einem rust crate namens [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m erstellt wurden.
[48;2;34;34;34m[38;2;255;255;255mPrinzipiell handelt es sich dabei um Themen für eine ältere Version von Sublime.
[48;2;34;34;34m[38;2;255;255;255mWenn [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m nicht angegeben wird, wird versucht, den Wert aus der Umgebungsvariable [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zu lesen. [0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mGibt das Dateiformat des Quelldokuments manuell an (standardmäßig wird es automatisch erkannt)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mDeaktiviert das Hintergrundbild des Themas (standardmäßig aktiviert). Wenn [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m aufgerufen wird, wird es deaktiviert.[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Die unkomprimierte Version hat Vorrang vor der komprimierten.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWenn [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m nicht angegeben wird, wird versucht, den Wert aus der Umgebungsvariable [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zu lesen. [0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mWenn [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m nicht angegeben wird, wird versucht, den Wert aus der Umgebungsvariable [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zu lesen.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWenn dieser Wert auf [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m gesetzt ist, wird er deaktiviert
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mGibt die Datei mit den unkomprimierten binären Themen an. Auch hier müssen Sie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verwenden, um den Namen des Themas anzugeben.[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mSie müssen den Namen der Shell manuell angeben, für die Sie Autovervollständigungsskripte generieren möchten.
[48;2;34;34;34m[38;2;255;255;255mZum Beispiel können Sie für PowerShell unter Windows Folgendes verwenden: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q: Was ist die Autovervollständigungsfunktion für eine Shell?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: Wenn wir Befehle auf der Kommandozeile eingeben und die Tab-Taste drücken, wird das Autovervollständigungsskript ausgelöst, um mögliche Vervollständigungsoptionen zu durchsuchen und diese Optionen zur Auswahl bereitzustellen. Diese Funktion kann die Effizienz und Genauigkeit der Eingabe auf der Kommandozeile erheblich verbessern.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDie Speicherorte für Autovervollständigungsskripte sind je nach Shell unterschiedlich. Für zsh müssen Sie sie beispielsweise in einem Verzeichnis speichern, das im [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m-Array enthalten ist.
[48;2;34;34;34m[38;2;255;255;255mSie können [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verwenden, um den Inhalt dieses Arrays abzurufen.
[48;2;34;34;34m[38;2;255;255;255mAngenommen, es enthält das Verzeichnis [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Dann können Sie den folgenden Befehl verwenden, um das Autovervollständigungsskript in die Datei [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m in diesem Verzeichnis zu schreiben.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mUm die Verwendung von Root-Rechten zu vermeiden, können Sie das Skript in einem Unterverzeichnis des Benutzerordners ablegen. Beispielsweise können Sie mit dem folgenden Befehl am Anfang des Arrays ein Element einfügen und ein entsprechendes Verzeichnis verwenden.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mWenn Daten aus stdin gelesen werden, z.B. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, wird zuerst versucht, das Dateiformat automatisch zu erkennen.
[48;2;34;34;34m[38;2;255;255;255mDas automatische Erkennen kann jedoch fehlschlagen, in diesem Fall muss das Format manuell angegeben werden.
[48;2;34;34;34m[38;2;255;255;255mBeispiel: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mWenn Sie nur [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verwenden, wird eine Liste der verfügbaren integrierten Stile angezeigt.
[48;2;34;34;34m[38;2;255;255;255mMit Ausnahme von Markdown, das als [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mmd[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m abgekürzt werden kann, müssen für andere Stile vollständige Namen verwendet werden.
[48;2;34;34;34m[38;2;255;255;255mBeispiel 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mBeispiel 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mHinweis: Derzeit wird das Sammeln von [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mTabellenstilen[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m aus einer Datei nicht unterstützt. Wenn Sie diese Funktion oder weitere Funktionen benötigen, können Sie gerne ein Issue erstellen, um Feedback zu geben. [0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mGibt den Stil für Tabellen an[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mWenn Sie nur [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verwenden, wird eine Liste der verfügbaren Themen angezeigt.
[48;2;34;34;34m[38;2;255;255;255mWenn kein Thema angegeben ist, werden die integrierten Themen verwendet.
[48;2;34;34;34m[38;2;255;255;255mAngenommen, es gibt ein Thema namens [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, dann können Sie es mit [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verwenden.
[48;2;34;34;34m[38;2;255;255;255mWenn der Themenname Leerzeichen enthält, sollten Sie ihn in Anführungszeichen setzen.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Hinweis: Es gibt ein spezielles Thema [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, das für einen reinen Textstil ohne Syntaxhervorhebung steht.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWenn [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m nicht angegeben wird, wird versucht, den Wert aus der Umgebungsvariable [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zu lesen.
[48;2;34;34;34m[38;2;255;255;255mDas Thema für die Syntaxhervorhebung in dieser Hilfe stammt ebenfalls aus dieser [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mUmgebungsvariable[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Die Standardhilfe wird hartcodiert geladen und ist sehr schnell.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Der Ladevorgang für hartcodierten Text und für Text mit [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m-Thema ist in derselben Größenordnung, aber der dynamische Ladevorgang ist zwei oder mehr Größenordnungen langsamer.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Das Setzen der [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m-Umgebungsvariable kann dazu führen, dass das Programm das Thema neu lädt, was die Startzeit erheblich verlangsamen kann.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Wenn Sie das Thema für die Hilfe nicht ändern oder die Syntaxhervorhebung deaktivieren möchten, verwenden Sie bitte die [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m-Option anstelle der Änderung von Umgebungsvariablen.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mBeispiel für die Verwendung von Umgebungsvariablen:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mBeispiel für die Verwendung von Befehlszeilenoptionen:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mFür manuelle Aktionen wie "conv/get/set" usw. hat das Thema, das mit [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m angegeben wurde, Vorrang vor dem Thema, das aus der Umgebungsvariable gelesen wird.
[48;2;34;34;34m[38;2;255;255;255mWenn [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m nicht aufgerufen wird, wird stattdessen die Umgebungsvariable verwendet; wenn die Umgebungsvariable leer ist, wird [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m verwendet.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mGibt den Namen des Themas für die Syntaxhervorhebung an[0m"##),
    ],
}
}

/// Language ID: de;
/// Map name: "conv";
/// Description: Deutsch, Lateinisch, Deutschland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Konvertiere zwischen den Formaten TOML, YAML, JSON und RON");
/// ```
pub(super) const fn get_de_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Konvertiere zwischen den Formaten TOML, YAML, JSON und RON"##),
        ("from", r##"Gibt die Eingabedatei an"##),
        ("conv-to", r##"Gibt das Ausgabeformat und/oder die Ausgabedatei an, kann mehrfach genutzt werden"##),
        ("save", r##"Wenn diese Option aktiviert ist, wird die mit `--to` angegebene Datei gespeichert."##),
        ("conv-to-help", r##"Wenn in dem aktuellen Verzeichnis eine Datei namens **test.json** vorhanden ist, können Sie sie mit `conv test.json -t yml` in das Format YML (YAML) konvertieren oder Sie können den Dateipfad manuell angeben: `conv test.json -t /tmp/test.yml`.

Die unterstützten Formate hängen von den Funktionen ab, die von diesem Softwarepaket aktiviert wurden. Nicht alle Funktionen sind standardmäßig aktiviert.

Es gibt jedoch einige wichtige Informationen, die vor der Verwendung dieser Anwendung beachtet werden sollten:
- 1. Die Anwendung benennt automatisch Dateien um, wenn keine spezifischen Dateipfade angegeben sind.
  - 1.1: Nur Ein-Schicht-Dateien werden als Dateiformat erkannt, nicht als Dateipfad.
    Wenn Sie beispielsweise `conv test.toml -t ron` aufrufen, wird die Ausgabedatei automatisch in `test.ron` umbenannt, anstatt `ron`. Wenn Sie diese Funktion nicht benötigen, verwenden Sie bitte `-t ./ron` oder `-t $PWD/ron`.
  - 1.2: Wenn auf ein Verzeichnis und nicht auf eine Datei verwiesen wird
    Zum Beispiel `-t /tmp/`, können wir nur `test.toml` in einem Unterordner von `/tmp/` platzieren, anstatt `/tmp` selbst zu überschreiben. Wenn Sie diese Funktion nicht benötigen, verwenden Sie bitte den vollständigen (nicht-verzeichnis-) Dateipfad, z.B. `-t /tmp/xx.toml` statt `-t /tmp/`.
- 2. Ohne `--save` wird die angegebene Datei nicht automatisch gespeichert.
- 3. Wenn `--save` aufgerufen wird, dürfen die Eingabe- und Ausgabedateien des `conv`-Befehls nicht identisch sein.
- 4. Konvertierte Dateien behalten keine Kommentare bei."##),
    ],
}
}

/// Language ID: de;
/// Map name: "conv_md";
/// Description: Deutsch, Lateinisch, Deutschland;
pub(super) const fn get_de_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mKonvertiere zwischen den Formaten TOML, YAML, JSON und RON[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mGibt die Eingabedatei an[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mGibt das Ausgabeformat und/oder die Ausgabedatei an, kann mehrfach genutzt werden[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mWenn diese Option aktiviert ist, wird die mit [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m angegebene Datei gespeichert.[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mWenn in dem aktuellen Verzeichnis eine Datei namens [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m vorhanden ist, können Sie sie mit [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m in das Format YML (YAML) konvertieren oder Sie können den Dateipfad manuell angeben: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDie unterstützten Formate hängen von den Funktionen ab, die von diesem Softwarepaket aktiviert wurden. Nicht alle Funktionen sind standardmäßig aktiviert.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEs gibt jedoch einige wichtige Informationen, die vor der Verwendung dieser Anwendung beachtet werden sollten:
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Die Anwendung benennt automatisch Dateien um, wenn keine spezifischen Dateipfade angegeben sind.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Nur Ein-Schicht-Dateien werden als Dateiformat erkannt, nicht als Dateipfad.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mWenn Sie beispielsweise [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m aufrufen, wird die Ausgabedatei automatisch in [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m umbenannt, anstatt [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Wenn Sie diese Funktion nicht benötigen, verwenden Sie bitte [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m oder [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: Wenn auf ein Verzeichnis und nicht auf eine Datei verwiesen wird
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mZum Beispiel [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, können wir nur [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m in einem Unterordner von [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m platzieren, anstatt [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m selbst zu überschreiben. Wenn Sie diese Funktion nicht benötigen, verwenden Sie bitte den vollständigen (nicht-verzeichnis-) Dateipfad, z.B. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m statt [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. Ohne [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m wird die angegebene Datei nicht automatisch gespeichert.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. Wenn [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m aufgerufen wird, dürfen die Eingabe- und Ausgabedateien des [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m-Befehls nicht identisch sein.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. Konvertierte Dateien behalten keine Kommentare bei.[0m"##),
    ],
}
}

/// Language ID: de;
/// Map name: "opt";
/// Description: Deutsch, Lateinisch, Deutschland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "Array von Tabellen (Verwendung: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_de_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"Im Gegensatz zu `-k` spaltet `--ck` nicht automatisch `.`.
Für [a.b.c] ist `-k a.b.c` gleichbedeutend mit `--ck a --ck b --ck c`.
Für einfache Fälle ist es bequemer, `-k` zu verwenden.

Wenn der Schlüsselname '.' enthält, verwenden Sie bitte `--ck` anstelle von `-k`.
Zum Beispiel:

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

In dieser Struktur ist "editor.defaultFormatter" ein vollständiger String, "defaultFormatter" ist kein Unterkey von "editor".

Wenn Sie `-k '[typescript]'.'editor.defaultFormatter'` verwenden, wird der Schlüssel wie folgt aufgeteilt:

- "[typescript]"
- "editor"
- "defaultFormatter"

Wenn Sie jedoch `--ck '[typescript]' --ck editor.defaultFormatter` verwenden, wird der Schlüssel als

- "[typescript]"
- "editor.defaultFormatter"

behandelt, was die korrekte Struktur ist."##),
        ("num-arr", r##"i64-Array"##),
        ("bool", r##"bool (Boolescher Typ), kann nur `true` oder `false` sein"##),
        ("num", r##"i64: 64-Bit-Integer-Typ, z.B.: 1145"##),
        ("datetime-help", r##"Beispiel:
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

Diese Option wird nur von `toml` unterstützt. Wenn wir am Ende des Befehls `-t json` hinzufügen, wird es so aussehen:

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"Speichern Sie es als ein bestimmtes Format / eine bestimmte Datei, jeweils nur eins."##),
        ("get-opts-help", r##"Beispiel 1: `get Cargo.toml -k package.name`
Beispiel 2: `get cfg.yaml -k .`
"##),
        ("none-help", r##"> Hinweis: Ron hat den echten `Option<T>`-Typ, der entweder `Some(T)` oder `None` ist.
> Hier bezieht sich das `--none` auf das Löschen des angegebenen Schlüssels und nicht auf den tatsächlichen `Option<T>`-Typ.

Angenommen, test.yml enthält das folgende Tabellenarray:

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

Wenn wir z.B. **name: test** entfernen möchten, können wir Folgendes ausführen:
```sh
set test.yml -k bin.1.name --none
```

> Hinweis: Ein leerer String und `None` sind nicht dasselbe.
> Der Aufruf von `set test.yml --kv bin.1.name ""` setzt es auf einen leeren String, während `--none` den gesamten Schlüssel löscht.

Wenn wir die zweite Tabelle (Index 1) in bin löschen möchten, können wir Folgendes tun:
```sh
set test.yml -k bin.1 --none
```

Oder benutzen Sie `--rm`
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"Double Precision Floating Point (IEEE 754-2008 Standard), z.B.: 3,14159265359"##),
        ("saot-help", r##"> Hinweis: Obwohl über `--saot` nur ein K-V-Paar pro Tabelle erstellt werden kann, können Sie den Index des Tabellenarrays zuerst spezifizieren und dann andere Optionen verwenden, um ihn zu erweitern.

Beispiel:
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"Setze k-v pair, `--kv x.y.z value` entspricht `-k x.y.z -s value`"##),
        ("set-save", r##"Wenn ohne `--to` verwendet, wird die aktuelle Datei überschrieben (geschrieben), wenn aufgerufen."##),
        ("array", r##"Zeichenfolgen-Array"##),
        ("aot-help", r##"Beispiel:
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

Wählen Sie zunächst durch Angabe des Schlüssels aus und bearbeiten Sie ihn dann durch Angabe des Werts des Datentyps.

> Wenn nur der Schlüssel angegeben wird und kein Wert angegeben wird, werden der entsprechende Datentyp und der Wert aufgelistet.
> Zum Beispiel gibt `set Cargo.toml -k package` die Tabelle von package und seinen Untertabellen an und bietet detailliertere Informationen als `get`.

## Beispiele

### Beispiel 1:
```sh
--ts md set test.toml --key package.edition --str 2021
```

Da `--str` statt `--num` angegeben ist, ist "2021" ein String-Typ und kein Integer-Typ.

| package | new value |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### Beispiel 2:
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### Beispiel 3:
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: Dies ist der erste Index*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> Hinweis: Die obigen Beispiele beginnen mit einem Argument (arg) / Unterbefehl (sub-command) und lassen den Namen des Stamm-Befehls weg. In der tatsächlichen Bedienung müssen Sie ihn manuell ergänzen.
"##),
        ("num-arr-help", r##"Beispiel:
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"Wenn Sie sich nicht sicher sind, ob Sie die Standardtabelle oder die Inlinetabelle verwenden sollen, dann verwenden Sie die Inlinetabelle!

Beispiel:
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> Mit Ausnahme von `toml` verwenden andere Formate eine Standardtabelle anstelle einer Inline-Tabelle.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"Abfrage des Werts durch den Schlüssel"##),
        ("f64-arr-help", r##"Beispiel:
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> JSON 1.0 unterstützt kein NaN und Inf.

Es gibt `null` aus, anstatt `NaN`.

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"Löschen des angegebenen Schlüssels"##),
        ("set-save-to-help", r##"Ähnlich wie das `-t` Argument in dem `conv` Sub-Befehl. Der größte Unterschied besteht darin, dass `-t` im `conv` Sub-Befehl mehrmals aufeinanderfolgend aufgerufen werden kann, um in mehrere Dateiformate / Dateien zu exportieren.
Während `-t` im `set` Sub-Befehl nur einmal aufgerufen werden kann, um in eine einzelne Datei zu exportieren.

> Hinweis: Nach der Umwandlung mit dem `conv` Befehl werden alle Dateiformate ohne Kommentare gespeichert.
> Währenddessen behält der `set` Befehl die Kommentare von **toml** Dateien bei. In allen anderen Formaten **werden keine** Kommentare beibehalten.

- Wenn Sie -t nicht aufrufen, überschreiben Sie sich selbst, wenn Sie `--save` aufrufen.
- Wenn Sie -t bereits aufgerufen haben und auch `--save` aufrufen, wird die modifizierte Datei in die von -t angegebene Datei gespeichert.
- Wenn sich die Quelldatei von der vom -t angegebenen Datei unterscheidet, verhält sich -t genauso wie der Teil -t von `conv`. Beide ändern den Dateinamen automatisch für Nicht-Konkrete Pfadangaben.
  - Angenommen from: **test.yml**, to: **json**, wird die Ausgabedatei zu **test.json** und nicht zu **json** umbenannt.
  - Wenn Sie diese Funktion nicht benötigen, verwenden Sie bitte `-t ./json` oder `-t $PWD/json`, anstelle von`-t json`."##),
        ("preview", r##"Nach Abschluss der Änderungen die gesamte Datei anzeigen"##),
        ("set-key", r##"Wähle einen bestimmten Schlüssel (key) aus, indem du "." als Trennzeichen verwendest."##),
        ("get-opts", r##"Holen Sie den Wert einer bestimmten Konfiguration"##),
        ("saot", r##"Array von Einzel-K-V-Paaren von Tabellen (Verwendung: `--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"Das Format lautet: "a.b.c", wobei der Name der Untertabelle oder der Array-Index durch "." (halber Punkt) verbunden ist.
Wenn es nur einen "." gibt, wird die gesamte Tabelle aufgelistet"##),
        ("set-opts", r##"Erstellen / Aktualisieren / Löschen der Struktur einer Konfigurationsdatei"##),
        ("bool-arr-help", r##"Beispiel:
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: Das erste Element ist true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"bool-Array"##),
        ("table", r##"Standardtabelle zum Speichern von mehreren K-V-Paaren."##),
        ("datetime", r##"Datums- und Uhrzeittyp"##),
        ("table-help", r##"Beispiel 1:
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

Beispiel 2:
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"f64-Array"##),
        ("string", r##"String (Zeichenfolgentyp)"##),
        ("array-help", r##"Verwenden Sie `-a` mehrmals, um Array-Elemente festzulegen.
Beispiel:
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> Legen Sie beim ersten Aufruf das Element mit Index 0 fest und legen Sie beim zweiten Aufruf das Element mit Index 1 fest usw.

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"Array von Tabellen (Verwendung: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"Inline-Tabelle, ähnlich wie eine kompaktere Standard-Tabelle."##),
        ("concat-key", r##"Gib den Schlüssel in Abschnitten ein, das Ergebnis wird automatisch zusammengesetzt, ohne "." als Trennzeichen zu verwenden."##),
    ],
}
}

/// Language ID: de;
/// Map name: "opt_md";
/// Description: Deutsch, Lateinisch, Deutschland;
pub(super) const fn get_de_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255mIm Gegensatz zu [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m spaltet [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m nicht automatisch [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m.[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mFür [a.b.c] ist [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m gleichbedeutend mit [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mFür einfache Fälle ist es bequemer, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zu verwenden.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWenn der Schlüsselname '.' enthält, verwenden Sie bitte [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m anstelle von [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mZum Beispiel:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIn dieser Struktur ist "editor.defaultFormatter" ein vollständiger String, "defaultFormatter" ist kein Unterkey von "editor".
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWenn Sie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verwenden, wird der Schlüssel wie folgt aufgeteilt:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWenn Sie jedoch [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verwenden, wird der Schlüssel als
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mbehandelt, was die korrekte Struktur ist.[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mi64-Array[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mbool (Boolescher Typ), kann nur [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m oder [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sein[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: 64-Bit-Integer-Typ, z.B.: 1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255mBeispiel:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDiese Option wird nur von [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m unterstützt. Wenn wir am Ende des Befehls [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m hinzufügen, wird es so aussehen:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255mSpeichern Sie es als ein bestimmtes Format / eine bestimmte Datei, jeweils nur eins.[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255mBeispiel 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mBeispiel 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Hinweis: Ron hat den echten [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m-Typ, der entweder [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m oder [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m ist.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Hier bezieht sich das [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m auf das Löschen des angegebenen Schlüssels und nicht auf den tatsächlichen [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m-Typ.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAngenommen, test.yml enthält das folgende Tabellenarray:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWenn wir z.B. [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mname: test[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m entfernen möchten, können wir Folgendes ausführen:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Hinweis: Ein leerer String und [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m sind nicht dasselbe.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Der Aufruf von [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m setzt es auf einen leeren String, während [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m den gesamten Schlüssel löscht.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWenn wir die zweite Tabelle (Index 1) in bin löschen möchten, können wir Folgendes tun:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOder benutzen Sie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255mDouble Precision Floating Point (IEEE 754-2008 Standard), z.B.: 3,14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Hinweis: Obwohl über [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m nur ein K-V-Paar pro Tabelle erstellt werden kann, können Sie den Index des Tabellenarrays zuerst spezifizieren und dann andere Optionen verwenden, um ihn zu erweitern.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mBeispiel:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mSetze k-v pair, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m entspricht [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255mWenn ohne [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verwendet, wird die aktuelle Datei überschrieben (geschrieben), wenn aufgerufen.[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255mZeichenfolgen-Array[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255mBeispiel:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWählen Sie zunächst durch Angabe des Schlüssels aus und bearbeiten Sie ihn dann durch Angabe des Werts des Datentyps.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Wenn nur der Schlüssel angegeben wird und kein Wert angegeben wird, werden der entsprechende Datentyp und der Wert aufgelistet.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Zum Beispiel gibt [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m die Tabelle von package und seinen Untertabellen an und bietet detailliertere Informationen als [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mBeispiele[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mBeispiel 1:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDa [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m statt [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m angegeben ist, ist "2021" ein String-Typ und kein Integer-Typ.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m new value [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mBeispiel 2:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mBeispiel 3:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: Dies ist der erste Index*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Hinweis: Die obigen Beispiele beginnen mit einem Argument (arg) / Unterbefehl (sub-command) und lassen den Namen des Stamm-Befehls weg. In der tatsächlichen Bedienung müssen Sie ihn manuell ergänzen.
[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mBeispiel:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255mWenn Sie sich nicht sicher sind, ob Sie die Standardtabelle oder die Inlinetabelle verwenden sollen, dann verwenden Sie die Inlinetabelle!
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mBeispiel:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Mit Ausnahme von [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m verwenden andere Formate eine Standardtabelle anstelle einer Inline-Tabelle.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mAbfrage des Werts durch den Schlüssel[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mBeispiel:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m JSON 1.0 unterstützt kein NaN und Inf.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEs gibt [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m aus, anstatt [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255mLöschen des angegebenen Schlüssels[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mÄhnlich wie das [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m Argument in dem [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m Sub-Befehl. Der größte Unterschied besteht darin, dass [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m im [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m Sub-Befehl mehrmals aufeinanderfolgend aufgerufen werden kann, um in mehrere Dateiformate / Dateien zu exportieren.
[48;2;34;34;34m[38;2;255;255;255mWährend [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m im [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m Sub-Befehl nur einmal aufgerufen werden kann, um in eine einzelne Datei zu exportieren.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Hinweis: Nach der Umwandlung mit dem [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m Befehl werden alle Dateiformate ohne Kommentare gespeichert.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Währenddessen behält der [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m Befehl die Kommentare von [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m Dateien bei. In allen anderen Formaten [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mwerden keine[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m Kommentare beibehalten.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mWenn Sie -t nicht aufrufen, überschreiben Sie sich selbst, wenn Sie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m aufrufen.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Wenn Sie -t bereits aufgerufen haben und auch [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m aufrufen, wird die modifizierte Datei in die von -t angegebene Datei gespeichert.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Wenn sich die Quelldatei von der vom -t angegebenen Datei unterscheidet, verhält sich -t genauso wie der Teil -t von [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Beide ändern den Dateinamen automatisch für Nicht-Konkrete Pfadangaben.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Angenommen from: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, to: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, wird die Ausgabedatei zu [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m und nicht zu [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m umbenannt.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Wenn Sie diese Funktion nicht benötigen, verwenden Sie bitte [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m oder [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, anstelle von[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255mNach Abschluss der Änderungen die gesamte Datei anzeigen[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255mWähle einen bestimmten Schlüssel (key) aus, indem du "." als Trennzeichen verwendest.[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255mHolen Sie den Wert einer bestimmten Konfiguration[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255mArray von Einzel-K-V-Paaren von Tabellen (Verwendung: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mDas Format lautet: "a.b.c", wobei der Name der Untertabelle oder der Array-Index durch "." (halber Punkt) verbunden ist.
[48;2;34;34;34m[38;2;255;255;255mWenn es nur einen "." gibt, wird die gesamte Tabelle aufgelistet[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255mErstellen / Aktualisieren / Löschen der Struktur einer Konfigurationsdatei[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mBeispiel:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: Das erste Element ist true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mbool-Array[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255mStandardtabelle zum Speichern von mehreren K-V-Paaren.[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255mDatums- und Uhrzeittyp[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255mBeispiel 1:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mBeispiel 2:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mf64-Array[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString (Zeichenfolgentyp)[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255mVerwenden Sie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m mehrmals, um Array-Elemente festzulegen.
[48;2;34;34;34m[38;2;255;255;255mBeispiel:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Legen Sie beim ersten Aufruf das Element mit Index 0 fest und legen Sie beim zweiten Aufruf das Element mit Index 1 fest usw.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255mArray von Tabellen (Verwendung: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mInline-Tabelle, ähnlich wie eine kompaktere Standard-Tabelle.[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mGib den Schlüssel in Abschnitten ein, das Ergebnis wird automatisch zusammengesetzt, ohne "." als Trennzeichen zu verwenden.[0m"##),
    ],
}
}

/// Language ID: de;
/// Map name: "parser";
/// Description: Deutsch, Lateinisch, Deutschland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "Das folgende ist die stderr-Ausgabe. Wenn Sie die Ausgabe des Autovervollständigungsskripts umleiten möchten, sollten Sie `>/path/to/file` anstelle von `2>/path/to/file` verwenden.");
/// ```
pub(super) const fn get_de_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"Das folgende ist die stderr-Ausgabe. Wenn Sie die Ausgabe des Autovervollständigungsskripts umleiten möchten, sollten Sie `>/path/to/file` anstelle von `2>/path/to/file` verwenden."##),
    ],
}
}

/// Language ID: de;
/// Map name: "theme";
/// Description: Deutsch, Lateinisch, Deutschland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ungültiger Themenname");
/// ```
pub(super) const fn get_de_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ungültiger Themenname"##),
    ],
}
}

/// de: Deutsch, Lateinisch, Deutschland
pub(super) const fn get_de_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_de_map_args),
        ("conv_md", get_de_map_conv_md),
        ("parser", get_de_map_parser),
        ("opt_md", get_de_map_opt_md),
        ("args_md", get_de_map_args_md),
        ("opt", get_de_map_opt),
        ("log-core", get_de_map_log_core),
        ("conv", get_de_map_conv),
        ("theme", get_de_map_theme),
    ],
}
}

/// Language ID: el;
/// Map name: "log-core";
/// Description: Ελληνικά, Ελληνικό, Ελλάδα;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "σφαλμάτων");
/// ```
pub(super) const fn get_el_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"σφαλμάτων"##),
        ("warn", r##"προειδοποιήστε"##),
        ("info", r##"Πληροφορίες"##),
        ("error", r##"Σφάλμα"##),
        ("trace", r##"ίχνος"##),
        ("init-logger", r##"Αρχικοποίηση του καταγραφέα ..."##),
    ],
}
}

/// Language ID: el;
/// Map name: "args";
/// Description: Ελληνικά, Ελληνικό, Ελλάδα;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Απενεργοποίηση φόντου θέματος (προεπιλογή είναι η ενεργοποίηση, εάν χρησιμοποιηθεί η επιλογή `--dis-theme-bg`, τότε απενεργοποιείται)");
/// ```
pub(super) const fn get_el_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"Αρχείο συλλογής προσαρμοσμένων θεμάτων, χρειάζεται επίσης να καθορίσετε το όνομα του θέματος με το `--theme`"##),
        ("shell-completion", r##"Δημιουργεί ένα σενάριο συμπλήρωσης για το shell"##),
        ("theme-file-help", r##"Υποστηριζόμενη μορφή αρχείων είναι δυαδικά δεδομένα τα οποία έχουν δημιουργηθεί με την εντολή `syntect` της βιβλιοθήκης Rust Crate. 
Βασικά, αυτά τα θέματα είναι θέματα της παλιάς έκδοσης του Sublime.
Εάν δεν καθοριστεί το `--tf`, τότε διαβάζεται η τιμή της μεταβλητής περιβάλλοντος `$TOMLYRE_THEME_FILE`."##),
        ("from-format", r##"Χειροκίνητη καθορισή της μορφής αρχείου προέλευσης (αυτόματη ανίχνευση αν δεν καθοριστεί)"##),
        ("disable-theme-background", r##"Απενεργοποίηση φόντου θέματος (προεπιλογή είναι η ενεργοποίηση, εάν χρησιμοποιηθεί η επιλογή `--dis-theme-bg`, τότε απενεργοποιείται)"##),
        ("theme-uncompressed-file-help", r##"> Η μη συμπιεσμένη έκδοση έχει προτεραιότητα από τη συμπιεσμένη. 

Εάν δεν καθοριστεί το `--tuf`, τότε διαβάζεται η τιμή της μεταβλητής περιβάλλοντος `$TOMLYRE_THEME_UNCOMPRESSED_FILE`."##),
        ("disable-theme-background-help", r##"Εάν δεν καθοριστεί το `--dis-theme-bg`, τότε διαβάζεται η τιμή της μεταβλητής περιβάλλοντος `$TOMLYRE_THEME_BG`.

Εάν αυτή η τιμή είναι false, τότε η απενεργοποίηση γίνεται ως εξής:

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"Παρόμοιο με το `theme-file`, αλλά απαιτεί μη συμπιεσμένα δεδομένα dump (αρχείο)"##),
        ("shell-completion-help", r##"Πρέπει να καθορίσετε χειροκίνητα το όνομα του shell που χρησιμοποιείτε, προκειμένου να δημιουργήσετε ένα σενάριο συμπλήρωσης.
Για παράδειγμα, για το powershell σε Windows, μπορείτε να χρησιμοποιήσετε: `--sh-comp pwsh`

> Ε: Τι είναι η λειτουργία συμπλήρωσης για το shell;

> Α: Όταν πληκτρολογούμε μια εντολή στο κέλυφος (shell), αν πατήσουμε το πλήκτρο Tab, θα ενεργοποιηθεί ένα σενάριο συμπλήρωσης που θα αναζητήσει τις πιθανές επιλογές συμπλήρωσης και θα τις παρουσιάσει για επιλογή. Αυτή η λειτουργία μπορεί να βελτιώσει σημαντικά την αποδοτικότητα και την ακρίβεια της εισαγωγής εντολών στο κέλυφος.

Τα σενάρια συμπλήρωσης για διαφορετικά shell αποθηκεύονται σε διαφορετικές διαδρομές. Για παράδειγμα, για το zsh, πρέπει να τοποθετήσετε το σενάριο συμπλήρωσης σε μια από τις διαδρομές που περιλαμβάνονται στον πίνακα `$fpath`.
Μπορείτε να χρησιμοποιήσετε την εντολή `print -l $fpath` για να πάρετε το περιεχόμενο αυτού του πίνακα.
Ας υποθέσουμε ότι περιλαμβάνει το φάκελο `/usr/local/share/zsh/site-functions`, τότε μπορείτε να χρησιμοποιήσετε την παρακάτω εντολή για να γράψετε το σενάριο συμπλήρωσης στο αρχείο `_tomlyre` στον εν λόγω φάκελο.

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

Για να αποφύγετε τη χρήση του root ως χρήστης, μπορείτε να τοποθετήσετε το αρχείο στον φάκελο του χρήστη. Για παράδειγμα, μπορείτε να χρησιμοποιήσετε την παρακάτω εντολή για να εισάγετε ένα νέο στοιχείο στην αρχή του πίνακα fpath και στη συνέχεια να χρησιμοποιήσετε τον αντίστοιχο φάκελο:

```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```
"##),
        ("from-format-help", r##"Όταν διαβάζεται είσοδος από το stdin, για παράδειγμα: `cat a | tomlyre conv -`, προσπαθεί αυτόματα να ανιχνεύσει τη μορφή του αρχείου.
Ωστόσο, η αυτόματη ανίχνευση μπορεί να αποτύχει, οπότε απαιτείται η χειροκίνητη καθορισή της μορφής.
Παράδειγμα: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Όταν χρησιμοποιείται μόνο το `--ts` παράμετρος, εμφανίζεται η λίστα των ενσωματωμένων συλλογών στυλ.
Εκτός από το markdown, το οποίο μπορεί να συντομευθεί σε md, για όλα τα άλλα στυλ πρέπει να χρησιμοποιείτε το πλήρες όνομα.
Παράδειγμα χρήσης 1: `--ts md`
Παράδειγμα χρήσης 2: `--table-style ascii`

Σημείωση: Το παρόν δεν υποστηρίζει την ανάγνωση συλλογών **στυλ πινάκων** από αρχεία. Εάν χρειάζεστε αυτήν τη λειτουργία ή άλλες πρόσθετες λειτουργίες, μπορείτε να υποβάλλετε ένα issue για ανατροφοδότηση."##),
        ("table-style", r##"Καθορίζει το στυλ του πίνακα"##),
        ("theme-help", r##"Όταν χρησιμοποιείται μόνο η επιλογή `-t`, θα εμφανίζονται τα διαθέσιμα θέματα.
Εάν δεν καθοριστεί το αρχείο θέματος, θα χρησιμοποιηθεί η προεπιλεγμένη συλλογή θεμάτων.
Για παράδειγμα, εάν υπάρχει ένα θέμα με το όνομα **ayu-dark**, μπορείτε να χρησιμοποιήσετε την `-t ayu-dark` για να χρησιμοποιήσετε αυτό το θέμα.
Εάν το όνομα του θέματος περιέχει κενά, χρησιμοποιήστε αποστρόφους ή εισαγωγικά για να το περικλείσετε.

> Σημείωση: Υπάρχει ένα ειδικό θέμα με το όνομα `None`, το οποίο αντιστοιχεί σε ένα απλό κείμενο χωρίς σημείωση συντακτικού.

Εάν δεν καθοριστεί η επιλογή `--theme`, θα διαβαστεί από τη μεταβλητή περιβάλλοντος `$TOMLYRE_THEME`.
Το θέμα που χρησιμοποιείται για την επισήμανση του συντακτικού της βοήθειας αυτής προέρχεται επίσης από αυτήν τη μεταβλητή περιβάλλοντος.

> Η προεπιλεγμένη βοήθεια είναι χωρίς σχόλια και φορτώνεται από τον κώδικα, για μεγαλύτερη ταχύτητα.  
> Η φόρτωση του σκληρού κώδικα και της βοήθειας που δεν έχει επισήμανση συντακτικού έχει την ίδια τάξη μεγέθους, ενώ η δυναμική φόρτωση έχει διαφορετική τάξη μεγέθους και είναι πιο αργή.

> Η ρύθμιση της μεταβλητής περιβάλλοντος `$TOMLYRE_THEME` μπορεί να οδηγήσει στην επαναφόρτωση του θέματος, πράγμα που μπορεί να επηρεάσει αρνητικά την ταχύτητα εκκίνησης.  
> Εκτός εάν πρόκειται να αλλάξετε το θέμα της βοήθειας ή να απενεργοποιήσετε την επισήμανση συντακτικού, χρησιμοποιήστε το `-t` αντί να τροποποιείτε τη μεταβλητή περιβάλλοντος.

Παράδειγμα με χρήση μεταβλητής περιβάλλοντος:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Παράδειγμα με χρήση παραμέτρου γραμμής εντολών:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Για τα περιεχόμενα που παράγονται από εντολές όπως `conv/get/set`, η προτεραιότητα δίνεται στο θέμα που καθορίζεται από την επιλογή `-t`. Εάν δε δοθεί η επιλογή `-t`, χρησιμοποιείται η μεταβλητή περιβάλλοντος. Εάν η μεταβλητή περιβάλλοντος δεν έχει τιμή, χρησιμοποιείται το θέμα **Monokai Extended**."##),
        ("theme", r##"Καθορίζει το όνομα του θέματος για την επισήμανση του συντακτικού. Αν δεν καθοριστεί, θα χρησιμοποιηθεί το προεπιλεγμένο θέμα."##),
    ],
}
}

/// Language ID: el;
/// Map name: "args_md";
/// Description: Ελληνικά, Ελληνικό, Ελλάδα;
pub(super) const fn get_el_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mΑρχείο συλλογής προσαρμοσμένων θεμάτων, χρειάζεται επίσης να καθορίσετε το όνομα του θέματος με το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mΔημιουργεί ένα σενάριο συμπλήρωσης για το shell[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mΥποστηριζόμενη μορφή αρχείων είναι δυαδικά δεδομένα τα οποία έχουν δημιουργηθεί με την εντολή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m της βιβλιοθήκης Rust Crate. 
[48;2;34;34;34m[38;2;255;255;255mΒασικά, αυτά τα θέματα είναι θέματα της παλιάς έκδοσης του Sublime.
[48;2;34;34;34m[38;2;255;255;255mΕάν δεν καθοριστεί το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, τότε διαβάζεται η τιμή της μεταβλητής περιβάλλοντος [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mΧειροκίνητη καθορισή της μορφής αρχείου προέλευσης (αυτόματη ανίχνευση αν δεν καθοριστεί)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mΑπενεργοποίηση φόντου θέματος (προεπιλογή είναι η ενεργοποίηση, εάν χρησιμοποιηθεί η επιλογή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, τότε απενεργοποιείται)[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Η μη συμπιεσμένη έκδοση έχει προτεραιότητα από τη συμπιεσμένη. 
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΕάν δεν καθοριστεί το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, τότε διαβάζεται η τιμή της μεταβλητής περιβάλλοντος [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mΕάν δεν καθοριστεί το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, τότε διαβάζεται η τιμή της μεταβλητής περιβάλλοντος [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΕάν αυτή η τιμή είναι false, τότε η απενεργοποίηση γίνεται ως εξής:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mΠαρόμοιο με το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, αλλά απαιτεί μη συμπιεσμένα δεδομένα dump (αρχείο)[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mΠρέπει να καθορίσετε χειροκίνητα το όνομα του shell που χρησιμοποιείτε, προκειμένου να δημιουργήσετε ένα σενάριο συμπλήρωσης.
[48;2;34;34;34m[38;2;255;255;255mΓια παράδειγμα, για το powershell σε Windows, μπορείτε να χρησιμοποιήσετε: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Ε: Τι είναι η λειτουργία συμπλήρωσης για το shell;
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Α: Όταν πληκτρολογούμε μια εντολή στο κέλυφος (shell), αν πατήσουμε το πλήκτρο Tab, θα ενεργοποιηθεί ένα σενάριο συμπλήρωσης που θα αναζητήσει τις πιθανές επιλογές συμπλήρωσης και θα τις παρουσιάσει για επιλογή. Αυτή η λειτουργία μπορεί να βελτιώσει σημαντικά την αποδοτικότητα και την ακρίβεια της εισαγωγής εντολών στο κέλυφος.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΤα σενάρια συμπλήρωσης για διαφορετικά shell αποθηκεύονται σε διαφορετικές διαδρομές. Για παράδειγμα, για το zsh, πρέπει να τοποθετήσετε το σενάριο συμπλήρωσης σε μια από τις διαδρομές που περιλαμβάνονται στον πίνακα [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mΜπορείτε να χρησιμοποιήσετε την εντολή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m για να πάρετε το περιεχόμενο αυτού του πίνακα.
[48;2;34;34;34m[38;2;255;255;255mΑς υποθέσουμε ότι περιλαμβάνει το φάκελο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, τότε μπορείτε να χρησιμοποιήσετε την παρακάτω εντολή για να γράψετε το σενάριο συμπλήρωσης στο αρχείο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m στον εν λόγω φάκελο.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΓια να αποφύγετε τη χρήση του root ως χρήστης, μπορείτε να τοποθετήσετε το αρχείο στον φάκελο του χρήστη. Για παράδειγμα, μπορείτε να χρησιμοποιήσετε την παρακάτω εντολή για να εισάγετε ένα νέο στοιχείο στην αρχή του πίνακα fpath και στη συνέχεια να χρησιμοποιήσετε τον αντίστοιχο φάκελο:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mΌταν διαβάζεται είσοδος από το stdin, για παράδειγμα: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, προσπαθεί αυτόματα να ανιχνεύσει τη μορφή του αρχείου.
[48;2;34;34;34m[38;2;255;255;255mΩστόσο, η αυτόματη ανίχνευση μπορεί να αποτύχει, οπότε απαιτείται η χειροκίνητη καθορισή της μορφής.
[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mΌταν χρησιμοποιείται μόνο το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m παράμετρος, εμφανίζεται η λίστα των ενσωματωμένων συλλογών στυλ.
[48;2;34;34;34m[38;2;255;255;255mΕκτός από το markdown, το οποίο μπορεί να συντομευθεί σε md, για όλα τα άλλα στυλ πρέπει να χρησιμοποιείτε το πλήρες όνομα.
[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα χρήσης 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα χρήσης 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΣημείωση: Το παρόν δεν υποστηρίζει την ανάγνωση συλλογών [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mστυλ πινάκων[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m από αρχεία. Εάν χρειάζεστε αυτήν τη λειτουργία ή άλλες πρόσθετες λειτουργίες, μπορείτε να υποβάλλετε ένα issue για ανατροφοδότηση.[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mΚαθορίζει το στυλ του πίνακα[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mΌταν χρησιμοποιείται μόνο η επιλογή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, θα εμφανίζονται τα διαθέσιμα θέματα.
[48;2;34;34;34m[38;2;255;255;255mΕάν δεν καθοριστεί το αρχείο θέματος, θα χρησιμοποιηθεί η προεπιλεγμένη συλλογή θεμάτων.
[48;2;34;34;34m[38;2;255;255;255mΓια παράδειγμα, εάν υπάρχει ένα θέμα με το όνομα [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, μπορείτε να χρησιμοποιήσετε την [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m για να χρησιμοποιήσετε αυτό το θέμα.
[48;2;34;34;34m[38;2;255;255;255mΕάν το όνομα του θέματος περιέχει κενά, χρησιμοποιήστε αποστρόφους ή εισαγωγικά για να το περικλείσετε.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Σημείωση: Υπάρχει ένα ειδικό θέμα με το όνομα [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, το οποίο αντιστοιχεί σε ένα απλό κείμενο χωρίς σημείωση συντακτικού.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΕάν δεν καθοριστεί η επιλογή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, θα διαβαστεί από τη μεταβλητή περιβάλλοντος [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mΤο θέμα που χρησιμοποιείται για την επισήμανση του συντακτικού της βοήθειας αυτής προέρχεται επίσης από αυτήν τη μεταβλητή περιβάλλοντος.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Η προεπιλεγμένη βοήθεια είναι χωρίς σχόλια και φορτώνεται από τον κώδικα, για μεγαλύτερη ταχύτητα.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Η φόρτωση του σκληρού κώδικα και της βοήθειας που δεν έχει επισήμανση συντακτικού έχει την ίδια τάξη μεγέθους, ενώ η δυναμική φόρτωση έχει διαφορετική τάξη μεγέθους και είναι πιο αργή.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Η ρύθμιση της μεταβλητής περιβάλλοντος [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m μπορεί να οδηγήσει στην επαναφόρτωση του θέματος, πράγμα που μπορεί να επηρεάσει αρνητικά την ταχύτητα εκκίνησης.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Εκτός εάν πρόκειται να αλλάξετε το θέμα της βοήθειας ή να απενεργοποιήσετε την επισήμανση συντακτικού, χρησιμοποιήστε το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m αντί να τροποποιείτε τη μεταβλητή περιβάλλοντος.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα με χρήση μεταβλητής περιβάλλοντος:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα με χρήση παραμέτρου γραμμής εντολών:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΓια τα περιεχόμενα που παράγονται από εντολές όπως [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv/get/set[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, η προτεραιότητα δίνεται στο θέμα που καθορίζεται από την επιλογή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Εάν δε δοθεί η επιλογή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, χρησιμοποιείται η μεταβλητή περιβάλλοντος. Εάν η μεταβλητή περιβάλλοντος δεν έχει τιμή, χρησιμοποιείται το θέμα [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mΚαθορίζει το όνομα του θέματος για την επισήμανση του συντακτικού. Αν δεν καθοριστεί, θα χρησιμοποιηθεί το προεπιλεγμένο θέμα.[0m"##),
    ],
}
}

/// Language ID: el;
/// Map name: "conv";
/// Description: Ελληνικά, Ελληνικό, Ελλάδα;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Μετατροπή μεταξύ των μορφών αρχείων, όπως toml, yaml, json και ron");
/// ```
pub(super) const fn get_el_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Μετατροπή μεταξύ των μορφών αρχείων, όπως toml, yaml, json και ron"##),
        ("from", r##"Υποδεικνύει το αρχείο εισόδου"##),
        ("conv-to", r##"Δηλώνει τη μορφή/αρχείο εξόδου. Υποστηρίζει πολλαπλές κλήσεις"##),
        ("save", r##"Όταν καλείται αυτή η επιλογή, το αρχείο που καθορίζεται από το `--to` θα αποθηκεύεται"##),
        ("conv-to-help", r##"Αν υπάρχει ήδη στον τρέχον φάκελο ένα αρχείο **test.json**, μπορείτε να το μετατρέψετε σε yml (yaml) καλώντας `conv test.json -t yml`,
ή να καθορίσετε χειροκίνητα τη διαδρομή του αρχείου έξοδου, βάζοντας για παράδειγμα `conv test.json -t /tmp/test.yml`.

Οι μορφές που υποστηρίζονται εξαρτώνται από τις λειτουργίες που ενεργοποιούνται σε αυτό το πακέτο και δεν θα είναι όλες οι λειτουργίες ενεργοποιημένες από προεπιλογή.

Επιπλέον, πρέπει να γνωρίζετε κάποια πράγματα πριν τη χρήση:
- 1. Θα αλλάξει αυτόματα το όνομα αρχείων αντί για διαδρομές αρχείων.
  - 1.1: Αρχεία με μία διαδρομή στο επίπεδο του φακέλου θα αναγνωρισθούν ως αρχεία και όχι διαδρομές αρχείων. Για παράδειγμα, καλώντας `conv test.toml -t ron`, το αρχείο εξόδου θα είναι το `test.ron` αντί για `ron`. Αν δεν θέλετε αυτήν τη λειτουργία, χρησιμοποιήστε `-t ./ron` ή `-t $PWD/ron`.
  - 1.2: Δείχνει προς φάκελο αντί για αρχείο.
    Για παράδειγμα, καλώντας `-t /tmp`, μπορείτε να τοποθετήσετε το `test.toml` σε υποφάκελο του `/tmp`, αλλά όχι στον ίδιο τον `/tmp`.
    Εάν δεν χρειάζεστε αυτήν τη λειτουργία, χρησιμοποιήστε την πλήρη (όχι κατάλογο) διαδρομή του αρχείου, π.χ. `-t /tmp/xx.toml` αντί για `-t /tmp/`.
- 2. Όταν δεν καλείται το `--save`, το καθορισμένο αρχείο δεν αποθηκεύεται αυτόματα.
- 3. Όταν καλείται το `--save`, η είσοδος και η έξοδος της υποεντολής conv δεν μπορούν να είναι το ίδιο αρχείο.
- 4. Τα σχόλια δεν διατηρούνται στο εξαγόμενο αρχείο."##),
    ],
}
}

/// Language ID: el;
/// Map name: "conv_md";
/// Description: Ελληνικά, Ελληνικό, Ελλάδα;
pub(super) const fn get_el_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mΜετατροπή μεταξύ των μορφών αρχείων, όπως toml, yaml, json και ron[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mΥποδεικνύει το αρχείο εισόδου[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mΔηλώνει τη μορφή/αρχείο εξόδου. Υποστηρίζει πολλαπλές κλήσεις[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mΌταν καλείται αυτή η επιλογή, το αρχείο που καθορίζεται από το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m θα αποθηκεύεται[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mΑν υπάρχει ήδη στον τρέχον φάκελο ένα αρχείο [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, μπορείτε να το μετατρέψετε σε yml (yaml) καλώντας [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m,
[48;2;34;34;34m[38;2;255;255;255mή να καθορίσετε χειροκίνητα τη διαδρομή του αρχείου έξοδου, βάζοντας για παράδειγμα [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΟι μορφές που υποστηρίζονται εξαρτώνται από τις λειτουργίες που ενεργοποιούνται σε αυτό το πακέτο και δεν θα είναι όλες οι λειτουργίες ενεργοποιημένες από προεπιλογή.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΕπιπλέον, πρέπει να γνωρίζετε κάποια πράγματα πριν τη χρήση:
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Θα αλλάξει αυτόματα το όνομα αρχείων αντί για διαδρομές αρχείων.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Αρχεία με μία διαδρομή στο επίπεδο του φακέλου θα αναγνωρισθούν ως αρχεία και όχι διαδρομές αρχείων. Για παράδειγμα, καλώντας [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, το αρχείο εξόδου θα είναι το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m αντί για [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Αν δεν θέλετε αυτήν τη λειτουργία, χρησιμοποιήστε [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: Δείχνει προς φάκελο αντί για αρχείο.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mΓια παράδειγμα, καλώντας [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, μπορείτε να τοποθετήσετε το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m σε υποφάκελο του [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, αλλά όχι στον ίδιο τον [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mΕάν δεν χρειάζεστε αυτήν τη λειτουργία, χρησιμοποιήστε την πλήρη (όχι κατάλογο) διαδρομή του αρχείου, π.χ. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m αντί για [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. Όταν δεν καλείται το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, το καθορισμένο αρχείο δεν αποθηκεύεται αυτόματα.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. Όταν καλείται το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, η είσοδος και η έξοδος της υποεντολής conv δεν μπορούν να είναι το ίδιο αρχείο.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. Τα σχόλια δεν διατηρούνται στο εξαγόμενο αρχείο.[0m"##),
    ],
}
}

/// Language ID: el;
/// Map name: "opt";
/// Description: Ελληνικά, Ελληνικό, Ελλάδα;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "Πίνακας πινάκων με διάφορα κλειδιά (array of tables – AOT). Χρησιμοποιήστε την παράμετρο `--aot 'k1, v1, k2, v2'` για να ορίσετε τα στοιχεία του πίνακα");
/// ```
pub(super) const fn get_el_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"Η διαφορά με το `-k` είναι ότι το `--ck` δεν θα διαχωρίσει αυτόματα το κλειδί στα ".". 
Για παράδειγμα, για το [a.b.c], το `-k a.b.c` και το `--ck a --ck b --ck c` είναι ισοδύναμα. 
Σε απλές περιπτώσεις, το `-k` είναι πιο εύχρηστο.

Εάν το όνομα του κλειδιού περιέχει '.' χρησιμοποιήστε το `--ck` αντί για `-k`. 
Για παράδειγμα:

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

Σε αυτήν τη δομή, το "editor.defaultFormatter" είναι μια πλήρης συμβολοσειρά, η "defaultFormatter" δεν είναι ένα υπο-κλειδί (sub-key) του "editor".

Όταν χρησιμοποιείτε το `-k '[typescript]'.'editor.defaultFormatter'`, θα χωρίσει το κλειδί σε:

- "[typescript]"
- "editor"
- "defaultFormatter"

Το να χρησιμοποιήσετε `--ck '[typescript]' --ck editor.defaultFormatter`, θα διαχωρίσει το κλειδί ως:

- "[typescript]"
- "editor.defaultFormatter"

Αυτό το δεύτερο είναι το σωστό σχήμα."##),
        ("num-arr", r##"Πίνακας ακεραίων i64"##),
        ("bool", r##"bool (λογικός τύπος), μπορεί να είναι μόνο `true` ή `false`"##),
        ("num", r##"i64: 64bit ακέραιος τύπος, για παράδειγμα: 1145"##),
        ("datetime-help", r##"Παράδειγμα:
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

Αυτή η επιλογή υποστηρίζεται μόνο στο `toml`. Εάν προσθέσουμε `-t json` στο τέλος της εντολής, τότε θα φαίνεται κάπως έτσι:

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"Αποθηκεύει σε συγκεκριμένο φορμάτ/αρχείο, μόνο ένα αρχείο μπορεί να αποθηκευτεί κάθε φορά."##),
        ("get-opts-help", r##"Παράδειγμα χρήσης 1: `get Cargo.toml -k package.name`
Παράδειγμα χρήσης 2: `get cfg.yaml -k .`
"##),
        ("none-help", r##"> Σημείωση: το ron διαθέτει πραγματικό τύπο `Option<T>` με τιμές `Some(T)` ή `None`
> Η παράμετρος `--none` αφορά τη διαγραφή ενός κλειδιού (key), όχι το πραγματικό τύπο `Option<T>`.

Ας υποθέσουμε ότι η ακόλουθη λίστα πινάκων υπάρχει στο `test.yml`:

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

Για να διαγράψουμε το `name: test`, μπορούμε να εκτελέσουμε:
```sh
set test.yml -k bin.1.name --none
```

> Σημείωση: Η κενή συμβολοσειρά και το None είναι διαφορετικά πράγματα.
> Με την κλήση `set test.yml --kv bin.1.name ""` μπορούμε να το ορίσουμε με κενή συμβολοσειρά, ενώ με την παράμετρο `--none` διαγράφουμε ολόκληρο το κλειδί.

Για να διαγράψουμε τον δεύτερο πίνακα του `bin` (με δείκτη 1), μπορούμε να εκτελέσουμε:
```sh
set test.yml -k bin.1 --none
```

Ή μπορούμε να χρησιμοποιήσουμε την παράμετρο `--rm`
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"Διπλής ακρίβειας κινητής υποδιαστολής (IEEE 754-2008 πρότυπο), για παράδειγμα: 3.14159265359"##),
        ("saot-help", r##"> Σημείωση: αν και μόνο μία τιμή K-V μπορεί να οριστεί με την παράμετρο `--saot`, μπορείτε να ορίσετε το δείκτη του πίνακα χρησιμοποιώντας άλλες επιλογές.

Παράδειγμα:
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"Ορισμός ζεύγους κλειδιού-τιμής (k-v)，`--kv x.y.z value` αντιστοιχεί σε `-k x.y.z -s value`"##),
        ("set-save", r##"Εάν καλεστεί χωρίς τη συνοδευτική επιλογή `--to`, αντικαθιστά (overwrite) το τρέχον αρχείο"##),
        ("array", r##"Πίνακας συμβολοσειρών (string array)"##),
        ("aot-help", r##"Παράδειγμα:
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

Επιλέξτε μία ρύθμιση με βάση το κλειδί και μετά τροποποιήστε την χρησιμοποιώντας μία τιμή της δοθείσας περίπτωσης.

> Όταν δεν δοθεί τιμή για τη ρύθμιση, αλλά δοθεί μόνο το κλειδί, επιστρέφει πληροφορίες σχετικά με τον τύπο της ρύθμισης και την τιμή της.
> Για παράδειγμα, `set Cargo.toml -k package` επιστρέφει μία λεπτομερή περιγραφή του πίνακα 'package' και των υποπινάκων του, παρέχοντας περισσότερες πληροφορίες από την εντολή `get`.

## Παραδείγματα

### Παράδειγμα 1:
```sh
--ts md set test.toml --key package.edition --str 2021
```

Δεδομένου ότι δόθηκε `--str` αντί για `--num`, το "2021" είναι τύπου string, όχι integer.

| package | νέα τιμή |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### Παράδειγμα 2:
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### Παράδειγμα 3:

```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: Αυτή είναι η πρώτη είσοδος*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> Σημείωση: Οι παραπάνω παραδείγματα ξεκινούν με όρους (argument)/ υπο-εντολές (sub-command) και παραλείπουν το όνομα της βασικής εντολής, στην πραγματικότητα θα πρέπει να προσθέσετε το όνομα αυτό χειροκίνητα."##),
        ("num-arr-help", r##"Παράδειγμα:
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"Παράδειγμα:
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> Εκτός από το `toml`, όλες οι άλλες μορφές αρχείων θα χρησιμοποιήσουν έναν κανονικό πίνακα αντί για εμφωλιασμένο.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"Αναζήτηση της τιμής με βάση το κλειδί"##),
        ("f64-arr-help", r##"Παράδειγμα:
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> Το json 1.0 δεν υποστηρίζει NaN και Inf, επομένως τυπώνει `null` αντί για `NaN`.

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"Διαγραφή ενός κλειδιού (key)"##),
        ("set-save-to-help", r##"Είναι παρόμοιο με το `-t` της υπο-εντολής `conv`, η μεγαλύτερη διαφορά είναι ότι το `-t` του `conv` μπορεί να κληθεί πολλές φορές σε σειρά, για να παράξει πολλαπλά αρχεία σε διαφορετικά φορμάτ. Αντίθετα, το `-t` της υπο-εντολής `set` μπορεί να κληθεί μόνο μία φορά και μόνο ένα αρχείο μπορεί να παρουσιαστεί σε αυτό.

>Σημείωση: Μετά την εκτέλεση της λειτουργίας μετατροπής της υπο-εντολής `conv`, όλα τα αρχεία χάνουν τα σχόλια τους.
> Αντίθετα, η υπο-εντολή `set` διατηρεί τα σχόλια **των αρχείων toml**, όπως και τα αρχεία σε άλλα φορματ **δεν διατηρούν** τα σχόλια τους.

- Αν δεν έχει κληθεί το `-t`, το `--save` θα αντικαταστήσει το ίδιο το αρχείο.
- Αν έχει κληθεί το `-t` και ο χρήστης έχει επιλέξει `--save`, τότε το τροποποιημένο αρχείο θα αποθηκευτεί στο αρχείο που καθορίζεται από το `-t`.

- Εκτός αν το αρχείο προέλευσης και αυτό που καθορίζεται με το `-t` δεν είναι το ίδιο, τότε το `-t` της υπο-εντολής `set` λειτουργεί όπως και το `-t` της υπο-εντολής `conv`, και συγκεκριμένα αναφέρεται σε μη συγκεκριμένα ονόματα αρχείων.
  - Για παράδειγμα, αν το αρχείο `from` είναι **test.yml** και το αρχείο `to` είναι **json**, τότε το αποτέλεσμα θα είναι το **test.json**, όχι απλά **json**.
  - Εάν δεν θέλετε αυτήν τη λειτουργία, τότε χρησιμοποιήστε `-t ./json` ή `-t $PWD/json`, αντί για `-t json`."##),
        ("preview", r##"Προεπισκόπηση του πλήρους αρχείου μετά την ολοκλήρωση των αλλαγών"##),
        ("set-key", r##"Επιλέξτε ένα συγκεκριμένο κλειδί (key), χρησιμοποιώντας το "." ως διαχωριστικό"##),
        ("get-opts", r##"Λήψη της τιμής που αντιστοιχεί στη δοσμένη ρύθμιση"##),
        ("saot", r##"Πίνακας με μοναδικές τιμές (single-array-of-tables, SAOT). Χρησιμοποιήστε την παράμετρο `--saot k1 v1` για να ορίσετε τα στοιχεία του πίνακα."##),
        ("key-help", r##"Η μορφή είναι: "a.b.c", όνομα υποπίνακα ή δείκτης πίνακας συνδέονται με ".".
Όταν χρησιμοποιείτε μόνο ένα ".", επιστρέφει ολόκληρο τον υποπίνακα"##),
        ("set-opts", r##"Δημιουργία/ενημέρωση/διαγραφή δομής αρχείου ρυθμίσεων"##),
        ("bool-arr-help", r##"Παράδειγμα:
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: το πρώτο στοιχείο είναι true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"Πίνακας boolean (bool)"##),
        ("table", r##"Κανονικός πίνακας (standard table), χρησιμοποιείται για να αποθηκεύσει πολλά ζευγάρια K-V."##),
        ("datetime", r##"Τύπος ημερομηνίας και ώρας"##),
        ("table-help", r##"Παράδειγμα 1:
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

Παράδειγμα 2:
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"Πίνακας διπλής ακρίβειας κινητής υποδιαστολής (f64)"##),
        ("string", r##"String (τύπος συμβολοσειράς)"##),
        ("array-help", r##"Χρησιμοποιήστε την παράμετρο `-a` για να ορίσετε τα στοιχεία του πίνακα.
Παράδειγμα:
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> Ορίζει το στοιχείο με δείκτη 0 στην πρώτη κλήση, το στοιχείο με δείκτη 1 στη δεύτερη κλήση και ούτω καθεξής.

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"Πίνακας πινάκων με διάφορα κλειδιά (array of tables – AOT). Χρησιμοποιήστε την παράμετρο `--aot 'k1, v1, k2, v2'` για να ορίσετε τα στοιχεία του πίνακα"##),
        ("inline-table", r##"Εμφωλιασμένος πίνακας (inline table), αντιστοιχεί σε έναν πιο συμπυκνωμένο κανονικό πίνακα."##),
        ("concat-key", r##"Διαχωρισμός του κλειδιού σε τμήματα, τα οποία θα ενωθούν αυτόματα στο τέλος"##),
    ],
}
}

/// Language ID: el;
/// Map name: "opt_md";
/// Description: Ελληνικά, Ελληνικό, Ελλάδα;
pub(super) const fn get_el_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255mΗ διαφορά με το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m είναι ότι το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m δεν θα διαχωρίσει αυτόματα το κλειδί στα ".". 
[48;2;34;34;34m[38;2;255;255;255mΓια παράδειγμα, για το [a.b.c], το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m και το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m είναι ισοδύναμα. 
[48;2;34;34;34m[38;2;255;255;255mΣε απλές περιπτώσεις, το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m είναι πιο εύχρηστο.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΕάν το όνομα του κλειδιού περιέχει '.' χρησιμοποιήστε το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m αντί για [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. 
[48;2;34;34;34m[38;2;255;255;255mΓια παράδειγμα:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΣε αυτήν τη δομή, το "editor.defaultFormatter" είναι μια πλήρης συμβολοσειρά, η "defaultFormatter" δεν είναι ένα υπο-κλειδί (sub-key) του "editor".
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΌταν χρησιμοποιείτε το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, θα χωρίσει το κλειδί σε:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΤο να χρησιμοποιήσετε [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, θα διαχωρίσει το κλειδί ως:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΑυτό το δεύτερο είναι το σωστό σχήμα.[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mΠίνακας ακεραίων i64[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mbool (λογικός τύπος), μπορεί να είναι μόνο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: 64bit ακέραιος τύπος, για παράδειγμα: 1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΑυτή η επιλογή υποστηρίζεται μόνο στο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Εάν προσθέσουμε [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m στο τέλος της εντολής, τότε θα φαίνεται κάπως έτσι:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255mΑποθηκεύει σε συγκεκριμένο φορμάτ/αρχείο, μόνο ένα αρχείο μπορεί να αποθηκευτεί κάθε φορά.[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα χρήσης 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα χρήσης 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Σημείωση: το ron διαθέτει πραγματικό τύπο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m με τιμές [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m ή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Η παράμετρος [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m αφορά τη διαγραφή ενός κλειδιού (key), όχι το πραγματικό τύπο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΑς υποθέσουμε ότι η ακόλουθη λίστα πινάκων υπάρχει στο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΓια να διαγράψουμε το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mname: test[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, μπορούμε να εκτελέσουμε:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Σημείωση: Η κενή συμβολοσειρά και το None είναι διαφορετικά πράγματα.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Με την κλήση [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m μπορούμε να το ορίσουμε με κενή συμβολοσειρά, ενώ με την παράμετρο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m διαγράφουμε ολόκληρο το κλειδί.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΓια να διαγράψουμε τον δεύτερο πίνακα του [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mbin[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m (με δείκτη 1), μπορούμε να εκτελέσουμε:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΉ μπορούμε να χρησιμοποιήσουμε την παράμετρο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255mΔιπλής ακρίβειας κινητής υποδιαστολής (IEEE 754-2008 πρότυπο), για παράδειγμα: 3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Σημείωση: αν και μόνο μία τιμή K-V μπορεί να οριστεί με την παράμετρο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, μπορείτε να ορίσετε το δείκτη του πίνακα χρησιμοποιώντας άλλες επιλογές.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mΟρισμός ζεύγους κλειδιού-τιμής (k-v)，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m αντιστοιχεί σε [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255mΕάν καλεστεί χωρίς τη συνοδευτική επιλογή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, αντικαθιστά (overwrite) το τρέχον αρχείο[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255mΠίνακας συμβολοσειρών (string array)[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΕπιλέξτε μία ρύθμιση με βάση το κλειδί και μετά τροποποιήστε την χρησιμοποιώντας μία τιμή της δοθείσας περίπτωσης.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Όταν δεν δοθεί τιμή για τη ρύθμιση, αλλά δοθεί μόνο το κλειδί, επιστρέφει πληροφορίες σχετικά με τον τύπο της ρύθμισης και την τιμή της.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Για παράδειγμα, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m επιστρέφει μία λεπτομερή περιγραφή του πίνακα 'package' και των υποπινάκων του, παρέχοντας περισσότερες πληροφορίες από την εντολή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mΠαραδείγματα[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mΠαράδειγμα 1:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΔεδομένου ότι δόθηκε [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m αντί για [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, το "2021" είναι τύπου string, όχι integer.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m νέα τιμή [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mΠαράδειγμα 2:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mΠαράδειγμα 3:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: Αυτή είναι η πρώτη είσοδος*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Σημείωση: Οι παραπάνω παραδείγματα ξεκινούν με όρους (argument)/ υπο-εντολές (sub-command) και παραλείπουν το όνομα της βασικής εντολής, στην πραγματικότητα θα πρέπει να προσθέσετε το όνομα αυτό χειροκίνητα.[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Εκτός από το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, όλες οι άλλες μορφές αρχείων θα χρησιμοποιήσουν έναν κανονικό πίνακα αντί για εμφωλιασμένο.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mΑναζήτηση της τιμής με βάση το κλειδί[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Το json 1.0 δεν υποστηρίζει NaN και Inf, επομένως τυπώνει [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m αντί για [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255mΔιαγραφή ενός κλειδιού (key)[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mΕίναι παρόμοιο με το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m της υπο-εντολής [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, η μεγαλύτερη διαφορά είναι ότι το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m του [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m μπορεί να κληθεί πολλές φορές σε σειρά, για να παράξει πολλαπλά αρχεία σε διαφορετικά φορμάτ. Αντίθετα, το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m της υπο-εντολής [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m μπορεί να κληθεί μόνο μία φορά και μόνο ένα αρχείο μπορεί να παρουσιαστεί σε αυτό.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239mΣημείωση: Μετά την εκτέλεση της λειτουργίας μετατροπής της υπο-εντολής [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, όλα τα αρχεία χάνουν τα σχόλια τους.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Αντίθετα, η υπο-εντολή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m διατηρεί τα σχόλια [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mτων αρχείων toml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m, όπως και τα αρχεία σε άλλα φορματ [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mδεν διατηρούν[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m τα σχόλια τους.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mΑν δεν έχει κληθεί το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m θα αντικαταστήσει το ίδιο το αρχείο.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Αν έχει κληθεί το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m και ο χρήστης έχει επιλέξει [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, τότε το τροποποιημένο αρχείο θα αποθηκευτεί στο αρχείο που καθορίζεται από το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mΕκτός αν το αρχείο προέλευσης και αυτό που καθορίζεται με το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m δεν είναι το ίδιο, τότε το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m της υπο-εντολής [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m λειτουργεί όπως και το [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m της υπο-εντολής [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, και συγκεκριμένα αναφέρεται σε μη συγκεκριμένα ονόματα αρχείων.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Για παράδειγμα, αν το αρχείο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfrom[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m είναι [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m και το αρχείο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mto[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m είναι [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, τότε το αποτέλεσμα θα είναι το [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, όχι απλά [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Εάν δεν θέλετε αυτήν τη λειτουργία, τότε χρησιμοποιήστε [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ή [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, αντί για [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255mΠροεπισκόπηση του πλήρους αρχείου μετά την ολοκλήρωση των αλλαγών[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255mΕπιλέξτε ένα συγκεκριμένο κλειδί (key), χρησιμοποιώντας το "." ως διαχωριστικό[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255mΛήψη της τιμής που αντιστοιχεί στη δοσμένη ρύθμιση[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255mΠίνακας με μοναδικές τιμές (single-array-of-tables, SAOT). Χρησιμοποιήστε την παράμετρο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m για να ορίσετε τα στοιχεία του πίνακα.[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mΗ μορφή είναι: "a.b.c", όνομα υποπίνακα ή δείκτης πίνακας συνδέονται με ".".
[48;2;34;34;34m[38;2;255;255;255mΌταν χρησιμοποιείτε μόνο ένα ".", επιστρέφει ολόκληρο τον υποπίνακα[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255mΔημιουργία/ενημέρωση/διαγραφή δομής αρχείου ρυθμίσεων[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: το πρώτο στοιχείο είναι true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mΠίνακας boolean (bool)[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255mΚανονικός πίνακας (standard table), χρησιμοποιείται για να αποθηκεύσει πολλά ζευγάρια K-V.[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255mΤύπος ημερομηνίας και ώρας[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα 1:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα 2:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mΠίνακας διπλής ακρίβειας κινητής υποδιαστολής (f64)[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString (τύπος συμβολοσειράς)[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255mΧρησιμοποιήστε την παράμετρο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m για να ορίσετε τα στοιχεία του πίνακα.
[48;2;34;34;34m[38;2;255;255;255mΠαράδειγμα:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Ορίζει το στοιχείο με δείκτη 0 στην πρώτη κλήση, το στοιχείο με δείκτη 1 στη δεύτερη κλήση και ούτω καθεξής.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255mΠίνακας πινάκων με διάφορα κλειδιά (array of tables – AOT). Χρησιμοποιήστε την παράμετρο [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m για να ορίσετε τα στοιχεία του πίνακα[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mΕμφωλιασμένος πίνακας (inline table), αντιστοιχεί σε έναν πιο συμπυκνωμένο κανονικό πίνακα.[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mΔιαχωρισμός του κλειδιού σε τμήματα, τα οποία θα ενωθούν αυτόματα στο τέλος[0m"##),
    ],
}
}

/// Language ID: el;
/// Map name: "parser";
/// Description: Ελληνικά, Ελληνικό, Ελλάδα;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "Ακολουθεί το stderr output. Όταν ανακατευθύνετε τη συμπλήρωση εξόδου σε ένα αρχείο, πρέπει να χρησιμοποιήσετε `> /path/to/file` αντί για `2> /path/to/file`.");
/// ```
pub(super) const fn get_el_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"Ακολουθεί το stderr output. Όταν ανακατευθύνετε τη συμπλήρωση εξόδου σε ένα αρχείο, πρέπει να χρησιμοποιήσετε `> /path/to/file` αντί για `2> /path/to/file`."##),
    ],
}
}

/// Language ID: el;
/// Map name: "theme";
/// Description: Ελληνικά, Ελληνικό, Ελλάδα;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Μη έγκυρο όνομα θέματος");
/// ```
pub(super) const fn get_el_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Μη έγκυρο όνομα θέματος"##),
    ],
}
}

/// el: Ελληνικά, Ελληνικό, Ελλάδα
pub(super) const fn get_el_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_el_map_args),
        ("conv_md", get_el_map_conv_md),
        ("parser", get_el_map_parser),
        ("opt_md", get_el_map_opt_md),
        ("args_md", get_el_map_args_md),
        ("opt", get_el_map_opt),
        ("log-core", get_el_map_log_core),
        ("conv", get_el_map_conv),
        ("theme", get_el_map_theme),
    ],
}
}

/// Language ID: en;
/// Map name: "log-core";
/// Description: English, Latin, United States;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "DEBUG");
/// ```
pub(super) const fn get_en_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"DEBUG"##),
        ("warn", r##"WARN"##),
        ("info", r##"INFO"##),
        ("error", r##"ERROR"##),
        ("trace", r##"TRACE"##),
        ("init-logger", r##"Initializing logger ..."##),
    ],
}
}

/// Language ID: en;
/// Map name: "args";
/// Description: English, Latin, United States;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Disables the theme background (enabled by default, disabled when `--dis-theme-bg` is called).");
/// ```
pub(super) const fn get_en_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"Custom theme collection file, you also need to specify a theme name with `--theme`."##),
        ("shell-completion", r##"Generates completion scripts for the shell."##),
        ("theme-file-help", r##"The supported file format is binary data dumped by the rust crate `syntect`.
In essence, these themes are themes of the old version of Sublime.
If `--tf` is not specified, the value of the `$TOMLYRE_THEME_FILE` environment variable is read."##),
        ("from-format", r##"Manually specify the file format of the source (it will be automatically determined by default)."##),
        ("disable-theme-background", r##"Disables the theme background (enabled by default, disabled when `--dis-theme-bg` is called)."##),
        ("theme-uncompressed-file-help", r##"> The uncompressed version has a higher priority than the compressed version.

If `--tuf` is not specified, the value of the `$TOMLYRE_THEME_UNCOMPRESSED_FILE` environment variable is read."##),
        ("disable-theme-background-help", r##"If `--dis-theme-bg` is not specified, the value of the `$TOMLYRE_THEME_BG` environment variable is read.

If the value is false, it will be disabled.
```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"Similar to `theme-file`, but requires uncompressed dump data (file)"##),
        ("shell-completion-help", r##"You need to manually specify the name of the shell to generate the completion script.
For example, for powershell on Windows, you can use: `--sh-comp pwsh`

> Q: What is the completion function of the shell?

> A: When we enter a command on the command line and press the Tab key, the completion script is triggered, which begins searching for possible completion options and presents these options for selection. This feature can greatly improve the efficiency and accuracy of command line input.

The location where the completion scripts for different shells are stored is different. For zsh, you need to store it in the path included in the `$fpath` array.
You can use `print -l $fpath` to get the contents of this array.
Assuming that the `/usr/local/share/zsh/site-functions` directory is included, then you can use the following command to write the completion script to the `_tomlyre` file in that directory.

```sh
tomlyre --sh-comp zsh | sudo tee /usr/local/share/zsh/site-functions/_tomlyre
```

To avoid using root privileges, you can put it in a user subdirectory. For example, you can use the following command to insert an element at the beginning of that array, and then use the relevant directory.
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"When reading data from stdin, for example: `cat a | tomlyre conv -`, it will first try to automatically determine the file format.
However, automatic identification may fail, and manual specification is required.
Example: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"When using only the `--ts` parameter, the built-in style collection will be displayed.
Except that markdown can be abbreviated as md, other styles must use the full name.
Usage Example 1: `--ts md`
Usage Example 2: `--table-style ascii`

Note: Currently, **table style** collections cannot be read from files. If you need this or other additional features, please submit an issue for feedback. "##),
        ("table-style", r##"Specifies the style for tables."##),
        ("theme-help", r##"When using only the `-t` parameter, the list of current themes will be displayed.
If no theme file is specified, the built-in theme collection will be used.
Assuming there is a theme called **ayu-dark**, you can use `-t ayu-dark` to use that theme.
If the theme name contains spaces, please enclose it in double quotes or single quotes.

> Note: There is a special theme called `None`, which represents plain text style without syntax highlighting.

When the `--theme` option is not specified, data will be read from the `$TOMLYRE_THEME` environment variable.
The **syntax highlighting** theme of this help information also comes from the **environment variable**.

> The default help information is hard-coded and loads very quickly.  
> Loading hard-coded text and loading text with `None` theme have the same speed, while dynamic loading will slow down by two orders of magnitude or more.  
> Setting the `TOMLYRE_THEME` environment variable may cause it to reload the theme, which will significantly slow down startup time.  
> Unless you are planning to modify the theme of the help information or disable syntax highlighting, please use the `-t` parameter instead of modifying the environment variables. 

Example using environment variables:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Example using command line parameters:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

For the content output manually executed by operations such as "conv/get/set", etc., the priority of the theme specified by `-t` > the environment variable.
If `-t` is not called, the environment variable is read; if the environment variable is empty, **Monokai Extended** is used."##),
        ("theme", r##"Specifies the theme name for syntax highlighting."##),
    ],
}
}

/// Language ID: en;
/// Map name: "args_md";
/// Description: English, Latin, United States;
pub(super) const fn get_en_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mCustom theme collection file, you also need to specify a theme name with [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mGenerates completion scripts for the shell.[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mThe supported file format is binary data dumped by the rust crate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mIn essence, these themes are themes of the old version of Sublime.
[48;2;34;34;34m[38;2;255;255;255mIf [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is not specified, the value of the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m environment variable is read.[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mManually specify the file format of the source (it will be automatically determined by default).[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mDisables the theme background (enabled by default, disabled when [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is called).[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m The uncompressed version has a higher priority than the compressed version.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIf [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is not specified, the value of the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m environment variable is read.[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mIf [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is not specified, the value of the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m environment variable is read.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIf the value is false, it will be disabled.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mSimilar to [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, but requires uncompressed dump data (file)[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mYou need to manually specify the name of the shell to generate the completion script.
[48;2;34;34;34m[38;2;255;255;255mFor example, for powershell on Windows, you can use: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q: What is the completion function of the shell?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: When we enter a command on the command line and press the Tab key, the completion script is triggered, which begins searching for possible completion options and presents these options for selection. This feature can greatly improve the efficiency and accuracy of command line input.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mThe location where the completion scripts for different shells are stored is different. For zsh, you need to store it in the path included in the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m array.
[48;2;34;34;34m[38;2;255;255;255mYou can use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m to get the contents of this array.
[48;2;34;34;34m[38;2;255;255;255mAssuming that the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m directory is included, then you can use the following command to write the completion script to the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m file in that directory.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255msudo[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTo avoid using root privileges, you can put it in a user subdirectory. For example, you can use the following command to insert an element at the beginning of that array, and then use the relevant directory.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mWhen reading data from stdin, for example: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, it will first try to automatically determine the file format.
[48;2;34;34;34m[38;2;255;255;255mHowever, automatic identification may fail, and manual specification is required.
[48;2;34;34;34m[38;2;255;255;255mExample: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mWhen using only the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m parameter, the built-in style collection will be displayed.
[48;2;34;34;34m[38;2;255;255;255mExcept that markdown can be abbreviated as md, other styles must use the full name.
[48;2;34;34;34m[38;2;255;255;255mUsage Example 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mUsage Example 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mNote: Currently, [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtable style[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m collections cannot be read from files. If you need this or other additional features, please submit an issue for feedback. [0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mSpecifies the style for tables.[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mWhen using only the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m parameter, the list of current themes will be displayed.
[48;2;34;34;34m[38;2;255;255;255mIf no theme file is specified, the built-in theme collection will be used.
[48;2;34;34;34m[38;2;255;255;255mAssuming there is a theme called [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, you can use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m to use that theme.
[48;2;34;34;34m[38;2;255;255;255mIf the theme name contains spaces, please enclose it in double quotes or single quotes.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Note: There is a special theme called [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, which represents plain text style without syntax highlighting.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWhen the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m option is not specified, data will be read from the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m environment variable.
[48;2;34;34;34m[38;2;255;255;255mThe [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114msyntax highlighting[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m theme of this help information also comes from the [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114menvironment variable[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m The default help information is hard-coded and loads very quickly.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Loading hard-coded text and loading text with [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m theme have the same speed, while dynamic loading will slow down by two orders of magnitude or more.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Setting the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m environment variable may cause it to reload the theme, which will significantly slow down startup time.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Unless you are planning to modify the theme of the help information or disable syntax highlighting, please use the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m parameter instead of modifying the environment variables. 
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExample using environment variables:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExample using command line parameters:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mFor the content output manually executed by operations such as "conv/get/set", etc., the priority of the theme specified by [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m > the environment variable.
[48;2;34;34;34m[38;2;255;255;255mIf [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is not called, the environment variable is read; if the environment variable is empty, [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m is used.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mSpecifies the theme name for syntax highlighting.[0m"##),
    ],
}
}

/// Language ID: en;
/// Map name: "conv";
/// Description: English, Latin, United States;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Convert between toml, yaml, json and ron formats.");
/// ```
pub(super) const fn get_en_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Convert between toml, yaml, json and ron formats."##),
        ("from", r##"Specifies the input file."##),
        ("conv-to", r##"Specifies the output format/file. Multiple calls are supported."##),
        ("save", r##"When this option is called, the file specified by `--to` will be saved."##),
        ("conv-to-help", r##"If a file named **test.json** exists in the current directory, you can call `conv test.json -t yml` to convert it to YAML,
or manually specify the file path, `conv test.json -t /tmp/test.yml`

The supported formats depend on the features enabled by this package, not all features are enabled by default.

In addition, there are some things you must be aware of before using:
- 1. It will automatically rename non-specific file paths.
  - 1.1: Only files with one level will be recognized as file formats, rather than file paths.
    When calling `conv test.toml -t ron`, the output file will be automatically renamed to `test.ron`, instead of `ron`.
    If you don't need this feature, please use `-t ./ron` or `-t $PWD/ron`.
  - 1.2: The target is a directory, not a file.
    For example, `-t /tmp/`, we can only put `test.toml` under the sub-level of `/tmp/`, not cover `/tmp` itself.
    If you don't need this feature, please use the complete (non-directory) file path, such as `-t /tmp/xx.toml`, instead of `-t /tmp/`.
- 2. When `--save` is not called, the specified file will not be saved automatically.
- 3. When `--save` is called, the input and output of the conv subcommand cannot be the same file.
- 4. The converted file will not retain any comments."##),
    ],
}
}

/// Language ID: en;
/// Map name: "conv_md";
/// Description: English, Latin, United States;
pub(super) const fn get_en_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mConvert between toml, yaml, json and ron formats.[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mSpecifies the input file.[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mSpecifies the output format/file. Multiple calls are supported.[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mWhen this option is called, the file specified by [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m will be saved.[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mIf a file named [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m exists in the current directory, you can call [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m to convert it to YAML,
[48;2;34;34;34m[38;2;255;255;255mor manually specify the file path, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mThe supported formats depend on the features enabled by this package, not all features are enabled by default.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIn addition, there are some things you must be aware of before using:
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m It will automatically rename non-specific file paths.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Only files with one level will be recognized as file formats, rather than file paths.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mWhen calling [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, the output file will be automatically renamed to [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, instead of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mIf you don't need this feature, please use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m or [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: The target is a directory, not a file.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mFor example, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, we can only put [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m under the sub-level of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, not cover [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m itself.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mIf you don't need this feature, please use the complete (non-directory) file path, such as [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, instead of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. When [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is not called, the specified file will not be saved automatically.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. When [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is called, the input and output of the conv subcommand cannot be the same file.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. The converted file will not retain any comments.[0m"##),
    ],
}
}

/// Language ID: en;
/// Map name: "opt";
/// Description: English, Latin, United States;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "Array of tables (Usage: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_en_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"The difference from `-k` is that `--ck` does not automatically `split('.')`.
For [a.b.c], `-k a.b.c` is the same as `--ck a --ck b --ck c`.
For simple cases, it may be more convenient to use `-k`.

If the key name contains '.', please use `--ck` instead of `-k`.
For example:

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

In this structure, "editor.defaultFormatter" is a complete string, and `defaultFormatter` is not a sub-key of `editor`.

When you use `-k '[typescript]'.'editor.defaultFormatter'`, it will split the key into:

- "[typescript]"
- "editor"
- "defaultFormatter"

But when using `--ck '[typescript]' --ck editor.defaultFormatter`, it will treat the key as:

- "[typescript]"
- "editor.defaultFormatter"

The latter is the correct structure."##),
        ("num-arr", r##"i64 Array"##),
        ("bool", r##"Boolean type, can only be `true` or `false`."##),
        ("num", r##"i64: 64-bit signed integer type, e.g., 1145"##),
        ("datetime-help", r##"Example:
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

This option only supports `toml`. If we add `-t json` at the end of the command, it will look like this:

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"Save to the specified format/file, one at a time"##),
        ("get-opts-help", r##"Usage Example 1: `get Cargo.toml -k package.name`
Usage Example 2: `get cfg.yaml -k .`
"##),
        ("none-help", r##"> Note: RON has a true `Option<T>` type, with a value of `Some(T)` or `None`.
> Here, `--none` means to delete the specified key, not the true `Option<T>` type.

Assuming that the following array of tables exists in `test.yml`:

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

If we want to delete `name: test`, we can execute:

```sh
set test.yml -k bin.1.name --none
```

> Note: An empty string and `None` are not the same thing.
> Calling `set test.yml --kv bin.1.name ""` will set it to an empty string, while `--none` will delete the entire key.

If we want to delete the second table of `bin` (index 1), we can execute:

```sh
set test.yml -k bin.1 --none
```

Or use `--rm`:

```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"Double-precision floating-point number (IEEE 754-2008 standard), e.g., 3.14159265359."##),
        ("saot-help", r##"> Note: Although only one K-V pair can be created for each table using `--saot`, you can first specify the index of the array of tables and then use other options to expand.

Example:
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"Set k-v pair, `--kv x.y.z value` is equivalent to `-k x.y.z -s value`"##),
        ("set-save", r##"If used without `--to`, it overwrites the current file."##),
        ("array", r##"String array"##),
        ("aot-help", r##"Example:
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

Select first by specifying a key, and then modify it by specifying a value of the data type.

> When only key is provided without value, the corresponding data type and value will be displayed. 
> For example, `set Cargo.toml -k package` will display tables for package and its sub-tables, and provide more detailed information than `get`.

## Examples

### Example 1:
```sh
--ts md set test.toml --key package.edition --str 2021
```

Because `--str` is specified instead of `--num`, "2021" is a string type, not an integer type.

| package | new value |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### Example 2:
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### Example 3:
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: This is the first index.*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> Note: The above examples start with arg/sub-command and omit the root command name. In actual operation, you need to manually supplement it. "##),
        ("num-arr-help", r##"Example:
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"If you're not sure whether you should use the standard table, or the inline table, then use the inline table!

Example:
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> Except for `toml`, other formats will use standard tables instead of inline tables.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"Query the value through the key."##),
        ("f64-arr-help", r##"Example:
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> JSON 1.0 does not support `NaN` and `Inf`.

It will output `null` instead of `NaN`.

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"Delete the specified key."##),
        ("set-save-to-help", r##"Similar to the `-t` option for the `conv` sub-command, the biggest difference is that the `-t` option for `conv` can be called multiple times in a row to output to multiple formats/files.
While the `-t` option for the `set` sub-command can only be called once to output to a single file.

> Note: After executing the conversion operation, all files of all formats will not retain comments in the `conv` sub-command. 
> While the `set` sub-command will retain comments for **toml**, but **none** for other formats.

- Without calling `-t`, calling `--save` will overwrite itself.
- If `-t` has been called and `--save` has also been called, the modified file will be saved to the file pointed to by `-t`.
- Unless the source file is the same as the file pointed to by `-t`, the behavior of the `-t` option for the `set` sub-command will be similar to that of the `-t` option for the `conv` sub-command. They will both automatically rename non-specific file paths.
  - Assuming from: **test.yml**, to: **json**, then the output file will become **test.json**, not **json**
  - If you don't need this feature, please use `-t ./json` or `-t $PWD/json` instead of `-t json`. "##),
        ("preview", r##"Preview the entire file after modification."##),
        ("set-key", r##"Select the specified key using "." as a separator"##),
        ("get-opts", r##"Get the value of a specified configuration."##),
        ("saot", r##"Array of tables with only one K-V pair (Usage: `--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"The format is "a.b.c", and sub-table names or array indices are connected by ".".
When there is only one ".", the entire table will be shown."##),
        ("set-opts", r##"Create/Update/Delete the structure of the configuration file."##),
        ("bool-arr-help", r##"Example:
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: the first element is true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"Bool array"##),
        ("table", r##"Standard table, used to store multiple K-V pairs."##),
        ("datetime", r##"Type for date and time."##),
        ("table-help", r##"Example 1：
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

Example 2：
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"f64 Array"##),
        ("string", r##"String type"##),
        ("array-help", r##"Set array elements by calling `-a` multiple times.
Example:
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> Set the element of index 0 on the first call, set the element of index 1 on the second call, and so on.

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"Array of tables (Usage: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"It is more compact than the standard table. In addition, for nested keys that do not exist, please use inline tables."##),
        ("concat-key", r##"Input the key in sections, and the result will be automatically concatenated without using "." as a separator."##),
    ],
}
}

/// Language ID: en;
/// Map name: "opt_md";
/// Description: English, Latin, United States;
pub(super) const fn get_en_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255mThe difference from [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is that [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m does not automatically [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msplit('.')[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mFor [a.b.c], [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is the same as [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mFor simple cases, it may be more convenient to use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIf the key name contains '.', please use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m instead of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mFor example:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIn this structure, "editor.defaultFormatter" is a complete string, and [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mdefaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is not a sub-key of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255meditor[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWhen you use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, it will split the key into:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mBut when using [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, it will treat the key as:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mThe latter is the correct structure.[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mi64 Array[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mBoolean type, can only be [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m or [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: 64-bit signed integer type, e.g., 1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255mExample:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mThis option only supports [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. If we add [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m at the end of the command, it will look like this:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255mSave to the specified format/file, one at a time[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255mUsage Example 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mUsage Example 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Note: RON has a true [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m type, with a value of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m or [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Here, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m means to delete the specified key, not the true [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m type.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAssuming that the following array of tables exists in [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIf we want to delete [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mname: test[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, we can execute:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Note: An empty string and [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m are not the same thing.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Calling [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m will set it to an empty string, while [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m will delete the entire key.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIf we want to delete the second table of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mbin[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m (index 1), we can execute:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOr use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255mDouble-precision floating-point number (IEEE 754-2008 standard), e.g., 3.14159265359.[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Note: Although only one K-V pair can be created for each table using [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, you can first specify the index of the array of tables and then use other options to expand.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExample:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mSet k-v pair, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is equivalent to [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255mIf used without [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, it overwrites the current file.[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255mString array[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255mExample:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSelect first by specifying a key, and then modify it by specifying a value of the data type.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m When only key is provided without value, the corresponding data type and value will be displayed. 
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m For example, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m will display tables for package and its sub-tables, and provide more detailed information than [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExamples[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExample 1:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mBecause [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is specified instead of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, "2021" is a string type, not an integer type.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m new value [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExample 2:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExample 3:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: This is the first index.*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Note: The above examples start with arg/sub-command and omit the root command name. In actual operation, you need to manually supplement it. [0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mExample:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255mIf you're not sure whether you should use the standard table, or the inline table, then use the inline table!
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExample:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Except for [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, other formats will use standard tables instead of inline tables.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mQuery the value through the key.[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mExample:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m JSON 1.0 does not support [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m and [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mInf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIt will output [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m instead of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255mDelete the specified key.[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mSimilar to the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m option for the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sub-command, the biggest difference is that the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m option for [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m can be called multiple times in a row to output to multiple formats/files.
[48;2;34;34;34m[38;2;255;255;255mWhile the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m option for the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sub-command can only be called once to output to a single file.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Note: After executing the conversion operation, all files of all formats will not retain comments in the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m sub-command. 
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m While the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m sub-command will retain comments for [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m, but [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mnone[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m for other formats.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mWithout calling [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, calling [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m will overwrite itself.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m If [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m has been called and [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m has also been called, the modified file will be saved to the file pointed to by [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Unless the source file is the same as the file pointed to by [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, the behavior of the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m option for the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sub-command will be similar to that of the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m option for the [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sub-command. They will both automatically rename non-specific file paths.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Assuming from: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, to: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, then the output file will become [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, not [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m If you don't need this feature, please use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m or [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m instead of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. [0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255mPreview the entire file after modification.[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255mSelect the specified key using "." as a separator[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255mGet the value of a specified configuration.[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255mArray of tables with only one K-V pair (Usage: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mThe format is "a.b.c", and sub-table names or array indices are connected by ".".
[48;2;34;34;34m[38;2;255;255;255mWhen there is only one ".", the entire table will be shown.[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255mCreate/Update/Delete the structure of the configuration file.[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mExample:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: the first element is true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mBool array[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255mStandard table, used to store multiple K-V pairs.[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255mType for date and time.[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255mExample 1：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExample 2：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mf64 Array[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString type[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255mSet array elements by calling [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m multiple times.
[48;2;34;34;34m[38;2;255;255;255mExample:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Set the element of index 0 on the first call, set the element of index 1 on the second call, and so on.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255mArray of tables (Usage: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mIt is more compact than the standard table. In addition, for nested keys that do not exist, please use inline tables.[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mInput the key in sections, and the result will be automatically concatenated without using "." as a separator.[0m"##),
    ],
}
}

/// Language ID: en;
/// Map name: "parser";
/// Description: English, Latin, United States;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "The following is stderr output. When redirecting the completion script's output, you should use `>/path/to/file`, not `2>/path/to/file`.");
/// ```
pub(super) const fn get_en_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"The following is stderr output. When redirecting the completion script's output, you should use `>/path/to/file`, not `2>/path/to/file`."##),
    ],
}
}

/// Language ID: en;
/// Map name: "theme";
/// Description: English, Latin, United States;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Invalid theme name");
/// ```
pub(super) const fn get_en_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Invalid theme name"##),
    ],
}
}

/// en: English, Latin, United States
pub(super) const fn get_en_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_en_map_args),
        ("conv_md", get_en_map_conv_md),
        ("parser", get_en_map_parser),
        ("opt_md", get_en_map_opt_md),
        ("args_md", get_en_map_args_md),
        ("opt", get_en_map_opt),
        ("log-core", get_en_map_log_core),
        ("conv", get_en_map_conv),
        ("theme", get_en_map_theme),
    ],
}
}

/// Language ID: en-GB;
/// Map name: "log-core";
/// Description: English, Latin, United Kingdom;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "init-logger");
///
/// assert_eq!(msg, "Initialising logger ...");
/// ```
pub(super) const fn get_en_gb_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("init-logger", r##"Initialising logger ..."##),
    ],
}
}

/// en-GB: English, Latin, United Kingdom
pub(super) const fn get_en_gb_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("log-core", get_en_gb_map_log_core),
    ],
}
}

/// Language ID: eo;
/// Map name: "log-core";
/// Description: esperanto, Latn, Mondo;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_eo_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"avertu"##),
        ("info", r##"Info"##),
        ("error", r##"eraro"##),
        ("trace", r##"spuro"##),
        ("init-logger", r##"Inicializanta logger ..."##),
    ],
}
}

/// Language ID: eo;
/// Map name: "theme";
/// Description: esperanto, Latn, Mondo;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nevalida temnomo");
/// ```
pub(super) const fn get_eo_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nevalida temnomo"##),
    ],
}
}

/// eo: esperanto, Latn, Mondo
pub(super) const fn get_eo_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_eo_map_log_core),
        ("theme", get_eo_map_theme),
    ],
}
}

/// Language ID: es;
/// Map name: "log-core";
/// Description: español, latino, España;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "DEPURACIÓN");
/// ```
pub(super) const fn get_es_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"DEPURACIÓN"##),
        ("warn", r##"ADVERTENCIA"##),
        ("info", r##"INFORMACIÓN"##),
        ("error", r##"ERROR"##),
        ("trace", r##"TRAZA"##),
        ("init-logger", r##"Inicializando el registro..."##),
    ],
}
}

/// Language ID: es;
/// Map name: "args";
/// Description: español, latino, España;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Deshabilita el fondo del tema (predeterminado: habilitado). Uso de `--dis-theme-bg` para deshabilitar el fondo del tema.");
/// ```
pub(super) const fn get_es_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"Especifica un archivo personalizado para el conjunto de temas. También debe especificar el nombre del tema con `--theme`."##),
        ("shell-completion", r##"Genera un script de autocompletado para la shell especificada."##),
        ("theme-file-help", r##"El formato de archivo compatible es la salida binaria volcada por la crate rust `syntect`.
En esencia, estos son temas de la versión antigua de Sublime.
Si no se especifica `--tf`, se leerá el valor de la variable de entorno `$TOMLYRE_THEME_FILE`."##),
        ("from-format", r##"Especifica manualmente el formato de archivo de origen (por defecto, se detecta automáticamente)."##),
        ("disable-theme-background", r##"Deshabilita el fondo del tema (predeterminado: habilitado). Uso de `--dis-theme-bg` para deshabilitar el fondo del tema."##),
        ("theme-uncompressed-file-help", r##"> La versión sin comprimir tiene una prioridad más alta que la versión comprimida.

Si no se especifica `--tuf`, se leerá el valor de la variable de entorno `$TOMLYRE_THEME_UNCOMPRESSED_FILE`."##),
        ("disable-theme-background-help", r##"Si no se especifica `--dis-theme-bg`, se leerá el valor de la variable de entorno `$TOMLYRE_THEME_BG`.

Si el valor es `false`, se desactivará.

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"Al igual que `theme-file`, pero los datos de volcado sin comprimir (archivo) deben estar presentes."##),
        ("shell-completion-help", r##"Debe especificar manualmente el nombre de la shell para generar un script de autocompletado.
Por ejemplo, en PowerShell para Windows, puede usar: `--sh-comp pwsh`

> P: ¿Qué es la función de autocompletado de shell?

> R: Cuando ingresamos un comando en la línea de comandos y presionamos la tecla Tab, se activa el script de autocompletado, que busca opciones de autocompletado posibles y las presenta para su selección. Esta función puede mejorar significativamente la eficiencia y precisión de la entrada en línea de comandos.

La ubicación del archivo de script de autocompletado es diferente para diferentes shells. Para zsh, debe guardarlo en una ruta incluida en la matriz `$fpath`.
Puede usar `print -l $fpath` para obtener el contenido de esta matriz.
Supongamos que incluye el directorio `/usr/local/share/zsh/site-functions`, puede usar el siguiente comando para escribir el script de autocompletado en el archivo `_tomlyre` en ese directorio.

```sh
tomlyre --sh-comp zsh | sudo tee /usr/local/share/zsh/site-functions/_tomlyre
```

Para evitar usar permisos de root, puede colocarlos en un subdirectorio dentro del directorio de inicio del usuario. Por ejemplo, puede usar el siguiente comando para insertar un elemento al principio de la matriz.
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"Cuando se lee datos desde stdin, por ejemplo: `cat a | tomlyre conv -`, intentará determinar automáticamente el formato del archivo primero.
Sin embargo, es posible que la detección automática falle y deba especificar manualmente el formato.
Ejemplo: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Cuando se usa solo `--ts`, se mostrará una lista de conjuntos de estilos integrados.
Además de Markdown, que se puede abreviar como md, los demás estilos deben especificarse con su nombre completo.
Ejemplo de uso 1: `--ts md`
Ejemplo de uso 2: `--table-style ascii`

Nota: Actualmente no se admite la lectura del conjunto de estilos de **tabla** desde un archivo. Si necesita esta función u otras funciones adicionales, no dude en informarnos a través de un problema (issue)."##),
        ("table-style", r##"Especifica el estilo de la tabla."##),
        ("theme-help", r##"Cuando se usa solo `-t`, se mostrará una lista de temas disponibles.
Si no se especifica un archivo de tema, se utilizarán los temas integrados.
Supongamos que existe un tema llamado **ayu-dark**, puede usar `-t ayu-dark` para usar este tema.
Si el nombre del tema contiene espacios, utilice comillas simples o dobles para envolverlo.

> Nota: Hay un tema especial llamado `None`, que representa un estilo de texto plano sin resaltado de sintaxis.

Si no se especifica la opción `--theme`, se leerá la variable de entorno `$TOMLYRE_THEME`.
El tema utilizado para el resaltado de sintaxis en esta información de ayuda también viene de esta variable de entorno.

> La información de ayuda predeterminada está codificada en duro y se carga muy rápido.
> Cargar texto codificado en duro y texto cargado dinámicamente sin resaltado de sintaxis tarda aproximadamente el mismo tiempo. 
> Sin embargo, cargar texto codificado dinámicamente con resaltado de sintaxis tarda algunos órdenes de magnitud más.
> Establecer la variable de entorno `$TOMLYRE_THEME` puede hacer que se tenga que volver a cargar el tema, lo que ralentiza **significativamente** el tiempo de inicio.
> A menos que esté modificando el tema para la información de ayuda o deshabilitando el resaltado de sintaxis, utilice la opción `-t` en lugar de modificar la variable de entorno.

Ejemplo de uso de la variable de entorno:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Ejemplo de uso de argumentos de línea de comandos:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Para las salidas manuales de operaciones como "conv/get/set", la prioridad del tema especificado por `-t` es mayor que la variable de entorno.
Si no se llama a `-t`, se leerá la variable de entorno. Si la variable de entorno está vacía, se utilizará el tema **Monokai Extended**."##),
        ("theme", r##"Especifica el nombre del tema para el resaltado de sintaxis."##),
    ],
}
}

/// Language ID: es;
/// Map name: "args_md";
/// Description: español, latino, España;
pub(super) const fn get_es_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifica un archivo personalizado para el conjunto de temas. También debe especificar el nombre del tema con [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mGenera un script de autocompletado para la shell especificada.[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mEl formato de archivo compatible es la salida binaria volcada por la crate rust [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mEn esencia, estos son temas de la versión antigua de Sublime.
[48;2;34;34;34m[38;2;255;255;255mSi no se especifica [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, se leerá el valor de la variable de entorno [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifica manualmente el formato de archivo de origen (por defecto, se detecta automáticamente).[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mDeshabilita el fondo del tema (predeterminado: habilitado). Uso de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m para deshabilitar el fondo del tema.[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m La versión sin comprimir tiene una prioridad más alta que la versión comprimida.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi no se especifica [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, se leerá el valor de la variable de entorno [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mSi no se especifica [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, se leerá el valor de la variable de entorno [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi el valor es [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, se desactivará.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mAl igual que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, pero los datos de volcado sin comprimir (archivo) deben estar presentes.[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mDebe especificar manualmente el nombre de la shell para generar un script de autocompletado.
[48;2;34;34;34m[38;2;255;255;255mPor ejemplo, en PowerShell para Windows, puede usar: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m P: ¿Qué es la función de autocompletado de shell?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m R: Cuando ingresamos un comando en la línea de comandos y presionamos la tecla Tab, se activa el script de autocompletado, que busca opciones de autocompletado posibles y las presenta para su selección. Esta función puede mejorar significativamente la eficiencia y precisión de la entrada en línea de comandos.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mLa ubicación del archivo de script de autocompletado es diferente para diferentes shells. Para zsh, debe guardarlo en una ruta incluida en la matriz [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mPuede usar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m para obtener el contenido de esta matriz.
[48;2;34;34;34m[38;2;255;255;255mSupongamos que incluye el directorio [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, puede usar el siguiente comando para escribir el script de autocompletado en el archivo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m en ese directorio.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255msudo[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPara evitar usar permisos de root, puede colocarlos en un subdirectorio dentro del directorio de inicio del usuario. Por ejemplo, puede usar el siguiente comando para insertar un elemento al principio de la matriz.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mCuando se lee datos desde stdin, por ejemplo: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, intentará determinar automáticamente el formato del archivo primero.
[48;2;34;34;34m[38;2;255;255;255mSin embargo, es posible que la detección automática falle y deba especificar manualmente el formato.
[48;2;34;34;34m[38;2;255;255;255mEjemplo: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mCuando se usa solo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, se mostrará una lista de conjuntos de estilos integrados.
[48;2;34;34;34m[38;2;255;255;255mAdemás de Markdown, que se puede abreviar como md, los demás estilos deben especificarse con su nombre completo.
[48;2;34;34;34m[38;2;255;255;255mEjemplo de uso 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEjemplo de uso 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mNota: Actualmente no se admite la lectura del conjunto de estilos de [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtabla[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m desde un archivo. Si necesita esta función u otras funciones adicionales, no dude en informarnos a través de un problema (issue).[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifica el estilo de la tabla.[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mCuando se usa solo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, se mostrará una lista de temas disponibles.
[48;2;34;34;34m[38;2;255;255;255mSi no se especifica un archivo de tema, se utilizarán los temas integrados.
[48;2;34;34;34m[38;2;255;255;255mSupongamos que existe un tema llamado [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, puede usar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m para usar este tema.
[48;2;34;34;34m[38;2;255;255;255mSi el nombre del tema contiene espacios, utilice comillas simples o dobles para envolverlo.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: Hay un tema especial llamado [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, que representa un estilo de texto plano sin resaltado de sintaxis.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi no se especifica la opción [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, se leerá la variable de entorno [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mEl tema utilizado para el resaltado de sintaxis en esta información de ayuda también viene de esta variable de entorno.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m La información de ayuda predeterminada está codificada en duro y se carga muy rápido.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Cargar texto codificado en duro y texto cargado dinámicamente sin resaltado de sintaxis tarda aproximadamente el mismo tiempo. 
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Sin embargo, cargar texto codificado dinámicamente con resaltado de sintaxis tarda algunos órdenes de magnitud más.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Establecer la variable de entorno [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m puede hacer que se tenga que volver a cargar el tema, lo que ralentiza [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114msignificativamente[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m el tiempo de inicio.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A menos que esté modificando el tema para la información de ayuda o deshabilitando el resaltado de sintaxis, utilice la opción [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m en lugar de modificar la variable de entorno.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEjemplo de uso de la variable de entorno:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEjemplo de uso de argumentos de línea de comandos:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPara las salidas manuales de operaciones como "conv/get/set", la prioridad del tema especificado por [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m es mayor que la variable de entorno.
[48;2;34;34;34m[38;2;255;255;255mSi no se llama a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, se leerá la variable de entorno. Si la variable de entorno está vacía, se utilizará el tema [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifica el nombre del tema para el resaltado de sintaxis.[0m"##),
    ],
}
}

/// Language ID: es;
/// Map name: "conv";
/// Description: español, latino, España;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Convierte entre formatos como toml, yaml, json y ron");
/// ```
pub(super) const fn get_es_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Convierte entre formatos como toml, yaml, json y ron"##),
        ("from", r##"Especifica el archivo de entrada"##),
        ("conv-to", r##"Especifica el formato/archivo de salida, se puede utilizar varias veces."##),
        ("save", r##"Cuando se llama a esta opción, el archivo especificado en `--to` se guardará."##),
        ("conv-to-help", r##"Si hay un archivo llamado **test.json** en el directorio actual, puede usar `conv test.json -t yml` para convertirlo a formato yml (yaml).
También puede especificar la ruta del archivo manualmente: `conv test.json -t /tmp/test.yml`

Los formatos compatibles dependen de las características habilitadas en este paquete. No todas las características están habilitadas de forma predeterminada.

Además, hay algunas cosas que debe saber antes de usar este comando.
- 1. Se renombrarán automáticamente las rutas que no sean específicas de archivos.
  - 1.1: Un archivo con solo un nivel de profundidad se considera un formato de archivo, no una ruta de archivo.
    Por ejemplo, al llamar a `conv test.toml -t ron`, el archivo de salida se renombrará automáticamente como `test.ron`, no como `ron`.
    Si no necesita esta función, utilice `-t ./ron` o `-t $PWD/ron`.
  - 1.2: La ruta especifica a un directorio en lugar de a un archivo
    Por ejemplo, `-t /tmp/`. Solo podemos colocar el archivo `test.toml` en un subdirectorio de `/tmp/`, pero no reemplazar `/tmp` en sí.
    Si no necesita esta función, especifique la ruta completa del archivo (en lugar del directorio), por ejemplo `-t /tmp/xx.toml` en lugar de `-t /tmp/`.
- 2. Cuando no se llama a `--save`, el archivo especificado no se guarda automáticamente.
- 3. Cuando se llama a `--save`, la entrada y salida del subcomando conv no pueden ser el mismo archivo.
- 4. Los comentarios no se conservan después de la conversión del archivo."##),
    ],
}
}

/// Language ID: es;
/// Map name: "conv_md";
/// Description: español, latino, España;
pub(super) const fn get_es_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mConvierte entre formatos como toml, yaml, json y ron[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifica el archivo de entrada[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifica el formato/archivo de salida, se puede utilizar varias veces.[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mCuando se llama a esta opción, el archivo especificado en [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m se guardará.[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mSi hay un archivo llamado [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m en el directorio actual, puede usar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m para convertirlo a formato yml (yaml).
[48;2;34;34;34m[38;2;255;255;255mTambién puede especificar la ruta del archivo manualmente: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mLos formatos compatibles dependen de las características habilitadas en este paquete. No todas las características están habilitadas de forma predeterminada.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAdemás, hay algunas cosas que debe saber antes de usar este comando.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Se renombrarán automáticamente las rutas que no sean específicas de archivos.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Un archivo con solo un nivel de profundidad se considera un formato de archivo, no una ruta de archivo.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mPor ejemplo, al llamar a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, el archivo de salida se renombrará automáticamente como [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, no como [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mSi no necesita esta función, utilice [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m o [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: La ruta especifica a un directorio en lugar de a un archivo
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mPor ejemplo, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Solo podemos colocar el archivo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m en un subdirectorio de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, pero no reemplazar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m en sí.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mSi no necesita esta función, especifique la ruta completa del archivo (en lugar del directorio), por ejemplo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m en lugar de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. Cuando no se llama a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, el archivo especificado no se guarda automáticamente.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. Cuando se llama a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, la entrada y salida del subcomando conv no pueden ser el mismo archivo.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. Los comentarios no se conservan después de la conversión del archivo.[0m"##),
    ],
}
}

/// Language ID: es;
/// Map name: "opt";
/// Description: español, latino, España;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "Array de tablas. (modo de uso: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_es_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"La diferencia entre `--ck` y `-k` es que `--ck` no dividirá automáticamente la clave con el símbolo ".".
Para [a.b.c], `-k a.b.c` es lo mismo que `--ck a --ck b --ck c`.
Para casos simples, usar `-k` puede ser más sencillo.

Si el nombre de la clave contiene ".", use `--ck` en lugar de `-k`.
Por ejemplo:

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

En esta estructura, "editor.defaultFormatter" es una cadena completa, no una subclave de "editor".

Cuando usa `-k '[typescript]'.'editor.defaultFormatter'`, la clave se divide en:

- "[typescript]"
- "editor"
- "defaultFormatter"

Mientras que con `--ck '[typescript]' --ck editor.defaultFormatter'`, la clave se trata como:

- "[typescript]"
- "editor.defaultFormatter"

Esta última es la estructura correcta."##),
        ("num-arr", r##"Array de números enteros con signo de 64 bits (i64)."##),
        ("bool", r##"Tipo bool (booleano), sólo puede ser `true` o `false`."##),
        ("num", r##"Tipo i64: número entero con signo de 64 bits, por ejemplo: 1145."##),
        ("datetime-help", r##"Ejemplo:
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

Esta opción sólo es compatible con `toml`. Si agregamos `-t json` al final del comando, se verá así:

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"Guardar como formato/archivo especificado. Solo se puede guardar uno a la vez."##),
        ("get-opts-help", r##"Ejemplo de uso 1: `get Cargo.toml -k package.name`
Ejemplo de uso 2: `get cfg.yaml -k .`

Nota: Cuando se proporciona solo la clave (`-k`), se mostrará el tipo de datos y los valores correspondientes en una tabla detallada.
Por ejemplo, `set Cargo.toml -k package` mostraría una tabla con información detallada sobre el paquete y sus subtablas. "##),
        ("none-help", r##"> Nota: RON tiene un tipo `Option<T>` real que puede ser `Some(T)` o `None`. 
> El uso de `--none` aquí es para eliminar una clave específica, no para un verdadero tipo `Option<T>`.

Supongamos que tenemos el siguiente array de tabla en `test.yml`:

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

Si deseamos eliminar la entrada `name: test`, podemos hacer lo siguiente:
```sh
set test.yml -k bin.1.name --none
```

> Nota: una cadena vacía y `None` son cosas diferentes.
> Usando `set test.yml --kv bin.1.name ""` podemos establecerla como una cadena vacía, mientras que `--none` elimina toda la clave.

Si queremos eliminar la segunda tabla (`index = 1`) de `bin`, podemos hacer lo siguiente:
```sh
set test.yml -k bin.1 --none
```

O use `--rm`:
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"Tipo flotante de doble precisión (estándar IEEE 754-2008), por ejemplo: 3.14159265359."##),
        ("saot-help", r##"> Nota: Aunque sólo se puede crear una pareja clave-valor por tabla con `--saot`, aún puede especificar el índice del array de tablas primero y luego usar otras opciones para ampliarlo.

Ejemplo:
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"Establecer pares clave-valor (`--kv x.y.z value` es equivalente a `-k x.y.z -s value`)."##),
        ("set-save", r##"Si no se especifica `--to`, sobrescribe (escribe) el archivo actual."##),
        ("array", r##"Array de cadenas de texto."##),
        ("aot-help", r##"Ejemplo:
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

Primero, seleccione la clave especificada y luego modifique la estructura del archivo especificando el tipo de datos del valor.

> Si solo proporciona la clave pero no el valor, se mostrarán los tipos de datos y los valores correspondientes.
> Por ejemplo, `set Cargo.toml -k package` mostrará una tabla detallada del paquete y sus subtablas.

## Ejemplos

### Ejemplo 1:
```sh
--ts md set test.toml --key package.edition --str 2021
```

Dado que se especificó `--str`, en lugar de `--num`, "2021" se trata como una cadena de texto en lugar de un número.

| package | nuevo valor |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### Ejemplo 2:
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### Ejemplo 3:
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: Este es el primer índice */ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

Nota: Los ejemplos anteriores omiten el nombre del comando raíz y comienzan con un argumento/subcomando, por lo que deberá especificarlo manualmente en la práctica."##),
        ("num-arr-help", r##"Ejemplo:
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"Ejemplo:
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> Excepto `toml`, otros formatos utilizarán una tabla estándar, no una tabla en línea.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"Consultar un valor mediante una clave."##),
        ("f64-arr-help", r##"Ejemplo:
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> JSON 1.0 no admite NaN e Inf

En su lugar, se imprimirá `null` en lugar de `NaN`.

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"Elimina la clave especificada."##),
        ("set-save-to-help", r##"Similar al uso de la opción `-t` para el sub-comando `conv`, la mayor diferencia es que la opción `-t` para el sub-comando `set` solo puede ser llamada una vez para guardar en un único archivo.
Mientras que la opción `-t` para el sub-comando `conv` se puede llamar varias veces seguidas para generar múltiples formatos/archivos.

> Nota: Después de ejecutar la operación de conversión, todos los archivos de todos los formatos perderán los comentarios en el sub-comando `conv`.
> Mientras que el sub-comando `set` conservará los comentarios para **toml**, pero no para otros formatos.

- Si no se utiliza la opción `-t`, llamar a `--save` sobrescribirá el propio archivo.
- Si se ha utilizado `-t` y también se llama a `--save`, el archivo modificado será guardado en el archivo apuntado por `-t`.
- A menos que el archivo origen sea el mismo que el archivo apuntado por `-t`, el comportamiento de la opción `-t` para el sub-comando `set` será similar al de la opción `-t` para el sub-comando `conv`. Ambas renombrarán automáticamente las rutas de archivo no específicas.
  - Suponiendo de: **test.yml**, a: **json**, entonces el archivo de salida se convertirá en **test.json**, no en **json**
  - Si no se necesita esta característica, utilice `-t ./json` o `-t $PWD/json` en lugar de `-t json`. "##),
        ("preview", r##"Vista previa del archivo completo después de las modificaciones."##),
        ("set-key", r##"Seleccionar una clave (key) específica, separada por puntos (".")"##),
        ("get-opts", r##"Obtener el valor de una configuración específica."##),
        ("saot", r##"Array de tablas con una sola pareja de clave-valor cada una. (Modo de uso: `--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"El formato es "a.b.c", donde las tablas secundarias o las indicaciones de matriz están conectadas por "." (puntos).
Cuando hay solo un ".", se mostrará toda la tabla."##),
        ("set-opts", r##"Crear/actualizar/borrar la estructura de un archivo de configuración."##),
        ("bool-arr-help", r##"Ejemplo:
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: El primer elemento es true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"Array de booleanos."##),
        ("table", r##"Tabla estándar utilizada para almacenar varias parejas clave-valor."##),
        ("datetime", r##"Tipo de fecha y hora."##),
        ("table-help", r##"Ejemplo 1:
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

Ejemplo 2:
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"Array de flotantes de doble precisión (f64)."##),
        ("string", r##"Tipo string (cadena de texto)."##),
        ("array-help", r##"Establezca los elementos del array llamando a `-a` varias veces.

Por ejemplo:
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> La primera vez se establece el elemento 0, la segunda vez se establece el elemento 1 y así sucesivamente.

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"Array de tablas. (modo de uso: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"Tabla en línea, esencialmente una versión más compacta de una tabla estándar."##),
        ("concat-key", r##"Introducir la clave en secciones y se unirán automáticamente, sin utilizar "." como separador."##),
    ],
}
}

/// Language ID: es;
/// Map name: "opt_md";
/// Description: español, latino, España;
pub(super) const fn get_es_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255mLa diferencia entre [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m y [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m es que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m no dividirá automáticamente la clave con el símbolo ".".
[48;2;34;34;34m[38;2;255;255;255mPara [a.b.c], [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m es lo mismo que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mPara casos simples, usar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m puede ser más sencillo.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi el nombre de la clave contiene ".", use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m en lugar de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mPor ejemplo:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEn esta estructura, "editor.defaultFormatter" es una cadena completa, no una subclave de "editor".
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mCuando usa [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, la clave se divide en:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mMientras que con [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, la clave se trata como:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEsta última es la estructura correcta.[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mArray de números enteros con signo de 64 bits (i64).[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mTipo bool (booleano), sólo puede ser [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m o [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mTipo i64: número entero con signo de 64 bits, por ejemplo: 1145.[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255mEjemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEsta opción sólo es compatible con [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Si agregamos [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m al final del comando, se verá así:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255mGuardar como formato/archivo especificado. Solo se puede guardar uno a la vez.[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255mEjemplo de uso 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEjemplo de uso 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mNota: Cuando se proporciona solo la clave ([48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m), se mostrará el tipo de datos y los valores correspondientes en una tabla detallada.
[48;2;34;34;34m[38;2;255;255;255mPor ejemplo, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m mostraría una tabla con información detallada sobre el paquete y sus subtablas. [0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: RON tiene un tipo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m real que puede ser [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m o [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m. 
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m El uso de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m aquí es para eliminar una clave específica, no para un verdadero tipo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSupongamos que tenemos el siguiente array de tabla en [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi deseamos eliminar la entrada [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mname: test[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, podemos hacer lo siguiente:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: una cadena vacía y [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m son cosas diferentes.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Usando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m podemos establecerla como una cadena vacía, mientras que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m elimina toda la clave.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi queremos eliminar la segunda tabla ([48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mindex = 1[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m) de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mbin[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, podemos hacer lo siguiente:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mO use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255mTipo flotante de doble precisión (estándar IEEE 754-2008), por ejemplo: 3.14159265359.[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: Aunque sólo se puede crear una pareja clave-valor por tabla con [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, aún puede especificar el índice del array de tablas primero y luego usar otras opciones para ampliarlo.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEjemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mEstablecer pares clave-valor ([48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m es equivalente a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m).[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255mSi no se especifica [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, sobrescribe (escribe) el archivo actual.[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255mArray de cadenas de texto.[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255mEjemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPrimero, seleccione la clave especificada y luego modifique la estructura del archivo especificando el tipo de datos del valor.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Si solo proporciona la clave pero no el valor, se mostrarán los tipos de datos y los valores correspondientes.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Por ejemplo, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m mostrará una tabla detallada del paquete y sus subtablas.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mEjemplos[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mEjemplo 1:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDado que se especificó [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, en lugar de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, "2021" se trata como una cadena de texto en lugar de un número.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m nuevo valor [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mEjemplo 2:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mEjemplo 3:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: Este es el primer índice */[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mNota: Los ejemplos anteriores omiten el nombre del comando raíz y comienzan con un argumento/subcomando, por lo que deberá especificarlo manualmente en la práctica.[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mEjemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255mEjemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Excepto [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, otros formatos utilizarán una tabla estándar, no una tabla en línea.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mConsultar un valor mediante una clave.[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mEjemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m JSON 1.0 no admite NaN e Inf
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEn su lugar, se imprimirá [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m en lugar de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255mElimina la clave especificada.[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mSimilar al uso de la opción [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m para el sub-comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, la mayor diferencia es que la opción [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m para el sub-comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m solo puede ser llamada una vez para guardar en un único archivo.
[48;2;34;34;34m[38;2;255;255;255mMientras que la opción [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m para el sub-comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m se puede llamar varias veces seguidas para generar múltiples formatos/archivos.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: Después de ejecutar la operación de conversión, todos los archivos de todos los formatos perderán los comentarios en el sub-comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Mientras que el sub-comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m conservará los comentarios para [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m, pero no para otros formatos.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mSi no se utiliza la opción [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, llamar a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sobrescribirá el propio archivo.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Si se ha utilizado [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m y también se llama a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, el archivo modificado será guardado en el archivo apuntado por [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m A menos que el archivo origen sea el mismo que el archivo apuntado por [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, el comportamiento de la opción [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m para el sub-comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m será similar al de la opción [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m para el sub-comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Ambas renombrarán automáticamente las rutas de archivo no específicas.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Suponiendo de: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, a: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, entonces el archivo de salida se convertirá en [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, no en [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Si no se necesita esta característica, utilice [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m o [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m en lugar de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. [0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255mVista previa del archivo completo después de las modificaciones.[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255mSeleccionar una clave (key) específica, separada por puntos (".")[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255mObtener el valor de una configuración específica.[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255mArray de tablas con una sola pareja de clave-valor cada una. (Modo de uso: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mEl formato es "a.b.c", donde las tablas secundarias o las indicaciones de matriz están conectadas por "." (puntos).
[48;2;34;34;34m[38;2;255;255;255mCuando hay solo un ".", se mostrará toda la tabla.[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255mCrear/actualizar/borrar la estructura de un archivo de configuración.[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mEjemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: El primer elemento es true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mArray de booleanos.[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255mTabla estándar utilizada para almacenar varias parejas clave-valor.[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255mTipo de fecha y hora.[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255mEjemplo 1:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEjemplo 2:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mArray de flotantes de doble precisión (f64).[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mTipo string (cadena de texto).[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255mEstablezca los elementos del array llamando a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m varias veces.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPor ejemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m La primera vez se establece el elemento 0, la segunda vez se establece el elemento 1 y así sucesivamente.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255mArray de tablas. (modo de uso: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mTabla en línea, esencialmente una versión más compacta de una tabla estándar.[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mIntroducir la clave en secciones y se unirán automáticamente, sin utilizar "." como separador.[0m"##),
    ],
}
}

/// Language ID: es;
/// Map name: "parser";
/// Description: español, latino, España;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "A continuación se muestra la salida stderr. Al redirigir la salida del script de autocompletado, debe utilizar `>/path/to/file` en lugar de `2>/path/to/file`.");
/// ```
pub(super) const fn get_es_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"A continuación se muestra la salida stderr. Al redirigir la salida del script de autocompletado, debe utilizar `>/path/to/file` en lugar de `2>/path/to/file`."##),
    ],
}
}

/// Language ID: es;
/// Map name: "theme";
/// Description: español, latino, España;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nombre de tema no válido");
/// ```
pub(super) const fn get_es_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nombre de tema no válido"##),
    ],
}
}

/// es: español, latino, España
pub(super) const fn get_es_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_es_map_args),
        ("conv_md", get_es_map_conv_md),
        ("parser", get_es_map_parser),
        ("opt_md", get_es_map_opt_md),
        ("args_md", get_es_map_args_md),
        ("opt", get_es_map_opt),
        ("log-core", get_es_map_log_core),
        ("conv", get_es_map_conv),
        ("theme", get_es_map_theme),
    ],
}
}

/// Language ID: et;
/// Map name: "log-core";
/// Description: eesti, ladina, Eesti;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "silumine");
/// ```
pub(super) const fn get_et_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"silumine"##),
        ("warn", r##"hoiatab"##),
        ("info", r##"teave"##),
        ("error", r##"viga"##),
        ("trace", r##"jälg"##),
        ("init-logger", r##"Logija initsialiseerimine ..."##),
    ],
}
}

/// Language ID: et;
/// Map name: "theme";
/// Description: eesti, ladina, Eesti;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Kehtetu teemanimi");
/// ```
pub(super) const fn get_et_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Kehtetu teemanimi"##),
    ],
}
}

/// et: eesti, ladina, Eesti
pub(super) const fn get_et_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_et_map_log_core),
        ("theme", get_et_map_theme),
    ],
}
}

/// Language ID: eu;
/// Map name: "log-core";
/// Description: euskara, latinoa, Espainia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "arazketa");
/// ```
pub(super) const fn get_eu_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"arazketa"##),
        ("warn", r##"ohartarazi"##),
        ("info", r##"informazioa"##),
        ("error", r##"errorea"##),
        ("trace", r##"arrastoa"##),
        ("init-logger", r##"Logger hasieran ..."##),
    ],
}
}

/// Language ID: eu;
/// Map name: "theme";
/// Description: euskara, latinoa, Espainia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Gaiaren izena baliogabea");
/// ```
pub(super) const fn get_eu_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Gaiaren izena baliogabea"##),
    ],
}
}

/// eu: euskara, latinoa, Espainia
pub(super) const fn get_eu_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_eu_map_log_core),
        ("theme", get_eu_map_theme),
    ],
}
}

/// Language ID: fa;
/// Map name: "log-core";
/// Description: فارسی, عربی, ایران;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "اشکال زدایی");
/// ```
pub(super) const fn get_fa_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"اشکال زدایی"##),
        ("warn", r##"هشدار دهید"##),
        ("info", r##"اطلاعات"##),
        ("error", r##"خطا"##),
        ("trace", r##"ردیابی"##),
        ("init-logger", r##"اولیه سازی logger ..."##),
    ],
}
}

/// Language ID: fa;
/// Map name: "theme";
/// Description: فارسی, عربی, ایران;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "نام موضوع نامعتبر");
/// ```
pub(super) const fn get_fa_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"نام موضوع نامعتبر"##),
    ],
}
}

/// fa: فارسی, عربی, ایران
pub(super) const fn get_fa_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_fa_map_log_core),
        ("theme", get_fa_map_theme),
    ],
}
}

/// Language ID: fi;
/// Map name: "log-core";
/// Description: suomi, latinalainen, Suomi;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "virheenkorjaus");
/// ```
pub(super) const fn get_fi_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"virheenkorjaus"##),
        ("warn", r##"varoita"##),
        ("info", r##"tiedot"##),
        ("error", r##"virhe"##),
        ("trace", r##"Jälki"##),
        ("init-logger", r##"Loggerin alustaminen ..."##),
    ],
}
}

/// Language ID: fi;
/// Map name: "theme";
/// Description: suomi, latinalainen, Suomi;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Virheellinen teeman nimi");
/// ```
pub(super) const fn get_fi_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Virheellinen teeman nimi"##),
    ],
}
}

/// fi: suomi, latinalainen, Suomi
pub(super) const fn get_fi_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_fi_map_log_core),
        ("theme", get_fi_map_theme),
    ],
}
}

/// Language ID: fil;
/// Map name: "log-core";
/// Description: Filipino, Latin, Pilipinas;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "DEBAG");
/// ```
pub(super) const fn get_fil_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"DEBAG"##),
        ("warn", r##"BABALA"##),
        ("info", r##"IMPORMASYON"##),
        ("error", r##"ERROR"##),
        ("trace", r##"TUMUTULONG"##),
        ("init-logger", r##"Pagsisimula ng logger..."##),
    ],
}
}

/// Language ID: fil;
/// Map name: "theme";
/// Description: Filipino, Latin, Pilipinas;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Hindi wastong pangalan ng tema");
/// ```
pub(super) const fn get_fil_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Hindi wastong pangalan ng tema"##),
    ],
}
}

/// fil: Filipino, Latin, Pilipinas
pub(super) const fn get_fil_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_fil_map_log_core),
        ("theme", get_fil_map_theme),
    ],
}
}

/// Language ID: fr;
/// Map name: "log-core";
/// Description: français, latin, France;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "DÉBOGAGE");
/// ```
pub(super) const fn get_fr_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"DÉBOGAGE"##),
        ("warn", r##"AVERTISSEMENT"##),
        ("info", r##"INFORMATIONS"##),
        ("error", r##"ERREUR"##),
        ("trace", r##"TRACER"##),
        ("init-logger", r##"Initialisation du journal..."##),
    ],
}
}

/// Language ID: fr;
/// Map name: "args";
/// Description: français, latin, France;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Désactiver l'arrière-plan du thème (activé par défaut, désactivé si `--dis-theme-bg` est appelé)");
/// ```
pub(super) const fn get_fr_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"Chemin vers un fichier contenant des thèmes personnalisés (doit être utilisé avec `--theme`)"##),
        ("shell-completion", r##"Générer un script de complétion pour shell"##),
        ("theme-file-help", r##"Le format de fichier pris en charge est celui des données binaires générées par la fonction dump du crate rust `syntect`.
Essentiellement, ces thèmes sont des anciens thèmes Sublime.
Si aucune option `--tf` n'est spécifiée, les données seront lues à partir de la variable d'environnement `$TOMLYRE_THEME_FILE`."##),
        ("from-format", r##"Spécifier manuellement le format du fichier source (il sera automatiquement détecté par défaut)"##),
        ("disable-theme-background", r##"Désactiver l'arrière-plan du thème (activé par défaut, désactivé si `--dis-theme-bg` est appelé)"##),
        ("theme-uncompressed-file-help", r##"> La version non compressée aura une priorité plus élevée que la version compressée.

Si aucune option `--tuf` n'est spécifiée, les données seront lues à partir de la variable d'environnement `$TOMLYRE_THEME_UNCOMPRESSED_FILE`."##),
        ("disable-theme-background-help", r##"Si l'option `--dis-theme-bg` n'est pas appelée, les données seront lues à partir de la variable d'environnement `$TOMLYRE_THEME_BG`.

Si cette valeur est fausse, l'arrière-plan du thème sera désactivé.

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"Chemin vers un fichier contenant des thèmes personnalisés non compressés (doit être utilisé avec `--theme`)"##),
        ("shell-completion-help", r##"Vous devez spécifier manuellement le nom du shell pour lequel vous voulez générer le script de complétion.
Par exemple, pour PowerShell sur Windows, utilisez : `--sh-comp pwsh`

> Q: Qu'est-ce que la fonctionnalité de complétion dans le shell ?

> R: Lorsque nous saisissons une commande dans la ligne de commande, si nous appuyons sur la touche Tab, le script de complétion sera déclenché, commençant à rechercher les options de complétion possibles et les présentant pour sélection. Cette fonctionnalité peut grandement améliorer l'efficacité et la précision de la saisie en ligne de commande.

Les scripts de complétion pour différents shells doivent être stockés à des emplacements différents. Pour Zsh, vous devrez les stocker dans des répertoires inclus dans la variable `$fpath`. Vous pouvez utiliser `print -l $fpath` pour obtenir le contenu de cette variable.
Supposons qu'elle comprenne le répertoire `/usr/local/share/zsh/site-functions`, vous pouvez alors utiliser la commande suivante pour écrire le script de complétion dans le fichier `_tomlyre` de ce répertoire.

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

Pour éviter d'utiliser des privilèges root, vous pouvez les stocker dans le répertoire de votre utilisateur. Par exemple, vous pouvez utiliser la commande suivante pour insérer un élément au début de l'array.

```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"Lors de la lecture de données depuis stdin, par exemple : `cat a | tomlyre conv -`, le format du fichier sera automatiquement détecté en premier lieu. Cependant, la détection automatique peut échouer et vous devrez alors spécifier manuellement le format.
Exemple : `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Lorsque vous utilisez l'option `--ts` sans spécifier de nom de style, une liste des styles intégrés sera affichée.
À l'exception de Markdown, qui peut être abrégé en md, tous les autres styles doivent être écrits en toutes lettres.
Exemple: `--ts md` 
Exemple: `--table-style ascii`

Note : actuellement, il n'est pas possible de lire la collection de **styles de tableau** à partir d'un fichier. Si vous avez besoin de cette fonctionnalité ou d'autres fonctionnalités supplémentaires, veuillez soumettre une issue pour donner votre avis."##),
        ("table-style", r##"Spécifier le style des tableaux"##),
        ("theme-help", r##"Lorsque vous utilisez l'option `-t` sans spécifier de nom de thème, une liste des thèmes disponibles sera affichée.
Si aucun fichier de thème n'est spécifié, le jeu de thèmes intégré sera utilisé.
Supposons qu'un thème appelé **ayu-dark** existe, vous pouvez utiliser `-t ayu-dark` pour utiliser ce thème.
Si le nom du thème contient des espaces, veuillez utiliser des guillemets simples ou doubles pour l'encadrer.

> Remarque: il y a un thème spécial appelé `None`, qui représente le style brut sans coloration syntaxique.

Si aucune option `--theme` n'est spécifiée, les données seront lues à partir de la variable d'environnement `$TOMLYRE_THEME`.
Le thème des exemples de code dans cette aide est également déterminé par la variable d'environnement `$TOMLYRE_THEME`.

> L'aide par défaut est codée en dur pour une vitesse de chargement maximale.
> Le temps de chargement pour le texte codé en dur et le texte sans coloration syntaxique (`None`) est similaire,
> tandis que le chargement dynamique prendra deux fois plus de temps ou plus.
> Définir la variable d'environnement `$TOMLYRE_THEME` peut ralentir considérablement le temps de démarrage car elle force
> un rechargement du thème.
> À moins que vous ne prévoyiez de modifier le thème de l'aide ou de désactiver la coloration syntaxique, utilisez l'option `-t` plutôt que de modifier la variable d'environnement.

Exemple d'utilisation de la variable d'environnement :
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Exemple d'utilisation avec les options en ligne de commande:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Pour les sorties manuelles telles que celles générées par les commandes "conv/get/set", etc., le thème spécifié par `-t` a la priorité sur la variable d'environnement.
Si l'option `-t` n'est pas appelée, la variable d'environnement est utilisée ; si la variable d'environnement est vide,
le thème **Monokai Extended** est utilisé."##),
        ("theme", r##"Nom du thème pour la coloration syntaxique"##),
    ],
}
}

/// Language ID: fr;
/// Map name: "args_md";
/// Description: français, latin, France;
pub(super) const fn get_fr_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mChemin vers un fichier contenant des thèmes personnalisés (doit être utilisé avec [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mGénérer un script de complétion pour shell[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mLe format de fichier pris en charge est celui des données binaires générées par la fonction dump du crate rust [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mEssentiellement, ces thèmes sont des anciens thèmes Sublime.
[48;2;34;34;34m[38;2;255;255;255mSi aucune option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m n'est spécifiée, les données seront lues à partir de la variable d'environnement [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mSpécifier manuellement le format du fichier source (il sera automatiquement détecté par défaut)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mDésactiver l'arrière-plan du thème (activé par défaut, désactivé si [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m est appelé)[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m La version non compressée aura une priorité plus élevée que la version compressée.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi aucune option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m n'est spécifiée, les données seront lues à partir de la variable d'environnement [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mSi l'option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m n'est pas appelée, les données seront lues à partir de la variable d'environnement [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi cette valeur est fausse, l'arrière-plan du thème sera désactivé.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mChemin vers un fichier contenant des thèmes personnalisés non compressés (doit être utilisé avec [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mVous devez spécifier manuellement le nom du shell pour lequel vous voulez générer le script de complétion.
[48;2;34;34;34m[38;2;255;255;255mPar exemple, pour PowerShell sur Windows, utilisez : [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q: Qu'est-ce que la fonctionnalité de complétion dans le shell ?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m R: Lorsque nous saisissons une commande dans la ligne de commande, si nous appuyons sur la touche Tab, le script de complétion sera déclenché, commençant à rechercher les options de complétion possibles et les présentant pour sélection. Cette fonctionnalité peut grandement améliorer l'efficacité et la précision de la saisie en ligne de commande.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mLes scripts de complétion pour différents shells doivent être stockés à des emplacements différents. Pour Zsh, vous devrez les stocker dans des répertoires inclus dans la variable [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Vous pouvez utiliser [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m pour obtenir le contenu de cette variable.
[48;2;34;34;34m[38;2;255;255;255mSupposons qu'elle comprenne le répertoire [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, vous pouvez alors utiliser la commande suivante pour écrire le script de complétion dans le fichier [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m de ce répertoire.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPour éviter d'utiliser des privilèges root, vous pouvez les stocker dans le répertoire de votre utilisateur. Par exemple, vous pouvez utiliser la commande suivante pour insérer un élément au début de l'array.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mLors de la lecture de données depuis stdin, par exemple : [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, le format du fichier sera automatiquement détecté en premier lieu. Cependant, la détection automatique peut échouer et vous devrez alors spécifier manuellement le format.
[48;2;34;34;34m[38;2;255;255;255mExemple : [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mLorsque vous utilisez l'option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sans spécifier de nom de style, une liste des styles intégrés sera affichée.
[48;2;34;34;34m[38;2;255;255;255mÀ l'exception de Markdown, qui peut être abrégé en md, tous les autres styles doivent être écrits en toutes lettres.
[48;2;34;34;34m[38;2;255;255;255mExemple: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 
[48;2;34;34;34m[38;2;255;255;255mExemple: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mNote : actuellement, il n'est pas possible de lire la collection de [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mstyles de tableau[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m à partir d'un fichier. Si vous avez besoin de cette fonctionnalité ou d'autres fonctionnalités supplémentaires, veuillez soumettre une issue pour donner votre avis.[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mSpécifier le style des tableaux[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mLorsque vous utilisez l'option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sans spécifier de nom de thème, une liste des thèmes disponibles sera affichée.
[48;2;34;34;34m[38;2;255;255;255mSi aucun fichier de thème n'est spécifié, le jeu de thèmes intégré sera utilisé.
[48;2;34;34;34m[38;2;255;255;255mSupposons qu'un thème appelé [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m existe, vous pouvez utiliser [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m pour utiliser ce thème.
[48;2;34;34;34m[38;2;255;255;255mSi le nom du thème contient des espaces, veuillez utiliser des guillemets simples ou doubles pour l'encadrer.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Remarque: il y a un thème spécial appelé [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, qui représente le style brut sans coloration syntaxique.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi aucune option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m n'est spécifiée, les données seront lues à partir de la variable d'environnement [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mLe thème des exemples de code dans cette aide est également déterminé par la variable d'environnement [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m L'aide par défaut est codée en dur pour une vitesse de chargement maximale.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Le temps de chargement pour le texte codé en dur et le texte sans coloration syntaxique ([48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m) est similaire,
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m tandis que le chargement dynamique prendra deux fois plus de temps ou plus.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Définir la variable d'environnement [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m peut ralentir considérablement le temps de démarrage car elle force
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m un rechargement du thème.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m À moins que vous ne prévoyiez de modifier le thème de l'aide ou de désactiver la coloration syntaxique, utilisez l'option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m plutôt que de modifier la variable d'environnement.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExemple d'utilisation de la variable d'environnement :
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExemple d'utilisation avec les options en ligne de commande:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPour les sorties manuelles telles que celles générées par les commandes "conv/get/set", etc., le thème spécifié par [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m a la priorité sur la variable d'environnement.
[48;2;34;34;34m[38;2;255;255;255mSi l'option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m n'est pas appelée, la variable d'environnement est utilisée ; si la variable d'environnement est vide,
[48;2;34;34;34m[38;2;255;255;255mle thème [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m est utilisé.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mNom du thème pour la coloration syntaxique[0m"##),
    ],
}
}

/// Language ID: fr;
/// Map name: "conv";
/// Description: français, latin, France;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Convertir entre les formats tels que toml, yaml, json et ron");
/// ```
pub(super) const fn get_fr_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Convertir entre les formats tels que toml, yaml, json et ron"##),
        ("from", r##"Spécifier le fichier d'entrée"##),
        ("conv-to", r##"Spécifier le format/fichier de sortie, peut être appelé plusieurs fois"##),
        ("save", r##"Lorsque cette option est appelée, le fichier spécifié avec `--to` sera enregistré."##),
        ("conv-to-help", r##"Si un fichier nommé **test.json** existe dans le répertoire en cours, vous pouvez appeler `conv test.json -t yml` pour le convertir en yml (yaml),
ou spécifier manuellement le chemin du fichier : `conv test.json -t /tmp/test.yml`

Les formats pris en charge dépendent des fonctionnalités activées par ce package et toutes les fonctionnalités ne sont pas activées par défaut.

De plus, il y a certaines choses à savoir avant de l'utiliser.
- 1. Il renomme automatiquement les chemins non spécifiques de fichiers.
  - 1.1: Seuls les fichiers avec un niveau de profondeur seront considérés comme des formats de fichier plutôt que des chemins de fichier.
    Lorsque vous appelez `conv test.toml -t ron`, le fichier de sortie sera automatiquement renommé en `test.ron`, plutôt qu'en `ron`.
    Si vous n'avez pas besoin de cette fonctionnalité, veuillez utiliser `-t ./ron` ou `-t $PWD/ron`.
  - 1.2: Pointe vers un dossier au lieu d'un fichier
    Par exemple, si vous utilisez `-t /tmp/`, nous ne pouvons placer `test.toml` que dans le sous-répertoire de `/tmp/`, plutôt que de remplacer `/tmp` lui-même.
    Si vous n'avez pas besoin de cette fonctionnalité, veuillez utiliser le chemin complet du fichier (pas seulement le dossier), par exemple `-t /tmp/xx.toml` au lieu de `-t /tmp/`.
- 2. Lorsque `--save` n'est pas appelé, les fichiers spécifiés ne seront pas enregistrés automatiquement.
- 3. Lorsque `--save` est appelé, l'entrée et la sortie de la commande conv ne peuvent pas être le même fichier.
- 4. Les commentaires ne seront pas conservés dans le fichier converti."##),
    ],
}
}

/// Language ID: fr;
/// Map name: "conv_md";
/// Description: français, latin, France;
pub(super) const fn get_fr_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mConvertir entre les formats tels que toml, yaml, json et ron[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mSpécifier le fichier d'entrée[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mSpécifier le format/fichier de sortie, peut être appelé plusieurs fois[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mLorsque cette option est appelée, le fichier spécifié avec [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sera enregistré.[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mSi un fichier nommé [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m existe dans le répertoire en cours, vous pouvez appeler [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m pour le convertir en yml (yaml),
[48;2;34;34;34m[38;2;255;255;255mou spécifier manuellement le chemin du fichier : [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mLes formats pris en charge dépendent des fonctionnalités activées par ce package et toutes les fonctionnalités ne sont pas activées par défaut.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDe plus, il y a certaines choses à savoir avant de l'utiliser.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Il renomme automatiquement les chemins non spécifiques de fichiers.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Seuls les fichiers avec un niveau de profondeur seront considérés comme des formats de fichier plutôt que des chemins de fichier.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mLorsque vous appelez [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, le fichier de sortie sera automatiquement renommé en [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, plutôt qu'en [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mSi vous n'avez pas besoin de cette fonctionnalité, veuillez utiliser [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ou [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: Pointe vers un dossier au lieu d'un fichier
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mPar exemple, si vous utilisez [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, nous ne pouvons placer [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m que dans le sous-répertoire de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, plutôt que de remplacer [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m lui-même.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mSi vous n'avez pas besoin de cette fonctionnalité, veuillez utiliser le chemin complet du fichier (pas seulement le dossier), par exemple [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m au lieu de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. Lorsque [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m n'est pas appelé, les fichiers spécifiés ne seront pas enregistrés automatiquement.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. Lorsque [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m est appelé, l'entrée et la sortie de la commande conv ne peuvent pas être le même fichier.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. Les commentaires ne seront pas conservés dans le fichier converti.[0m"##),
    ],
}
}

/// Language ID: fr;
/// Map name: "opt";
/// Description: français, latin, France;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "Tableau de tableaux (utilisation : `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_fr_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"Contrairement à `-k`, `--ck` ne divise pas automatiquement la chaîne en utilisant '.'.
Pour [a.b.c], `-k a.b.c` est équivalent à `--ck a --ck b --ck c`.
Pour les cas simples, il est plus facile d'utiliser `-k`.

Si le nom de la clé contient '.', utilisez `--ck` plutôt que `-k`.
Par exemple :

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

Dans cette structure, "editor.defaultFormatter" est une chaîne complète, `defaultFormatter` n'est pas une sous-clé de `editor`.

Lorsque vous utilisez `-k '[typescript]'.'editor.defaultFormatter'`, il va diviser la clé en :

- "[typescript]"
- "editor"
- "defaultFormatter"

Alors qu'en utilisant `--ck '[typescript]' --ck editor.defaultFormatter`, il divise la clé en :

- "[typescript]"
- "editor.defaultFormatter"

La deuxième est la structure correcte."##),
        ("num-arr", r##"Tableau d'entiers i64"##),
        ("bool", r##"bool (type booléen), peut être `true` ou `false`"##),
        ("num", r##"i64 : type entier signé 64 bits, par exemple : 1145"##),
        ("datetime-help", r##"Exemple :
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

Cette option n'est prise en charge que pour `toml`. Si nous ajoutons `-t json` à la fin de la commande, cela donnera :

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"Enregistre sous un format/fichier spécifié, une opération à la fois"##),
        ("get-opts-help", r##"Exemple d'utilisation 1 : `get Cargo.toml -k package.name`
Exemple d'utilisation 2 : `get cfg.yaml -k .`
"##),
        ("none-help", r##"Remarque : RON possède un véritable type `Option<T>` qui peut être soit `Some(T)` (quelque chose), soit `None` (rien). Ici, l'option `--none` supprime simplement la clé spécifiée, elle n'a pas le même comportement qu'un véritable type `Option<T>`.

Imaginons que nous avons une table de tableaux comme celle-ci dans `test.yml` :

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

Si nous voulons supprimer la clé `name` de la deuxième table, nous pouvons exécuter :

```sh
set test.yml -k bin.1.name --none
```

> Remarque : une chaîne vide et None sont différents.
> L'appel `set test.yml --kv bin.1.name ""` définit la valeur sur une chaîne vide, mais `--none` supprime complètement la clé.

Si nous voulons supprimer complètement la deuxième table de `bin` (indice 1), nous pouvons exécuter :

```sh
set test.yml -k bin.1 --none
```

ou utiliser `--rm`:

```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"Double précision à virgule flottante (norme IEEE 754-2008), par exemple : 3.14159265359"##),
        ("saot-help", r##"> Remarque : bien que `--saot` ne permette qu'une seule paire clé-valeur par tableau, vous pouvez d'abord spécifier l'index du tableau, puis utiliser d'autres options pour étendre le tableau.

Exemple :
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"Définir des paires clé-valeur. `--kv x.y.z valeur` est équivalent à `-k x.y.z -s valeur`."##),
        ("set-save", r##"Si utilisée sans l'option `--to`, elle écrase simplement (écrit) le fichier actuel"##),
        ("array", r##"Tableau de chaînes de caractères"##),
        ("aot-help", r##"Exemple :
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# Set

Tout d'abord, sélectionnez à l'aide de la clé spécifiée, puis modifiez à l'aide de la valeur du type de données spécifié.

> Si vous ne spécifiez que la clé et pas la valeur, les types de données et les valeurs correspondantes seront listés.
> Par exemple, `set Cargo.toml -k package`, listera le tableau package et ses sous-tableaux, ainsi que des informations plus détaillées que `get`.

## Exemples

### Exemple 1:
```sh
--ts md set test.toml --key package.edition --str 2021
```

Comme `--str` a été spécifié plutôt que `--num`, "2021" est une chaîne de caractères plutôt qu'un nombre entier.

| package | new value |
|---------|-----------|
| edition | "2021"    |

```toml
[package]
edition = "2021"
```

### Exemple 2:
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### Exemple 3:
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: C'est le premier index*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> Remarque : Les exemples ci-dessus commencent par un argument/sub-command et omettent le nom de la commande racine. En pratique, vous devrez le spécifier manuellement."##),
        ("num-arr-help", r##"Exemple :
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"Exemple :
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> Tous les formats autres que `toml` utiliseront des tables standards plutôt que des tableaux intégrés.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"Rechercher une valeur à partir de sa clé"##),
        ("f64-arr-help", r##"Exemple :
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> Json 1.0 ne prend pas en charge NaN et Inf.

Il les affiche comme `null` plutôt que `NaN`.

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"Supprime la clé spécifiée"##),
        ("set-save-to-help", r##"Ceci est très similaire à l'option `-t` de la sous-commande `conv`, la différence étant que `-t` de `conv` peut être appelée plusieurs fois consécutives pour produire plusieurs formats/fichiers, tandis que `-t` de `set` ne peut être appelée qu'une seule fois pour produire un seul fichier.

> Remarque : toutes les annotations sont supprimées après la conversion des fichiers par la sous-commande `conv`. Les commentaires sont conservés dans le format **toml** par la sous-commande `set`, mais pas dans tous les autres formats.

- Si vous n'utilisez pas `-t`, l'option `--save` écrase simplement le fichier actuel.
- Si vous utilisez `-t` et que vous ajoutez `--save`, le fichier modifié sera enregistré dans le fichier spécifié par `-t`.
- À moins que le fichier source et le fichier spécifié par `-t` soient identiques, le comportement de `-t` de la sous-commande `set` sera le même que celui de `-t` de la sous-commande `conv`, c'est-à-dire qu'ils renommeront automatiquement les chemins qui ne sont pas spécifiques.
  - Si, par exemple, vous avez `from: test.yml`, et `to: json`, alors le fichier de sortie sera renommé en `test.json`, et non en `json`.
  - Si vous ne voulez pas cette fonctionnalité, veuillez utiliser `-t ./json` ou `-t $PWD/json` plutôt que `-t json`."##),
        ("preview", r##"Prévisualiser le fichier complet après modification"##),
        ("set-key", r##"Sélectionnez une clé spécifique avec "." comme séparateur"##),
        ("get-opts", r##"Obtenir la valeur de la configuration spécifiée"##),
        ("saot", r##"Tableau de paires clé-valeur pour une seule table (utilisation : `--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"Le format est : 'a.b.c'. Les noms de sous-tableaux ou les index de tableau sont reliés par '.' (point). 
Lorsque le point est unique, l'ensemble de la table est listé."##),
        ("set-opts", r##"Créer/Mettre à jour/Supprimer la structure du fichier de configuration"##),
        ("bool-arr-help", r##"Exemple :
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: le premier élément est `true`
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"Tableau de booléens"##),
        ("table", r##"Table standard utilisée pour stocker plusieurs paires clé-valeur"##),
        ("datetime", r##"Type de date et d'heure"##),
        ("table-help", r##"Exemple 1 :
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

Exemple 2 :
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"Tableau de nombres à virgule flottante f64"##),
        ("string", r##"String (type chaîne de caractères)"##),
        ("array-help", r##"Utilisez plusieurs appels à l'option `-a` pour ajouter des éléments au tableau.
Exemple :
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> Le premier appel ajoute l'élément d'index 0, le deuxième ajoute l'élément d'index 1, etc.

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"Tableau de tableaux (utilisation : `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"Tableau intégré, équivalent à une table standard plus compacte"##),
        ("concat-key", r##"Entrez la clé en segments et elle sera automatiquement concaténée sans utiliser "." comme séparateur"##),
    ],
}
}

/// Language ID: fr;
/// Map name: "opt_md";
/// Description: français, latin, France;
pub(super) const fn get_fr_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255mContrairement à [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ne divise pas automatiquement la chaîne en utilisant '.'.
[48;2;34;34;34m[38;2;255;255;255mPour [a.b.c], [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m est équivalent à [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mPour les cas simples, il est plus facile d'utiliser [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi le nom de la clé contient '.', utilisez [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m plutôt que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mPar exemple :
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDans cette structure, "editor.defaultFormatter" est une chaîne complète, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mdefaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m n'est pas une sous-clé de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255meditor[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mLorsque vous utilisez [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, il va diviser la clé en :
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAlors qu'en utilisant [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, il divise la clé en :
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mLa deuxième est la structure correcte.[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mTableau d'entiers i64[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mbool (type booléen), peut être [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ou [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64 : type entier signé 64 bits, par exemple : 1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemple :
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mCette option n'est prise en charge que pour [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Si nous ajoutons [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m à la fin de la commande, cela donnera :
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255mEnregistre sous un format/fichier spécifié, une opération à la fois[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemple d'utilisation 1 : [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExemple d'utilisation 2 : [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;255;255;255mRemarque : RON possède un véritable type [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m qui peut être soit [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m (quelque chose), soit [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m (rien). Ici, l'option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m supprime simplement la clé spécifiée, elle n'a pas le même comportement qu'un véritable type [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mImaginons que nous avons une table de tableaux comme celle-ci dans [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m :
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi nous voulons supprimer la clé [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mname[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m de la deuxième table, nous pouvons exécuter :
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Remarque : une chaîne vide et None sont différents.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m L'appel [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m définit la valeur sur une chaîne vide, mais [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m supprime complètement la clé.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi nous voulons supprimer complètement la deuxième table de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mbin[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m (indice 1), nous pouvons exécuter :
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mou utiliser [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255mDouble précision à virgule flottante (norme IEEE 754-2008), par exemple : 3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Remarque : bien que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m ne permette qu'une seule paire clé-valeur par tableau, vous pouvez d'abord spécifier l'index du tableau, puis utiliser d'autres options pour étendre le tableau.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExemple :
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mDéfinir des paires clé-valeur. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z valeur[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m est équivalent à [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s valeur[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255mSi utilisée sans l'option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, elle écrase simplement (écrit) le fichier actuel[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255mTableau de chaînes de caractères[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemple :
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mSet[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTout d'abord, sélectionnez à l'aide de la clé spécifiée, puis modifiez à l'aide de la valeur du type de données spécifié.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Si vous ne spécifiez que la clé et pas la valeur, les types de données et les valeurs correspondantes seront listés.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Par exemple, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, listera le tableau package et ses sous-tableaux, ainsi que des informations plus détaillées que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExemples[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExemple 1:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mComme [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m a été spécifié plutôt que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, "2021" est une chaîne de caractères plutôt qu'un nombre entier.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m new value [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExemple 2:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExemple 3:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: C'est le premier index*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Remarque : Les exemples ci-dessus commencent par un argument/sub-command et omettent le nom de la commande racine. En pratique, vous devrez le spécifier manuellement.[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemple :
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemple :
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Tous les formats autres que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m utiliseront des tables standards plutôt que des tableaux intégrés.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mRechercher une valeur à partir de sa clé[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemple :
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Json 1.0 ne prend pas en charge NaN et Inf.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIl les affiche comme [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m plutôt que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255mSupprime la clé spécifiée[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mCeci est très similaire à l'option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m de la sous-commande [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, la différence étant que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m peut être appelée plusieurs fois consécutives pour produire plusieurs formats/fichiers, tandis que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ne peut être appelée qu'une seule fois pour produire un seul fichier.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Remarque : toutes les annotations sont supprimées après la conversion des fichiers par la sous-commande [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m. Les commentaires sont conservés dans le format [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m par la sous-commande [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, mais pas dans tous les autres formats.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mSi vous n'utilisez pas [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, l'option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m écrase simplement le fichier actuel.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Si vous utilisez [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m et que vous ajoutez [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, le fichier modifié sera enregistré dans le fichier spécifié par [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m À moins que le fichier source et le fichier spécifié par [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m soient identiques, le comportement de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m de la sous-commande [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sera le même que celui de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m de la sous-commande [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, c'est-à-dire qu'ils renommeront automatiquement les chemins qui ne sont pas spécifiques.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Si, par exemple, vous avez [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfrom: test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, et [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mto: json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, alors le fichier de sortie sera renommé en [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, et non en [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mjson[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Si vous ne voulez pas cette fonctionnalité, veuillez utiliser [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ou [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m plutôt que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255mPrévisualiser le fichier complet après modification[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255mSélectionnez une clé spécifique avec "." comme séparateur[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255mObtenir la valeur de la configuration spécifiée[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255mTableau de paires clé-valeur pour une seule table (utilisation : [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mLe format est : 'a.b.c'. Les noms de sous-tableaux ou les index de tableau sont reliés par '.' (point). 
[48;2;34;34;34m[38;2;255;255;255mLorsque le point est unique, l'ensemble de la table est listé.[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255mCréer/Mettre à jour/Supprimer la structure du fichier de configuration[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemple :
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: le premier élément est `true`
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mTableau de booléens[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255mTable standard utilisée pour stocker plusieurs paires clé-valeur[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255mType de date et d'heure[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemple 1 :
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExemple 2 :
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mTableau de nombres à virgule flottante f64[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString (type chaîne de caractères)[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255mUtilisez plusieurs appels à l'option [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m pour ajouter des éléments au tableau.
[48;2;34;34;34m[38;2;255;255;255mExemple :
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Le premier appel ajoute l'élément d'index 0, le deuxième ajoute l'élément d'index 1, etc.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255mTableau de tableaux (utilisation : [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mTableau intégré, équivalent à une table standard plus compacte[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mEntrez la clé en segments et elle sera automatiquement concaténée sans utiliser "." comme séparateur[0m"##),
    ],
}
}

/// Language ID: fr;
/// Map name: "parser";
/// Description: français, latin, France;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "Ce qui suit est une sortie STDERR. Lors de la redirection de la sortie vers un fichier pour générer des scripts de complétion pour shell, vous devriez utiliser `> /path/to/file` au lieu de `2> /path/to/file`.");
/// ```
pub(super) const fn get_fr_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"Ce qui suit est une sortie STDERR. Lors de la redirection de la sortie vers un fichier pour générer des scripts de complétion pour shell, vous devriez utiliser `> /path/to/file` au lieu de `2> /path/to/file`."##),
    ],
}
}

/// Language ID: fr;
/// Map name: "theme";
/// Description: français, latin, France;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nom de thème invalide.");
/// ```
pub(super) const fn get_fr_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nom de thème invalide."##),
    ],
}
}

/// fr: français, latin, France
pub(super) const fn get_fr_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_fr_map_args),
        ("conv_md", get_fr_map_conv_md),
        ("parser", get_fr_map_parser),
        ("opt_md", get_fr_map_opt_md),
        ("args_md", get_fr_map_args_md),
        ("opt", get_fr_map_opt),
        ("log-core", get_fr_map_log_core),
        ("conv", get_fr_map_conv),
        ("theme", get_fr_map_theme),
    ],
}
}

/// Language ID: fy;
/// Map name: "log-core";
/// Description: Frysk, Latyn, Nederlân;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_fy_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"warskôgje"##),
        ("info", r##"ynfo"##),
        ("error", r##"flater"##),
        ("trace", r##"Trace"##),
        ("init-logger", r##"Inisjalisearjen fan logger ..."##),
    ],
}
}

/// Language ID: fy;
/// Map name: "theme";
/// Description: Frysk, Latyn, Nederlân;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Unjildige tema-namme");
/// ```
pub(super) const fn get_fy_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Unjildige tema-namme"##),
    ],
}
}

/// fy: Frysk, Latyn, Nederlân
pub(super) const fn get_fy_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_fy_map_log_core),
        ("theme", get_fy_map_theme),
    ],
}
}

/// Language ID: ga;
/// Map name: "log-core";
/// Description: Gaeilge, Laidineach, Éire;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "dífhabhtaithe");
/// ```
pub(super) const fn get_ga_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"dífhabhtaithe"##),
        ("warn", r##"rabhadh"##),
        ("info", r##"Eolas"##),
        ("error", r##"Earráid"##),
        ("trace", r##"rian"##),
        ("init-logger", r##"Tús a chur le logálaí ..."##),
    ],
}
}

/// Language ID: ga;
/// Map name: "theme";
/// Description: Gaeilge, Laidineach, Éire;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ainm téama neamhbhailí");
/// ```
pub(super) const fn get_ga_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ainm téama neamhbhailí"##),
    ],
}
}

/// ga: Gaeilge, Laidineach, Éire
pub(super) const fn get_ga_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ga_map_log_core),
        ("theme", get_ga_map_theme),
    ],
}
}

/// Language ID: gd;
/// Map name: "log-core";
/// Description: Gàidhlig, Laideann, An Rìoghachd Aonaichte;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "deasbad");
/// ```
pub(super) const fn get_gd_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"deasbad"##),
        ("warn", r##"rabhadh"##),
        ("info", r##"fiosrachadh"##),
        ("error", r##"mearachd"##),
        ("trace", r##"lorg"##),
        ("init-logger", r##"A 'tòiseachadh logger ..."##),
    ],
}
}

/// Language ID: gd;
/// Map name: "theme";
/// Description: Gàidhlig, Laideann, An Rìoghachd Aonaichte;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ainm a 'chuspair neo-dhligheach");
/// ```
pub(super) const fn get_gd_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ainm a 'chuspair neo-dhligheach"##),
    ],
}
}

/// gd: Gàidhlig, Laideann, An Rìoghachd Aonaichte
pub(super) const fn get_gd_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_gd_map_log_core),
        ("theme", get_gd_map_theme),
    ],
}
}

/// Language ID: gl;
/// Map name: "log-core";
/// Description: galego, latino, España;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Depuración");
/// ```
pub(super) const fn get_gl_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Depuración"##),
        ("warn", r##"ADVERTIR"##),
        ("info", r##"información"##),
        ("error", r##"Erro"##),
        ("trace", r##"rastrexo"##),
        ("init-logger", r##"Inicializar o rexistro ..."##),
    ],
}
}

/// Language ID: gl;
/// Map name: "theme";
/// Description: galego, latino, España;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nome do tema non válido");
/// ```
pub(super) const fn get_gl_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nome do tema non válido"##),
    ],
}
}

/// gl: galego, latino, España
pub(super) const fn get_gl_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_gl_map_log_core),
        ("theme", get_gl_map_theme),
    ],
}
}

/// Language ID: gu;
/// Map name: "log-core";
/// Description: ગુજરાતી, ગુજરાતી, ભારત;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ડિબગ");
/// ```
pub(super) const fn get_gu_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ડિબગ"##),
        ("warn", r##"ચેતવણી"##),
        ("info", r##"માહિતી"##),
        ("error", r##"ભૂલ"##),
        ("trace", r##"ટ્રેસ"##),
        ("init-logger", r##"પ્રારંભિક લોગર ..."##),
    ],
}
}

/// Language ID: gu;
/// Map name: "theme";
/// Description: ગુજરાતી, ગુજરાતી, ભારત;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "અમાન\u{acd}ય થીમ નામ");
/// ```
pub(super) const fn get_gu_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"અમાન્ય થીમ નામ"##),
    ],
}
}

/// gu: ગુજરાતી, ગુજરાતી, ભારત
pub(super) const fn get_gu_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_gu_map_log_core),
        ("theme", get_gu_map_theme),
    ],
}
}

/// Language ID: ha;
/// Map name: "log-core";
/// Description: Hausa, Latin, Nijeriya;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "DEBUG");
/// ```
pub(super) const fn get_ha_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"DEBUG"##),
        ("warn", r##"gargadi"##),
        ("info", r##"Bayani"##),
        ("error", r##"kuskure"##),
        ("trace", r##"gano"##),
        ("init-logger", r##"Fara logger ..."##),
    ],
}
}

/// Language ID: ha;
/// Map name: "theme";
/// Description: Hausa, Latin, Nijeriya;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Sunan Jigo mara kyau");
/// ```
pub(super) const fn get_ha_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Sunan Jigo mara kyau"##),
    ],
}
}

/// ha: Hausa, Latin, Nijeriya
pub(super) const fn get_ha_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ha_map_log_core),
        ("theme", get_ha_map_theme),
    ],
}
}

/// Language ID: haw;
/// Map name: "log-core";
/// Description: ʻŌlelo Hawaiʻi, Latn, ʻAmelika Hui Pū ʻIa;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_haw_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"Haeō"##),
        ("info", r##"ʻike"##),
        ("error", r##"Haki"##),
        ("trace", r##"trace"##),
        ("init-logger", r##"Ke hoʻomakaʻana me ka logger ..."##),
    ],
}
}

/// haw: ʻŌlelo Hawaiʻi, Latn, ʻAmelika Hui Pū ʻIa
pub(super) const fn get_haw_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("log-core", get_haw_map_log_core),
    ],
}
}

/// Language ID: he;
/// Map name: "log-core";
/// Description: עברית, עברי, ישראל;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ניפוי באגים");
/// ```
pub(super) const fn get_he_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ניפוי באגים"##),
        ("warn", r##"להזהיר"##),
        ("info", r##"מידע"##),
        ("error", r##"שגיאה"##),
        ("trace", r##"עקבות"##),
        ("init-logger", r##"אתחול לוגר ..."##),
    ],
}
}

/// Language ID: he;
/// Map name: "theme";
/// Description: עברית, עברי, ישראל;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "שם נושא לא חוקי");
/// ```
pub(super) const fn get_he_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"שם נושא לא חוקי"##),
    ],
}
}

/// he: עברית, עברי, ישראל
pub(super) const fn get_he_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_he_map_log_core),
        ("theme", get_he_map_theme),
    ],
}
}

/// Language ID: hi;
/// Map name: "log-core";
/// Description: हिन्दी, देवनागरी, भारत;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "डिबग");
/// ```
pub(super) const fn get_hi_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"डिबग"##),
        ("warn", r##"चेतावनी"##),
        ("info", r##"जानकारी"##),
        ("error", r##"त्रुटि"##),
        ("trace", r##"ट्रेस"##),
        ("init-logger", r##"आरंभिक लकड़हारा ..."##),
    ],
}
}

/// Language ID: hi;
/// Map name: "args";
/// Description: हिन्दी, देवनागरी, भारत;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "थीम ब\u{948}कग\u{94d}राउ\u{902}ड अक\u{94d}षम कर\u{947}\u{902} (डिफ\u{93c}ॉल\u{94d}ट र\u{942}प स\u{947} सक\u{94d}षम होता ह\u{948}, यदि `--dis-theme-bg` को कॉल किया गया होता ह\u{948} तो अक\u{94d}षम होता ह\u{948})");
/// ```
pub(super) const fn get_hi_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"अपनी थीम सेट का फ़ाइल, आपको उसके साथ `--theme` नाम निर्दिष्ट करने की आवश्यकता होगी"##),
        ("shell-completion", r##"शैली के लिए पूर्णता स्क्रिप्ट उत्पन्न करें"##),
        ("theme-file-help", r##"यह फ़ाइल rust crate 'syntect' द्वारा dump किए गए बाइनरी डेटा के समरूप है। 
मूल रूप से, ये थीम पुरानी संस्करण की sublime की थीम हैं।
यदि `--tf` निर्दिष्ट नहीं है, तो एनवायरमेंट वेरिएबल `$ TOMLYRE_THEME_FILE` का मूल्य पढ़ा जाएगा।"##),
        ("from-format", r##"स्रोत फ़ाइल प्रारूप (डिफ़ॉल्ट रूप से स्वचालित रूप से जाँच किया जाएगा)"##),
        ("disable-theme-background", r##"थीम बैकग्राउंड अक्षम करें (डिफ़ॉल्ट रूप से सक्षम होता है, यदि `--dis-theme-bg` को कॉल किया गया होता है तो अक्षम होता है)"##),
        ("theme-uncompressed-file-help", r##"> Uncompressed वर्जन Compressed वर्जन की तुलना में उच्चतम प्राथमिकता होती है।

यदि `--tuf` निर्दिष्ट नहीं है, तो एनवायरमेंट वेरिएबल `$ TOMLYRE_THEME_UNCOMPRESSED_FILE` का मूल्य पढ़ा जाएगा।"##),
        ("disable-theme-background-help", r##"यदि `--dis-theme-bg` निर्दिष्ट नहीं है, तो एनवायरमेंट वेरिएबल `$ TOMLYRE_THEME_BG` का मूल्य पढ़ा जाएगा।

यदि इस मान का मूल्य false है तो अक्षम होता है

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"इस `theme-file` फ़ाइल की तुलना में uncompressed dump डेटा (फ़ाइल) आवश्यक है"##),
        ("shell-completion-help", r##"आपको पूर्णता स्क्रिप्ट उत्पन्न करने के लिए शैली का नाम मैनुअल रूप से निर्दिष्ट करना होगा।
उदाहरण के लिए, Windows पर powershell के लिए आप इस्तेमाल कर सकते हैं: `--sh-comp pwsh`

> प्रश्न: शेल का पूरक क्या होता है?

> उत्तर: जब हम कमांड लाइन में कमांड टाइप करते हैं तो, अगर हम Tab कुंजी दबाते हैं तो पूरक स्क्रिप्ट ट्रिगर हो जाता है, जो संभवतः पूरक विकल्पों को खोजना शुरू करता है और इन विकल्पों को चुनने के लिए प्रस्तुत करता है। यह सुविधा कमांड लाइन इनपुट की दक्षता और सटीकता को बढ़ा सकती है।

विभिन्न शेल के पूरक स्क्रिप्ट के स्थान भिन्न होते हैं। zsh के लिए, आपको इसे `$fpath` अरे में शामिल निर्देशित पथ में रखना होगा।
आप `print -l $fpath` का उपयोग करके इस अरे की सामग्री प्राप्त कर सकते हैं।
मान लीजिए कि इसमें `/usr/local/share/zsh/site-functions` निर्देशितिका शामिल है, तो आप निम्नलिखित कमांड का उपयोग करके `_tomlyre` फ़ाइल में इस निर्देशिका में पूरक स्क्रिप्ट लिख सकते हैं।

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

रूट के अधिकारों का उपयोग बचने के लिए, आप इसे उपयोगकर्ता उपनिर्देशित निर्देशिका में रख सकते हैं। उदाहरण के लिए, आप इस अरे के सिरे में एक तत्व डालने और संबंधित निर्देशिका का उपयोग कर सकते हैं।

```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"stdin से डेटा पढ़ने के लिए, जैसे: `cat a | tomlyre conv -`, यह फ़ाइल प्रारूप स्वचालित रूप से जानने का प्रयास करेगा।
हालांकि, स्वचालित जाँच विफल हो सकती है, इस स्थिति में आपको प्रारूप को मैनुअल रूप से निर्दिष्ट करना होगा।
उदाहरण: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"`--ts` पैरामीटर का उपयोग करते हुए, केवल अंतर्निहित शैली सेट को लिस्ट किया जाएगा।
मार्कडाउन को md के रूप में छोटा किया जा सकता है, लेकिन अन्य शैलियों को पूरी नाम की आवश्यकता होगी।
उपयोग उदाहरण1: `--ts md`
उपयोग उदाहरण2: `--table-style ascii`

नोट: फ़ाइल से **टेबल स्टाइल** सेट पढ़ना वर्तमान में समर्थित नहीं है, यदि आपको इस सुविधा या अन्य अतिरिक्त सुविधाओं की आवश्यकता है, तो आप issue सबमिट कर सकते हैं।"##),
        ("table-style", r##"टेबल के स्टाइल को निर्दिष्ट करें"##),
        ("theme-help", r##"जब केवल `-t` पैरामीटर का उपयोग किया जाता है, तो वर्तमान थीम की सूची दी जाएगी।
यदि थीम फ़ाइल निर्दिष्ट नहीं है, तो आंतरिक थीम संग्रह का उपयोग किया जाएगा।
यदि एक थीम ** ayu-dark ** मौजूद है, तो आप ` -t ayu-dark ` का उपयोग करके उसे उपयोग कर सकते हैं।
यदि थीम नाम में अंतरिक्ष शामिल हैं, तो कृपया उसे डबल कोट या सिंगल कोट में बंधने का प्रयास करें।

> टिप्पणी: `None` एक विशेष थीम है, जो कि कोई वर्ण संकेत उच्चारण नहीं है।

जब `--theme` विकल्प निर्दिष्ट नहीं होता है, तो डेटा को पर्याप्तता से पढ़ने के लिए एनवायरनमेंट वेरिएबल `$TOMLYRE_THEME` से पढ़ा जाएगा।
इस मदद जानकारी के **वर्ण संकेत उच्चारण** थीम भी इस **एनवायरनमेंट वेरिएबल** से आता है।

> डिफ़ॉल्ट मदद जानकारी हार्ड कोड होती है, जो त्वरित लोड होती है।
> हार्ड कोडेड पाठ लोड करना ` None` थीम टेक्स्ट लोड करने से एक ही स्तर के होते हैं, जबकि डायनामिक लोड दो स्तर या उससे अधिक धीमे होता है।
```
> `$TOMLYRE_THEME` एनवायरनमेंट वेरिएबल सेट करने से यह मदद जानकारी के थीम को फिर से लोड करने से शुरू हो सकता है, जो **अत्यधिक तेज़** प्रणाली शुरू करने की गति को कम कर सकता है।
> यदि आप मदद जानकारी के थीम में बदलाव करने का इरादा नहीं रखते हैं या वर्ण संकेत उच्चारण को अक्षम करने का इरादा नहीं रखते हैं, तो कृपया एनवायरमेंट वेरिएबल को संपादित करने की बजाय `-t` पैरामीटर का उपयोग करें।

एनवायरमेंट वेरिएबल का उपयोग करने का उदाहरण:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

कमांड लाइन आधारित पैरामीटर का उपयोग करने का उदाहरण:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

"conv/get/set" आदि हस्तक्षेपों के लिए खुद से निर्माण किए गए सामग्री के लिए, `-t` द्वारा निर्दिष्ट थीम की प्राथमिकता होगी> एनवायरमेंट वेरिएबल । यदि `-t` को नहीं बुलाया गया है, तो एनवायरमेंट वेरिएबल को पढ़ा जाएगा; यदि एनवायरमेंट वेरिएबल खाली है, तो ** Monokai Extended ** का उपयोग किया जाएगा।"##),
        ("theme", r##"वर्ण संकेत उच्चारण के लिए थीम नाम निर्दिष्ट करें"##),
    ],
}
}

/// Language ID: hi;
/// Map name: "args_md";
/// Description: हिन्दी, देवनागरी, भारत;
pub(super) const fn get_hi_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mअपनी थीम सेट का फ़ाइल, आपको उसके साथ [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m नाम निर्दिष्ट करने की आवश्यकता होगी[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mशैली के लिए पूर्णता स्क्रिप्ट उत्पन्न करें[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mयह फ़ाइल rust crate 'syntect' द्वारा dump किए गए बाइनरी डेटा के समरूप है। 
[48;2;34;34;34m[38;2;255;255;255mमूल रूप से, ये थीम पुरानी संस्करण की sublime की थीम हैं।
[48;2;34;34;34m[38;2;255;255;255mयदि [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m निर्दिष्ट नहीं है, तो एनवायरमेंट वेरिएबल [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$ TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m का मूल्य पढ़ा जाएगा।[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mस्रोत फ़ाइल प्रारूप (डिफ़ॉल्ट रूप से स्वचालित रूप से जाँच किया जाएगा)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mथीम बैकग्राउंड अक्षम करें (डिफ़ॉल्ट रूप से सक्षम होता है, यदि [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m को कॉल किया गया होता है तो अक्षम होता है)[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Uncompressed वर्जन Compressed वर्जन की तुलना में उच्चतम प्राथमिकता होती है।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mयदि [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m निर्दिष्ट नहीं है, तो एनवायरमेंट वेरिएबल [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$ TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m का मूल्य पढ़ा जाएगा।[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mयदि [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m निर्दिष्ट नहीं है, तो एनवायरमेंट वेरिएबल [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$ TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m का मूल्य पढ़ा जाएगा।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mयदि इस मान का मूल्य false है तो अक्षम होता है
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mइस [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m फ़ाइल की तुलना में uncompressed dump डेटा (फ़ाइल) आवश्यक है[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mआपको पूर्णता स्क्रिप्ट उत्पन्न करने के लिए शैली का नाम मैनुअल रूप से निर्दिष्ट करना होगा।
[48;2;34;34;34m[38;2;255;255;255mउदाहरण के लिए, Windows पर powershell के लिए आप इस्तेमाल कर सकते हैं: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m प्रश्न: शेल का पूरक क्या होता है?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m उत्तर: जब हम कमांड लाइन में कमांड टाइप करते हैं तो, अगर हम Tab कुंजी दबाते हैं तो पूरक स्क्रिप्ट ट्रिगर हो जाता है, जो संभवतः पूरक विकल्पों को खोजना शुरू करता है और इन विकल्पों को चुनने के लिए प्रस्तुत करता है। यह सुविधा कमांड लाइन इनपुट की दक्षता और सटीकता को बढ़ा सकती है।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mविभिन्न शेल के पूरक स्क्रिप्ट के स्थान भिन्न होते हैं। zsh के लिए, आपको इसे [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m अरे में शामिल निर्देशित पथ में रखना होगा।
[48;2;34;34;34m[38;2;255;255;255mआप [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m का उपयोग करके इस अरे की सामग्री प्राप्त कर सकते हैं।
[48;2;34;34;34m[38;2;255;255;255mमान लीजिए कि इसमें [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m निर्देशितिका शामिल है, तो आप निम्नलिखित कमांड का उपयोग करके [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m फ़ाइल में इस निर्देशिका में पूरक स्क्रिप्ट लिख सकते हैं।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mरूट के अधिकारों का उपयोग बचने के लिए, आप इसे उपयोगकर्ता उपनिर्देशित निर्देशिका में रख सकते हैं। उदाहरण के लिए, आप इस अरे के सिरे में एक तत्व डालने और संबंधित निर्देशिका का उपयोग कर सकते हैं।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mstdin से डेटा पढ़ने के लिए, जैसे: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, यह फ़ाइल प्रारूप स्वचालित रूप से जानने का प्रयास करेगा।
[48;2;34;34;34m[38;2;255;255;255mहालांकि, स्वचालित जाँच विफल हो सकती है, इस स्थिति में आपको प्रारूप को मैनुअल रूप से निर्दिष्ट करना होगा।
[48;2;34;34;34m[38;2;255;255;255mउदाहरण: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m पैरामीटर का उपयोग करते हुए, केवल अंतर्निहित शैली सेट को लिस्ट किया जाएगा।
[48;2;34;34;34m[38;2;255;255;255mमार्कडाउन को md के रूप में छोटा किया जा सकता है, लेकिन अन्य शैलियों को पूरी नाम की आवश्यकता होगी।
[48;2;34;34;34m[38;2;255;255;255mउपयोग उदाहरण1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mउपयोग उदाहरण2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mनोट: फ़ाइल से [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mटेबल स्टाइल[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m सेट पढ़ना वर्तमान में समर्थित नहीं है, यदि आपको इस सुविधा या अन्य अतिरिक्त सुविधाओं की आवश्यकता है, तो आप issue सबमिट कर सकते हैं।[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mटेबल के स्टाइल को निर्दिष्ट करें[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mजब केवल [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m पैरामीटर का उपयोग किया जाता है, तो वर्तमान थीम की सूची दी जाएगी।
[48;2;34;34;34m[38;2;255;255;255mयदि थीम फ़ाइल निर्दिष्ट नहीं है, तो आंतरिक थीम संग्रह का उपयोग किया जाएगा।
[48;2;34;34;34m[38;2;255;255;255mयदि एक थीम ** ayu-dark ** मौजूद है, तो आप [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m -t ayu-dark [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m का उपयोग करके उसे उपयोग कर सकते हैं।
[48;2;34;34;34m[38;2;255;255;255mयदि थीम नाम में अंतरिक्ष शामिल हैं, तो कृपया उसे डबल कोट या सिंगल कोट में बंधने का प्रयास करें।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m टिप्पणी: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m एक विशेष थीम है, जो कि कोई वर्ण संकेत उच्चारण नहीं है।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mजब [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m विकल्प निर्दिष्ट नहीं होता है, तो डेटा को पर्याप्तता से पढ़ने के लिए एनवायरनमेंट वेरिएबल [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m से पढ़ा जाएगा।
[48;2;34;34;34m[38;2;255;255;255mइस मदद जानकारी के [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mवर्ण संकेत उच्चारण[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m थीम भी इस [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mएनवायरनमेंट वेरिएबल[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m से आता है।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m डिफ़ॉल्ट मदद जानकारी हार्ड कोड होती है, जो त्वरित लोड होती है।
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m हार्ड कोडेड पाठ लोड करना [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m None[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m थीम टेक्स्ट लोड करने से एक ही स्तर के होते हैं, जबकि डायनामिक लोड दो स्तर या उससे अधिक धीमे होता है।
[48;2;34;34;34m[38;2;102;217;239m```[48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m> `$TOMLYRE_THEME` एनवायरनमेंट वेरिएबल सेट करने से यह मदद जानकारी के थीम को फिर से लोड करने से शुरू हो सकता है, जो **अत्यधिक तेज़** प्रणाली शुरू करने की गति को कम कर सकता है।
[48;2;34;34;34m[38;2;102;217;239m> यदि आप मदद जानकारी के थीम में बदलाव करने का इरादा नहीं रखते हैं या वर्ण संकेत उच्चारण को अक्षम करने का इरादा नहीं रखते हैं, तो कृपया एनवायरमेंट वेरिएबल को संपादित करने की बजाय `-t` पैरामीटर का उपयोग करें।
[48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239mएनवायरमेंट वेरिएबल का उपयोग करने का उदाहरण:
[48;2;34;34;34m[38;2;102;217;239m```sh
[48;2;34;34;34m[38;2;102;217;239mTOMLYRE_THEME="Monokai Extended" tomlyre --help
[48;2;34;34;34m[38;2;102;217;239m```[48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mकमांड लाइन आधारित पैरामीटर का उपयोग करने का उदाहरण:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m"conv/get/set" आदि हस्तक्षेपों के लिए खुद से निर्माण किए गए सामग्री के लिए, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m द्वारा निर्दिष्ट थीम की प्राथमिकता होगी> एनवायरमेंट वेरिएबल । यदि [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m को नहीं बुलाया गया है, तो एनवायरमेंट वेरिएबल को पढ़ा जाएगा; यदि एनवायरमेंट वेरिएबल खाली है, तो ** Monokai Extended ** का उपयोग किया जाएगा।[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mवर्ण संकेत उच्चारण के लिए थीम नाम निर्दिष्ट करें[0m"##),
    ],
}
}

/// Language ID: hi;
/// Map name: "conv";
/// Description: हिन्दी, देवनागरी, भारत;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "टॉमल, याम\u{94d}ल, ज\u{947}सन और रोन फ\u{93c}ॉर\u{94d}म\u{947}ट क\u{947} बीच परिवर\u{94d}तन कर\u{947}\u{902}");
/// ```
pub(super) const fn get_hi_map_conv() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("conv-opts", r##"टॉमल, याम्ल, जेसन और रोन फ़ॉर्मेट के बीच परिवर्तन करें"##),
        ("from", r##"इनपुट फ़ाइल का नाम निर्दिष्ट करें"##),
        ("conv-to", r##"आउटपुट फॉर्मेट/फ़ाइल का नाम निर्दिष्ट करें, एक से अधिक बार कॉल कैसे करें"##),
        ("conv-to-help", r##"यदि वर्तमान निर्देशिका में **test.json** फ़ाइल मौजूद है, तो आप `conv test.json -t yml` कॉल करके इसे yml (yaml) में बदल सकते हैं,
या आप मैन्युअली फ़ाइल पथ निर्दिष्ट कर सकते हैं: `conv test.json -t /tmp/test.yml`

यह पैकेज इनग्रीडिएंट्स के आधार पर समर्थित फ़ॉर्मेट का निर्धारण करता है, सभी फ़ंक्शंस डिफ़ॉल्ट रूप से समर्थित नहीं होते हैं।

इसके अतिरिक्त, आपको इस्तेमाल से पहले जानना आवश्यक है कि -
- 1. यह गैर विशिष्ट फ़ाइल पथों को स्वतः नामित करेगा।
  - 1.1: एक स्तर की केवल एक फ़ाइल को फ़ाइल फ़ॉर्मेट के रूप में गणना किया जाएगा, न कि फ़ाइल पथ के रूप में
    `conv test.toml -t ron` कॉल करते समय, उत्पादित फ़ाइल अपने आपको `test.ron` बनाएगी, न कि `ron`।
    यदि आपकी ज़रूरत नहीं है, तो कृपया `-t ./ron` या `-t $PWD/ron` के रूप में स्पष्ट फ़ाइल पथ का उपयोग करें।
  - 1.2: एक निर्दिष्ट डायरेक्टरी को लक्ष्य बनाया गया है, न कि फ़ाइल
    उदाहरण के लिए, `-t /tmp/` केवल `/tmp/` के उप-स्तर में `test.toml` रख सकते हैं, `/tmp"##),
    ],
}
}

/// Language ID: hi;
/// Map name: "conv_md";
/// Description: हिन्दी, देवनागरी, भारत;
pub(super) const fn get_hi_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mटॉमल, याम्ल, जेसन और रोन फ़ॉर्मेट के बीच परिवर्तन करें[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mइनपुट फ़ाइल का नाम निर्दिष्ट करें[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mआउटपुट फॉर्मेट/फ़ाइल का नाम निर्दिष्ट करें, एक से अधिक बार कॉल कैसे करें[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mयदि वर्तमान निर्देशिका में [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m फ़ाइल मौजूद है, तो आप [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m कॉल करके इसे yml (yaml) में बदल सकते हैं,
[48;2;34;34;34m[38;2;255;255;255mया आप मैन्युअली फ़ाइल पथ निर्दिष्ट कर सकते हैं: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mयह पैकेज इनग्रीडिएंट्स के आधार पर समर्थित फ़ॉर्मेट का निर्धारण करता है, सभी फ़ंक्शंस डिफ़ॉल्ट रूप से समर्थित नहीं होते हैं।
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mइसके अतिरिक्त, आपको इस्तेमाल से पहले जानना आवश्यक है कि -
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m यह गैर विशिष्ट फ़ाइल पथों को स्वतः नामित करेगा।
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: एक स्तर की केवल एक फ़ाइल को फ़ाइल फ़ॉर्मेट के रूप में गणना किया जाएगा, न कि फ़ाइल पथ के रूप में
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m कॉल करते समय, उत्पादित फ़ाइल अपने आपको [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m बनाएगी, न कि [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m।
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mयदि आपकी ज़रूरत नहीं है, तो कृपया [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m या [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m के रूप में स्पष्ट फ़ाइल पथ का उपयोग करें।
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: एक निर्दिष्ट डायरेक्टरी को लक्ष्य बनाया गया है, न कि फ़ाइल
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mउदाहरण के लिए, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m केवल [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m के उप-स्तर में [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m रख सकते हैं, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[0m"##),
    ],
}
}

/// Language ID: hi;
/// Map name: "parser";
/// Description: हिन्दी, देवनागरी, भारत;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "नीच\u{947} stderr आउटप\u{941}ट ह\u{948}। आउटप\u{941}ट क\u{902}प\u{94d}लीशन स\u{94d}क\u{94d}रिप\u{94d}ट पर रीडायर\u{947}क\u{94d}ट करत\u{947} समय, आपको `2>/path/to/file` की जगह `/path/to/file` का उपयोग करना चाहिए।");
/// ```
pub(super) const fn get_hi_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"नीचे stderr आउटपुट है। आउटपुट कंप्लीशन स्क्रिप्ट पर रीडायरेक्ट करते समय, आपको `2>/path/to/file` की जगह `/path/to/file` का उपयोग करना चाहिए।"##),
    ],
}
}

/// Language ID: hi;
/// Map name: "theme";
/// Description: हिन्दी, देवनागरी, भारत;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "अमान\u{94d}य थीम नाम");
/// ```
pub(super) const fn get_hi_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"अमान्य थीम नाम"##),
    ],
}
}

/// hi: हिन्दी, देवनागरी, भारत
pub(super) const fn get_hi_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (3, 4),
        (0, 0),
    ],
    entries: &[
        ("parser", get_hi_map_parser),
        ("conv_md", get_hi_map_conv_md),
        ("args", get_hi_map_args),
        ("args_md", get_hi_map_args_md),
        ("theme", get_hi_map_theme),
        ("conv", get_hi_map_conv),
        ("log-core", get_hi_map_log_core),
    ],
}
}

/// Language ID: hr;
/// Map name: "log-core";
/// Description: hrvatski, latinica, Hrvatska;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "uklanjanje pogrešaka");
/// ```
pub(super) const fn get_hr_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"uklanjanje pogrešaka"##),
        ("warn", r##"upozoriti"##),
        ("info", r##"Informacije"##),
        ("error", r##"Pogreška"##),
        ("trace", r##"PRACE"##),
        ("init-logger", r##"Inicijaliziranje zapisnika ..."##),
    ],
}
}

/// Language ID: hr;
/// Map name: "theme";
/// Description: hrvatski, latinica, Hrvatska;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nevažeći naziv teme");
/// ```
pub(super) const fn get_hr_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nevažeći naziv teme"##),
    ],
}
}

/// hr: hrvatski, latinica, Hrvatska
pub(super) const fn get_hr_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_hr_map_log_core),
        ("theme", get_hr_map_theme),
    ],
}
}

/// Language ID: ht;
/// Map name: "log-core";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_ht_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"avèti"##),
        ("info", r##"Info"##),
        ("error", r##"Erè"##),
        ("trace", r##"Trace"##),
        ("init-logger", r##"Inisyalize anrejistrè ..."##),
    ],
}
}

/// Language ID: ht;
/// Map name: "theme";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Non tèm valab");
/// ```
pub(super) const fn get_ht_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Non tèm valab"##),
    ],
}
}

/// ht: ht-Latn-HT
pub(super) const fn get_ht_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ht_map_log_core),
        ("theme", get_ht_map_theme),
    ],
}
}

/// Language ID: hu;
/// Map name: "log-core";
/// Description: magyar, Latin, Magyarország;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "hibakeresési");
/// ```
pub(super) const fn get_hu_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"hibakeresési"##),
        ("warn", r##"Figyelmeztetés"##),
        ("info", r##"Info"##),
        ("error", r##"hiba"##),
        ("trace", r##"nyomkövetés"##),
        ("init-logger", r##"A fakitermelő inicializálása ..."##),
    ],
}
}

/// Language ID: hu;
/// Map name: "theme";
/// Description: magyar, Latin, Magyarország;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Érvénytelen téma neve");
/// ```
pub(super) const fn get_hu_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Érvénytelen téma neve"##),
    ],
}
}

/// hu: magyar, Latin, Magyarország
pub(super) const fn get_hu_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_hu_map_log_core),
        ("theme", get_hu_map_theme),
    ],
}
}

/// Language ID: hy;
/// Map name: "log-core";
/// Description: հայերեն, հայկական, Հայաստան;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "կարգաբերում");
/// ```
pub(super) const fn get_hy_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"կարգաբերում"##),
        ("warn", r##"նախազգուշացում"##),
        ("info", r##"Տեղեկություն"##),
        ("error", r##"սխալ"##),
        ("trace", r##"հետք"##),
        ("init-logger", r##"Նախաձեռնող անտառահատ ..."##),
    ],
}
}

/// Language ID: hy;
/// Map name: "theme";
/// Description: հայերեն, հայկական, Հայաստան;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Անվավեր թեմայի անվանումը");
/// ```
pub(super) const fn get_hy_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Անվավեր թեմայի անվանումը"##),
    ],
}
}

/// hy: հայերեն, հայկական, Հայաստան
pub(super) const fn get_hy_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_hy_map_log_core),
        ("theme", get_hy_map_theme),
    ],
}
}

/// Language ID: id;
/// Map name: "log-core";
/// Description: Indonesia, Latin, Indonesia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_id_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"memperingatkan"##),
        ("info", r##"info"##),
        ("error", r##"kesalahan"##),
        ("trace", r##"jejak"##),
        ("init-logger", r##"Menginisialisasi logger ..."##),
    ],
}
}

/// Language ID: id;
/// Map name: "theme";
/// Description: Indonesia, Latin, Indonesia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nama tema tidak valid");
/// ```
pub(super) const fn get_id_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nama tema tidak valid"##),
    ],
}
}

/// id: Indonesia, Latin, Indonesia
pub(super) const fn get_id_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_id_map_log_core),
        ("theme", get_id_map_theme),
    ],
}
}

/// Language ID: ig;
/// Map name: "log-core";
/// Description: Igbo, Latin, Naịjịrịa;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_ig_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"dọrọ aka na ntị"##),
        ("info", r##"info"##),
        ("error", r##"njehie"##),
        ("trace", r##"trace"##),
        ("init-logger", r##"Mgbakwunye Ndekọ ..."##),
    ],
}
}

/// Language ID: ig;
/// Map name: "theme";
/// Description: Igbo, Latin, Naịjịrịa;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Aha ezighi ezi");
/// ```
pub(super) const fn get_ig_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Aha ezighi ezi"##),
    ],
}
}

/// ig: Igbo, Latin, Naịjịrịa
pub(super) const fn get_ig_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ig_map_log_core),
        ("theme", get_ig_map_theme),
    ],
}
}

/// Language ID: is;
/// Map name: "log-core";
/// Description: íslenska, latneskt, Ísland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "kembiforrit");
/// ```
pub(super) const fn get_is_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"kembiforrit"##),
        ("warn", r##"vara við"##),
        ("info", r##"upplýsingar"##),
        ("error", r##"villa"##),
        ("trace", r##"rekja"##),
        ("init-logger", r##"Frumstilla skógarhöggsmann ..."##),
    ],
}
}

/// Language ID: is;
/// Map name: "theme";
/// Description: íslenska, latneskt, Ísland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ógilt þemanafn");
/// ```
pub(super) const fn get_is_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ógilt þemanafn"##),
    ],
}
}

/// is: íslenska, latneskt, Ísland
pub(super) const fn get_is_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_is_map_log_core),
        ("theme", get_is_map_theme),
    ],
}
}

/// Language ID: it;
/// Map name: "log-core";
/// Description: italiano, latino, Italia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "DEBUG");
/// ```
pub(super) const fn get_it_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"DEBUG"##),
        ("warn", r##"AVVERTIMENTO"##),
        ("info", r##"INFORMAZIONE"##),
        ("error", r##"ERRORE"##),
        ("trace", r##"TRACCIA"##),
        ("init-logger", r##"Inizializzazione del logger..."##),
    ],
}
}

/// Language ID: it;
/// Map name: "theme";
/// Description: italiano, latino, Italia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nome a tema non valido");
/// ```
pub(super) const fn get_it_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nome a tema non valido"##),
    ],
}
}

/// Language ID: it;
/// Map name: "args";
/// Description: italiano, latino, Italia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Disabilita lo sfondo del tema (default attivo, se chiamato `--dis-theme-bg` disattiva)");
/// ```
pub(super) const fn get_it_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"File di raccolta temi personalizzato, è necessario specificare anche il nome del tema tramite `--theme`"##),
        ("shell-completion", r##"Generare uno script di completamento per shell"##),
        ("theme-file-help", r##"I formati di file supportati sono dati binari dump dal rust crate `syntect`.
Essenzialmente, questi temi sono i vecchi temi di sublime.
Se `--tf` non è specificato, il valore della variabile d'ambiente `$TOMLYRE_THEME_FILE` verrà letto."##),
        ("from-format", r##"Formato del file di origine specificato manualmente (per impostazione predefinita viene rilevato automaticamente)"##),
        ("disable-theme-background", r##"Disabilita lo sfondo del tema (default attivo, se chiamato `--dis-theme-bg` disattiva)"##),
        ("theme-uncompressed-file-help", r##"> Le versioni non compresse hanno una priorità più alta rispetto alle versioni compressi.

Se `--tuf` non è specificato, il valore della variabile d'ambiente `$TOMLYRE_THEME_UNCOMPRESSED_FILE` verrà letto. "##),
        ("disable-theme-background-help", r##"Se `--dis-theme-bg` non viene specificato, verrà letto il valore dalla variabile d'ambiente `$TOMLYRE_THEME_BG`.

Se il valore è falso, sarà disabilitato

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"simile a `theme-file`, ma richiede dati dump non compressi (file)"##),
        ("shell-completion-help", r##"È necessario specificare manualmente il nome della shell per generare lo script di completamento.
Ad esempio, per powershell su Windows, puoi usare: `--sh-comp pwsh`

> Domanda: Cos'è la funzione di completamento della shell?

> Risposta: Quando digitiamo un comando sulla riga di comando, se premiamo il tasto Tab, lo script di completamento verrà attivato, inizierà a cercare le possibili opzioni di completamento e mostrerà queste opzioni per la selezione. Questa funzione può migliorare notevolmente l'efficienza e l'accuratezza dell'input sulla riga di comando.

Lo script di completamento per diverse shell è memorizzato in posizioni diverse. Per zsh, dovresti salvarlo nelle directory incluse nell'array `$fpath`.
Puoi usare `print -l $fpath` per ottenere il contenuto di quest'array.
Supponiamo che ci sia la cartella `/usr/local/share/zsh/site-functions`. Quindi, puoi usare il seguente comando per scrivere lo script di completamento nel file `_tomlyre` in questa cartella.

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

Per evitare di utilizzare i privilegi di root, puoi salvarlo nella sotto-directory dell'utente. Ad esempio, puoi usare il seguente comando per inserire un elemento all'inizio di questo array e poi usare la directory relativa.
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"Quando si leggono i dati da stdin, ad esempio: `cat a | tomlyre conv -`, verrà tentato di rilevare automaticamente il formato del file.
Tuttavia, l'autodetect potrebbe fallire e in tal caso sarà necessario specificare manualmente il formato.
Esempio: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Quando viene utilizzato solo il parametro `--ts`, verranno elencate le raccolte di stili integrate.
Tranne markdown che può essere abbreviato in md, gli altri stili devono essere usati con il loro nome completo.
Esempio di utilizzo 1: `--ts md`
Esempio di utilizzo 2: `--table-style ascii`

> Nota: Attualmente non è supportata la lettura di insiemi di **stili tabella** da file. 
> Se hai bisogno di questa funzionalità o di altre funzionalità aggiuntive, ti preghiamo di segnalarlo tramite un issue."##),
        ("table-style", r##"Specifica lo stile della tabella"##),
        ("theme-help", r##"Quando si utilizza solo `-t`, verrà elencato l'elenco dei temi attuali.
Se non viene specificato un file di tema, verrà utilizzata una raccolta di temi integrati.
Supponiamo che ci sia un tema chiamato **ayu-dark**, è possibile utilizzare `-t ayu-dark` per utilizzare quel tema.
Se il nome del tema contiene uno spazio, utilizzare le virgolette o gli apici.

> Nota: c'è un tema speciale `None` che rappresenta uno stile di testo normale senza evidenziazione della sintassi.

Se l'opzione `--theme` non è specificata, i dati verranno letti dalla variabile d'ambiente `$TOMLYRE_THEME`.
Il **tema di evidenziazione della sintassi** di questo messaggio di aiuto proviene anche da questa **variabile d'ambiente**.

> Il messaggio di aiuto predefinito è codificato duramente e ha velocità di caricamento molto veloce.
> Caricare il testo codificato duramente e il testo con il tema `None` richiederà lo stesso ordine di grandezza di tempo,
> mentre il caricamento dinamico rallenterà di due o più ordini di grandezza.
> Impostare la variabile d'ambiente `$TOMLYRE_THEME` può causare un ritardo significativo nel tempo di avvio se riprogettiamo il tema.
> A meno che non si modifichi il tema delle informazioni di aiuto o si disattivi l'evidenziazione della sintassi, si consiglia di utilizzare il parametro `-t` invece di modificare la variabile d'ambiente.

Esempio di utilizzo con una variabile d'ambiente:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Esempio di utilizzo con un parametro sulla riga di comando:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Per i contenuti prodotti manualmente dalle operazioni correlate `conv / get / set`, ecc., La priorità del tema specificato da `-t`> variabile d'ambiente.
Se `-t` non è chiamato, leggere la variabile d'ambiente; se la variabile d'ambiente è vuota, utilizzare ** Monokai Extended **."##),
        ("theme", r##"Specifica il nome del tema per l'evidenziazione della sintassi"##),
    ],
}
}

/// Language ID: it;
/// Map name: "args_md";
/// Description: italiano, latino, Italia;
pub(super) const fn get_it_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mFile di raccolta temi personalizzato, è necessario specificare anche il nome del tema tramite [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mGenerare uno script di completamento per shell[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mI formati di file supportati sono dati binari dump dal rust crate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mEssenzialmente, questi temi sono i vecchi temi di sublime.
[48;2;34;34;34m[38;2;255;255;255mSe [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m non è specificato, il valore della variabile d'ambiente [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verrà letto.[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mFormato del file di origine specificato manualmente (per impostazione predefinita viene rilevato automaticamente)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mDisabilita lo sfondo del tema (default attivo, se chiamato [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m disattiva)[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Le versioni non compresse hanno una priorità più alta rispetto alle versioni compressi.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSe [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m non è specificato, il valore della variabile d'ambiente [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verrà letto. [0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mSe [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m non viene specificato, verrà letto il valore dalla variabile d'ambiente [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSe il valore è falso, sarà disabilitato
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255msimile a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ma richiede dati dump non compressi (file)[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mÈ necessario specificare manualmente il nome della shell per generare lo script di completamento.
[48;2;34;34;34m[38;2;255;255;255mAd esempio, per powershell su Windows, puoi usare: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Domanda: Cos'è la funzione di completamento della shell?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Risposta: Quando digitiamo un comando sulla riga di comando, se premiamo il tasto Tab, lo script di completamento verrà attivato, inizierà a cercare le possibili opzioni di completamento e mostrerà queste opzioni per la selezione. Questa funzione può migliorare notevolmente l'efficienza e l'accuratezza dell'input sulla riga di comando.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mLo script di completamento per diverse shell è memorizzato in posizioni diverse. Per zsh, dovresti salvarlo nelle directory incluse nell'array [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mPuoi usare [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m per ottenere il contenuto di quest'array.
[48;2;34;34;34m[38;2;255;255;255mSupponiamo che ci sia la cartella [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Quindi, puoi usare il seguente comando per scrivere lo script di completamento nel file [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m in questa cartella.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPer evitare di utilizzare i privilegi di root, puoi salvarlo nella sotto-directory dell'utente. Ad esempio, puoi usare il seguente comando per inserire un elemento all'inizio di questo array e poi usare la directory relativa.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mQuando si leggono i dati da stdin, ad esempio: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, verrà tentato di rilevare automaticamente il formato del file.
[48;2;34;34;34m[38;2;255;255;255mTuttavia, l'autodetect potrebbe fallire e in tal caso sarà necessario specificare manualmente il formato.
[48;2;34;34;34m[38;2;255;255;255mEsempio: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mQuando viene utilizzato solo il parametro [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, verranno elencate le raccolte di stili integrate.
[48;2;34;34;34m[38;2;255;255;255mTranne markdown che può essere abbreviato in md, gli altri stili devono essere usati con il loro nome completo.
[48;2;34;34;34m[38;2;255;255;255mEsempio di utilizzo 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEsempio di utilizzo 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: Attualmente non è supportata la lettura di insiemi di [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mstili tabella[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m da file. 
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Se hai bisogno di questa funzionalità o di altre funzionalità aggiuntive, ti preghiamo di segnalarlo tramite un issue.[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mSpecifica lo stile della tabella[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mQuando si utilizza solo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, verrà elencato l'elenco dei temi attuali.
[48;2;34;34;34m[38;2;255;255;255mSe non viene specificato un file di tema, verrà utilizzata una raccolta di temi integrati.
[48;2;34;34;34m[38;2;255;255;255mSupponiamo che ci sia un tema chiamato [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, è possibile utilizzare [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m per utilizzare quel tema.
[48;2;34;34;34m[38;2;255;255;255mSe il nome del tema contiene uno spazio, utilizzare le virgolette o gli apici.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: c'è un tema speciale [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m che rappresenta uno stile di testo normale senza evidenziazione della sintassi.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSe l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m non è specificata, i dati verranno letti dalla variabile d'ambiente [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mIl [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtema di evidenziazione della sintassi[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m di questo messaggio di aiuto proviene anche da questa [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mvariabile d'ambiente[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Il messaggio di aiuto predefinito è codificato duramente e ha velocità di caricamento molto veloce.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Caricare il testo codificato duramente e il testo con il tema [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m richiederà lo stesso ordine di grandezza di tempo,
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m mentre il caricamento dinamico rallenterà di due o più ordini di grandezza.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Impostare la variabile d'ambiente [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m può causare un ritardo significativo nel tempo di avvio se riprogettiamo il tema.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A meno che non si modifichi il tema delle informazioni di aiuto o si disattivi l'evidenziazione della sintassi, si consiglia di utilizzare il parametro [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m invece di modificare la variabile d'ambiente.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEsempio di utilizzo con una variabile d'ambiente:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEsempio di utilizzo con un parametro sulla riga di comando:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPer i contenuti prodotti manualmente dalle operazioni correlate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv / get / set[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ecc., La priorità del tema specificato da [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m> variabile d'ambiente.
[48;2;34;34;34m[38;2;255;255;255mSe [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m non è chiamato, leggere la variabile d'ambiente; se la variabile d'ambiente è vuota, utilizzare ** Monokai Extended [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mSpecifica il nome del tema per l'evidenziazione della sintassi[0m"##),
    ],
}
}

/// Language ID: it;
/// Map name: "conv";
/// Description: italiano, latino, Italia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Convertire tra i formati toml, yaml, json e ron");
/// ```
pub(super) const fn get_it_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Convertire tra i formati toml, yaml, json e ron"##),
        ("from", r##"Specifica il file di input"##),
        ("conv-to", r##"Specificare il formato / file di output, chiamabile più volte"##),
        ("save", r##"Quando viene chiamata questa opzione, il file specificato da `--to` verrà salvato."##),
        ("conv-to-help", r##"Se il file **test.json** esiste nella directory corrente, è possibile chiamare `conv test.json -t yml` per convertirlo in yml (yaml),
oppure specificare manualmente il percorso del file: `conv test.json -t /tmp/test.yml`

I formati supportati dipendono dalle funzionalità abilitate in questo pacchetto e non tutte le funzionalità sono abilitate per impostazione predefinita.

Inoltre, ci sono alcune cose che devi sapere prima di utilizzarlo.
- 1. Rinominerà automaticamente i percorsi dei file non specifici.
  - 1.1: Un file con un solo livello verrà considerato un formato di file, non un percorso di file
    Quando si chiama `conv test.toml -t ron`, il file di output verrà automaticamente rinominato in `test.ron`, non in `ron`.
    Se non hai bisogno di questa funzionalità, utilizzare `-t ./ron` o `-t $PWD/ron`.
  - 1.2: Si sta facendo riferimento a una directory, non a un file
    Ad esempio, `-t /tmp/`, potremmo mettere `test.toml` solo sotto la sottodirectory di `/tmp/`, ma non sovrascrivere `/tmp` stesso.
    Se non hai bisogno di questa funzionalità, utilizzare il percorso completo del file (non la directory), ad esempio `-t /tmp/xx.toml` invece di `-t /tmp/`.
- 2. Quando `--save` non viene chiamato, il file specificato non verrà salvato automaticamente.
- 3. Quando si chiama `--save`, l'input e l'output del sottocomando conv non possono essere lo stesso file.
- 4. I commenti non saranno conservati nel file convertito"##),
    ],
}
}

/// Language ID: it;
/// Map name: "conv_md";
/// Description: italiano, latino, Italia;
pub(super) const fn get_it_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mConvertire tra i formati toml, yaml, json e ron[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mSpecifica il file di input[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mSpecificare il formato / file di output, chiamabile più volte[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mQuando viene chiamata questa opzione, il file specificato da [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verrà salvato.[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mSe il file [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m esiste nella directory corrente, è possibile chiamare [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m per convertirlo in yml (yaml),
[48;2;34;34;34m[38;2;255;255;255moppure specificare manualmente il percorso del file: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mI formati supportati dipendono dalle funzionalità abilitate in questo pacchetto e non tutte le funzionalità sono abilitate per impostazione predefinita.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mInoltre, ci sono alcune cose che devi sapere prima di utilizzarlo.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Rinominerà automaticamente i percorsi dei file non specifici.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Un file con un solo livello verrà considerato un formato di file, non un percorso di file
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mQuando si chiama [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, il file di output verrà automaticamente rinominato in [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, non in [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mSe non hai bisogno di questa funzionalità, utilizzare [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m o [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: Si sta facendo riferimento a una directory, non a un file
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mAd esempio, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, potremmo mettere [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m solo sotto la sottodirectory di [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ma non sovrascrivere [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m stesso.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mSe non hai bisogno di questa funzionalità, utilizzare il percorso completo del file (non la directory), ad esempio [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m invece di [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. Quando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m non viene chiamato, il file specificato non verrà salvato automaticamente.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. Quando si chiama [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, l'input e l'output del sottocomando conv non possono essere lo stesso file.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. I commenti non saranno conservati nel file convertito[0m"##),
    ],
}
}

/// Language ID: it;
/// Map name: "opt";
/// Description: italiano, latino, Italia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "Array di tabelle (utilizzo: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_it_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"A differenza di `-k`, `--ck` non farà automaticamente uno `split('.')`.
Per [a.b.c], `-k a.b.c` e `--ck a --ck b --ck c` sono equivalenti.
Per situazioni più semplici, `-k` sarà più facile.

Se il nome della chiave contiene '.', usare `--ck` invece di `-k`.
Ad esempio:

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

In questa struttura,"editor.defaultFormatter" è una stringa completa e "defaultFormatter" non è una sotto chiave di "editor".

Usando `-k '[typescript]'.'editor.defaultFormatter'`, verrà divisa in:

- "[typescript]"
- "editor"
- "defaultFormatter"

Mentre usando `--ck '[typescript]' --ck editor.defaultFormatter`, verrà trattata come:

- "[typescript]"
- "editor.defaultFormatter"

Che è la struttura giusta."##),
        ("num-arr", r##"Array di interi con segno a 64 bit"##),
        ("bool", r##"Bool (tipo booleano), può essere solo `true` o `false`"##),
        ("num", r##"i64: intero con segno a 64 bit, ad esempio: 1145"##),
        ("datetime-help", r##"Esempio:
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

Questa opzione è supportata solo per il formato `toml`. Se si specifica l'opzione `-t json`, verrà utilizzata una rappresentazione diversa:

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"Salva in un formato/file specifico, solo uno alla volta"##),
        ("get-opts-help", r##"Esempio d'uso 1: `get Cargo.toml -k package.name`
Esempio d'uso 2: `get cfg.yaml -k .`
"##),
        ("none-help", r##"> Nota: RON ha un vero tipo `Option<T>` che può avere valore `Some(T)` o `None`. 
> L'opzione `--none` rimuove una chiave specificata, ma non corrisponde veramente al tipo `Option<T>`.

Supponiamo che ci sia un array di tabelle nel file `test.yml`:

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

Per rimuovere la chiave `name: test`, è possibile eseguire il seguente comando:
```sh
set test.yml -k bin.1.name --none
```

> Nota: una stringa vuota e `None` sono due cose diverse.
> Utilizzare `set test.yml --kv bin.1.name ""` per impostare la chiave su una stringa vuota, mentre `--none` rimuove completamente la chiave.

Per rimuovere completamente la seconda tabella dell'array `bin`, è possibile eseguire:
```sh
set test.yml -k bin.1 --none
```

Oppure si può utilizzare l'opzione `--rm`:
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"Numero in virgola mobile doppia precisione (standard IEEE 754-2008), ad esempio: 3.14159265359"##),
        ("saot-help", r##"> Nota che anche se è possibile creare solo una coppia K-V tramite l'opzione `--saot`, è possibile specificare l'indice dell'array di tabelle e poi utilizzare altre opzioni per espandere.

Esempio:
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"Imposta la coppia k-v, `--kv x.y.z value` equivale a `-k x.y.z -s value`"##),
        ("set-save", r##"Se chiamato senza l'opzione `--to`, sovrascrive (scrive) il file corrente"##),
        ("array", r##"Array di stringhe"##),
        ("aot-help", r##"Esempio:
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

Scegliere prima la chiave specificata e quindi modificare il valore del tipo di dati specificato.

> Se viene fornita solo la chiave e non il valore, verranno elencati il tipo di dati corrispondente e il valore.
> Ad esempio, `set Cargo.toml -k package` elencherà la tabella package e le sue sotto-tabelle, fornendo maggiori informazioni rispetto a `get`.

## Esempi

### Esempio 1:
```sh
--ts md set test.toml --key package.edition --str 2021
```

Dato che è stata specificata l'opzione `--str`, invece di `--num`, "2021" è di tipo stringa, non intero.

| package | new value |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### Esempio 2:
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### Esempio 3:
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: Questo è il primo indice*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> Nota: Gli esempi sopra iniziano con un parametro/sotto-comando e omettono il nome del comando radice. Nell'uso reale, è necessario aggiungerlo manualmente."##),
        ("num-arr-help", r##"Esempio:
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"Esempio:
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> Tutti i formati ad eccezione di `toml` utilizzeranno una tabella standard invece di una tabella inline.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"Cercare il valore attraverso la chiave"##),
        ("f64-arr-help", r##"Esempio:
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> Il formato JSON 1.0 non supporta NaN e Inf. 
> Verrà emesso `null` invece di `NaN`.

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```"##),
        ("none", r##"Rimuove la chiave specificata"##),
        ("set-save-to-help", r##"Simile all'opzione `-t` del sotto-comando `conv`, la differenza principale è che l'opzione `-t` di `conv` può essere chiamata più volte consecutivamente per produrre diversi formati o file.
Invece, l'opzione `-t` del sotto-comando `set` può essere chiamata solo una volta e produce un singolo file.

> Nota che dopo la conversione, il comando `conv` non mantiene i commenti presenti nei file convertiti. Invece, il comando `set` mantiene i commenti solo nei file **toml**, mentre per gli altri formati i commenti **non vengono** mantenuti.

- Senza l'opzione `-t`, l'opzione `--save` sovrascrive il file sorgente.
- Se l'opzione `-t` è stata chiamata e viene successivamente chiamata anche l'opzione `--save`, il file modificato viene salvato nel file specificato dall'opzione `-t`.
- A meno che il file sorgente e quello specificato dall'opzione `-t` coincidano, il comportamento dell'opzione `-t` del sotto-comando `set` è simile a quello dell'opzione `-t` del sotto-comando `conv`, in quanto entrambi rinominano automaticamente i file non specifici.
  - Ad esempio, supponiamo che l'opzione `-t` sia stata impostata su **json**, e che il file sorgente sia **test.yml**. Il file di output sarà rinominato come **test.json**, anziché **json**.
  - Se questa funzionalità non è necessaria, utilizzare `-t ./json` o `-t $PWD/json` invece di `-t json`. "##),
        ("preview", r##"Dopo aver effettuato le modifiche, visualizza l'intero file"##),
        ("set-key", r##"Scegliere la chiave specificata utilizzando il '.' come separatore"##),
        ("get-opts", r##"Ottenere il valore della configurazione specificata"##),
        ("saot", r##"Array di coppie chiave-valore per ogni tabella (utilizzo: `--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"Il formato è: "a.b.c", i nomi di sotto-tabelle o gli indici dell'array sono separati dal "." (punto).
Se c'è solo un ".", verrà elencata l'intera tabella."##),
        ("set-opts", r##"Creare/aggiornare/eliminare la struttura del file di configurazione"##),
        ("bool-arr-help", r##"Esempio:
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: il primo elemento è true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"Array di valori booleani"##),
        ("table", r##"Tabella standard, utilizzata per memorizzare coppie K-V multiple"##),
        ("datetime", r##"Tipo di data e ora"##),
        ("table-help", r##"Esempio 1:
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

Esempio 2:
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"Array di numeri in virgola mobile doppia precisione"##),
        ("string", r##"String (tipo stringa)"##),
        ("array-help", r##"Imposta gli elementi dell'array mediante chiamate multiple all'opzione `-a`.
Ad esempio:
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> La prima chiamata imposta l'elemento all'indice 0, la seconda imposta l'elemento all'indice 1 e così via.

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"Array di tabelle (utilizzo: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"Tabella inline, equivalente alla tabella standard ma più compatta"##),
        ("concat-key", r##"Inserire la chiave per parti, verrà automaticamente concatenata, non utilizzare il '.' come separatore"##),
    ],
}
}

/// Language ID: it;
/// Map name: "opt_md";
/// Description: italiano, latino, Italia;
pub(super) const fn get_it_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255mA differenza di [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m non farà automaticamente uno [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msplit('.')[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mPer [a.b.c], [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m e [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sono equivalenti.
[48;2;34;34;34m[38;2;255;255;255mPer situazioni più semplici, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sarà più facile.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSe il nome della chiave contiene '.', usare [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m invece di [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mAd esempio:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIn questa struttura,"editor.defaultFormatter" è una stringa completa e "defaultFormatter" non è una sotto chiave di "editor".
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mUsando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, verrà divisa in:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mMentre usando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, verrà trattata come:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mChe è la struttura giusta.[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mArray di interi con segno a 64 bit[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mBool (tipo booleano), può essere solo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m o [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: intero con segno a 64 bit, ad esempio: 1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255mEsempio:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mQuesta opzione è supportata solo per il formato [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Se si specifica l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, verrà utilizzata una rappresentazione diversa:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255mSalva in un formato/file specifico, solo uno alla volta[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255mEsempio d'uso 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEsempio d'uso 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: RON ha un vero tipo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m che può avere valore [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m o [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m. 
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m L'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m rimuove una chiave specificata, ma non corrisponde veramente al tipo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSupponiamo che ci sia un array di tabelle nel file [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPer rimuovere la chiave [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mname: test[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, è possibile eseguire il seguente comando:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: una stringa vuota e [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m sono due cose diverse.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Utilizzare [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m per impostare la chiave su una stringa vuota, mentre [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m rimuove completamente la chiave.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPer rimuovere completamente la seconda tabella dell'array [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mbin[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, è possibile eseguire:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOppure si può utilizzare l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255mNumero in virgola mobile doppia precisione (standard IEEE 754-2008), ad esempio: 3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota che anche se è possibile creare solo una coppia K-V tramite l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, è possibile specificare l'indice dell'array di tabelle e poi utilizzare altre opzioni per espandere.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEsempio:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mImposta la coppia k-v, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m equivale a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255mSe chiamato senza l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, sovrascrive (scrive) il file corrente[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255mArray di stringhe[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255mEsempio:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mScegliere prima la chiave specificata e quindi modificare il valore del tipo di dati specificato.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Se viene fornita solo la chiave e non il valore, verranno elencati il tipo di dati corrispondente e il valore.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Ad esempio, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m elencherà la tabella package e le sue sotto-tabelle, fornendo maggiori informazioni rispetto a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mEsempi[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mEsempio 1:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDato che è stata specificata l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, invece di [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, "2021" è di tipo stringa, non intero.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m new value [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mEsempio 2:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mEsempio 3:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: Questo è il primo indice*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: Gli esempi sopra iniziano con un parametro/sotto-comando e omettono il nome del comando radice. Nell'uso reale, è necessario aggiungerlo manualmente.[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mEsempio:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255mEsempio:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Tutti i formati ad eccezione di [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m utilizzeranno una tabella standard invece di una tabella inline.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mCercare il valore attraverso la chiave[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mEsempio:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Il formato JSON 1.0 non supporta NaN e Inf. 
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Verrà emesso [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m invece di [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255mRimuove la chiave specificata[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mSimile all'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m del sotto-comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, la differenza principale è che l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m di [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m può essere chiamata più volte consecutivamente per produrre diversi formati o file.
[48;2;34;34;34m[38;2;255;255;255mInvece, l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m del sotto-comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m può essere chiamata solo una volta e produce un singolo file.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota che dopo la conversione, il comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m non mantiene i commenti presenti nei file convertiti. Invece, il comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m mantiene i commenti solo nei file [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m, mentre per gli altri formati i commenti [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mnon vengono[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m mantenuti.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mSenza l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sovrascrive il file sorgente.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Se l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m è stata chiamata e viene successivamente chiamata anche l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, il file modificato viene salvato nel file specificato dall'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m A meno che il file sorgente e quello specificato dall'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m coincidano, il comportamento dell'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m del sotto-comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m è simile a quello dell'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m del sotto-comando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, in quanto entrambi rinominano automaticamente i file non specifici.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Ad esempio, supponiamo che l'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sia stata impostata su [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, e che il file sorgente sia [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m. Il file di output sarà rinominato come [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, anziché [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Se questa funzionalità non è necessaria, utilizzare [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m o [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m invece di [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. [0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255mDopo aver effettuato le modifiche, visualizza l'intero file[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255mScegliere la chiave specificata utilizzando il '.' come separatore[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255mOttenere il valore della configurazione specificata[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255mArray di coppie chiave-valore per ogni tabella (utilizzo: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mIl formato è: "a.b.c", i nomi di sotto-tabelle o gli indici dell'array sono separati dal "." (punto).
[48;2;34;34;34m[38;2;255;255;255mSe c'è solo un ".", verrà elencata l'intera tabella.[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255mCreare/aggiornare/eliminare la struttura del file di configurazione[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mEsempio:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: il primo elemento è true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mArray di valori booleani[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255mTabella standard, utilizzata per memorizzare coppie K-V multiple[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255mTipo di data e ora[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255mEsempio 1:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEsempio 2:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mArray di numeri in virgola mobile doppia precisione[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString (tipo stringa)[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255mImposta gli elementi dell'array mediante chiamate multiple all'opzione [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mAd esempio:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m La prima chiamata imposta l'elemento all'indice 0, la seconda imposta l'elemento all'indice 1 e così via.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255mArray di tabelle (utilizzo: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mTabella inline, equivalente alla tabella standard ma più compatta[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mInserire la chiave per parti, verrà automaticamente concatenata, non utilizzare il '.' come separatore[0m"##),
    ],
}
}

/// Language ID: it;
/// Map name: "parser";
/// Description: italiano, latino, Italia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "Questo è l'output di stderr. Quando si reindirizza l'output dello script di completamento, si dovrebbe utilizzare `>/path/to/file` invece di `2>/path/to/file`.");
/// ```
pub(super) const fn get_it_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"Questo è l'output di stderr. Quando si reindirizza l'output dello script di completamento, si dovrebbe utilizzare `>/path/to/file` invece di `2>/path/to/file`."##),
    ],
}
}

/// it: italiano, latino, Italia
pub(super) const fn get_it_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_it_map_args),
        ("conv_md", get_it_map_conv_md),
        ("parser", get_it_map_parser),
        ("opt_md", get_it_map_opt_md),
        ("args_md", get_it_map_args_md),
        ("opt", get_it_map_opt),
        ("log-core", get_it_map_log_core),
        ("conv", get_it_map_conv),
        ("theme", get_it_map_theme),
    ],
}
}

/// Language ID: iw;
/// Map name: "log-core";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ניפוי באגים");
/// ```
pub(super) const fn get_iw_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ניפוי באגים"##),
        ("warn", r##"להזהיר"##),
        ("info", r##"מידע"##),
        ("error", r##"שגיאה"##),
        ("trace", r##"עקבות"##),
        ("init-logger", r##"אתחול לוגר ..."##),
    ],
}
}

/// Language ID: iw;
/// Map name: "theme";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "שם נושא לא חוקי");
/// ```
pub(super) const fn get_iw_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"שם נושא לא חוקי"##),
    ],
}
}

/// iw: iw-Hebr-IL
pub(super) const fn get_iw_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_iw_map_log_core),
        ("theme", get_iw_map_theme),
    ],
}
}

/// Language ID: ja;
/// Map name: "log-core";
/// Description: 日本語, 日本語の文字, 日本;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "デバッグ");
/// ```
pub(super) const fn get_ja_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"デバッグ"##),
        ("warn", r##"警告"##),
        ("info", r##"情報"##),
        ("error", r##"エラー"##),
        ("trace", r##"トレース"##),
        ("init-logger", r##"ロガーの初期化..."##),
    ],
}
}

/// Language ID: ja;
/// Map name: "args";
/// Description: 日本語, 日本語の文字, 日本;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "テーマ背景を無効にする（デフォルトは有効ですが、 `--dis-theme-bg` を使用して無効化できます）");
/// ```
pub(super) const fn get_ja_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"カスタムテーマセットファイル。 `--theme` でテーマ名を指定する必要があります"##),
        ("shell-completion", r##"シェルの補完スクリプトを生成します"##),
        ("theme-file-help", r##"サポートされているファイル形式は、rust crate `syntect` がダンプしたバイナリデータです。
本質的に、これらのテーマは古い版の sublime のテーマです。
`--tf` が指定されていない場合、環境変数 `$TOMLYRE_THEME_FILE` の値が読み取られます。"##),
        ("from-format", r##"ソースファイルの形式を手動指定する（デフォルトは自動判別）"##),
        ("disable-theme-background", r##"テーマ背景を無効にする（デフォルトは有効ですが、 `--dis-theme-bg` を使用して無効化できます）"##),
        ("theme-uncompressed-file-help", r##"> 未圧縮のバージョンは、圧縮バージョンよりも優先されます。

`--tuf` が指定されていない場合は、環境変数 `$TOMLYRE_THEME_UNCOMPRESSED_FILE` の値が読み込まれます。"##),
        ("disable-theme-background-help", r##"`--dis-theme-bg` が指定されていない場合は、環境変数 `$TOMLYRE_THEME_BG` の値が読み込まれます。

値が false の場合は無効になります。

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"`theme-file` に似ていますが、圧縮されていない dump データ(ファイル)が必要です"##),
        ("shell-completion-help", r##"補完スクリプトを生成するために、シェルの名前を手動で指定する必要があります。
たとえば、Windows 上の PowerShell の場合、 `--sh-comp pwsh` を使用できます。

> Q. 何が shell の補完機能ですか？

> A: コマンドラインにコマンドを入力している時、Tabキーを押すと、補完スクリプトがトリガーされ、可能性のある補完オプションが検索され、これらのオプションが選択できるように表示されます。この機能は、コマンドライン入力の効率と正確性を大幅に向上させることができます。

異なるシェルの補完スクリプトは、保存される場所が異なります。zsh の場合、 `$fpath` 配列に含まれるパスに保存する必要があります。
`print -l $fpath` を使用して、この配列の内容を取得できます。
たとえば、 `/usr/local/share/zsh/site-functions` ディレクトリが含まれているとします。その場合、次のコマンドを使用して、補完スクリプトを `_tomlyre` ファイルに書き込むことができます。

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

root 権限を使用しないようにするために、ユーザのサブディレクトリに配置することもできます。たとえば、次のコマンドを使用して、配列の先頭に要素を挿入し、関連するディレクトリを使用できます。
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"stdin からデータを読み取る場合、例： `cat a | tomlyre conv -`、それは最初にファイル形式を自動的に判断しようとします。
ただし、自動判断が失敗する場合があります。その場合、手動で形式を指定する必要があります。
例：`cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"`--ts` のパラメーターだけで使用する場合は、組み込みのスタイルのセットが表示されます。
markdown 以外は md に省略形があります。
現在、**テーブルスタイル** のコレクションをファイルから読み取る機能はサポートされていません。 追加の機能が必要な場合は、issueを提出してフィードバックしてください。"##),
        ("table-style", r##"表のスタイルを指定します"##),
        ("theme-help", r##"`-t` パラメータだけを使用する場合、現在のテーマのリストが表示されます。
テーマファイルが指定されていない場合、組み込みのテーマセットが使用されます。
例えば、**ayu-dark** というテーマがある場合、`-t ayu-dark` を使用してそのテーマを使用できます。
テーマ名に空白が含まれている場合は、ダブルクォートまたはシングルクォートで囲む必要があります。

> 注： `None` という特殊なテーマがあります。これは、シンタックスハイライトのない純テキストスタイルを表します。

`--theme` オプションが指定されていない場合、データは環境変数 `$TOMLYRE_THEME` から読み取られます。
このヘルプ情報の **シンタックスハイライト** テーマも、この **環境変数** から読み取られます。

> デフォルトのヘルプ情報はハードコーディングされており、読み込み速度が非常に速いです。
> ハードコーディングされたテキストを読み込む速度と `None` テーマのテキストを読み込む速度は同じですが、動的に読み込む場合は2桁以上遅くなります。
> `TOMLYRE_THEME` 環境変数を設定すると、テーマが再度読み込まれるため、起動速度が **著しく** 遅くなります。
> テーマを変更するか、シンタックスハイライトを無効にする場合を除き、環境変数を変更する代わりに `-t` パラメータを使用してください。

環境変数を使用した例：
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

コマンドライン引数を使用した例:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

手動で `conv/get/set` などの操作を実行した場合の出力については、`-t` で指定されたテーマ > 環境変数の優先順位があります。
`-t` が呼び出されない場合、環境変数が読み取られます。環境変数が空の場合は、**Monokai Extended** が使用されます。"##),
        ("theme", r##"指定されたシンタックスハイライトのテーマ名"##),
    ],
}
}

/// Language ID: ja;
/// Map name: "args_md";
/// Description: 日本語, 日本語の文字, 日本;
pub(super) const fn get_ja_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mカスタムテーマセットファイル。 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m でテーマ名を指定する必要があります[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mシェルの補完スクリプトを生成します[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mサポートされているファイル形式は、rust crate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m がダンプしたバイナリデータです。
[48;2;34;34;34m[38;2;255;255;255m本質的に、これらのテーマは古い版の sublime のテーマです。
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m が指定されていない場合、環境変数 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m の値が読み取られます。[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mソースファイルの形式を手動指定する（デフォルトは自動判別）[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mテーマ背景を無効にする（デフォルトは有効ですが、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を使用して無効化できます）[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 未圧縮のバージョンは、圧縮バージョンよりも優先されます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m が指定されていない場合は、環境変数 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m の値が読み込まれます。[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m が指定されていない場合は、環境変数 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m の値が読み込まれます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m値が false の場合は無効になります。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m に似ていますが、圧縮されていない dump データ(ファイル)が必要です[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255m補完スクリプトを生成するために、シェルの名前を手動で指定する必要があります。
[48;2;34;34;34m[38;2;255;255;255mたとえば、Windows 上の PowerShell の場合、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を使用できます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q. 何が shell の補完機能ですか？
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: コマンドラインにコマンドを入力している時、Tabキーを押すと、補完スクリプトがトリガーされ、可能性のある補完オプションが検索され、これらのオプションが選択できるように表示されます。この機能は、コマンドライン入力の効率と正確性を大幅に向上させることができます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m異なるシェルの補完スクリプトは、保存される場所が異なります。zsh の場合、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 配列に含まれるパスに保存する必要があります。
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を使用して、この配列の内容を取得できます。
[48;2;34;34;34m[38;2;255;255;255mたとえば、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ディレクトリが含まれているとします。その場合、次のコマンドを使用して、補完スクリプトを [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ファイルに書き込むことができます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mroot 権限を使用しないようにするために、ユーザのサブディレクトリに配置することもできます。たとえば、次のコマンドを使用して、配列の先頭に要素を挿入し、関連するディレクトリを使用できます。
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mstdin からデータを読み取る場合、例： [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m、それは最初にファイル形式を自動的に判断しようとします。
[48;2;34;34;34m[38;2;255;255;255mただし、自動判断が失敗する場合があります。その場合、手動で形式を指定する必要があります。
[48;2;34;34;34m[38;2;255;255;255m例：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m のパラメーターだけで使用する場合は、組み込みのスタイルのセットが表示されます。
[48;2;34;34;34m[38;2;255;255;255mmarkdown 以外は md に省略形があります。
[48;2;34;34;34m[38;2;255;255;255m現在、[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mテーブルスタイル[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m のコレクションをファイルから読み取る機能はサポートされていません。 追加の機能が必要な場合は、issueを提出してフィードバックしてください。[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255m表のスタイルを指定します[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m パラメータだけを使用する場合、現在のテーマのリストが表示されます。
[48;2;34;34;34m[38;2;255;255;255mテーマファイルが指定されていない場合、組み込みのテーマセットが使用されます。
[48;2;34;34;34m[38;2;255;255;255m例えば、[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m というテーマがある場合、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を使用してそのテーマを使用できます。
[48;2;34;34;34m[38;2;255;255;255mテーマ名に空白が含まれている場合は、ダブルクォートまたはシングルクォートで囲む必要があります。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注： [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m という特殊なテーマがあります。これは、シンタックスハイライトのない純テキストスタイルを表します。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m オプションが指定されていない場合、データは環境変数 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m から読み取られます。
[48;2;34;34;34m[38;2;255;255;255mこのヘルプ情報の [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mシンタックスハイライト[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m テーマも、この [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m環境変数[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m から読み取られます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m デフォルトのヘルプ情報はハードコーディングされており、読み込み速度が非常に速いです。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m ハードコーディングされたテキストを読み込む速度と [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m テーマのテキストを読み込む速度は同じですが、動的に読み込む場合は2桁以上遅くなります。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 環境変数を設定すると、テーマが再度読み込まれるため、起動速度が [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m著しく[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 遅くなります。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m テーマを変更するか、シンタックスハイライトを無効にする場合を除き、環境変数を変更する代わりに [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m パラメータを使用してください。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m環境変数を使用した例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mコマンドライン引数を使用した例:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m手動で [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv/get/set[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m などの操作を実行した場合の出力については、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m で指定されたテーマ > 環境変数の優先順位があります。
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m が呼び出されない場合、環境変数が読み取られます。環境変数が空の場合は、[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m が使用されます。[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255m指定されたシンタックスハイライトのテーマ名[0m"##),
    ],
}
}

/// Language ID: ja;
/// Map name: "conv";
/// Description: 日本語, 日本語の文字, 日本;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "toml、yaml、json、ronなどの形式間で変換する");
/// ```
pub(super) const fn get_ja_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"toml、yaml、json、ronなどの形式間で変換する"##),
        ("from", r##"入力ファイルを指定します"##),
        ("conv-to", r##"出力形式/ファイルを指定します。複数回呼び出すことができます。"##),
        ("save", r##"このオプションを呼び出すと、`--to` で指定されたファイルが保存されます。"##),
        ("conv-to-help", r##"現在のディレクトリに **test.json** ファイルがある場合、`conv test.json -t yml` を呼び出してyml (yaml)に変換することができます。
また、ファイルパスを手動で指定することもできます。`conv test.json -t /tmp/test.yml`

サポートされている形式は、このパッケージで有効になっている機能によって異なります。全機能が自動的に有効になるわけではありません。

さらに、使用前に知っておく必要があることがあります。
- 1. 具体的なファイルパス以外にも自動的に名前を変更します。
  - 1.1: ファイルパスが一段階だけの場合、ファイル形式と判断されます。
    `conv test.toml -t ron` を呼び出すと、出力ファイルが `test.ron` に自動変更されるため、 `ron` にはなりません。
    この機能が必要ない場合は、`-t ./ron`または`-t $PWD/ron`を使用してください。
  - 1.2: ディレクトリを指す場合
    `-t /tmp/` のように指定した場合、 `test.toml` を `/tmp/` のサブディレクトリに置く必要があります。`/tmp` 自体を上書きすることはできません。
    この機能が必要ない場合は、完全なファイルパス（ディレクトリではない）を指定してください。たとえば、`-t /tmp/xx.toml`のように、`-t /tmp/`ではなく、`-t /tmp/xx.toml`を使用します。
- 2. `--save` を呼び出さない場合、指定されたファイルは自動的に保存されません。
- 3. `--save` を呼び出す場合、convサブコマンドの入力と出力が同じファイルになることはできません。
- 4. 変換後のファイルにはコメントが含まれません。"##),
    ],
}
}

/// Language ID: ja;
/// Map name: "conv_md";
/// Description: 日本語, 日本語の文字, 日本;
pub(super) const fn get_ja_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mtoml、yaml、json、ronなどの形式間で変換する[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255m入力ファイルを指定します[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255m出力形式/ファイルを指定します。複数回呼び出すことができます。[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mこのオプションを呼び出すと、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m で指定されたファイルが保存されます。[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255m現在のディレクトリに [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m ファイルがある場合、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を呼び出してyml (yaml)に変換することができます。
[48;2;34;34;34m[38;2;255;255;255mまた、ファイルパスを手動で指定することもできます。[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mサポートされている形式は、このパッケージで有効になっている機能によって異なります。全機能が自動的に有効になるわけではありません。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mさらに、使用前に知っておく必要があることがあります。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m 具体的なファイルパス以外にも自動的に名前を変更します。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: ファイルパスが一段階だけの場合、ファイル形式と判断されます。
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を呼び出すと、出力ファイルが [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m に自動変更されるため、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m にはなりません。
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mこの機能が必要ない場合は、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255mまたは[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255mを使用してください。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: ディレクトリを指す場合
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m のように指定した場合、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m のサブディレクトリに置く必要があります。[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 自体を上書きすることはできません。
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mこの機能が必要ない場合は、完全なファイルパス（ディレクトリではない）を指定してください。たとえば、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255mのように、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255mではなく、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255mを使用します。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を呼び出さない場合、指定されたファイルは自動的に保存されません。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を呼び出す場合、convサブコマンドの入力と出力が同じファイルになることはできません。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. 変換後のファイルにはコメントが含まれません。[0m"##),
    ],
}
}

/// Language ID: ja;
/// Map name: "opt";
/// Description: 日本語, 日本語の文字, 日本;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "テーブルの配列（使用方法：`--aot 'k1, v1, k2, v2' --aot 'k1, v1'`）");
/// ```
pub(super) const fn get_ja_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"`-k` との違いは、 `--ck` は自動的に `split('.')` を行わないことです。
[a.b.c] の場合、 `-k a.b.c` と `--ck a --ck b --ck c` は同じです。
簡単な場合には、`-k` を使用する方が便利です。

ただし、キー名に '.' が含まれる場合は、`-k` ではなく `--ck` を使用してください。
例：

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

この構造体では、"editor.defaultFormatter" は完全な文字列であり、`defaultFormatter` は `editor` の sub-key ではありません。

`-k '[typescript]'.'editor.defaultFormatter'` を使用すると、キーが分割されます：

- "[typescript]"
- "editor"
- "defaultFormatter"

一方、 `--ck '[typescript]' --ck editor.defaultFormatter` を使用すると、以下のようになります：

- "[typescript]"
- "editor.defaultFormatter"

後者が正しい構造体です。"##),
        ("num-arr", r##"i64型の配列"##),
        ("bool", r##"ブール型(Bool)，`true`または`false`のみ可能です。"##),
        ("num", r##"i64: 64ビット符号付き整数型, 例：1145"##),
        ("datetime-help", r##"例：

```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

このオプションは `toml` のみをサポートしています。コマンドの末尾に `-t json` を追加すると、次のようになります。

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"指定されたフォーマット/ファイル名で保存します。一度に1つのみ保存できます。"##),
        ("get-opts-help", r##"使用例1：`get Cargo.toml -k package.name`
使用例2：`get cfg.yaml -k .`
"##),
        ("none-help", r##"> 注意：Ronには本当の `Option<T>` 型があり、値は `Some(T)` または `None` です。
> ここでの `--none` は、真の `Option<T>` 型ではなく、指定されたキーを削除することを意味します。

test.ymlに次のテーブル配列がある場合、

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

次のようにして **name: test** を削除できます：

```sh
set test.yml -k bin.1.name --none
```

> 注：空文字列とNoneは異なります。
> `set test.yml --kv bin.1.name ""` を使用することで空文字列に設定できますが、`--none` はキー全体を削除します。


2番目のテーブル（インデックス1）を削除するには、次のようにします。

```sh
set test.yml -k bin.1 --none
```

または `--rm` を使用することもできます。

```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"倍精度浮動小数点数（IEEE 754-2008標準），例：3.14159265359"##),
        ("saot-help", r##"> 注意：`--saot` を使用して、各テーブルに対して1つのK-Vペアしか作成できませんが、他のオプションを使用してテーブル配列のインデックスを指定し、拡張することができます。

例：

```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"k-v ペアを設定します。`--kv x.y.z 値` は `-k x.y.z -s 値` に相当します。"##),
        ("set-save", r##"`--to` を指定せずに呼び出される場合、現在のファイルを上書き（書き込み）します。"##),
        ("array", r##"文字列の配列"##),
        ("aot-help", r##"例：

```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

まず、指定されたキーを選択し、次に指定されたデータ型の値を変更します。

> key のみが提供され、value が提供されていない場合、対応するデータ型と value を表示します。
> たとえば、 `set Cargo.toml -k package` のようにすると、package およびそのサブテーブルのテーブルが表示され、 `get` より詳細な情報が提供されます。

## 使用例

### 使用例1：
```sh
--ts md set test.toml --key package.edition --str 2021
```

`--num`ではなく`--str`が指定されているため、"2021"は整数型ではなく文字列型です。

| package | new value |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### 使用例2：
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### 使用例3：
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: 1つ目のインデックス*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> 注：上記の例は、args/サブコマンドで始まり、ルートコマンド名が省略されています。実際の操作では、手動で補完してください。"##),
        ("num-arr-help", r##"例：

```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"例：

```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> `toml` の場合、他のフォーマットではデフォルトで標準テーブルが使用されます。

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"キー（key）を使ってバリュー（value）を検索する"##),
        ("f64-arr-help", r##"例：

```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> JSON 1.0 はNaNとInfをサポートしていません。代わりに、nullが出力されます。

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"指定されたキーを削除する"##),
        ("set-save-to-help", r##"`conv` サブコマンドの `-t` に非常に似ていますが、最大の違いは `conv` の `-t` は複数回連続呼び出しが可能で、複数のフォーマット/ファイルに出力できることです。
一方、 `set` サブコマンドの `-t` は1回だけ呼び出すことができます。単一のファイルに出力されます。

> 注： `conv` サブコマンドは変換操作の後、すべてのファイル形式でコメントを保持しません。
> ただし、 `set` サブコマンドは **toml** を除くすべてのフォーマットでコメントを保持します。

- `-t` を呼び出さない前提で `--save` を呼び出すと、自身を上書きします。
- `-t` が呼び出され、 `--save` も呼び出された場合、変更後のファイルは `-t` で指定されたファイルに保存されます。
- 元のファイルと `-t` の指定するファイルが異なる場合、 `set` サブコマンドの `-t` 指定は `conv` の `-t` 指定に非常に似た動作をすることになります。どちらも具体的なファイルパス以外は自動的に名前を変更します。
  - たとえば、from: **test.yml**、to: **json** の場合、出力ファイルは **test.json** ではなく **json** になります。
  - この機能は必要ない場合は、`-t json` ではなく `-t ./json` や `-t $PWD/json` を使用してください。"##),
        ("preview", r##"変更を完了した後、ファイル全体をプレビューします"##),
        ("set-key", r##"指定されたキーを"."で区切って選択します"##),
        ("get-opts", r##"指定した設定値を取得します"##),
        ("saot", r##"単一のキーと値で構成されるテーブルの配列（使用方法：`--saot k1 v1 --saot k2 v2...`）"##),
        ("key-help", r##"フォーマットは「a.b.c」で、サブテーブル名または配列のインデックスは「.」でつながります。
"."が1つしかない場合は、表全体が表示されます。"##),
        ("set-opts", r##"設定ファイルの構造を作成/更新/削除します"##),
        ("bool-arr-help", r##"例：

```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // 注: [0]: 最初の要素は true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"bool型の配列"##),
        ("table", r##"複数のK-Vペアを格納するための標準テーブル（standard table）"##),
        ("datetime", r##"日時の型"##),
        ("table-help", r##"例1：
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

例2：
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"f64型の配列"##),
        ("string", r##"文字列型（String）"##),
        ("array-help", r##"`-a`を複数回呼び出すことで、配列要素を設定します。
例：

```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> 最初の呼び出しでインデックス0の要素を設定し、2番目の呼び出しでインデックス1、以此類推。

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"テーブルの配列（使用方法：`--aot 'k1, v1, k2, v2' --aot 'k1, v1'`）"##),
        ("inline-table", r##"インラインテーブル、よりコンパクトな標準テーブル"##),
        ("concat-key", r##"キーを分割して入力し、自動的に連結されます。「.」は区切り文字として使用されません。"##),
    ],
}
}

/// Language ID: ja;
/// Map name: "opt_md";
/// Description: 日本語, 日本語の文字, 日本;
pub(super) const fn get_ja_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m との違いは、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m は自動的に [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msplit('.')[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を行わないことです。
[48;2;34;34;34m[38;2;255;255;255m[a.b.c] の場合、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m と [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m は同じです。
[48;2;34;34;34m[38;2;255;255;255m簡単な場合には、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を使用する方が便利です。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mただし、キー名に '.' が含まれる場合は、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ではなく [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を使用してください。
[48;2;34;34;34m[38;2;255;255;255m例：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mこの構造体では、"editor.defaultFormatter" は完全な文字列であり、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mdefaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m は [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255meditor[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m の sub-key ではありません。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を使用すると、キーが分割されます：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m一方、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を使用すると、以下のようになります：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m後者が正しい構造体です。[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mi64型の配列[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mブール型(Bool)，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255mまたは[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255mのみ可能です。[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: 64ビット符号付き整数型, 例：1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255m例：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mこのオプションは [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m のみをサポートしています。コマンドの末尾に [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を追加すると、次のようになります。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255m指定されたフォーマット/ファイル名で保存します。一度に1つのみ保存できます。[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255m使用例1：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m使用例2：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注意：Ronには本当の [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 型があり、値は [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m または [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m です。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m ここでの [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m は、真の [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 型ではなく、指定されたキーを削除することを意味します。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtest.ymlに次のテーブル配列がある場合、
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m次のようにして [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mname: test[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m を削除できます：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：空文字列とNoneは異なります。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m を使用することで空文字列に設定できますが、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m はキー全体を削除します。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m2番目のテーブル（インデックス1）を削除するには、次のようにします。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mまたは [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を使用することもできます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255m倍精度浮動小数点数（IEEE 754-2008標準），例：3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注意：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m を使用して、各テーブルに対して1つのK-Vペアしか作成できませんが、他のオプションを使用してテーブル配列のインデックスを指定し、拡張することができます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m例：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mk-v ペアを設定します。[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z 値[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m は [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s 値[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m に相当します。[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を指定せずに呼び出される場合、現在のファイルを上書き（書き込み）します。[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255m文字列の配列[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255m例：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mまず、指定されたキーを選択し、次に指定されたデータ型の値を変更します。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m key のみが提供され、value が提供されていない場合、対応するデータ型と value を表示します。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m たとえば、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m のようにすると、package およびそのサブテーブルのテーブルが表示され、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m より詳細な情報が提供されます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m使用例[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m使用例1：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255mではなく[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255mが指定されているため、"2021"は整数型ではなく文字列型です。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m new value [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m使用例2：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m使用例3：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: 1つ目のインデックス*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：上記の例は、args/サブコマンドで始まり、ルートコマンド名が省略されています。実際の操作では、手動で補完してください。[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m例：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255m例：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m の場合、他のフォーマットではデフォルトで標準テーブルが使用されます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mキー（key）を使ってバリュー（value）を検索する[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m例：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m JSON 1.0 はNaNとInfをサポートしていません。代わりに、nullが出力されます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255m指定されたキーを削除する[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m サブコマンドの [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m に非常に似ていますが、最大の違いは [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m の [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m は複数回連続呼び出しが可能で、複数のフォーマット/ファイルに出力できることです。
[48;2;34;34;34m[38;2;255;255;255m一方、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m サブコマンドの [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m は1回だけ呼び出すことができます。単一のファイルに出力されます。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注： [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m サブコマンドは変換操作の後、すべてのファイル形式でコメントを保持しません。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m ただし、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m サブコマンドは [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m を除くすべてのフォーマットでコメントを保持します。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を呼び出さない前提で [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を呼び出すと、自身を上書きします。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m が呼び出され、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m も呼び出された場合、変更後のファイルは [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m で指定されたファイルに保存されます。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 元のファイルと [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m の指定するファイルが異なる場合、 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m サブコマンドの [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 指定は [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m の [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 指定に非常に似た動作をすることになります。どちらも具体的なファイルパス以外は自動的に名前を変更します。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m たとえば、from: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m、to: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m の場合、出力ファイルは [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m ではなく [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m になります。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m この機能は必要ない場合は、[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ではなく [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m や [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m を使用してください。[0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255m変更を完了した後、ファイル全体をプレビューします[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255m指定されたキーを"."で区切って選択します[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255m指定した設定値を取得します[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255m単一のキーと値で構成されるテーブルの配列（使用方法：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m）[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mフォーマットは「a.b.c」で、サブテーブル名または配列のインデックスは「.」でつながります。
[48;2;34;34;34m[38;2;255;255;255m"."が1つしかない場合は、表全体が表示されます。[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255m設定ファイルの構造を作成/更新/削除します[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m例：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m 注: [0]: 最初の要素は true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mbool型の配列[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255m複数のK-Vペアを格納するための標準テーブル（standard table）[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255m日時の型[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255m例1：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m例2：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mf64型の配列[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255m文字列型（String）[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255mを複数回呼び出すことで、配列要素を設定します。
[48;2;34;34;34m[38;2;255;255;255m例：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 最初の呼び出しでインデックス0の要素を設定し、2番目の呼び出しでインデックス1、以此類推。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255mテーブルの配列（使用方法：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m）[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mインラインテーブル、よりコンパクトな標準テーブル[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mキーを分割して入力し、自動的に連結されます。「.」は区切り文字として使用されません。[0m"##),
    ],
}
}

/// Language ID: ja;
/// Map name: "parser";
/// Description: 日本語, 日本語の文字, 日本;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "以下は stderr の出力です。補完スクリプトの出力をリダイレクトする場合、`2>/path/to/file`ではなく、`>/path/to/file`を使用する必要があります。");
/// ```
pub(super) const fn get_ja_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"以下は stderr の出力です。補完スクリプトの出力をリダイレクトする場合、`2>/path/to/file`ではなく、`>/path/to/file`を使用する必要があります。"##),
    ],
}
}

/// Language ID: ja;
/// Map name: "theme";
/// Description: 日本語, 日本語の文字, 日本;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "無効なトピック名です");
/// ```
pub(super) const fn get_ja_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"無効なトピック名です"##),
    ],
}
}

/// ja: 日本語, 日本語の文字, 日本
pub(super) const fn get_ja_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_ja_map_args),
        ("conv_md", get_ja_map_conv_md),
        ("parser", get_ja_map_parser),
        ("opt_md", get_ja_map_opt_md),
        ("args_md", get_ja_map_args_md),
        ("opt", get_ja_map_opt),
        ("log-core", get_ja_map_log_core),
        ("conv", get_ja_map_conv),
        ("theme", get_ja_map_theme),
    ],
}
}

/// Language ID: jw;
/// Map name: "log-core";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_jw_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"ngelingake"##),
        ("info", r##"Info"##),
        ("error", r##"Kesalahan"##),
        ("trace", r##"tilak"##),
        ("init-logger", r##"Inisialisasi logger ..."##),
    ],
}
}

/// Language ID: jw;
/// Map name: "theme";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Jeneng tema sing ora bener");
/// ```
pub(super) const fn get_jw_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Jeneng tema sing ora bener"##),
    ],
}
}

/// jw: jw-Latn-ID
pub(super) const fn get_jw_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_jw_map_log_core),
        ("theme", get_jw_map_theme),
    ],
}
}

/// Language ID: ka;
/// Map name: "log-core";
/// Description: ქართული, ქართული, საქართველო;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "გამართვა");
/// ```
pub(super) const fn get_ka_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"გამართვა"##),
        ("warn", r##"აფრთხილებს"##),
        ("info", r##"ინფორმაცია"##),
        ("error", r##"შეცდომა"##),
        ("trace", r##"კვალი"##),
        ("init-logger", r##"ლოგინის ინიციალიზაცია ..."##),
    ],
}
}

/// Language ID: ka;
/// Map name: "theme";
/// Description: ქართული, ქართული, საქართველო;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "არასწორი თემის სახელი");
/// ```
pub(super) const fn get_ka_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"არასწორი თემის სახელი"##),
    ],
}
}

/// ka: ქართული, ქართული, საქართველო
pub(super) const fn get_ka_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ka_map_log_core),
        ("theme", get_ka_map_theme),
    ],
}
}

/// Language ID: kk;
/// Map name: "log-core";
/// Description: қазақ тілі, кирилл жазуы, Қазақстан;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Атқару");
/// ```
pub(super) const fn get_kk_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Атқару"##),
        ("warn", r##"ескертіңіз"##),
        ("info", r##"ақпарат"##),
        ("error", r##"қате"##),
        ("trace", r##"із"##),
        ("init-logger", r##"Бастауды бастау ..."##),
    ],
}
}

/// Language ID: kk;
/// Map name: "theme";
/// Description: қазақ тілі, кирилл жазуы, Қазақстан;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Тақырыптың жарамсыздығы");
/// ```
pub(super) const fn get_kk_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Тақырыптың жарамсыздығы"##),
    ],
}
}

/// kk: қазақ тілі, кирилл жазуы, Қазақстан
pub(super) const fn get_kk_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_kk_map_log_core),
        ("theme", get_kk_map_theme),
    ],
}
}

/// Language ID: km;
/// Map name: "log-core";
/// Description: ខ្មែរ, ខ្មែរ, កម្ពុជា;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ប\u{17c6}បាត\u{17cb}ក\u{17c6}ហ\u{17bb}ស");
/// ```
pub(super) const fn get_km_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"បំបាត់កំហុស"##),
        ("warn", r##"ព្រមាន"##),
        ("info", r##"ព័ត៌មាន"##),
        ("error", r##"កំហុស"##),
        ("trace", r##"ដាន"##),
        ("init-logger", r##"ចាប់ផ្តើមអ្នកកាប់ឈើ ..."##),
    ],
}
}

/// Language ID: km;
/// Map name: "theme";
/// Description: ខ្មែរ, ខ្មែរ, កម្ពុជា;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "ឈ\u{17d2}មោះស\u{17d2}បែកម\u{17b7}នត\u{17d2}រ\u{17b9}មត\u{17d2}រ\u{17bc}វ");
/// ```
pub(super) const fn get_km_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"ឈ្មោះស្បែកមិនត្រឹមត្រូវ"##),
    ],
}
}

/// km: ខ្មែរ, ខ្មែរ, កម្ពុជា
pub(super) const fn get_km_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_km_map_log_core),
        ("theme", get_km_map_theme),
    ],
}
}

/// Language ID: kn;
/// Map name: "log-core";
/// Description: ಕನ್ನಡ, ಕನ್ನಡ, ಭಾರತ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ಡೀಬಗ\u{ccd}");
/// ```
pub(super) const fn get_kn_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ಡೀಬಗ್"##),
        ("warn", r##"ಎಚ್ಚರಿಸಿ"##),
        ("info", r##"ಮಾಹಿತಿ"##),
        ("error", r##"ದೋಷ"##),
        ("trace", r##"ಜಾಡಿನ"##),
        ("init-logger", r##"ಲಾಗರ್ ಅನ್ನು ಪ್ರಾರಂಭಿಸುವುದು ..."##),
    ],
}
}

/// Language ID: kn;
/// Map name: "theme";
/// Description: ಕನ್ನಡ, ಕನ್ನಡ, ಭಾರತ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "ಅಮಾನ\u{ccd}ಯ ಥೀಮ\u{ccd} ಹ\u{cc6}ಸರು");
/// ```
pub(super) const fn get_kn_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"ಅಮಾನ್ಯ ಥೀಮ್ ಹೆಸರು"##),
    ],
}
}

/// kn: ಕನ್ನಡ, ಕನ್ನಡ, ಭಾರತ
pub(super) const fn get_kn_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_kn_map_log_core),
        ("theme", get_kn_map_theme),
    ],
}
}

/// Language ID: ko;
/// Map name: "log-core";
/// Description: 한국어, 한국 문자, 대한민국;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "디버그");
/// ```
pub(super) const fn get_ko_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"디버그"##),
        ("warn", r##"경고"##),
        ("info", r##"정보"##),
        ("error", r##"오류"##),
        ("trace", r##"추적"##),
        ("init-logger", r##"로거 초기화 ..."##),
    ],
}
}

/// Language ID: ko;
/// Map name: "theme";
/// Description: 한국어, 한국 문자, 대한민국;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "잘못된 테마 이름");
/// ```
pub(super) const fn get_ko_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"잘못된 테마 이름"##),
    ],
}
}

/// Language ID: ko;
/// Map name: "args";
/// Description: 한국어, 한국 문자, 대한민국;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "테마 배경을 사용하지 않도록 설정합니다(기본값은 활성화됨). `--dis-theme-bg`를 호출하지 않으면 `$TOMLYRE_THEME_BG` 값을 읽습니다.");
/// ```
pub(super) const fn get_ko_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"사용자 정의 테마 집합 파일입니다. `--theme`으로 테마 이름을 지정해야 합니다."##),
        ("shell-completion", r##"쉘 완성 스크립트를 생성합니다."##),
        ("theme-file-help", r##"지원되는 파일 형식은 rust crate `syntect`에서 덤프한 바이너리 데이터입니다.
본질적으로 이러한 테마들은 이전 버전의 sublime 테마입니다.
`--tf`가 지정되지 않은 경우, 환경 변수 `$TOMLYRE_THEME_FILE`의 값을 읽습니다. "##),
        ("from-format", r##"수동으로 소스 파일 형식을 지정합니다(자동으로 감지됩니다)."##),
        ("disable-theme-background", r##"테마 배경을 사용하지 않도록 설정합니다(기본값은 활성화됨). `--dis-theme-bg`를 호출하지 않으면 `$TOMLYRE_THEME_BG` 값을 읽습니다."##),
        ("theme-uncompressed-file-help", r##"> 압축이 풀린 버전보다 우선 순위가 높습니다.

`--tuf`가 지정되지 않은 경우, 환경 변수 `$TOMLYRE_THEME_UNCOMPRESSED_FILE`의 값을 읽습니다. "##),
        ("disable-theme-background-help", r##"활성화되면 false로 설정하세요.

환경 변수 사용 예:
```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"`theme-file`과 유사하지만 압축이 풀리지 않은 덤프 데이터(파일)를 요구합니다."##),
        ("shell-completion-help", r##"쉘 이름을 지정하여 직접 완성 스크립트를 생성하십시오.
예: 윈도우즈의 파워셸에서는 `--sh-comp pwsh`를 사용할 수 있습니다.

> Q: 쉘 완성이란 무엇인가요?

> A: 우리가 명령어를 입력할 때, 탭 키를 누르면 완성 기능이 작동하여 가능한 완성 옵션을 검색하고 제안합니다. 이러한 기능은 CLI 입력의 효율과 정확도를 크게 향상시킬 수 있습니다.

각 쉘에 대한 완성 스크립트 파일의 위치는 다릅니다.
zsh의 경우 `$fpath` 배열에 포함된 경로에 저장해야 합니다.
`print -l $fpath`을 사용하여 해당 배열의 내용을 가져올 수 있습니다.
예를 들어 `/usr/local/share/zsh/site-functions` 디렉토리가 포함되어 있다고 가정하면 다음 명령을 사용하여 완성 스크립트를 `_tomlyre` 파일에 작성할 수 있습니다.

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

루트 권한을 피하기 위해 사용자 하위 디렉터리에 설치할 수도 있습니다.
예를 들어 다음 명령을 사용하여 해당 배열의 맨 앞에 요소를 삽입하고 관련 디렉터리를 사용할 수 있습니다.
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"stdin에서 데이터를 읽는 경우(`cat a | tomlyre conv -`와 같은 경우), 먼저 파일 형식을 자동으로 감지하려고 시도합니다.
그러나 자동 감지가 실패할 수 있으므로 수동으로 형식을 지정해야 하는 경우가 있습니다.
예: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"`--ts` 매개 변수만 사용하면 내부 스타일 셋이 표시됩니다.
Markdown은 축약형으로 `md`를 사용할 수 있으며, 다른 스타일은 모두 전체 이름을 사용해야 합니다.
사용 방법 예1：`--ts md`
사용 방법 예2：`--table-style ascii`

참고: 현재 **표 스타일** 셋을 파일에서 읽어 오는 것은 지원하지 않습니다. 이 기능이 필요하거나 추가 기능이 필요한 경우, issue를 보내주시기 바랍니다."##),
        ("table-style", r##"표의 스타일을 지정합니다."##),
        ("theme-help", r##"`-t` 매개 변수만 사용하면 현재 테마 목록이 나열됩니다.
테마 파일이 지정되지 않은 경우 내장된 테마 집합이 사용됩니다.
예를 들어 **ayu-dark** 라는 테마가 있다면 `-t ayu-dark`를(를) 사용하여 해당 테마를 사용할 수 있습니다.
테마 이름에 공백이 포함된 경우 큰따옴표나 작은따옴표로 묶어주십시오.

> 참고: `None` 이라는 특수한 테마가 있으며, 구문 강조 없이 일반 텍스트 스타일입니다.

`--theme` 옵션이 지정되지 않은 경우 환경 변수 `$TOMLYRE_THEME`에서 데이터를 읽습니다.
이 도움말 정보의 **구문 강조** 테마도 이 **환경 변수**에서 가져옵니다.

> 기본 도움말은 하드코딩되어 있어 로드 속도가 매우 빠릅니다.
> 하드코딩된 텍스트와 `None` 테마 텍스트를 로드하는 속도는 같은 수준이며, 동적으로 로드하면 속도가 2단계 이상 느려집니다.
> `$TOMLYRE_THEME` 환경 변수를 설정하면 다시 테마를 로드하므로 시작 속도가 **매우** 느려질 수 있습니다.
> 테마 정보를 수정하거나 구문 강조 기능을 비활성화하지 않는 한, 환경 변수 대신 `-t` 매개 변수를 사용하십시오.

환경 변수 사용 예:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

명령행 인수 사용 예:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

`conv/get/set` 등과 같은 작업을 수동으로 실행할 때 출력되는 내용의 경우 `-t`로 지정된 테마가 우선합니다.
`-t`를 호출하지 않으면 환경 변수를 읽고, 환경 변수가 비어 있는 경우 **Monokai Extended**를 사용합니다."##),
        ("theme", r##"구문 강조에 사용할 테마 이름을 지정합니다."##),
    ],
}
}

/// Language ID: ko;
/// Map name: "args_md";
/// Description: 한국어, 한국 문자, 대한민국;
pub(super) const fn get_ko_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255m사용자 정의 테마 집합 파일입니다. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m으로 테마 이름을 지정해야 합니다.[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255m쉘 완성 스크립트를 생성합니다.[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255m지원되는 파일 형식은 rust crate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m에서 덤프한 바이너리 데이터입니다.
[48;2;34;34;34m[38;2;255;255;255m본질적으로 이러한 테마들은 이전 버전의 sublime 테마입니다.
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m가 지정되지 않은 경우, 환경 변수 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m의 값을 읽습니다. [0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255m수동으로 소스 파일 형식을 지정합니다(자동으로 감지됩니다).[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255m테마 배경을 사용하지 않도록 설정합니다(기본값은 활성화됨). [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 호출하지 않으면 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 값을 읽습니다.[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 압축이 풀린 버전보다 우선 순위가 높습니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m가 지정되지 않은 경우, 환경 변수 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m의 값을 읽습니다. [0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255m활성화되면 false로 설정하세요.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m환경 변수 사용 예:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m과 유사하지만 압축이 풀리지 않은 덤프 데이터(파일)를 요구합니다.[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255m쉘 이름을 지정하여 직접 완성 스크립트를 생성하십시오.
[48;2;34;34;34m[38;2;255;255;255m예: 윈도우즈의 파워셸에서는 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 사용할 수 있습니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q: 쉘 완성이란 무엇인가요?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: 우리가 명령어를 입력할 때, 탭 키를 누르면 완성 기능이 작동하여 가능한 완성 옵션을 검색하고 제안합니다. 이러한 기능은 CLI 입력의 효율과 정확도를 크게 향상시킬 수 있습니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m각 쉘에 대한 완성 스크립트 파일의 위치는 다릅니다.
[48;2;34;34;34m[38;2;255;255;255mzsh의 경우 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 배열에 포함된 경로에 저장해야 합니다.
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m을 사용하여 해당 배열의 내용을 가져올 수 있습니다.
[48;2;34;34;34m[38;2;255;255;255m예를 들어 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 디렉토리가 포함되어 있다고 가정하면 다음 명령을 사용하여 완성 스크립트를 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 파일에 작성할 수 있습니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m루트 권한을 피하기 위해 사용자 하위 디렉터리에 설치할 수도 있습니다.
[48;2;34;34;34m[38;2;255;255;255m예를 들어 다음 명령을 사용하여 해당 배열의 맨 앞에 요소를 삽입하고 관련 디렉터리를 사용할 수 있습니다.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mstdin에서 데이터를 읽는 경우([48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m와 같은 경우), 먼저 파일 형식을 자동으로 감지하려고 시도합니다.
[48;2;34;34;34m[38;2;255;255;255m그러나 자동 감지가 실패할 수 있으므로 수동으로 형식을 지정해야 하는 경우가 있습니다.
[48;2;34;34;34m[38;2;255;255;255m예: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 매개 변수만 사용하면 내부 스타일 셋이 표시됩니다.
[48;2;34;34;34m[38;2;255;255;255mMarkdown은 축약형으로 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mmd[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 사용할 수 있으며, 다른 스타일은 모두 전체 이름을 사용해야 합니다.
[48;2;34;34;34m[38;2;255;255;255m사용 방법 예1：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m사용 방법 예2：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m참고: 현재 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m표 스타일[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 셋을 파일에서 읽어 오는 것은 지원하지 않습니다. 이 기능이 필요하거나 추가 기능이 필요한 경우, issue를 보내주시기 바랍니다.[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255m표의 스타일을 지정합니다.[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 매개 변수만 사용하면 현재 테마 목록이 나열됩니다.
[48;2;34;34;34m[38;2;255;255;255m테마 파일이 지정되지 않은 경우 내장된 테마 집합이 사용됩니다.
[48;2;34;34;34m[38;2;255;255;255m예를 들어 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 라는 테마가 있다면 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를(를) 사용하여 해당 테마를 사용할 수 있습니다.
[48;2;34;34;34m[38;2;255;255;255m테마 이름에 공백이 포함된 경우 큰따옴표나 작은따옴표로 묶어주십시오.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 참고: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 이라는 특수한 테마가 있으며, 구문 강조 없이 일반 텍스트 스타일입니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 옵션이 지정되지 않은 경우 환경 변수 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m에서 데이터를 읽습니다.
[48;2;34;34;34m[38;2;255;255;255m이 도움말 정보의 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m구문 강조[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 테마도 이 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m환경 변수[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m에서 가져옵니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 기본 도움말은 하드코딩되어 있어 로드 속도가 매우 빠릅니다.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 하드코딩된 텍스트와 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 테마 텍스트를 로드하는 속도는 같은 수준이며, 동적으로 로드하면 속도가 2단계 이상 느려집니다.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 환경 변수를 설정하면 다시 테마를 로드하므로 시작 속도가 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m매우[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 느려질 수 있습니다.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 테마 정보를 수정하거나 구문 강조 기능을 비활성화하지 않는 한, 환경 변수 대신 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 매개 변수를 사용하십시오.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m환경 변수 사용 예:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m명령행 인수 사용 예:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv/get/set[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 등과 같은 작업을 수동으로 실행할 때 출력되는 내용의 경우 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m로 지정된 테마가 우선합니다.
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 호출하지 않으면 환경 변수를 읽고, 환경 변수가 비어 있는 경우 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m를 사용합니다.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255m구문 강조에 사용할 테마 이름을 지정합니다.[0m"##),
    ],
}
}

/// Language ID: ko;
/// Map name: "conv";
/// Description: 한국어, 한국 문자, 대한민국;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "toml, yaml, json, ron 등의 형식 간 변환");
/// ```
pub(super) const fn get_ko_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"toml, yaml, json, ron 등의 형식 간 변환"##),
        ("from", r##"입력 파일을 지정합니다."##),
        ("conv-to", r##"지원되는 출력 형식/파일을 지정하고, 여러 번 사용할 수 있습니다."##),
        ("save", r##"이 옵션을 호출하면 `--to`로 지정된 파일이 저장됩니다."##),
        ("conv-to-help", r##"현재 디렉토리에 **test.json** 파일이 있다면, `conv test.json -t yml`을(를) 호출하여 yml(yaml)로 변환할 수 있습니다.
또한 수동으로 파일 경로를 지정할 수도 있습니다: `conv test.json -t /tmp/test.yml`

지원되는 형식은 패키지에서 활성화된 기능에 따라 다릅니다. 모든 기능이 기본적으로 활성화되지는 않습니다.

또한 사용하기 전에 알아둬야 할 사항이 있습니다.
- 1. 명시적인 파일 경로가 아닌 경우 자동으로 이름이 변경됩니다.
    - 1.1: 단일 파일 계층 구조만 파일 형식으로 인식됩니다.
    `conv test.toml -t ron`을(를) 호출하면 출력 파일은 `ron`이 아니라 자동으로 `test.ron`으로 변경됩니다.
    이 기능이 필요하지 않은 경우 `-t ./ron` 또는 `-t $PWD/ron`을(를) 사용하세요.
    - 1.2: 디렉토리 대상이 아닌 경우
    예를 들어 `-t /tmp/`을(를) 호출하면 `test.toml` 파일은 `/tmp/`의 하위 디렉토리에만 저장할 수 있으며, `/tmp` 자체를 덮어쓸 수 없습니다.
    이 기능이 필요하지 않은 경우 `-t /tmp/xx.toml`과 같은 완전한(디렉토리가 아닌) 파일 경로를 사용하세요.
- 2. `--save`를 호출하지 않은 경우 지정된 파일이 자동으로 저장되지 않습니다.
- 3. `--save`를 호출한 경우, conv 하위 명령의 입력 및 출력은 동일한 파일이 될 수 없습니다.
- 4. 변환된 파일에서 주석은 보존되지 않습니다."##),
    ],
}
}

/// Language ID: ko;
/// Map name: "conv_md";
/// Description: 한국어, 한국 문자, 대한민국;
pub(super) const fn get_ko_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mtoml, yaml, json, ron 등의 형식 간 변환[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255m입력 파일을 지정합니다.[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255m지원되는 출력 형식/파일을 지정하고, 여러 번 사용할 수 있습니다.[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255m이 옵션을 호출하면 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m로 지정된 파일이 저장됩니다.[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255m현재 디렉토리에 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 파일이 있다면, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m을(를) 호출하여 yml(yaml)로 변환할 수 있습니다.
[48;2;34;34;34m[38;2;255;255;255m또한 수동으로 파일 경로를 지정할 수도 있습니다: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m지원되는 형식은 패키지에서 활성화된 기능에 따라 다릅니다. 모든 기능이 기본적으로 활성화되지는 않습니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m또한 사용하기 전에 알아둬야 할 사항이 있습니다.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m 명시적인 파일 경로가 아닌 경우 자동으로 이름이 변경됩니다.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: 단일 파일 계층 구조만 파일 형식으로 인식됩니다.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m을(를) 호출하면 출력 파일은 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m이 아니라 자동으로 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m으로 변경됩니다.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m이 기능이 필요하지 않은 경우 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 또는 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m을(를) 사용하세요.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: 디렉토리 대상이 아닌 경우
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m예를 들어 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m을(를) 호출하면 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 파일은 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m의 하위 디렉토리에만 저장할 수 있으며, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 자체를 덮어쓸 수 없습니다.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m이 기능이 필요하지 않은 경우 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m과 같은 완전한(디렉토리가 아닌) 파일 경로를 사용하세요.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 호출하지 않은 경우 지정된 파일이 자동으로 저장되지 않습니다.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 호출한 경우, conv 하위 명령의 입력 및 출력은 동일한 파일이 될 수 없습니다.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. 변환된 파일에서 주석은 보존되지 않습니다.[0m"##),
    ],
}
}

/// Language ID: ko;
/// Map name: "opt";
/// Description: 한국어, 한국 문자, 대한민국;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "테이블 배열(사용법: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_ko_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"`-k`와 달리 `--ck`는 `split('.')`를 자동으로 수행하지 않습니다.
[a.b.c]와 같은 경우, `-k a.b.c`와 `--ck a --ck b --ck c`는 동일합니다. 
간단한 경우에는 `-k`를 사용하는 것이 더 간편합니다.

키 이름에 '.'이 포함되어 있으면 `-k` 대신 `--ck`를 사용하세요.
예: 

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

이 구조에서 "editor.defaultFormatter"는 전체 문자열입니다. `defaultFormatter`는 `editor`의 하위 키(sub-key)가 아닙니다.

`-k '[typescript]'.'editor.defaultFormatter'`를 사용하면 키가 다음과 같이 분할됩니다.

- "[typescript]"
- "editor"
- "defaultFormatter"

반면, `--ck '[typescript]' --ck editor.defaultFormatter`는 다음과 같이 키를 처리합니다.

- "[typescript]"
- "editor.defaultFormatter"

후자가 올바른 구조입니다."##),
        ("num-arr", r##"i64 배열"##),
        ("bool", r##"bool (불리언 유형)으로, `true` 또는 `false`만 사용할 수 있습니다."##),
        ("num", r##"i64: 64 비트 부호 있는 정수 유형입니다. 예시: 1145"##),
        ("datetime-help", r##"예시:
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

이 옵션은 `toml` 만 지원하며, 명령 끝에 `-t json`을 추가하면 다음과 같이 표시됩니다.

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"지정된 형식/파일로 저장하며, 한 번에 하나의 파일만 저장할 수 있습니다."##),
        ("get-opts-help", r##"사용 방법 예1：`get Cargo.toml -k package.name`
사용 방법 예2：`get cfg.yaml -k .`
"##),
        ("none-help", r##"> 참고: ron에는 실제로 `Option<T>` 타입이 있으며 값은 `Some(T)` 또는 `None`입니다.
> 여기서 `--none`은 실제 `Option<T>` 타입이 아닌 지정된 키를 삭제하는 것을 의미합니다.

test.yml에 다음과 같은 표 배열이 있다고 가정해 봅시다:

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

우리는 **name: test**를 삭제하려면 다음을 실행할 수 있습니다.
```sh
set test.yml -k bin.1.name --none
```

> 참고: 빈 문자열과 None은 다릅니다.
> `set test.yml --kv bin.1.name ""`를 호출하면 빈 문자열로 설정됩니다.`--none`은 전체 키가 제거됩니다.

두 번째 표 (인덱스 1)를 제거하려면 다음을 실행하면 됩니다.
```sh
set test.yml -k bin.1 --none
```

또는 `--rm`을 사용하여 삭제할 수 있습니다.
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"배정밀도 부동 소수점 숫자 (IEEE 754-2008 표준)로, 예시: 3.14159265359"##),
        ("saot-help", r##"> 참고: `--saot`을 사용하여 각 테이블에 대해 하나의 K-V pair만 지정할 수 있지만 다른 옵션을 사용하여 테이블 배열 인덱스를 먼저 지정하고 확장할 수 있습니다.

예시:
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"k-v 쌍(k-v pair)을 설정합니다. `--kv x.y.z value`는 `-k x.y.z -s value`와 동일합니다."##),
        ("set-save", r##"현재 파일을 덮어쓰기(overwrite)합니다. `--to`와 함께 사용되지 않은 경우"##),
        ("array", r##"문자열 배열"##),
        ("aot-help", r##"예시:
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

지정된 key로 이동한 다음 지정된 데이터 유형의 값을 수정합니다.

> value가 지정되지 않은 경우, 해당 데이터 유형과 값을 나열합니다.
> 예: `set Cargo.toml -k package`를 사용하면 패키지 및 하위 테이블의 테이블이 나열되며 `get`보다 자세한 정보가 제공됩니다.

## Examples

### 예시1：
```sh
--ts md set test.toml --key package.edition --str 2021
```

`--str`이 `--num` 대신 지정되었으므로 "2021"은 정수 타입이 아닌 문자열 타입입니다.

| package | new value |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### 예시2：
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### 예시3：
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: 이것은 첫 번째 인덱스입니다.*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> 注：위의 예는 매개 변수(argument)/서브 명령(sub-command)로 시작하여 루트 명령 이름을 생략하였습니다. 실제 작업에서는 수동으로 추가해야 합니다."##),
        ("num-arr-help", r##"예제:
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"예시:
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> `toml` 이외의 다른 형식에서는 인라인 테이블 대신 표준 테이블을 사용합니다.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"key(키)를 사용하여 value(값)를 검색합니다."##),
        ("f64-arr-help", r##"예제:
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> JSON 1.0은 NaN과 Inf를 지원하지 않습니다.

`null`이 출력됩니다.

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"지정된 키를 삭제"##),
        ("set-save-to-help", r##"`conv` 하위 명령어의 `-t`와 매우 유사하지만 가장 큰 차이점은 `conv`가 `-t`를 여러 번 연속 호출하여 다양한 형식/여러 파일로 출력할 수 있지만 `set` 하위 명령어의 `-t`는 한 번만 호출하고 단일 파일로 출력합니다.

> 참고: `conv` 하위 명령어는 변환 작업을 완료한 후 모든 형식의 파일에서 주석을 유지하지 않습니다.
> 반면에 `set` 하위 명령어는 **toml**을 제외하고 모든 형식의 주석을 유지합니다.

- `-t`를 호출하지 않은 상태에서 `--save`를 호출하면 자신을 덮어쓰게 됩니다.
- 이미 `-t`를 호출하고 `--save`도 호출한 경우 수정된 파일은 `-t`가 지정한 파일에 저장됩니다.
- 소스 파일과 `-t`가 지정한 파일이 동일하지 않은 경우, `set` 하위 명령어의 `-t` 동작은 `conv`의 `-t`와 유사해지며, 구체적인 파일 경로에 대해서만 자동으로 이름을 바꿉니다.
  - 예를 들어 from: **test.yml**, to: **json** 인 경우, 출력 파일은 **test.json**이 되고 **json**이 아닙니다.
  - 이 기능이 필요하지 않은 경우 `-t ./json` 또는 `-t $PWD/json`을 사용하십시오. `-t json`이 아닙니다"##),
        ("preview", r##"수정 후 전체 파일 미리보기"##),
        ("set-key", r##"지정된 key(키)를 선택합니다. "."을 구분 기호로 사용합니다."##),
        ("get-opts", r##"지정된 구성 항목의 값(value)을 가져옵니다."##),
        ("saot", r##"하나의 key-value 쌍만 저장하는 테이블 배열 (사용법: `--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"형식: "a.b.c", 하위 테이블 이름 또는 배열 인덱스는 "."(점)으로 연결됩니다.
"." 만 제공하는 경우 전체 테이블을 표시합니다."##),
        ("set-opts", r##"구성 파일의 구조를 만들거나 업데이트하거나 삭제합니다."##),
        ("bool-arr-help", r##"예시:
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: 첫 번째 요소는 참(true)입니다.
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"boolean 배열"##),
        ("table", r##"여러 개의 K-V 쌍을 저장하는 데 사용되는 표준 테이블입니다."##),
        ("datetime", r##"날짜 및 시간 유형"##),
        ("table-help", r##"예시1:
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

예시2:
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"f64 배열"##),
        ("string", r##"String(문자열 유형)"##),
        ("array-help", r##"여러번의 `-a` 호출을 통해 배열 요소를 설정합니다.
예제:
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> 첫 번째 호출은 인덱스 0의 요소를 설정하고 두 번째 호출은 인덱스 1의 요소를 설정합니다.

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"테이블 배열(사용법: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"인라인 테이블, 보다 간결한 표준 테이블"##),
        ("concat-key", r##"키(key)를 분할하여 입력하면 결과가 자동으로 연결됩니다. 구분 기호 "."을 사용하지 않습니다."##),
    ],
}
}

/// Language ID: ko;
/// Map name: "opt_md";
/// Description: 한국어, 한국 문자, 대한민국;
pub(super) const fn get_ko_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m와 달리 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m는 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msplit('.')[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 자동으로 수행하지 않습니다.
[48;2;34;34;34m[38;2;255;255;255m[a.b.c]와 같은 경우, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m와 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m는 동일합니다. 
[48;2;34;34;34m[38;2;255;255;255m간단한 경우에는 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 사용하는 것이 더 간편합니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m키 이름에 '.'이 포함되어 있으면 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 대신 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 사용하세요.
[48;2;34;34;34m[38;2;255;255;255m예: 
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m이 구조에서 "editor.defaultFormatter"는 전체 문자열입니다. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mdefaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m는 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255meditor[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m의 하위 키(sub-key)가 아닙니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 사용하면 키가 다음과 같이 분할됩니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m반면, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m는 다음과 같이 키를 처리합니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m후자가 올바른 구조입니다.[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mi64 배열[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mbool (불리언 유형)으로, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 또는 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m만 사용할 수 있습니다.[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: 64 비트 부호 있는 정수 유형입니다. 예시: 1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255m예시:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m이 옵션은 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 만 지원하며, 명령 끝에 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m을 추가하면 다음과 같이 표시됩니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255m지정된 형식/파일로 저장하며, 한 번에 하나의 파일만 저장할 수 있습니다.[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255m사용 방법 예1：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m사용 방법 예2：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 참고: ron에는 실제로 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 타입이 있으며 값은 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 또는 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m입니다.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 여기서 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m은 실제 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 타입이 아닌 지정된 키를 삭제하는 것을 의미합니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtest.yml에 다음과 같은 표 배열이 있다고 가정해 봅시다:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m우리는 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mname: test[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m를 삭제하려면 다음을 실행할 수 있습니다.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 참고: 빈 문자열과 None은 다릅니다.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m를 호출하면 빈 문자열로 설정됩니다.[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m은 전체 키가 제거됩니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m두 번째 표 (인덱스 1)를 제거하려면 다음을 실행하면 됩니다.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m또는 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m을 사용하여 삭제할 수 있습니다.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255m배정밀도 부동 소수점 숫자 (IEEE 754-2008 표준)로, 예시: 3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 참고: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m을 사용하여 각 테이블에 대해 하나의 K-V pair만 지정할 수 있지만 다른 옵션을 사용하여 테이블 배열 인덱스를 먼저 지정하고 확장할 수 있습니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m예시:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mk-v 쌍(k-v pair)을 설정합니다. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m는 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m와 동일합니다.[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255m현재 파일을 덮어쓰기(overwrite)합니다. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m와 함께 사용되지 않은 경우[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255m문자열 배열[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255m예시:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m지정된 key로 이동한 다음 지정된 데이터 유형의 값을 수정합니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m value가 지정되지 않은 경우, 해당 데이터 유형과 값을 나열합니다.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 예: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m를 사용하면 패키지 및 하위 테이블의 테이블이 나열되며 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m보다 자세한 정보가 제공됩니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExamples[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m예시1：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m이 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 대신 지정되었으므로 "2021"은 정수 타입이 아닌 문자열 타입입니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m new value [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m예시2：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m예시3：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: 이것은 첫 번째 인덱스입니다.*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：위의 예는 매개 변수(argument)/서브 명령(sub-command)로 시작하여 루트 명령 이름을 생략하였습니다. 실제 작업에서는 수동으로 추가해야 합니다.[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m예제:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255m예시:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 이외의 다른 형식에서는 인라인 테이블 대신 표준 테이블을 사용합니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mkey(키)를 사용하여 value(값)를 검색합니다.[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m예제:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m JSON 1.0은 NaN과 Inf를 지원하지 않습니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m이 출력됩니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255m지정된 키를 삭제[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 하위 명령어의 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m와 매우 유사하지만 가장 큰 차이점은 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m가 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 여러 번 연속 호출하여 다양한 형식/여러 파일로 출력할 수 있지만 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 하위 명령어의 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m는 한 번만 호출하고 단일 파일로 출력합니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 참고: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 하위 명령어는 변환 작업을 완료한 후 모든 형식의 파일에서 주석을 유지하지 않습니다.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 반면에 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 하위 명령어는 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m을 제외하고 모든 형식의 주석을 유지합니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 호출하지 않은 상태에서 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 호출하면 자신을 덮어쓰게 됩니다.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 이미 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m를 호출하고 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m도 호출한 경우 수정된 파일은 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m가 지정한 파일에 저장됩니다.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 소스 파일과 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m가 지정한 파일이 동일하지 않은 경우, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 하위 명령어의 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 동작은 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m의 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m와 유사해지며, 구체적인 파일 경로에 대해서만 자동으로 이름을 바꿉니다.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 예를 들어 from: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, to: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 인 경우, 출력 파일은 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m이 되고 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m이 아닙니다.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 이 기능이 필요하지 않은 경우 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 또는 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m을 사용하십시오. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m이 아닙니다[0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255m수정 후 전체 파일 미리보기[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255m지정된 key(키)를 선택합니다. "."을 구분 기호로 사용합니다.[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255m지정된 구성 항목의 값(value)을 가져옵니다.[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255m하나의 key-value 쌍만 저장하는 테이블 배열 (사용법: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255m형식: "a.b.c", 하위 테이블 이름 또는 배열 인덱스는 "."(점)으로 연결됩니다.
[48;2;34;34;34m[38;2;255;255;255m"." 만 제공하는 경우 전체 테이블을 표시합니다.[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255m구성 파일의 구조를 만들거나 업데이트하거나 삭제합니다.[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m예시:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: 첫 번째 요소는 참(true)입니다.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mboolean 배열[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255m여러 개의 K-V 쌍을 저장하는 데 사용되는 표준 테이블입니다.[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255m날짜 및 시간 유형[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255m예시1:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m예시2:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mf64 배열[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString(문자열 유형)[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255m여러번의 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 호출을 통해 배열 요소를 설정합니다.
[48;2;34;34;34m[38;2;255;255;255m예제:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 첫 번째 호출은 인덱스 0의 요소를 설정하고 두 번째 호출은 인덱스 1의 요소를 설정합니다.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255m테이블 배열(사용법: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255m인라인 테이블, 보다 간결한 표준 테이블[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255m키(key)를 분할하여 입력하면 결과가 자동으로 연결됩니다. 구분 기호 "."을 사용하지 않습니다.[0m"##),
    ],
}
}

/// Language ID: ko;
/// Map name: "parser";
/// Description: 한국어, 한국 문자, 대한민국;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "아래는 stderr 출력입니다. 완성 스크립트를 출력으로 리디렉션 할 때 `2>/path/to/file` 대신 `>/path/to/file`을(를) 사용해야 합니다.");
/// ```
pub(super) const fn get_ko_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"아래는 stderr 출력입니다. 완성 스크립트를 출력으로 리디렉션 할 때 `2>/path/to/file` 대신 `>/path/to/file`을(를) 사용해야 합니다."##),
    ],
}
}

/// ko: 한국어, 한국 문자, 대한민국
pub(super) const fn get_ko_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_ko_map_args),
        ("conv_md", get_ko_map_conv_md),
        ("parser", get_ko_map_parser),
        ("opt_md", get_ko_map_opt_md),
        ("args_md", get_ko_map_args_md),
        ("opt", get_ko_map_opt),
        ("log-core", get_ko_map_log_core),
        ("conv", get_ko_map_conv),
        ("theme", get_ko_map_theme),
    ],
}
}

/// Language ID: ku;
/// Map name: "log-core";
/// Description: kurdî, latînî, Tirkiye;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_ku_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"hişyar kirin"##),
        ("info", r##"Agahdariya"##),
        ("error", r##"Erewtî"##),
        ("trace", r##"trace"##),
        ("init-logger", r##"Destpêkirina logger ..."##),
    ],
}
}

/// Language ID: ku;
/// Map name: "theme";
/// Description: kurdî, latînî, Tirkiye;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Navê mijara çewt");
/// ```
pub(super) const fn get_ku_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Navê mijara çewt"##),
    ],
}
}

/// ku: kurdî, latînî, Tirkiye
pub(super) const fn get_ku_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ku_map_log_core),
        ("theme", get_ku_map_theme),
    ],
}
}

/// Language ID: ky;
/// Map name: "log-core";
/// Description: кыргызча, Кирилл, Кыргызстан;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_ky_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"Эскертүү"##),
        ("info", r##"Маалымат"##),
        ("error", r##"Error"##),
        ("trace", r##"изин"##),
        ("init-logger", r##"Loggerди ишке киргизүү ..."##),
    ],
}
}

/// Language ID: ky;
/// Map name: "theme";
/// Description: кыргызча, Кирилл, Кыргызстан;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Жараксыз тема аты");
/// ```
pub(super) const fn get_ky_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Жараксыз тема аты"##),
    ],
}
}

/// ky: кыргызча, Кирилл, Кыргызстан
pub(super) const fn get_ky_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ky_map_log_core),
        ("theme", get_ky_map_theme),
    ],
}
}

/// Language ID: la;
/// Map name: "log-core";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_la_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"monere"##),
        ("info", r##"Info"##),
        ("error", r##"Error"##),
        ("trace", r##"vestigium"##),
        ("init-logger", r##"Initializing Logger ..."##),
    ],
}
}

/// Language ID: la;
/// Map name: "theme";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Aliquam Nomen Nomen");
/// ```
pub(super) const fn get_la_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Aliquam Nomen Nomen"##),
    ],
}
}

/// la: la-Latn-VA
pub(super) const fn get_la_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_la_map_log_core),
        ("theme", get_la_map_theme),
    ],
}
}

/// Language ID: lb;
/// Map name: "log-core";
/// Description: Lëtzebuergesch, Laténgesch, Lëtzebuerg;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_lb_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"warnen"##),
        ("info", r##"Info"##),
        ("error", r##"Feeler"##),
        ("trace", r##"Spur"##),
        ("init-logger", r##"Initialiséiere Logger ..."##),
    ],
}
}

/// Language ID: lb;
/// Map name: "theme";
/// Description: Lëtzebuergesch, Laténgesch, Lëtzebuerg;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ongëltegen Thema Numm");
/// ```
pub(super) const fn get_lb_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ongëltegen Thema Numm"##),
    ],
}
}

/// lb: Lëtzebuergesch, Laténgesch, Lëtzebuerg
pub(super) const fn get_lb_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_lb_map_log_core),
        ("theme", get_lb_map_theme),
    ],
}
}

/// Language ID: lo;
/// Map name: "log-core";
/// Description: ລາວ, ລາວ, ລາວ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_lo_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"ເຕືອນ"##),
        ("info", r##"ຂໍ້ມູນ"##),
        ("error", r##"ຜິດພາດ"##),
        ("trace", r##"ຮ່ອງຮອຍ"##),
        ("init-logger", r##"ເລີ່ມຕົ້ນຜູ້ຕັດໄມ້ ..."##),
    ],
}
}

/// Language ID: lo;
/// Map name: "theme";
/// Description: ລາວ, ລາວ, ລາວ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "ຊ\u{eb7}\u{ec8}ຫ\u{ebb}ວຂ\u{ecd}\u{ec9}ທ\u{eb5}\u{ec8}ບ\u{ecd}\u{ec8}ຖ\u{eb7}ກຕ\u{ec9}ອງ");
/// ```
pub(super) const fn get_lo_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"ຊື່ຫົວຂໍ້ທີ່ບໍ່ຖືກຕ້ອງ"##),
    ],
}
}

/// lo: ລາວ, ລາວ, ລາວ
pub(super) const fn get_lo_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_lo_map_log_core),
        ("theme", get_lo_map_theme),
    ],
}
}

/// Language ID: lt;
/// Map name: "log-core";
/// Description: lietuvių, lotynų, Lietuva;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Derintis");
/// ```
pub(super) const fn get_lt_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Derintis"##),
        ("warn", r##"perspėti"##),
        ("info", r##"Informacija"##),
        ("error", r##"Klaida"##),
        ("trace", r##"pėdsakas"##),
        ("init-logger", r##"Inicijuoti kaupiklį ..."##),
    ],
}
}

/// Language ID: lt;
/// Map name: "theme";
/// Description: lietuvių, lotynų, Lietuva;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Neteisingas temos pavadinimas");
/// ```
pub(super) const fn get_lt_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Neteisingas temos pavadinimas"##),
    ],
}
}

/// lt: lietuvių, lotynų, Lietuva
pub(super) const fn get_lt_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_lt_map_log_core),
        ("theme", get_lt_map_theme),
    ],
}
}

/// Language ID: lv;
/// Map name: "log-core";
/// Description: latviešu, latīņu, Latvija;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "atkļūdošana");
/// ```
pub(super) const fn get_lv_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"atkļūdošana"##),
        ("warn", r##"brīdināt"##),
        ("info", r##"Informācija"##),
        ("error", r##"Kļūda"##),
        ("trace", r##"izsekot"##),
        ("init-logger", r##"Inicializējot mežizstrādātāju ..."##),
    ],
}
}

/// Language ID: lv;
/// Map name: "theme";
/// Description: latviešu, latīņu, Latvija;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nederīgs tēmas nosaukums");
/// ```
pub(super) const fn get_lv_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nederīgs tēmas nosaukums"##),
    ],
}
}

/// lv: latviešu, latīņu, Latvija
pub(super) const fn get_lv_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_lv_map_log_core),
        ("theme", get_lv_map_theme),
    ],
}
}

/// Language ID: mg;
/// Map name: "log-core";
/// Description: Malagasy, Latn, Madagasikara;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_mg_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"mampitandrina"##),
        ("info", r##"Info"##),
        ("error", r##"Hadisoana"##),
        ("trace", r##"trace"##),
        ("init-logger", r##"Initialing Logger ..."##),
    ],
}
}

/// Language ID: mg;
/// Map name: "theme";
/// Description: Malagasy, Latn, Madagasikara;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Anaran'ny lohahevitra tsy mety");
/// ```
pub(super) const fn get_mg_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Anaran'ny lohahevitra tsy mety"##),
    ],
}
}

/// mg: Malagasy, Latn, Madagasikara
pub(super) const fn get_mg_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_mg_map_log_core),
        ("theme", get_mg_map_theme),
    ],
}
}

/// Language ID: mi;
/// Map name: "log-core";
/// Description: Māori, Rātina, Aotearoa;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_mi_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"Warn"##),
        ("info", r##"Korero"##),
        ("error", r##"Hapa"##),
        ("trace", r##"Wāhi hūkahu"##),
        ("init-logger", r##"Te kōkiri i te kaitirotiro ..."##),
    ],
}
}

/// Language ID: mi;
/// Map name: "theme";
/// Description: Māori, Rātina, Aotearoa;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ingoa Kaupapa Kore");
/// ```
pub(super) const fn get_mi_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ingoa Kaupapa Kore"##),
    ],
}
}

/// mi: Māori, Rātina, Aotearoa
pub(super) const fn get_mi_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_mi_map_log_core),
        ("theme", get_mi_map_theme),
    ],
}
}

/// Language ID: mk;
/// Map name: "log-core";
/// Description: македонски, кирилско писмо, Северна Македонија;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Дебагирање");
/// ```
pub(super) const fn get_mk_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Дебагирање"##),
        ("warn", r##"предупредува"##),
        ("info", r##"информации"##),
        ("error", r##"грешка"##),
        ("trace", r##"трага"##),
        ("init-logger", r##"Иницијализирање на дрвосечач ..."##),
    ],
}
}

/// Language ID: mk;
/// Map name: "theme";
/// Description: македонски, кирилско писмо, Северна Македонија;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Невалидно име на темата");
/// ```
pub(super) const fn get_mk_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Невалидно име на темата"##),
    ],
}
}

/// mk: македонски, кирилско писмо, Северна Македонија
pub(super) const fn get_mk_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_mk_map_log_core),
        ("theme", get_mk_map_theme),
    ],
}
}

/// Language ID: ml;
/// Map name: "log-core";
/// Description: മലയാളം, മലയാളം, ഇന്ത്യ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ഡീബഗ\u{d4d}");
/// ```
pub(super) const fn get_ml_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ഡീബഗ്"##),
        ("warn", r##"മുന്നറിയിപ്പ്"##),
        ("info", r##"വിവരം"##),
        ("error", r##"പിശക്"##),
        ("trace", r##"ട്രേസ്"##),
        ("init-logger", r##"ലോഗർ സമാരംഭിക്കുന്നു ..."##),
    ],
}
}

/// Language ID: ml;
/// Map name: "theme";
/// Description: മലയാളം, മലയാളം, ഇന്ത്യ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "തീം ന\u{d3e}മം അസ\u{d3e}ധ\u{d41}വ\u{d3e}ണ\u{d4d}");
/// ```
pub(super) const fn get_ml_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"തീം നാമം അസാധുവാണ്"##),
    ],
}
}

/// ml: മലയാളം, മലയാളം, ഇന്ത്യ
pub(super) const fn get_ml_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ml_map_log_core),
        ("theme", get_ml_map_theme),
    ],
}
}

/// Language ID: mn;
/// Map name: "log-core";
/// Description: монгол, кирилл, Монгол;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "дебаг");
/// ```
pub(super) const fn get_mn_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"дебаг"##),
        ("warn", r##"сэрэмжлүүлэг"##),
        ("info", r##"мэдээлэл"##),
        ("error", r##"алдаа"##),
        ("trace", r##"ул мөр"##),
        ("init-logger", r##"Бүртгэлийг эхлүүлэх ..."##),
    ],
}
}

/// Language ID: mn;
/// Map name: "theme";
/// Description: монгол, кирилл, Монгол;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Буруу сэдэв нэр");
/// ```
pub(super) const fn get_mn_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Буруу сэдэв нэр"##),
    ],
}
}

/// mn: монгол, кирилл, Монгол
pub(super) const fn get_mn_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_mn_map_log_core),
        ("theme", get_mn_map_theme),
    ],
}
}

/// Language ID: mr;
/// Map name: "log-core";
/// Description: मराठी, देवनागरी, भारत;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "डीबग");
/// ```
pub(super) const fn get_mr_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"डीबग"##),
        ("warn", r##"चेतावणी द्या"##),
        ("info", r##"माहिती"##),
        ("error", r##"त्रुटी"##),
        ("trace", r##"ट्रेस"##),
        ("init-logger", r##"लॉगर आरंभ करीत आहे ..."##),
    ],
}
}

/// Language ID: mr;
/// Map name: "theme";
/// Description: मराठी, देवनागरी, भारत;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "अव\u{948}ध थीम नाव");
/// ```
pub(super) const fn get_mr_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"अवैध थीम नाव"##),
    ],
}
}

/// mr: मराठी, देवनागरी, भारत
pub(super) const fn get_mr_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_mr_map_log_core),
        ("theme", get_mr_map_theme),
    ],
}
}

/// Language ID: ms;
/// Map name: "log-core";
/// Description: Melayu, Latin, Malaysia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_ms_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"memberi amaran"##),
        ("info", r##"Maklumat"##),
        ("error", r##"ralat"##),
        ("trace", r##"jejak"##),
        ("init-logger", r##"Memulakan logger ..."##),
    ],
}
}

/// Language ID: ms;
/// Map name: "theme";
/// Description: Melayu, Latin, Malaysia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nama tema tidak sah");
/// ```
pub(super) const fn get_ms_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nama tema tidak sah"##),
    ],
}
}

/// ms: Melayu, Latin, Malaysia
pub(super) const fn get_ms_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ms_map_log_core),
        ("theme", get_ms_map_theme),
    ],
}
}

/// Language ID: mt;
/// Map name: "log-core";
/// Description: Malti, Latin, Malta;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_mt_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"twissi"##),
        ("info", r##"Info"##),
        ("error", r##"Żball"##),
        ("trace", r##"Traċċa"##),
        ("init-logger", r##"Inizjalizza l-logger ..."##),
    ],
}
}

/// Language ID: mt;
/// Map name: "theme";
/// Description: Malti, Latin, Malta;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Isem tat-tema invalidu");
/// ```
pub(super) const fn get_mt_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Isem tat-tema invalidu"##),
    ],
}
}

/// mt: Malti, Latin, Malta
pub(super) const fn get_mt_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_mt_map_log_core),
        ("theme", get_mt_map_theme),
    ],
}
}

/// Language ID: my;
/// Map name: "log-core";
/// Description: မြန်မာ, မြန်မာ, မြန်မာ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_my_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"သတိပေး"##),
        ("info", r##"အချက်အလက်"##),
        ("error", r##"အမှား"##),
        ("trace", r##"သဲလွန်စ"##),
        ("init-logger", r##"logger ကိုစတင်ရန် ..."##),
    ],
}
}

/// Language ID: my;
/// Map name: "theme";
/// Description: မြန်မာ, မြန်မာ, မြန်မာ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "မမ\u{103e}န\u{103a}ကန\u{103a}သောအကြောင\u{103a}းအရာအမည\u{103a}");
/// ```
pub(super) const fn get_my_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"မမှန်ကန်သောအကြောင်းအရာအမည်"##),
    ],
}
}

/// my: မြန်မာ, မြန်မာ, မြန်မာ
pub(super) const fn get_my_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_my_map_log_core),
        ("theme", get_my_map_theme),
    ],
}
}

/// Language ID: ne;
/// Map name: "log-core";
/// Description: नेपाली, देवानागरी, नेपाल;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "डिबग");
/// ```
pub(super) const fn get_ne_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"डिबग"##),
        ("warn", r##"चेतावनी"##),
        ("info", r##"जानकारी"##),
        ("error", r##"त्रुटि"##),
        ("trace", r##"ट्रेस"##),
        ("init-logger", r##"Logger आरम्भिक"##),
    ],
}
}

/// Language ID: ne;
/// Map name: "theme";
/// Description: नेपाली, देवानागरी, नेपाल;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "अव\u{948}ध विषय नाम");
/// ```
pub(super) const fn get_ne_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"अवैध विषय नाम"##),
    ],
}
}

/// ne: नेपाली, देवानागरी, नेपाल
pub(super) const fn get_ne_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ne_map_log_core),
        ("theme", get_ne_map_theme),
    ],
}
}

/// Language ID: nl;
/// Map name: "log-core";
/// Description: Nederlands, Latijns, Nederland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_nl_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"waarschuwen"##),
        ("info", r##"info"##),
        ("error", r##"Fout"##),
        ("trace", r##"Trace"##),
        ("init-logger", r##"Logger initialiseren ..."##),
    ],
}
}

/// Language ID: nl;
/// Map name: "theme";
/// Description: Nederlands, Latijns, Nederland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ongeldige themamaam");
/// ```
pub(super) const fn get_nl_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ongeldige themamaam"##),
    ],
}
}

/// Language ID: nl;
/// Map name: "args";
/// Description: Nederlands, Latijns, Nederland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Schakel thema achtergrond uit (default is ingeschakeld, als `--dis-theme-bg` wordt gebruikt, dan uitgeschakeld)");
/// ```
pub(super) const fn get_nl_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"Aangepaste themaset-bestand, moet worden gebruikt samen met `--theme`-optie"##),
        ("shell-completion", r##"Genereer autocompletie-scripts voor de shell"##),
        ("theme-file-help", r##"Het ondersteunde bestandsformaat is binaire gegevens die zijn gedumpt door rust crate `syntect`.
Deze thema's zijn in wezen thema's voor de oude versie van sublime.
Als er geen `--tf` is opgegeven, wordt er gelezen uit de `$TOMLYRE_THEME_FILE`-omgevingsvariabele. "##),
        ("from-format", r##"Specificeer het formaat van het bronbestand (standaard zal dit automatisch worden gedetecteerd)"##),
        ("disable-theme-background", r##"Schakel thema achtergrond uit (default is ingeschakeld, als `--dis-theme-bg` wordt gebruikt, dan uitgeschakeld)"##),
        ("theme-uncompressed-file-help", r##"> Ongecomprimeerde versies hebben hogere prioriteit dan gecomprimeerde versies.

Als er geen `--tuf` is opgegeven, wordt er gelezen uit de `$TOMLYRE_THEME_UNCOMPRESSED_FILE`-omgevingsvariabele. "##),
        ("disable-theme-background-help", r##"Als er geen `--dis-theme-bg` is gespecificeerd, wordt er gelezen uit de `$TOMLYRE_THEME_BG`-omgevingsvariabele.

Als deze variabele onwaar is, wordt het uitgeschakeld

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"Vergelijkbaar met `theme-file`, maar ongecomprimeerde dumpgegevens (bestand) vereist"##),
        ("shell-completion-help", r##"U moet de naam van de shell opgeven om autocompletie-scripts te genereren.
Bijvoorbeeld, voor PowerShell op Windows, kunt u `--sh-comp pwsh` gebruiken.

> V: Wat is de functie van autocompletie in de shell?

> A: Wanneer we op de opdrachtregel typen, als we op de Tab-toets drukken, wordt het autocompletiescript gestart om mogelijke voltooiingsopties te zoeken en deze opties weer te geven om uit te kiezen. Deze functie kan de efficiëntie en nauwkeurigheid van het invoeren van opdrachtregels sterk verbeteren.

Autocompletie-scripts voor verschillende shells moeten worden opgeslagen in verschillende locaties. Voor zsh moet u het opslaan in een pad dat is opgenomen in de `$fpath` array.
U kunt `print -l $fpath` gebruiken om de inhoud van deze array te verkrijgen.
Als bijvoorbeeld de map `/usr/local/share/zsh/site-functions` hierin is opgenomen, kunt u het volgende commando gebruiken om het autocompletiescript naar het bestand `_tomlyre` in die map te schrijven.

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

Om root-toegang te vermijden, kunt u het opslaan in de submap van de gebruiker. Bijvoorbeeld, u kunt het volgende commando gebruiken om een element aan het begin van deze array toe te voegen en vervolgens de relevante directory te gebruiken.
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"Wanneer gegevens van stdin worden gelezen, bijvoorbeeld `cat a | tomlyre conv -`, wordt eerst geprobeerd om het bestandsformaat automatisch te detecteren.
Automatische detectie kan echter mislukken, waardoor handmatige specificatie nodig is.
Voorbeeld: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Wanneer alleen de parameter `--ts` wordt gebruikt, wordt er een lijst weergegeven met de ingebouwde set stijlen.
Behalve markdown die kan worden afgekort tot md, moeten andere stijlnamen volledig worden geschreven.
Voorbeeld 1:`--ts md`
Voorbeeld 2:`--table-style ascii`

Opmerking: Het is momenteel niet mogelijk om de set **tabelstijlen** uit een bestand te lezen. Als u deze functie of andere extra functies nodig heeft, kunt u een issue indienen om feedback te geven."##),
        ("table-style", r##"Specificeer de stijl van de tabel"##),
        ("theme-help", r##"Wanneer alleen de parameter `-t` wordt gebruikt, wordt er een lijst weergegeven met de huidige thema's.
Als er geen themabestand is opgegeven, wordt de ingebouwde set thema's gebruikt.
Als er bijvoorbeeld een thema genaamd **ayu-dark** bestaat, kunt u `-t ayu-dark` gebruiken om dit thema te gebruiken.
Als de naam van het thema spaties bevat, moet u het tussen dubbele of enkele aanhalingstekens zetten.

> Opmerking: er is een speciaal thema genaamd `None`, dat staat voor de pure tekststijl zonder syntax highlighting.

Als de optie `--theme` niet is gespecificeerd, wordt er gelezen uit de `$TOMLYRE_THEME`-omgevingsvariabele.
Het thema voor syntax highlighting in deze help-informatie komt ook van deze omgevingsvariabele.

> De standaard help-informatie is hard gecodeerd en laadt zeer snel.
> Het laden van de hardgecodeerde tekst en het laden van de tekst zonder syntax highlighting (via `None`-thema) is vergelijkbaar qua snelheid, terwijl dynamisch laden veel trager zal zijn.
> Het instellen van de `$TOMLYRE_THEME`-omgevingsvariabele kan leiden tot herladen van het thema, wat de opstarttijd **sterk** zal vertragen.
> Gebruik de `-t`-parameter in plaats van het wijzigen van de omgevingsvariabele, tenzij u van plan bent het thema voor de help-informatie te wijzigen of syntax highlighting uit te schakelen.

Voorbeeld van het gebruik van een omgevingsvariabele:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Voorbeeld van het gebruik van de opdrachtregelparameters:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Voor handmatige bewerkingen zoals "conv/get/set" en gerelateerde, heeft het opgegeven thema met `-t` prioriteit boven de omgevingsvariabele.
Als er geen `-t` is opgegeven, wordt de omgevingsvariabele gelezen. Als deze variabele leeg is, wordt **Monokai Extended** gebruikt."##),
        ("theme", r##"Specificeer de naam van het thema voor syntax highlighting"##),
    ],
}
}

/// Language ID: nl;
/// Map name: "args_md";
/// Description: Nederlands, Latijns, Nederland;
pub(super) const fn get_nl_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mAangepaste themaset-bestand, moet worden gebruikt samen met [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m-optie[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mGenereer autocompletie-scripts voor de shell[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mHet ondersteunde bestandsformaat is binaire gegevens die zijn gedumpt door rust crate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mDeze thema's zijn in wezen thema's voor de oude versie van sublime.
[48;2;34;34;34m[38;2;255;255;255mAls er geen [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is opgegeven, wordt er gelezen uit de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m-omgevingsvariabele. [0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mSpecificeer het formaat van het bronbestand (standaard zal dit automatisch worden gedetecteerd)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mSchakel thema achtergrond uit (default is ingeschakeld, als [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m wordt gebruikt, dan uitgeschakeld)[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Ongecomprimeerde versies hebben hogere prioriteit dan gecomprimeerde versies.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAls er geen [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is opgegeven, wordt er gelezen uit de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m-omgevingsvariabele. [0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mAls er geen [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is gespecificeerd, wordt er gelezen uit de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m-omgevingsvariabele.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAls deze variabele onwaar is, wordt het uitgeschakeld
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mVergelijkbaar met [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, maar ongecomprimeerde dumpgegevens (bestand) vereist[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mU moet de naam van de shell opgeven om autocompletie-scripts te genereren.
[48;2;34;34;34m[38;2;255;255;255mBijvoorbeeld, voor PowerShell op Windows, kunt u [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m gebruiken.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m V: Wat is de functie van autocompletie in de shell?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: Wanneer we op de opdrachtregel typen, als we op de Tab-toets drukken, wordt het autocompletiescript gestart om mogelijke voltooiingsopties te zoeken en deze opties weer te geven om uit te kiezen. Deze functie kan de efficiëntie en nauwkeurigheid van het invoeren van opdrachtregels sterk verbeteren.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAutocompletie-scripts voor verschillende shells moeten worden opgeslagen in verschillende locaties. Voor zsh moet u het opslaan in een pad dat is opgenomen in de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m array.
[48;2;34;34;34m[38;2;255;255;255mU kunt [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m gebruiken om de inhoud van deze array te verkrijgen.
[48;2;34;34;34m[38;2;255;255;255mAls bijvoorbeeld de map [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m hierin is opgenomen, kunt u het volgende commando gebruiken om het autocompletiescript naar het bestand [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m in die map te schrijven.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOm root-toegang te vermijden, kunt u het opslaan in de submap van de gebruiker. Bijvoorbeeld, u kunt het volgende commando gebruiken om een element aan het begin van deze array toe te voegen en vervolgens de relevante directory te gebruiken.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mWanneer gegevens van stdin worden gelezen, bijvoorbeeld [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, wordt eerst geprobeerd om het bestandsformaat automatisch te detecteren.
[48;2;34;34;34m[38;2;255;255;255mAutomatische detectie kan echter mislukken, waardoor handmatige specificatie nodig is.
[48;2;34;34;34m[38;2;255;255;255mVoorbeeld: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mWanneer alleen de parameter [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m wordt gebruikt, wordt er een lijst weergegeven met de ingebouwde set stijlen.
[48;2;34;34;34m[38;2;255;255;255mBehalve markdown die kan worden afgekort tot md, moeten andere stijlnamen volledig worden geschreven.
[48;2;34;34;34m[38;2;255;255;255mVoorbeeld 1:[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mVoorbeeld 2:[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOpmerking: Het is momenteel niet mogelijk om de set [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtabelstijlen[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m uit een bestand te lezen. Als u deze functie of andere extra functies nodig heeft, kunt u een issue indienen om feedback te geven.[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mSpecificeer de stijl van de tabel[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mWanneer alleen de parameter [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m wordt gebruikt, wordt er een lijst weergegeven met de huidige thema's.
[48;2;34;34;34m[38;2;255;255;255mAls er geen themabestand is opgegeven, wordt de ingebouwde set thema's gebruikt.
[48;2;34;34;34m[38;2;255;255;255mAls er bijvoorbeeld een thema genaamd [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m bestaat, kunt u [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m gebruiken om dit thema te gebruiken.
[48;2;34;34;34m[38;2;255;255;255mAls de naam van het thema spaties bevat, moet u het tussen dubbele of enkele aanhalingstekens zetten.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Opmerking: er is een speciaal thema genaamd [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, dat staat voor de pure tekststijl zonder syntax highlighting.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAls de optie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m niet is gespecificeerd, wordt er gelezen uit de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m-omgevingsvariabele.
[48;2;34;34;34m[38;2;255;255;255mHet thema voor syntax highlighting in deze help-informatie komt ook van deze omgevingsvariabele.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m De standaard help-informatie is hard gecodeerd en laadt zeer snel.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Het laden van de hardgecodeerde tekst en het laden van de tekst zonder syntax highlighting (via [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m-thema) is vergelijkbaar qua snelheid, terwijl dynamisch laden veel trager zal zijn.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Het instellen van de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m-omgevingsvariabele kan leiden tot herladen van het thema, wat de opstarttijd [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114msterk[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m zal vertragen.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Gebruik de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m-parameter in plaats van het wijzigen van de omgevingsvariabele, tenzij u van plan bent het thema voor de help-informatie te wijzigen of syntax highlighting uit te schakelen.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mVoorbeeld van het gebruik van een omgevingsvariabele:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mVoorbeeld van het gebruik van de opdrachtregelparameters:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mVoor handmatige bewerkingen zoals "conv/get/set" en gerelateerde, heeft het opgegeven thema met [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m prioriteit boven de omgevingsvariabele.
[48;2;34;34;34m[38;2;255;255;255mAls er geen [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is opgegeven, wordt de omgevingsvariabele gelezen. Als deze variabele leeg is, wordt [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m gebruikt.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mSpecificeer de naam van het thema voor syntax highlighting[0m"##),
    ],
}
}

/// Language ID: nl;
/// Map name: "conv";
/// Description: Nederlands, Latijns, Nederland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Tussen formaten zoals toml, yaml, json en ron converteren");
/// ```
pub(super) const fn get_nl_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Tussen formaten zoals toml, yaml, json en ron converteren"##),
        ("from", r##"Geef het invoerbestand op"##),
        ("conv-to", r##"Geef het uitvoerformaat/bestand op; meerdere aanroepen worden ondersteund"##),
        ("save", r##"Als deze optie wordt gebruikt, wordt het bestand dat wordt gespecificeerd door `--to` opgeslagen"##),
        ("conv-to-help", r##"Als er bijvoorbeeld een bestand **test.json** in de huidige map staat, kunt u `conv test.json -t yml` bellen om het te converteren naar yaml (yml),
of u kunt handmatig het bestandspad opgeven: `conv test.json -t /tmp/test.yml`

Ondersteunde indelingen zijn afhankelijk van de functies die zijn ingeschakeld voor dit softwarepakket, niet alle functies zijn standaard ingeschakeld.

Bovendien zijn er enkele dingen die u moet weten voordat u deze optie gebruikt.
- 1. Het zal automatisch hernoemen voor niet-specifieke bestandspaden.
  - 1.1: Enkelvoudige bestanden worden als bestandsindeling beschouwd in plaats van een bestandspad
    Wanneer u `conv test.toml -t ron` oproept, wordt het uitvoerbestand automatisch `test.ron` genoemd in plaats van `ron`.
    Als u deze functionaliteit niet nodig heeft, gebruik dan `-t ./ron` of `-t $PWD/ron`.
  - 1.2: Verwijst naar een directory in plaats van een bestand
    Bijv. `-t /tmp/`, we kunnen `test.toml` alleen in de submap van `/tmp/` plaatsen, niet in `/tmp` zelf overschrijven.
    Als u deze functionaliteit niet nodig heeft, gebruik dan het volledige (niet-directory) bestandspad, bv. `-t /tmp/xx.toml` in plaats van `-t /tmp/`.
- 2. Wanneer `--save` niet wordt aangeroepen, wordt het opgegeven bestand niet automatisch opgeslagen.
- 3. Bij het aanroepen van `--save`, kan de invoer van de conv-subopdracht niet hetzelfde zijn als de uitvoer.
- 4. Het geconverteerde bestand behoudt geen opmerkingen."##),
    ],
}
}

/// Language ID: nl;
/// Map name: "conv_md";
/// Description: Nederlands, Latijns, Nederland;
pub(super) const fn get_nl_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mTussen formaten zoals toml, yaml, json en ron converteren[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mGeef het invoerbestand op[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mGeef het uitvoerformaat/bestand op; meerdere aanroepen worden ondersteund[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mAls deze optie wordt gebruikt, wordt het bestand dat wordt gespecificeerd door [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m opgeslagen[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mAls er bijvoorbeeld een bestand [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m in de huidige map staat, kunt u [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m bellen om het te converteren naar yaml (yml),
[48;2;34;34;34m[38;2;255;255;255mof u kunt handmatig het bestandspad opgeven: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOndersteunde indelingen zijn afhankelijk van de functies die zijn ingeschakeld voor dit softwarepakket, niet alle functies zijn standaard ingeschakeld.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mBovendien zijn er enkele dingen die u moet weten voordat u deze optie gebruikt.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Het zal automatisch hernoemen voor niet-specifieke bestandspaden.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Enkelvoudige bestanden worden als bestandsindeling beschouwd in plaats van een bestandspad
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mWanneer u [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m oproept, wordt het uitvoerbestand automatisch [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m genoemd in plaats van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mAls u deze functionaliteit niet nodig heeft, gebruik dan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: Verwijst naar een directory in plaats van een bestand
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mBijv. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, we kunnen [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m alleen in de submap van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m plaatsen, niet in [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zelf overschrijven.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mAls u deze functionaliteit niet nodig heeft, gebruik dan het volledige (niet-directory) bestandspad, bv. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m in plaats van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. Wanneer [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m niet wordt aangeroepen, wordt het opgegeven bestand niet automatisch opgeslagen.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. Bij het aanroepen van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, kan de invoer van de conv-subopdracht niet hetzelfde zijn als de uitvoer.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. Het geconverteerde bestand behoudt geen opmerkingen.[0m"##),
    ],
}
}

/// Language ID: nl;
/// Map name: "opt";
/// Description: Nederlands, Latijns, Nederland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "Tabel-array (gebruik: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_nl_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"In tegenstelling tot `-k`, split `--ck` geen strings met '.'
Voor [a.b.c], `-k a.b.c` en `--ck a --ck b --ck c` zijn hetzelfde.
Voor eenvoudige gevallen zal `-k` handiger zijn.

Als de sleutel '.' bevat, gebruik dan `--ck` in plaats van `-k`.
Bijvoorbeeld:

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

In deze structuur is "editor.defaultFormatter" een complete string, en is `defaultFormatter` geen sub-sleutel van `editor`.

Wanneer u `-k '[typescript]'.'editor.defaultFormatter'` gebruikt, splitst het de sleutel in:

- "[typescript]"
- "editor"
- "defaultFormatter"

Terwijl `--ck '[typescript]' --ck editor.defaultFormatter` de sleutel beschouwt als:

- "[typescript]"
- "editor.defaultFormatter"

Dit laatste is de juiste structuur."##),
        ("num-arr", r##"Array van i64's"##),
        ("bool", r##"Bool (booleaanse type), kan alleen `true` of `false` zijn"##),
        ("num", r##"i64: 64-bit signed integer type, bijv. 1145"##),
        ("datetime-help", r##"Voorbeeld:
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

Deze optie is alleen beschikbaar voor `toml`. Als we `-t json` aan het einde van het commando toevoegen, wordt het als volgt weergegeven:

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"Opslaan in het opgegeven formaat / bestand, slechts één keer per keer"##),
        ("get-opts-help", r##"Voorbeeld 1: `get Cargo.toml -k package.name`
Voorbeeld 2: `get cfg.yaml -k .`
"##),
        ("none-help", r##"Opmerking: Ron heeft echt het type `Option<T>` dat kan worden `Some(T)` of `None`.
Hier verwijst `--none` naar het verwijderen van de opgegeven sleutel (key), niet het echte type `Option<T>`.

Stel dat we de volgende tabel-array hebben in `test.yml`:

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

Als we bijvoorbeeld **name: test** willen verwijderen, kunnen we dit doen met het volgende commando:
```sh
set test.yml -k bin.1.name --none
```

Opmerking: Een lege tekenreeks en `None` zijn niet hetzelfde.
Het gebruik van `set test.yml --kv bin.1.name ""` zal het instellen op een lege tekenreeks, terwijl `--none` de hele sleutel zal verwijderen.

Als we de tweede tabel (index 1) van `bin` willen verwijderen, kunnen we het volgende commando gebruiken:
```sh
set test.yml -k bin.1 --none
```

Of we kunnen `--rm` gebruiken:
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"Double-precision floating-point number (IEEE 754-2008 standard), bijv. 3.14159265359"##),
        ("saot-help", r##"Opmerking: Hoewel u met `--saot` slechts één K-V-paar per tabel kunt maken, kunt u eerst de index van de tabelarray opgeven en vervolgens andere opties gebruiken om uit te breiden.

Voorbeeld:
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"Stel de k-v pair in, `--kv x.y.z value` staat gelijk aan `-k x.y.z -s value`"##),
        ("set-save", r##"Oproepen zonder `--to`, overschrijft het huidige bestand als deze opgeroepen wordt"##),
        ("array", r##"String array"##),
        ("aot-help", r##"Voorbeeld:
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, version, 0.2.84, version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

Selecteer eerst met de opgegeven sleutel en wijzig vervolgens met de opgegeven datatype-waarde.

> Wanneer alleen de sleutel wordt verstrekt en geen waarde, worden de respectieve datatypen en waarden weergegeven.
> Bijvoorbeeld `set Cargo.toml -k package`, toont de tabel voor package en de onderliggende tabellen, en biedt meer gedetailleerde informatie dan `get`.

## Voorbeelden

### Voorbeeld 1：
```sh
--ts md set test.toml --key package.edition --str 2021
```

Omdat `--str` in plaats van `--num` is gespecificeerd, is "2021" een string-type, geen integer-type.

| package | nieuwe waarde |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### Voorbeeld 2：
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### Voorbeeld 3：
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: Dit is de eerste index*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> Opmerking: Deze voorbeelden beginnen met een argument of subopdracht en laten de naam van de hoofdopdracht weg. In werkelijke bewerkingen moet u deze handmatig aanvullen."##),
        ("num-arr-help", r##"Voorbeeld:
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"Voorbeeld:
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

Opmerking: Behalve in `toml` zullen andere formaten de standaardtabel gebruiken in plaats van de inline-tabel.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"Zoek de waarde op met behulp van de sleutel"##),
        ("f64-arr-help", r##"Voorbeeld:
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> JSON 1.0 ondersteunt geen NaN en Inf, deze produceert `null` in plaats van `NaN`.

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"Verwijder de opgegeven sleutel (key)"##),
        ("set-save-to-help", r##"Lijkt erg op de `-t` van de `conv` sub-opdracht, maar het grootste verschil is dat `-t` van `conv` meerdere keren achter elkaar kan worden opgeroepen om naar verschillende formaten / bestanden te converteren.
De `-t` van de `set` sub-opdracht kan slechts één keer worden aangeroepen en output naar een enkel bestand.

> Opmerking: Na het uitvoeren van de conversie-operatie verwijdert `conv` alle opmerkingen in alle bestandsindelingen.
> Terwijl de `set` sub-opdracht de **toml** opmerkingen behoudt, alle andere indelingen behouden geen opmerkingen.

- Als `--save` wordt aangeroepen zonder `-t`, wordt het huidige bestand overschreven.
- Als `-t` is aangeroepen en `--save` ook, wordt het gewijzigde bestand opgeslagen in het bestand waarnaar `-t` verwijst.
- Tenzij het bronbestand overeenkomt met het bestand waarnaar `-t` verwijst, zal het gedrag van `-t` van de `set` sub-opdracht vergelijkbaar zijn met dat van `-t` van de `conv` sub-opdracht, waarbij beide automatisch niet-schrijfbare bestanden hernoemen.
  - Stel bijvoorbeeld dat "from" is **test.yml** en "to" is **json**, dan wordt het uitvoerbestand **test.json** in plaats van **json**.
  - Als u deze functie niet nodig heeft, gebruik dan `-t ./json` of `-t $PWD/json`, in plaats van `-t json`. "##),
        ("preview", r##"Bekijk het volledige bestand na de wijzigingen vooraf"##),
        ("set-key", r##"Selecteer de opgegeven sleutel, gebruik "." als scheidingsteken"##),
        ("get-opts", r##"Haal de waarde op van de opgegeven configuratie"##),
        ("saot", r##"Tabel-array met één key-value-paar (gebruik: `--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"De notatie is als volgt: "a.b.c", gebruik de "." (dot) om subtabellen of array-indexen te verbinden.
Als er maar één "." is, wordt de hele tabel weergegeven."##),
        ("set-opts", r##"Maak / update / verwijder de structuur van het configuratiebestand"##),
        ("bool-arr-help", r##"Voorbeeld:
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: Het eerste element is waar
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"Array van bool's"##),
        ("table", r##"Standaardtabel voor het opslaan van meerdere K-V-paren"##),
        ("datetime", r##"Type voor datum en tijd"##),
        ("table-help", r##"Voorbeeld 1:
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

Voorbeeld 2:
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"Array van f64's"##),
        ("string", r##"String (string-type)"##),
        ("array-help", r##"Gebruik `-a` meerdere keren om array-elementen in te stellen.
Voorbeeld:
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> Stel het element van index 0 in tijdens de eerste aanroep, stel het element van index 1 in tijdens de tweede oproep, enzovoort.

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"Tabel-array (gebruik: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"Inline-tabel, vergelijkbaar met een compactere standaardtabel"##),
        ("concat-key", r##"Voer de sleutel in in segmenten, de resultaten worden automatisch samengevoegd zonder "." als scheidingsteken"##),
    ],
}
}

/// Language ID: nl;
/// Map name: "opt_md";
/// Description: Nederlands, Latijns, Nederland;
pub(super) const fn get_nl_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255mIn tegenstelling tot [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, split [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m geen strings met '.'
[48;2;34;34;34m[38;2;255;255;255mVoor [a.b.c], [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m en [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zijn hetzelfde.
[48;2;34;34;34m[38;2;255;255;255mVoor eenvoudige gevallen zal [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m handiger zijn.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAls de sleutel '.' bevat, gebruik dan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m in plaats van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mBijvoorbeeld:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mIn deze structuur is "editor.defaultFormatter" een complete string, en is [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mdefaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m geen sub-sleutel van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255meditor[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mWanneer u [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m gebruikt, splitst het de sleutel in:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTerwijl [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m de sleutel beschouwt als:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDit laatste is de juiste structuur.[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mArray van i64's[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mBool (booleaanse type), kan alleen [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zijn[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: 64-bit signed integer type, bijv. 1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255mVoorbeeld:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDeze optie is alleen beschikbaar voor [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Als we [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m aan het einde van het commando toevoegen, wordt het als volgt weergegeven:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255mOpslaan in het opgegeven formaat / bestand, slechts één keer per keer[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255mVoorbeeld 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mVoorbeeld 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;255;255;255mOpmerking: Ron heeft echt het type [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m dat kan worden [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mHier verwijst [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m naar het verwijderen van de opgegeven sleutel (key), niet het echte type [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mStel dat we de volgende tabel-array hebben in [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAls we bijvoorbeeld [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mname: test[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m willen verwijderen, kunnen we dit doen met het volgende commando:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOpmerking: Een lege tekenreeks en [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zijn niet hetzelfde.
[48;2;34;34;34m[38;2;255;255;255mHet gebruik van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zal het instellen op een lege tekenreeks, terwijl [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m de hele sleutel zal verwijderen.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAls we de tweede tabel (index 1) van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mbin[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m willen verwijderen, kunnen we het volgende commando gebruiken:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOf we kunnen [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m gebruiken:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255mDouble-precision floating-point number (IEEE 754-2008 standard), bijv. 3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;255;255;255mOpmerking: Hoewel u met [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m slechts één K-V-paar per tabel kunt maken, kunt u eerst de index van de tabelarray opgeven en vervolgens andere opties gebruiken om uit te breiden.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mVoorbeeld:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mStel de k-v pair in, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m staat gelijk aan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255mOproepen zonder [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, overschrijft het huidige bestand als deze opgeroepen wordt[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255mString array[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255mVoorbeeld:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, version, 0.2.84, version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSelecteer eerst met de opgegeven sleutel en wijzig vervolgens met de opgegeven datatype-waarde.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Wanneer alleen de sleutel wordt verstrekt en geen waarde, worden de respectieve datatypen en waarden weergegeven.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Bijvoorbeeld [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, toont de tabel voor package en de onderliggende tabellen, en biedt meer gedetailleerde informatie dan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mVoorbeelden[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mVoorbeeld 1：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOmdat [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m in plaats van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is gespecificeerd, is "2021" een string-type, geen integer-type.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m nieuwe waarde [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mVoorbeeld 2：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mVoorbeeld 3：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: Dit is de eerste index*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Opmerking: Deze voorbeelden beginnen met een argument of subopdracht en laten de naam van de hoofdopdracht weg. In werkelijke bewerkingen moet u deze handmatig aanvullen.[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mVoorbeeld:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255mVoorbeeld:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOpmerking: Behalve in [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zullen andere formaten de standaardtabel gebruiken in plaats van de inline-tabel.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mZoek de waarde op met behulp van de sleutel[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mVoorbeeld:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m JSON 1.0 ondersteunt geen NaN en Inf, deze produceert [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m in plaats van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255mVerwijder de opgegeven sleutel (key)[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mLijkt erg op de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m van de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sub-opdracht, maar het grootste verschil is dat [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m meerdere keren achter elkaar kan worden opgeroepen om naar verschillende formaten / bestanden te converteren.
[48;2;34;34;34m[38;2;255;255;255mDe [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m van de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sub-opdracht kan slechts één keer worden aangeroepen en output naar een enkel bestand.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Opmerking: Na het uitvoeren van de conversie-operatie verwijdert [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m alle opmerkingen in alle bestandsindelingen.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Terwijl de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m sub-opdracht de [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m opmerkingen behoudt, alle andere indelingen behouden geen opmerkingen.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mAls [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m wordt aangeroepen zonder [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, wordt het huidige bestand overschreven.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Als [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m is aangeroepen en [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ook, wordt het gewijzigde bestand opgeslagen in het bestand waarnaar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verwijst.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Tenzij het bronbestand overeenkomt met het bestand waarnaar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m verwijst, zal het gedrag van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m van de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sub-opdracht vergelijkbaar zijn met dat van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m van de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sub-opdracht, waarbij beide automatisch niet-schrijfbare bestanden hernoemen.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Stel bijvoorbeeld dat "from" is [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m en "to" is [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, dan wordt het uitvoerbestand [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m in plaats van [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Als u deze functie niet nodig heeft, gebruik dan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m of [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, in plaats van [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. [0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255mBekijk het volledige bestand na de wijzigingen vooraf[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255mSelecteer de opgegeven sleutel, gebruik "." als scheidingsteken[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255mHaal de waarde op van de opgegeven configuratie[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255mTabel-array met één key-value-paar (gebruik: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mDe notatie is als volgt: "a.b.c", gebruik de "." (dot) om subtabellen of array-indexen te verbinden.
[48;2;34;34;34m[38;2;255;255;255mAls er maar één "." is, wordt de hele tabel weergegeven.[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255mMaak / update / verwijder de structuur van het configuratiebestand[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mVoorbeeld:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: Het eerste element is waar
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mArray van bool's[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255mStandaardtabel voor het opslaan van meerdere K-V-paren[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255mType voor datum en tijd[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255mVoorbeeld 1:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mVoorbeeld 2:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mArray van f64's[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString (string-type)[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255mGebruik [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m meerdere keren om array-elementen in te stellen.
[48;2;34;34;34m[38;2;255;255;255mVoorbeeld:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Stel het element van index 0 in tijdens de eerste aanroep, stel het element van index 1 in tijdens de tweede oproep, enzovoort.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255mTabel-array (gebruik: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mInline-tabel, vergelijkbaar met een compactere standaardtabel[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mVoer de sleutel in in segmenten, de resultaten worden automatisch samengevoegd zonder "." als scheidingsteken[0m"##),
    ],
}
}

/// Language ID: nl;
/// Map name: "parser";
/// Description: Nederlands, Latijns, Nederland;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "Hieronder vindt u de standaardfout (stderr) uitvoer. Bij het omleiden van de uitvoer van de automatisch-aanvullen script, moet u `>/path/to/file` gebruiken in plaats van `2>/path/to/file`.");
/// ```
pub(super) const fn get_nl_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"Hieronder vindt u de standaardfout (stderr) uitvoer. Bij het omleiden van de uitvoer van de automatisch-aanvullen script, moet u `>/path/to/file` gebruiken in plaats van `2>/path/to/file`."##),
    ],
}
}

/// nl: Nederlands, Latijns, Nederland
pub(super) const fn get_nl_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_nl_map_args),
        ("conv_md", get_nl_map_conv_md),
        ("parser", get_nl_map_parser),
        ("opt_md", get_nl_map_opt_md),
        ("args_md", get_nl_map_args_md),
        ("opt", get_nl_map_opt),
        ("log-core", get_nl_map_log_core),
        ("conv", get_nl_map_conv),
        ("theme", get_nl_map_theme),
    ],
}
}

/// Language ID: no;
/// Map name: "log-core";
/// Description: norsk, latinsk, Norge;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "DEBUG");
/// ```
pub(super) const fn get_no_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"DEBUG"##),
        ("warn", r##"Advarsel"##),
        ("info", r##"Info"##),
        ("error", r##"Feil"##),
        ("trace", r##"Spor"##),
        ("init-logger", r##"Initialisere logger ..."##),
    ],
}
}

/// Language ID: no;
/// Map name: "theme";
/// Description: norsk, latinsk, Norge;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ugyldig temanavn");
/// ```
pub(super) const fn get_no_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ugyldig temanavn"##),
    ],
}
}

/// no: norsk, latinsk, Norge
pub(super) const fn get_no_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_no_map_log_core),
        ("theme", get_no_map_theme),
    ],
}
}

/// Language ID: ny;
/// Map name: "log-core";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_ny_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"kuchenjeza"##),
        ("info", r##"Kuchuluka"##),
        ("error", r##"cholakwika"##),
        ("trace", r##"Kufufuza"##),
        ("init-logger", r##"Poyambitsa Logger ..."##),
    ],
}
}

/// Language ID: ny;
/// Map name: "theme";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Dzina Losavomerezeka");
/// ```
pub(super) const fn get_ny_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Dzina Losavomerezeka"##),
    ],
}
}

/// ny: ny-Latn-MW
pub(super) const fn get_ny_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ny_map_log_core),
        ("theme", get_ny_map_theme),
    ],
}
}

/// Language ID: or;
/// Map name: "log-core";
/// Description: ଓଡ଼ିଆ, ଓଡ଼ିଆ, ଭାରତ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ଡ\u{b3f}ବଗ\u{b4d}");
/// ```
pub(super) const fn get_or_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ଡିବଗ୍"##),
        ("warn", r##"ଚେତାବନୀ"##),
        ("info", r##"ସୂଚନା"##),
        ("error", r##"ତ୍ରୁଟି"##),
        ("trace", r##"ଟ୍ରେସ୍"##),
        ("init-logger", r##"ଲଗର୍ ଆରମ୍ଭ କରିବା ..."##),
    ],
}
}

/// Language ID: or;
/// Map name: "theme";
/// Description: ଓଡ଼ିଆ, ଓଡ଼ିଆ, ଭାରତ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "ଅବ val ଧ ଥ\u{b3f}ମ\u{b4d} ନ\u{b3e}ମ |");
/// ```
pub(super) const fn get_or_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"ଅବ val ଧ ଥିମ୍ ନାମ |"##),
    ],
}
}

/// or: ଓଡ଼ିଆ, ଓଡ଼ିଆ, ଭାରତ
pub(super) const fn get_or_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_or_map_log_core),
        ("theme", get_or_map_theme),
    ],
}
}

/// Language ID: pa;
/// Map name: "log-core";
/// Description: ਪੰਜਾਬੀ, ਗੁਰਮੁਖੀ, ਭਾਰਤ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ਡੀਬ\u{a71}ਗ");
/// ```
pub(super) const fn get_pa_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ਡੀਬੱਗ"##),
        ("warn", r##"ਚੇਤਾਵਨੀ"##),
        ("info", r##"ਜਾਣਕਾਰੀ"##),
        ("error", r##"ਗਲਤੀ"##),
        ("trace", r##"ਟਰੇਸ"##),
        ("init-logger", r##"ਸ਼ੁਰੂਆਤੀ ਲਾਗਰ ..."##),
    ],
}
}

/// Language ID: pa;
/// Map name: "theme";
/// Description: ਪੰਜਾਬੀ, ਗੁਰਮੁਖੀ, ਭਾਰਤ;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "ਗਲਤ ਥੀਮ ਦਾ ਨਾਮ");
/// ```
pub(super) const fn get_pa_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"ਗਲਤ ਥੀਮ ਦਾ ਨਾਮ"##),
    ],
}
}

/// pa: ਪੰਜਾਬੀ, ਗੁਰਮੁਖੀ, ਭਾਰਤ
pub(super) const fn get_pa_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_pa_map_log_core),
        ("theme", get_pa_map_theme),
    ],
}
}

/// Language ID: pl;
/// Map name: "log-core";
/// Description: polski, łacińskie, Polska;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "DEBUGOWANIE");
/// ```
pub(super) const fn get_pl_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"DEBUGOWANIE"##),
        ("warn", r##"OSTRZEŻENIE"##),
        ("info", r##"INFORMACJA"##),
        ("error", r##"BŁĄD"##),
        ("trace", r##"ŚLEDZENIE"##),
        ("init-logger", r##"Inicjowanie loggera..."##),
    ],
}
}

/// Language ID: pl;
/// Map name: "theme";
/// Description: polski, łacińskie, Polska;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nieprawidłowa nazwa motywu");
/// ```
pub(super) const fn get_pl_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nieprawidłowa nazwa motywu"##),
    ],
}
}

/// Language ID: pl;
/// Map name: "args";
/// Description: polski, łacińskie, Polska;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Wyłącz tło tematu (domyślnie jest włączone, można wyłączyć za pomocą `--dis-theme-bg`)");
/// ```
pub(super) const fn get_pl_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"plik z niestandardowymi zestawami tematów; wymaga podania nazwy tematu za pomocą `--theme`"##),
        ("shell-completion", r##"Generuje skrypt uzupełniania dla powłoki"##),
        ("theme-file-help", r##"Obsługiwany format pliku to binarny dump danych utworzony przez paczkę rustową `syntect`.
W gruncie rzeczy są to tematy starej wersji programu Sublime.
Jeśli nie podasz opcji `--tf`, wartość zostanie pobrana z zmiennej środowiskowej `$TOMLYRE_THEME_FILE`."##),
        ("from-format", r##"Ręcznie określ format pliku źródłowego (domyślnie zostanie on automatycznie wykryty)"##),
        ("disable-theme-background", r##"Wyłącz tło tematu (domyślnie jest włączone, można wyłączyć za pomocą `--dis-theme-bg`)"##),
        ("theme-uncompressed-file-help", r##"> Nieskompresowane wersje plików mają wyższy priorytet niż te skompresowane.  

Jeśli nie podasz opcji `--tuf`, wartość zostanie pobrana z zmiennej środowiskowej `$TOMLYRE_THEME_UNCOMPRESSED_FILE`."##),
        ("disable-theme-background-help", r##"Jeśli nie podasz opcji `--dis-theme-bg`, wartość zostanie pobrana z zmiennej środowiskowej `$TOMLYRE_THEME_BG`.

Jeśli ta wartość wynosi false, tło będzie wyłączone.

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"plik z niestandardowymi zestawami tematów; wymaga podania nazwy tematu za pomocą `--theme`; wersja skompresowana"##),
        ("shell-completion-help", r##"Musisz ręcznie podać nazwę powłoki (`shell`) w celu wygenerowania skryptu uzupełniania.
Na przykład, dla powershell na Windowsie, możesz użyć: `--sh-comp pwsh`

> Q: Co to jest funkcja uzupełniania powłoki (shell completion)?

> A: Gdy wprowadzamy polecenia w wierszu poleceń i naciśniemy klawisz Tab, skrypt uzupełniania zostanie uruchomiony, aby wyszukać możliwe opcje uzupełnienia i przedstawić je do wyboru. Ta funkcja może znacznie zwiększyć wydajność i dokładność wprowadzania poleceń w wierszu poleceń.

Dla różnych powłok skrypty uzupełniania są przechowywane w innych miejscach. Dla przykładu, dla powłoki `zsh`, musisz umieścić pliki w katalogach zawartych w tablicy `$fpath`.
Możesz użyć `print -l $fpath`, aby uzyskać zawartość tej tablicy.
Załóżmy, że katalog `/usr/local/share/zsh/site-functions` znajduje się wewnątrz tej tablicy.
Możesz użyć poniższej komendy, aby zapisać skrypt uzupełniania w pliku `_tomlyre` w tym katalogu.

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

Aby uniknąć używania uprawnień `root`, możesz umieścić ten plik w podkatalogu użytkownika. Na przykład, możesz dodać element na początek tablicy za pomocą następującego polecenia, a następnie użyć odpowiedniego katalogu.
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"Jeśli odczytujesz dane ze standardowego wejścia (`stdin`), np. `cat a | tomlyre conv -`, program spróbuje automatycznie określić format pliku.
Jednak automatyczne wykrywanie może czasem zawodzić, wtedy trzeba określić format ręcznie.
Przykład użycia: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Jeśli użyjesz tylko opcji `--ts`, wyświetli się lista dostępnych stylów.
Poza markdown, który można zapisać jako md, pozostałe style wymagają pełnej nazwy.
Przykład użycia 1: `--ts md`
Przykład użycia 2: `--table-style ascii`

Uwaga: obecnie nie ma obsługi dla kolekcji **stylów tabel** z pliku. Jeśli potrzebujesz tej funkcji lub innych dodatkowych funkcji, zachęcamy do zgłaszania problemów."##),
        ("table-style", r##"Określa styl tabeli"##),
        ("theme-help", r##"Jeśli użyjesz tylko opcji `-t`, wyświetli się lista dostępnych tematów.
Jeśli nie podasz pliku z tematem, zostanie użyty zestaw wewnętrznych tematów.
Załóżmy, że istnieje temat o nazwie **ayu-dark**, możesz użyć opcji `-t ayu-dark`, aby użyć tego tematu.
Jeśli nazwa tematu zawiera spacje, należy ją umieścić w cudzysłowach lub apostrofach.

> Uwaga: Istnieje specjalny temat o nazwie `None`, który oznacza styl tekstu czystego bez kolorowania składni.

Jeśli nie podasz opcji `--theme`, wartość zostanie pobrana z zmiennej środowiskowej `$TOMLYRE_THEME`.
Ten opis pomocy dla tematu **kolorowania składni** również pochodzi z tej samej **zmiennej środowiskowej**.

> Domyślny opis pomocy jest zakodowany na stałe, co oznacza, że ładowanie jest bardzo szybkie.  
> Ładowanie tekstów zakodowanych na stałe jest porównywalne do ładowania tekstów bez kolorowania składni (`None`), a ładowanie dynamiczne jest wolniejsze o dwa lub więcej rzędów wielkości.  
> Ustawienie zmiennej środowiskowej `$TOMLYRE_THEME` może spowodować ponowne ładowanie tematów, co **znacznie** spowolni czas uruchamiania.  
> Jeśli nie masz zamiaru zmieniać tematu opisu pomocy lub wyłączać kolorowania składni, używaj opcji `-t` zamiast modyfikować zmienną środowiskową.

Przykłady użycia ze zmiennej środowiskowej:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Przykłady użycia ze wiersza poleceń:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Dla zawartości generowanej przez ręczne polecenia takie jak "conv/get/set", itp., priorytet ma temat podany przez `-t`, a jeśli nie podano `--theme`, wartość zostanie pobrana z zmiennej środowiskowej. Jeśli zmienna jest pusta, użyty zostanie temat **Monokai Extended**."##),
        ("theme", r##"nazwa tematu, który ma być użyty do kolorowania składni"##),
    ],
}
}

/// Language ID: pl;
/// Map name: "args_md";
/// Description: polski, łacińskie, Polska;
pub(super) const fn get_pl_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mplik z niestandardowymi zestawami tematów; wymaga podania nazwy tematu za pomocą [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mGeneruje skrypt uzupełniania dla powłoki[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mObsługiwany format pliku to binarny dump danych utworzony przez paczkę rustową [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mW gruncie rzeczy są to tematy starej wersji programu Sublime.
[48;2;34;34;34m[38;2;255;255;255mJeśli nie podasz opcji [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, wartość zostanie pobrana z zmiennej środowiskowej [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mRęcznie określ format pliku źródłowego (domyślnie zostanie on automatycznie wykryty)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mWyłącz tło tematu (domyślnie jest włączone, można wyłączyć za pomocą [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nieskompresowane wersje plików mają wyższy priorytet niż te skompresowane.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mJeśli nie podasz opcji [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, wartość zostanie pobrana z zmiennej środowiskowej [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mJeśli nie podasz opcji [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, wartość zostanie pobrana z zmiennej środowiskowej [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mJeśli ta wartość wynosi false, tło będzie wyłączone.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mplik z niestandardowymi zestawami tematów; wymaga podania nazwy tematu za pomocą [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m; wersja skompresowana[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mMusisz ręcznie podać nazwę powłoki ([48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mshell[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m) w celu wygenerowania skryptu uzupełniania.
[48;2;34;34;34m[38;2;255;255;255mNa przykład, dla powershell na Windowsie, możesz użyć: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q: Co to jest funkcja uzupełniania powłoki (shell completion)?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: Gdy wprowadzamy polecenia w wierszu poleceń i naciśniemy klawisz Tab, skrypt uzupełniania zostanie uruchomiony, aby wyszukać możliwe opcje uzupełnienia i przedstawić je do wyboru. Ta funkcja może znacznie zwiększyć wydajność i dokładność wprowadzania poleceń w wierszu poleceń.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDla różnych powłok skrypty uzupełniania są przechowywane w innych miejscach. Dla przykładu, dla powłoki [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mzsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, musisz umieścić pliki w katalogach zawartych w tablicy [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mMożesz użyć [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, aby uzyskać zawartość tej tablicy.
[48;2;34;34;34m[38;2;255;255;255mZałóżmy, że katalog [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m znajduje się wewnątrz tej tablicy.
[48;2;34;34;34m[38;2;255;255;255mMożesz użyć poniższej komendy, aby zapisać skrypt uzupełniania w pliku [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m w tym katalogu.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAby uniknąć używania uprawnień [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mroot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, możesz umieścić ten plik w podkatalogu użytkownika. Na przykład, możesz dodać element na początek tablicy za pomocą następującego polecenia, a następnie użyć odpowiedniego katalogu.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mJeśli odczytujesz dane ze standardowego wejścia ([48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mstdin[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m), np. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, program spróbuje automatycznie określić format pliku.
[48;2;34;34;34m[38;2;255;255;255mJednak automatyczne wykrywanie może czasem zawodzić, wtedy trzeba określić format ręcznie.
[48;2;34;34;34m[38;2;255;255;255mPrzykład użycia: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mJeśli użyjesz tylko opcji [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, wyświetli się lista dostępnych stylów.
[48;2;34;34;34m[38;2;255;255;255mPoza markdown, który można zapisać jako md, pozostałe style wymagają pełnej nazwy.
[48;2;34;34;34m[38;2;255;255;255mPrzykład użycia 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPrzykład użycia 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mUwaga: obecnie nie ma obsługi dla kolekcji [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mstylów tabel[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m z pliku. Jeśli potrzebujesz tej funkcji lub innych dodatkowych funkcji, zachęcamy do zgłaszania problemów.[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mOkreśla styl tabeli[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mJeśli użyjesz tylko opcji [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, wyświetli się lista dostępnych tematów.
[48;2;34;34;34m[38;2;255;255;255mJeśli nie podasz pliku z tematem, zostanie użyty zestaw wewnętrznych tematów.
[48;2;34;34;34m[38;2;255;255;255mZałóżmy, że istnieje temat o nazwie [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, możesz użyć opcji [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, aby użyć tego tematu.
[48;2;34;34;34m[38;2;255;255;255mJeśli nazwa tematu zawiera spacje, należy ją umieścić w cudzysłowach lub apostrofach.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Uwaga: Istnieje specjalny temat o nazwie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, który oznacza styl tekstu czystego bez kolorowania składni.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mJeśli nie podasz opcji [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, wartość zostanie pobrana z zmiennej środowiskowej [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mTen opis pomocy dla tematu [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mkolorowania składni[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m również pochodzi z tej samej [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mzmiennej środowiskowej[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Domyślny opis pomocy jest zakodowany na stałe, co oznacza, że ładowanie jest bardzo szybkie.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Ładowanie tekstów zakodowanych na stałe jest porównywalne do ładowania tekstów bez kolorowania składni ([48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m), a ładowanie dynamiczne jest wolniejsze o dwa lub więcej rzędów wielkości.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Ustawienie zmiennej środowiskowej [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m może spowodować ponowne ładowanie tematów, co [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mznacznie[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m spowolni czas uruchamiania.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Jeśli nie masz zamiaru zmieniać tematu opisu pomocy lub wyłączać kolorowania składni, używaj opcji [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m zamiast modyfikować zmienną środowiskową.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPrzykłady użycia ze zmiennej środowiskowej:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPrzykłady użycia ze wiersza poleceń:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDla zawartości generowanej przez ręczne polecenia takie jak "conv/get/set", itp., priorytet ma temat podany przez [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, a jeśli nie podano [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, wartość zostanie pobrana z zmiennej środowiskowej. Jeśli zmienna jest pusta, użyty zostanie temat [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mnazwa tematu, który ma być użyty do kolorowania składni[0m"##),
    ],
}
}

/// Language ID: pl;
/// Map name: "conv";
/// Description: polski, łacińskie, Polska;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Konwertuje między formatami takimi jak toml, yaml, json i ron.");
/// ```
pub(super) const fn get_pl_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Konwertuje między formatami takimi jak toml, yaml, json i ron."##),
        ("from", r##"Określa plik wejściowy"##),
        ("conv-to", r##"Określa format/plik wyjściowy, można użyć wielokrotnie"##),
        ("save", r##"Po użyciu tej opcji plik określony w `--to` zostanie zapisany"##),
        ("conv-to-help", r##"Jeśli istnieje plik **test.json** w bieżącym katalogu, możesz użyć `conv test.json -t yml`, aby przekonwertować go na format yml (yaml).
Możesz także ręcznie określić ścieżkę do pliku: `conv test.json -t /tmp/test.yml`

Obsługiwane formaty zależą od funkcji, które zostały włączone w tej paczce, a nie wszystkie funkcje są włączone domyślnie.

Ponadto musisz wiedzieć przed rozpoczęciem korzystania:
- 1. Automatyczna zmiana nazwy plików dla nieprecyzyjnych ścieżek.
  - 1.1: Tylko pojedynczy poziom plików jest uznawany za format pliku, a nie ścieżka pliku
    Gdy używasz `conv test.toml -t ron`, plik wyjściowy zostanie automatycznie zmieniony na `test.ron`, a nie na `ron`.
    Jeśli nie potrzebujesz tej funkcji, użyj `-t ./ron` lub `-t $PWD/ron`.
  - 1.2: Odwołanie do katalogu, a nie do pliku
    Na przykład, `-t /tmp/`, możemy umieścić tylko `test.toml` jako podkatalog `/tmp/`, a nie nadpisać sam katalog `/tmp`.
    Jeśli nie potrzebujesz tej funkcji, użyj pełnej ścieżki do pliku (a nie katalogu), np. `-t /tmp/xx.toml`, a nie `-t /tmp/`.
- 2. Gdy nie wywołasz opcji `--save`, określony plik nie zostanie automatycznie zapisany.
- 3. Gdy wywołasz opcję `--save`, dane wejściowe i wyjściowe polecenia conv nie mogą być tym samym plikiem.
- 4. Skomentowane wersje plików nie zostaną zachowane po konwersji."##),
    ],
}
}

/// Language ID: pl;
/// Map name: "conv_md";
/// Description: polski, łacińskie, Polska;
pub(super) const fn get_pl_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mKonwertuje między formatami takimi jak toml, yaml, json i ron.[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mOkreśla plik wejściowy[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mOkreśla format/plik wyjściowy, można użyć wielokrotnie[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mPo użyciu tej opcji plik określony w [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zostanie zapisany[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mJeśli istnieje plik [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m w bieżącym katalogu, możesz użyć [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, aby przekonwertować go na format yml (yaml).
[48;2;34;34;34m[38;2;255;255;255mMożesz także ręcznie określić ścieżkę do pliku: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mObsługiwane formaty zależą od funkcji, które zostały włączone w tej paczce, a nie wszystkie funkcje są włączone domyślnie.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPonadto musisz wiedzieć przed rozpoczęciem korzystania:
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Automatyczna zmiana nazwy plików dla nieprecyzyjnych ścieżek.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Tylko pojedynczy poziom plików jest uznawany za format pliku, a nie ścieżka pliku
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mGdy używasz [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, plik wyjściowy zostanie automatycznie zmieniony na [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, a nie na [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mJeśli nie potrzebujesz tej funkcji, użyj [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m lub [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: Odwołanie do katalogu, a nie do pliku
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mNa przykład, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, możemy umieścić tylko [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m jako podkatalog [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, a nie nadpisać sam katalog [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mJeśli nie potrzebujesz tej funkcji, użyj pełnej ścieżki do pliku (a nie katalogu), np. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, a nie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. Gdy nie wywołasz opcji [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, określony plik nie zostanie automatycznie zapisany.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. Gdy wywołasz opcję [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, dane wejściowe i wyjściowe polecenia conv nie mogą być tym samym plikiem.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. Skomentowane wersje plików nie zostaną zachowane po konwersji.[0m"##),
    ],
}
}

/// Language ID: pl;
/// Map name: "opt";
/// Description: polski, łacińskie, Polska;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "Tablica tabel z wieloma kluczami i wartościami (użycie: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_pl_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"Różnica między `-k` polega na tym, że `--ck` nie wykonuje automatycznie `split('.')`.
Dla [a.b.c], `-k a.b.c` i `--ck a --ck b --ck c` są identyczne.
Dla prostszych przypadków `-k` byłby wygodniejszy.

Jeśli nazwa klucza zawiera ".", należy użyć `--ck`, a nie `-k`.
Na przykład:

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

W tej strukturze "editor.defaultFormatter" jest pełnym ciągiem znaków, a "defaultFormatter" nie jest podkluczem "editor".

Jeśli użyjesz `-k '[typescript]'.'editor.defaultFormatter'`, zostanie podzielone na:

- "[typescript]"
- "editor"
- "defaultFormatter"

Natomiast z użyciem `--ck '[typescript]' --ck editor.defaultFormatter`, będzie traktowane jako:

- "[typescript]"
- "editor.defaultFormatter"

To drugie jest właściwą strukturą."##),
        ("num-arr", r##"Tablica liczb całkowitych ze znakiem typu i64"##),
        ("bool", r##"bool (typ logiczny), może być tylko `true` lub `false`."##),
        ("num", r##"i64: typ liczby całkowitej ze znakiem 64-bitową, np. 1145"##),
        ("datetime-help", r##"Przykład:
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

Ta opcja aplikuje się tylko do formatu `toml`. Jeśli dodamy `-t json` na końcu, to będzie wyglądać tak:

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"Zapisuje plik w określonym formacie / ścieżce, można zapisać tylko jeden plik naraz."##),
        ("get-opts-help", r##"Przykład użycia 1: `get Cargo.toml -k package.name`
Przykład użycia 2: `get cfg.yaml -k .`
"##),
        ("none-help", r##"> Uwaga: format `ron` posiada rzeczywisty typ `Option<T>`, który może mieć wartość `Some(T)` lub `None`.
> Opcja `--none` służy do usunięcia określonego klucza, a nie do rzeczywistego typu `Option<T>`.

Załóżmy, że w pliku `test.yml` istnieje następująca tablica tabel:

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

Jeśli chcemy usunąć klucz **name: test**, wykonujemy polecenie:
```sh
set test.yml -k bin.1.name --none
```

> Uwaga: pusty ciąg znaków i `None` są różne.
> Wywołanie `set test.yml --kv bin.1.name ""` ustawia wartość na pusty ciąg znaków, a `--none` usuwa cały klucz.

Jeśli chcemy usunąć drugą tabelę z tablicy `bin` (o indeksie 1), wykonujemy:

```sh
set test.yml -k bin.1 --none
```

lub używając `--rm`:
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"Liczby zmiennoprzecinkowe podwójnej precyzji (standard IEEE 754-2008), np. 3.14159265359"##),
        ("saot-help", r##"> Uwaga: chociaż można utworzyć tylko jedną parę K-V za pomocą `--saot`, można najpierw ustawić indeks tablicy, a następnie rozszerzyć go za pomocą innych opcji.

Przykład:
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"Ustawia pary klucz-wartość (k-v), `--kv x.y.z wartość` jest równoważne z `-k x.y.z -s wartość`."##),
        ("set-save", r##"Jeśli nie podano parametru `--to`, zastępuje (nadpisuje) bieżący plik"##),
        ("array", r##"Tablica ciągów znaków"##),
        ("aot-help", r##"Przykład:
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

Najpierw wybierz klucz, a następnie wprowadź wartość o określonym typie danych.

> Jeśli podany jest tylko klucz, a nie wartość, zostanie wyświetlony typ danych i wartość.
> Na przykład, `set Cargo.toml -k package`, wyświetli tabelę "package" i jej podtabelki z bardziej szczegółowymi informacjami niż polecenie `get`.

## Przykłady

### Przykład 1:
```sh
--ts md set test.toml --key package.edition --str 2021
```

Ze względu na użycie `--str` zamiast `--num`, "2021" będzie traktowane jako ciąg znaków, a nie liczba całkowita.

| package | nowa wartość |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### Przykład 2:
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### Przykład 3:
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: to jest pierwszy indeks*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> Uwaga: powyższe przykłady zaczynają się od argumentu/podkomendy, pomijając nazwę głównego polecenia. W rzeczywistym działaniu musisz jawnie podać nazwę."##),
        ("num-arr-help", r##"Przykład:
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"Przykład:
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> Poza formatem `toml`, pozostałe formaty będą używać standardowych tabel zamiast tabel wewnętrznych.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"Zapytaj o wartość dla klucza (key)"##),
        ("f64-arr-help", r##"Przykład:
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> JSON 1.0 nie obsługuje wartości NaN i Inf.
> Zamiast tego zostanie wyświetlone `null` zamiast `NaN`.

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```
```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"Usunięcie określonego klucza"##),
        ("set-save-to-help", r##"Bardzo podobne do `-t` w przypadku polecenia `conv`, ale największa różnica polega na tym, że `-t` w `conv` może być wywoływane wiele razy, aby wyeksportować dane do wielu formatów / plików naraz.
Podczas gdy `-t` w `set` może być wywołane tylko raz i eksportuje do jednego pliku.

> Uwaga: po wykonaniu operacji konwersji przez podkomendę `conv`, wszystkie pliki zostaną pozbawione komentarzy.
> Jednak `set` zachowa komentarze dla plików **toml**, a dla innych formatów **nie**.

- Jeśli nie użyto `-t`, to wywołanie `--save` spowoduje zastąpienie bieżącego pliku.
- Jeśli wywołano `-t` oraz `--save`, zmodyfikowany plik zostanie zapisany w miejscu wskazanym przez parametr `-t`.
- Chyba że źródło pliku jest takie samo jak plik wskazany przez `-t`, zachowanie `-t` w `set` będzie podobne do `-t` w `conv`, które automatycznie zmienia nazwy plików, na których nie wykonano żadnych operacji.
  - Na przykład dla from: **test.yml**, to: **json**, wyjściowy plik zostanie zmieniony na **test.json**, a nie **json**.
  - Jeśli ta funkcja nie jest potrzebna, należy użyć `-t ./json` lub `-t $PWD/json`, a nie `-t json`. "##),
        ("preview", r##"Po dokonaniu zmian, wyświetla pełny plik"##),
        ("set-key", r##"Wybiera określony klucz (key) z użyciem kropki "." jako separatora."##),
        ("get-opts", r##"Pobiera wartość określonej konfiguracji"##),
        ("saot", r##"Jedna para klucz-wartość dla każdej tabeli w tablicy (użycie: `--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"Format to "a.b.c", gdzie nazwy podtabel lub indeksy tablic są połączone kropką ".".
Jeśli wykorzystasz tylko jeden znak ".", zostanie wyświetlona pełna tabela."##),
        ("set-opts", r##"Tworzy / aktualizuje / usuwa struktury plików konfiguracyjnych"##),
        ("bool-arr-help", r##"Przykład:
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: pierwszy element to true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"Tablica wartości logicznych typu bool"##),
        ("table", r##"Tabela standardowa, służy do przechowywania wielu par K-V"##),
        ("datetime", r##"Typ daty i czasu"##),
        ("table-help", r##"Przykład 1:
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

Przykład 2:
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"Tablica liczb zmiennoprzecinkowych podwójnej precyzji typu f64"##),
        ("string", r##"String (typ ciągu znaków)"##),
        ("array-help", r##"Użyj wielokrotnie `-a`, aby ustawić elementy tablicy.
Na przykład:

```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> Pierwsze wywołanie ustawi element o indeksie 0, drugie ustawi element o indeksie 1 i tak dalej.

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"Tablica tabel z wieloma kluczami i wartościami (użycie: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"Tabela wewnętrzna, odpowiada bardziej zwartej standardowej tabeli"##),
        ("concat-key", r##"Podaj klucz sekwencyjnie, program sam połączy je w całość. Nie używaj kropki "." jako separatora."##),
    ],
}
}

/// Language ID: pl;
/// Map name: "opt_md";
/// Description: polski, łacińskie, Polska;
pub(super) const fn get_pl_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255mRóżnica między [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m polega na tym, że [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m nie wykonuje automatycznie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msplit('.')[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mDla [a.b.c], [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m i [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m są identyczne.
[48;2;34;34;34m[38;2;255;255;255mDla prostszych przypadków [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m byłby wygodniejszy.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mJeśli nazwa klucza zawiera ".", należy użyć [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, a nie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mNa przykład:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mW tej strukturze "editor.defaultFormatter" jest pełnym ciągiem znaków, a "defaultFormatter" nie jest podkluczem "editor".
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mJeśli użyjesz [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, zostanie podzielone na:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mNatomiast z użyciem [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, będzie traktowane jako:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTo drugie jest właściwą strukturą.[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mTablica liczb całkowitych ze znakiem typu i64[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mbool (typ logiczny), może być tylko [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m lub [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: typ liczby całkowitej ze znakiem 64-bitową, np. 1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255mPrzykład:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTa opcja aplikuje się tylko do formatu [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Jeśli dodamy [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m na końcu, to będzie wyglądać tak:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255mZapisuje plik w określonym formacie / ścieżce, można zapisać tylko jeden plik naraz.[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255mPrzykład użycia 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPrzykład użycia 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Uwaga: format [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m posiada rzeczywisty typ [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, który może mieć wartość [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m lub [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Opcja [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m służy do usunięcia określonego klucza, a nie do rzeczywistego typu [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mZałóżmy, że w pliku [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m istnieje następująca tablica tabel:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mJeśli chcemy usunąć klucz [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mname: test[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, wykonujemy polecenie:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Uwaga: pusty ciąg znaków i [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m są różne.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Wywołanie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m ustawia wartość na pusty ciąg znaków, a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m usuwa cały klucz.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mJeśli chcemy usunąć drugą tabelę z tablicy [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mbin[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m (o indeksie 1), wykonujemy:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mlub używając [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255mLiczby zmiennoprzecinkowe podwójnej precyzji (standard IEEE 754-2008), np. 3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Uwaga: chociaż można utworzyć tylko jedną parę K-V za pomocą [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, można najpierw ustawić indeks tablicy, a następnie rozszerzyć go za pomocą innych opcji.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPrzykład:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mUstawia pary klucz-wartość (k-v), [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z wartość[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m jest równoważne z [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s wartość[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255mJeśli nie podano parametru [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, zastępuje (nadpisuje) bieżący plik[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255mTablica ciągów znaków[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255mPrzykład:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mNajpierw wybierz klucz, a następnie wprowadź wartość o określonym typie danych.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Jeśli podany jest tylko klucz, a nie wartość, zostanie wyświetlony typ danych i wartość.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Na przykład, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, wyświetli tabelę "package" i jej podtabelki z bardziej szczegółowymi informacjami niż polecenie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mPrzykłady[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mPrzykład 1:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mZe względu na użycie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m zamiast [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, "2021" będzie traktowane jako ciąg znaków, a nie liczba całkowita.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m nowa wartość [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mPrzykład 2:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mPrzykład 3:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: to jest pierwszy indeks*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Uwaga: powyższe przykłady zaczynają się od argumentu/podkomendy, pomijając nazwę głównego polecenia. W rzeczywistym działaniu musisz jawnie podać nazwę.[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mPrzykład:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255mPrzykład:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Poza formatem [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, pozostałe formaty będą używać standardowych tabel zamiast tabel wewnętrznych.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mZapytaj o wartość dla klucza (key)[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mPrzykład:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m JSON 1.0 nie obsługuje wartości NaN i Inf.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Zamiast tego zostanie wyświetlone [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m zamiast [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255mUsunięcie określonego klucza[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mBardzo podobne do [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m w przypadku polecenia [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ale największa różnica polega na tym, że [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m w [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m może być wywoływane wiele razy, aby wyeksportować dane do wielu formatów / plików naraz.
[48;2;34;34;34m[38;2;255;255;255mPodczas gdy [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m w [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m może być wywołane tylko raz i eksportuje do jednego pliku.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Uwaga: po wykonaniu operacji konwersji przez podkomendę [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, wszystkie pliki zostaną pozbawione komentarzy.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Jednak [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m zachowa komentarze dla plików [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m, a dla innych formatów [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mnie[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mJeśli nie użyto [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, to wywołanie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m spowoduje zastąpienie bieżącego pliku.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Jeśli wywołano [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m oraz [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, zmodyfikowany plik zostanie zapisany w miejscu wskazanym przez parametr [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Chyba że źródło pliku jest takie samo jak plik wskazany przez [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, zachowanie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m w [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m będzie podobne do [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m w [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, które automatycznie zmienia nazwy plików, na których nie wykonano żadnych operacji.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Na przykład dla from: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, to: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, wyjściowy plik zostanie zmieniony na [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, a nie [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Jeśli ta funkcja nie jest potrzebna, należy użyć [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m lub [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, a nie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. [0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255mPo dokonaniu zmian, wyświetla pełny plik[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255mWybiera określony klucz (key) z użyciem kropki "." jako separatora.[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255mPobiera wartość określonej konfiguracji[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255mJedna para klucz-wartość dla każdej tabeli w tablicy (użycie: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mFormat to "a.b.c", gdzie nazwy podtabel lub indeksy tablic są połączone kropką ".".
[48;2;34;34;34m[38;2;255;255;255mJeśli wykorzystasz tylko jeden znak ".", zostanie wyświetlona pełna tabela.[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255mTworzy / aktualizuje / usuwa struktury plików konfiguracyjnych[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mPrzykład:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: pierwszy element to true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mTablica wartości logicznych typu bool[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255mTabela standardowa, służy do przechowywania wielu par K-V[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255mTyp daty i czasu[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255mPrzykład 1:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPrzykład 2:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mTablica liczb zmiennoprzecinkowych podwójnej precyzji typu f64[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString (typ ciągu znaków)[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255mUżyj wielokrotnie [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, aby ustawić elementy tablicy.
[48;2;34;34;34m[38;2;255;255;255mNa przykład:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Pierwsze wywołanie ustawi element o indeksie 0, drugie ustawi element o indeksie 1 i tak dalej.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255mTablica tabel z wieloma kluczami i wartościami (użycie: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mTabela wewnętrzna, odpowiada bardziej zwartej standardowej tabeli[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mPodaj klucz sekwencyjnie, program sam połączy je w całość. Nie używaj kropki "." jako separatora.[0m"##),
    ],
}
}

/// Language ID: pl;
/// Map name: "parser";
/// Description: polski, łacińskie, Polska;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "Poniżej znajduje się wyjście `stderr`. Gdy przekierowujesz wynik skryptu uzupełniania do pliku, powinieneś użyć `>/path/to/file`, a nie `2>/path/to/file`.");
/// ```
pub(super) const fn get_pl_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"Poniżej znajduje się wyjście `stderr`. Gdy przekierowujesz wynik skryptu uzupełniania do pliku, powinieneś użyć `>/path/to/file`, a nie `2>/path/to/file`."##),
    ],
}
}

/// pl: polski, łacińskie, Polska
pub(super) const fn get_pl_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_pl_map_args),
        ("conv_md", get_pl_map_conv_md),
        ("parser", get_pl_map_parser),
        ("opt_md", get_pl_map_opt_md),
        ("args_md", get_pl_map_args_md),
        ("opt", get_pl_map_opt),
        ("log-core", get_pl_map_log_core),
        ("conv", get_pl_map_conv),
        ("theme", get_pl_map_theme),
    ],
}
}

/// Language ID: ps;
/// Map name: "log-core";
/// Description: پښتو, عربي, افغانستان;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ډیبګ");
/// ```
pub(super) const fn get_ps_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ډیبګ"##),
        ("warn", r##"خبرداری"##),
        ("info", r##"معلومات"##),
        ("error", r##"تېروتنه"##),
        ("trace", r##"ټریس"##),
        ("init-logger", r##"د لاګر پیل پیل کړئ ..."##),
    ],
}
}

/// Language ID: ps;
/// Map name: "theme";
/// Description: پښتو, عربي, افغانستان;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "د موضوع ناباوره نوم");
/// ```
pub(super) const fn get_ps_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"د موضوع ناباوره نوم"##),
    ],
}
}

/// ps: پښتو, عربي, افغانستان
pub(super) const fn get_ps_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ps_map_log_core),
        ("theme", get_ps_map_theme),
    ],
}
}

/// Language ID: pt;
/// Map name: "log-core";
/// Description: português, latim, Brasil;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_pt_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"avisar"##),
        ("info", r##"informações"##),
        ("error", r##"Erro"##),
        ("trace", r##"Trace"##),
        ("init-logger", r##"Inicializando o Logger ..."##),
    ],
}
}

/// Language ID: pt;
/// Map name: "args";
/// Description: português, latim, Brasil;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Desativa o plano de fundo do tema (ativado por padrão; se chamado com `--dis-theme-bg`, ele será desativado)");
/// ```
pub(super) const fn get_pt_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"Especifica um arquivo personalizado para o conjunto de temas. Você também precisa especificar o nome do tema com `--theme`"##),
        ("shell-completion", r##"Gere um script de conclusão para o shell especificado"##),
        ("theme-file-help", r##"O formato do arquivo suportado é binário, e é o mesmo que o usado pelo rust crate `syntect`.
Esses temas são essencialmente temas antigos do sublime.
Se `--tf` não for especificado, ele tentará ler os dados da variável de ambiente `$TOMLYRE_THEME_FILE`. "##),
        ("from-format", r##"Especifique manualmente o formato do arquivo de origem (ele será detectado automaticamente por padrão)"##),
        ("disable-theme-background", r##"Desativa o plano de fundo do tema (ativado por padrão; se chamado com `--dis-theme-bg`, ele será desativado)"##),
        ("theme-uncompressed-file-help", r##"> A versão não compactada tem uma prioridade mais alta do que a versão compactada.

Se `--tuf` não for especificado, ele tentará ler os dados da variável de ambiente `$TOMLYRE_THEME_UNCOMPRESSED_FILE`. "##),
        ("disable-theme-background-help", r##"Se `--dis-theme-bg` não for especificado, ele tentará ler os dados da variável de ambiente `$TOMLYRE_THEME_BG`.

Se o valor for falso, será desativado:

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"Semelhante a `theme-file`, mas requer dados de despejo (arquivo) não compactados"##),
        ("shell-completion-help", r##"Você precisa especificar manualmente o nome do shell para gerar um script de conclusão.
Por exemplo, para powershell no Windows, você pode usar: `--sh-comp pwsh`

> P: O que é a função de conclusão automática do shell?

> R: Quando digitamos comandos na linha de comando e pressionamos a tecla Tab, o script de conclusão automático é acionado e começa a pesquisar possíveis opções de conclusão, apresentando essas opções para escolha. Essa função pode aumentar muito a eficiência e precisão da entrada de linhas de comando.

Os scripts de conclusão para shells diferentes são armazenados em locais diferentes. Para zsh, você precisa salvá-lo no caminho incluído na matriz `$fpath`.
Você pode usar `print -l $fpath` para obter o conteúdo dessa matriz.
Suponha que ele contenha o diretório `/usr/local/share/zsh/site-functions`, então você pode usar o seguinte comando para gravar o script de conclusão no arquivo `_tomlyre` nesse diretório.

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

Para evitar o uso de privilégios de root, você pode colocá-lo no subdiretório do usuário. Por exemplo, você pode inserir um elemento no início da matriz da seguinte maneira e usar o diretório relevante.
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"Ao ler dados do stdin, por exemplo: `cat a | tomlyre conv -`, ele tentará automaticamente detectar o formato do arquivo primeiro.
No entanto, a detecção automática pode falhar, e é quando você precisa especificar o formato manualmente.
Exemplo: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Quando usado apenas com `--ts`, ele lista os conjuntos de estilos internos.
Exceto markdown que pode ser abreviado como md, todos os outros estilos devem usar o nome completo.
Exemplo de uso 1: `--ts md`
Exemplo de uso 2: `--table-style ascii`

Observação: Atualmente não é possível ler o conjunto de estilos das tabelas de um arquivo. Se você precisar dessa funcionalidade ou de outras funcionalidades adicionais, sinta-se à vontade para enviar um problema de feedback."##),
        ("table-style", r##"Especifica o estilo da tabela"##),
        ("theme-help", r##"Quando usado apenas com `-t`, ele lista os temas disponíveis atualmente.
Se nenhum arquivo de tema for especificado, ele usará um conjunto de temas embutido.
Suponha que haja um tema chamado **ayu-dark**, você pode usá-lo com `-t ayu-dark`.
Se o nome do tema contiver espaços em branco, use aspas duplas ou simples ao redor dele.

> Observação: há um tema especial `None`, que representa o estilo de texto simples sem destaque de sintaxe.

Se `--theme` não for especificado, ele tentará ler os dados da variável de ambiente `$TOMLYRE_THEME`.
O tema de destaque de sintaxe nesta mensagem de ajuda também vem dessa **variável de ambiente**.

> O texto codificado rígido tem uma velocidade de carregamento muito rápida.  
> Carregar o texto codificado rígido e o texto com tema `None` é da mesma ordem de grandeza, enquanto carregar dinamicamente temas leva mais tempo.  
> Definir a variável de ambiente `$TOMLYRE_THEME` pode fazer com que o tema seja recarregado, o que diminui **significativamente** a velocidade de inicialização.  
> A menos que você pretenda modificar o tema das informações de ajuda ou desativar o destaque de sintaxe, use `-t` em vez de modificar a variável de ambiente.  

Exemplo usando variáveis de ambiente:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Exemplo usando parâmetros da linha de comando:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Para o conteúdo produzido manualmente por operações como "conv / get / set", etc., a prioridade do tema especificado por `-t` é maior que a variável de ambiente.
Se `-t` não for chamado, ele lerá da variável de ambiente. Se a variável de ambiente estiver vazia, usará **Monokai Extended**."##),
        ("theme", r##"Especifica o nome do tema para a sintaxe de destaque"##),
    ],
}
}

/// Language ID: pt;
/// Map name: "args_md";
/// Description: português, latim, Brasil;
pub(super) const fn get_pt_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifica um arquivo personalizado para o conjunto de temas. Você também precisa especificar o nome do tema com [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mGere um script de conclusão para o shell especificado[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mO formato do arquivo suportado é binário, e é o mesmo que o usado pelo rust crate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mEsses temas são essencialmente temas antigos do sublime.
[48;2;34;34;34m[38;2;255;255;255mSe [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m não for especificado, ele tentará ler os dados da variável de ambiente [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. [0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifique manualmente o formato do arquivo de origem (ele será detectado automaticamente por padrão)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mDesativa o plano de fundo do tema (ativado por padrão; se chamado com [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ele será desativado)[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A versão não compactada tem uma prioridade mais alta do que a versão compactada.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSe [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m não for especificado, ele tentará ler os dados da variável de ambiente [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. [0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mSe [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m não for especificado, ele tentará ler os dados da variável de ambiente [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSe o valor for falso, será desativado:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mSemelhante a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, mas requer dados de despejo (arquivo) não compactados[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mVocê precisa especificar manualmente o nome do shell para gerar um script de conclusão.
[48;2;34;34;34m[38;2;255;255;255mPor exemplo, para powershell no Windows, você pode usar: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m P: O que é a função de conclusão automática do shell?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m R: Quando digitamos comandos na linha de comando e pressionamos a tecla Tab, o script de conclusão automático é acionado e começa a pesquisar possíveis opções de conclusão, apresentando essas opções para escolha. Essa função pode aumentar muito a eficiência e precisão da entrada de linhas de comando.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOs scripts de conclusão para shells diferentes são armazenados em locais diferentes. Para zsh, você precisa salvá-lo no caminho incluído na matriz [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mVocê pode usar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m para obter o conteúdo dessa matriz.
[48;2;34;34;34m[38;2;255;255;255mSuponha que ele contenha o diretório [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, então você pode usar o seguinte comando para gravar o script de conclusão no arquivo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m nesse diretório.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPara evitar o uso de privilégios de root, você pode colocá-lo no subdiretório do usuário. Por exemplo, você pode inserir um elemento no início da matriz da seguinte maneira e usar o diretório relevante.
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mAo ler dados do stdin, por exemplo: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ele tentará automaticamente detectar o formato do arquivo primeiro.
[48;2;34;34;34m[38;2;255;255;255mNo entanto, a detecção automática pode falhar, e é quando você precisa especificar o formato manualmente.
[48;2;34;34;34m[38;2;255;255;255mExemplo: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mQuando usado apenas com [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ele lista os conjuntos de estilos internos.
[48;2;34;34;34m[38;2;255;255;255mExceto markdown que pode ser abreviado como md, todos os outros estilos devem usar o nome completo.
[48;2;34;34;34m[38;2;255;255;255mExemplo de uso 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExemplo de uso 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mObservação: Atualmente não é possível ler o conjunto de estilos das tabelas de um arquivo. Se você precisar dessa funcionalidade ou de outras funcionalidades adicionais, sinta-se à vontade para enviar um problema de feedback.[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifica o estilo da tabela[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mQuando usado apenas com [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ele lista os temas disponíveis atualmente.
[48;2;34;34;34m[38;2;255;255;255mSe nenhum arquivo de tema for especificado, ele usará um conjunto de temas embutido.
[48;2;34;34;34m[38;2;255;255;255mSuponha que haja um tema chamado [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, você pode usá-lo com [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mSe o nome do tema contiver espaços em branco, use aspas duplas ou simples ao redor dele.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Observação: há um tema especial [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, que representa o estilo de texto simples sem destaque de sintaxe.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSe [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m não for especificado, ele tentará ler os dados da variável de ambiente [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mO tema de destaque de sintaxe nesta mensagem de ajuda também vem dessa [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mvariável de ambiente[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m O texto codificado rígido tem uma velocidade de carregamento muito rápida.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Carregar o texto codificado rígido e o texto com tema [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m é da mesma ordem de grandeza, enquanto carregar dinamicamente temas leva mais tempo.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Definir a variável de ambiente [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m pode fazer com que o tema seja recarregado, o que diminui [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114msignificativamente[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m a velocidade de inicialização.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A menos que você pretenda modificar o tema das informações de ajuda ou desativar o destaque de sintaxe, use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m em vez de modificar a variável de ambiente.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExemplo usando variáveis de ambiente:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExemplo usando parâmetros da linha de comando:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPara o conteúdo produzido manualmente por operações como "conv / get / set", etc., a prioridade do tema especificado por [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m é maior que a variável de ambiente.
[48;2;34;34;34m[38;2;255;255;255mSe [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m não for chamado, ele lerá da variável de ambiente. Se a variável de ambiente estiver vazia, usará [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifica o nome do tema para a sintaxe de destaque[0m"##),
    ],
}
}

/// Language ID: pt;
/// Map name: "conv";
/// Description: português, latim, Brasil;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Converter entre formatos como toml, yaml, json e ron");
/// ```
pub(super) const fn get_pt_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Converter entre formatos como toml, yaml, json e ron"##),
        ("from", r##"Especifica o arquivo de entrada"##),
        ("conv-to", r##"Especifica o formato/arquivo de saída; pode ser usado várias vezes"##),
        ("save", r##"Quando esta opção for chamada, o arquivo especificado com `--to` será salvo automaticamente"##),
        ("conv-to-help", r##"Se houver um arquivo **test.json** no diretório atual, você pode usar `conv test.json -t yml` para convertê-lo em yml (yaml),
ou especificar manualmente o caminho do arquivo: `conv test.json -t /tmp/test.yml`

Os formatos suportados dependem das funcionalidades ativadas neste pacote e nem todas estarão ativadas por padrão.

Além disso, há algumas coisas que você precisa saber antes de usá-lo.
- 1. Ele renomeia automaticamente caminhos que não são arquivos específicos.
  - 1.1: Apenas arquivos com um nível serão tratados como um arquivo, não como um caminho
    Quando você usa `conv test.toml -t ron`, a saída será renomeada automaticamente para `test.ron`, não para `ron`.
    Se você não deseja essa funcionalidade, use `-t ./ron` ou `-t $PWD/ron`.
  - 1.2: Aponta para um diretório, não para um arquivo
    Por exemplo, `-t /tmp/`, podemos apenas colocar `test.toml` como um subdiretório de `/tmp/`, não sobrescrever o próprio `/tmp`.
    Se você não quiser esta funcionalidade, use o caminho completo do arquivo (não um diretório), por exemplo, `-t /tmp/xx.toml`, não `-t /tmp/`.
- 2. Quando `--save` não é chamado, o arquivo especificado não será salvo automaticamente.
- 3. Quando `--save` é usado, a entrada e saída do subcomando conv não podem ser o mesmo arquivo.
- 4. Os comentários não serão preservados após a conversão. "##),
    ],
}
}

/// Language ID: pt;
/// Map name: "conv_md";
/// Description: português, latim, Brasil;
pub(super) const fn get_pt_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mConverter entre formatos como toml, yaml, json e ron[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifica o arquivo de entrada[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mEspecifica o formato/arquivo de saída; pode ser usado várias vezes[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mQuando esta opção for chamada, o arquivo especificado com [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m será salvo automaticamente[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mSe houver um arquivo [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m no diretório atual, você pode usar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m para convertê-lo em yml (yaml),
[48;2;34;34;34m[38;2;255;255;255mou especificar manualmente o caminho do arquivo: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOs formatos suportados dependem das funcionalidades ativadas neste pacote e nem todas estarão ativadas por padrão.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mAlém disso, há algumas coisas que você precisa saber antes de usá-lo.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Ele renomeia automaticamente caminhos que não são arquivos específicos.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Apenas arquivos com um nível serão tratados como um arquivo, não como um caminho
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mQuando você usa [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, a saída será renomeada automaticamente para [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, não para [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mSe você não deseja essa funcionalidade, use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ou [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: Aponta para um diretório, não para um arquivo
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mPor exemplo, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, podemos apenas colocar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m como um subdiretório de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, não sobrescrever o próprio [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mSe você não quiser esta funcionalidade, use o caminho completo do arquivo (não um diretório), por exemplo, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, não [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. Quando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m não é chamado, o arquivo especificado não será salvo automaticamente.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. Quando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m é usado, a entrada e saída do subcomando conv não podem ser o mesmo arquivo.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. Os comentários não serão preservados após a conversão. [0m"##),
    ],
}
}

/// Language ID: pt;
/// Map name: "opt";
/// Description: português, latim, Brasil;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "表数组(用法：`--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_pt_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"A diferença entre `--ck` e `-k` é que `--ck` não divide a chave automaticamente com `split('.')`.  
Para [a.b.c], `-k a.b.c` e `--ck a --ck b --ck c` são iguais.  
Para casos simples, é mais conveniente usar `-k`.

Se o nome da chave contém '.', então use `--ck` em vez de `-k`.
Por exemplo:

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

Nessa estrutura, "editor.defaultFormatter" é uma string completa, `defaultFormatter` não é uma sub-chave de `editor`.

Quando você usa `-k '[typescript]'.'editor.defaultFormatter'`, ele dividirá a chave em:

- "[typescript]"
- "editor"
- "defaultFormatter"

Mas quando você usa `--ck '[typescript]' --ck editor.defaultFormatter`, ele considerará a chave como:

- "[typescript]"
- "editor.defaultFormatter"

Este último é a estrutura correta.
"##),
        ("num-arr", r##"Array de i64"##),
        ("bool", r##"bool (tipo booleano), somente pode ser `true` ou `false`"##),
        ("num", r##"i64: tipo inteiro de 64 bits com sinal, por exemplo: 1145"##),
        ("datetime-help", r##"Exemplo:
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

Este parâmetro só é suportado em `toml`. Se adicionarmos `-t json` no final do comando, o resultado seria:

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"Salva em um formato/arquivo especificado, apenas um pode ser salvo de cada vez"##),
        ("get-opts-help", r##"Exemplo de uso 1: `get Cargo.toml -k package.name`
Exemplo de uso 2: `get cfg.yaml -k .`
"##),
        ("none-help", r##"> Observação: o formato RON tem um tipo `Option<T>`, que pode ser `Some(T)` ou `None`.
> Aqui, `--none` se refere a remover a chave especificada, em vez do tipo `Option<T>`.

Suponha que tenhamos o seguinte array de tabelas em `test.yml`:

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

Se quisermos remover a chave **name: test**, podemos executar o seguinte comando:
```sh
set test.yml -k bin.1.name --none
```

> Observação: uma string vazia e `None` são coisas diferentes.
> Chamar `set test.yml --kv bin.1.name ""` definirá a chave como uma string vazia, enquanto `--none` removerá completamente a chave.

Se quisermos remover a segunda tabela de `bin` (índice 1), podemos executar:
```sh
set test.yml -k bin.1 --none
```

ou usar `--rm`
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"número de ponto flutuante duplo (padrão IEEE 754-2008), por exemplo: 3.14159265359"##),
        ("saot-help", r##"> Observação: embora usando `--saot` só seja possível criar um único par K-V por tabela, você pode primeiro especificar o índice do array da tabela e, em seguida, usar outras opções para expandi-lo.

Exemplo:
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"Definir um par chave-valor. `--kv x.y.z valor` é equivalente a `-k x.y.z -s valor`"##),
        ("set-save", r##"Se chamado sem `--to`, sobrepõe (escreve) o arquivo atual"##),
        ("array", r##"Array de strings"##),
        ("aot-help", r##"示例：
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

Selecione primeiro com a chave especificada e, em seguida, modifique com o valor do tipo de dados especificado.

> Quando apenas a chave é fornecida e nenhum valor é fornecido, ele exibe informações detalhadas do tipo de dados e do valor correspondente.
> Por exemplo, `set Cargo.toml -k package`, ele exibe a tabela package e suas sub-tabelas em uma tabela e fornece informações mais detalhadas do que 'get'.

## Exemplos

### Exemplo 1:

```sh
--ts md set test.toml --key package.edition --str 2021
```

Como `--str` é especificado em vez de `--num`, "2021" é do tipo de string, não de inteiro.

| package | new value |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### Exemplo 2:

```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### Exemplo 3:

```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: Este é o primeiro índice*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> Nota: os exemplos acima começam com argumentos/subcomandos, omitindo o nome do comando raiz. Na operação real, você precisa digitá-lo manualmente no início."##),
        ("num-arr-help", r##"Exemplo:
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"Exemplo:
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> Exceto em `toml`, outros formatos usarão tabelas padrão, não internas.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"Consultar um valor através da chave (key)"##),
        ("f64-arr-help", r##"Exemplo:
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> JSON 1.0 não suporta NaN e Inf

Ele sairá como `null`, não `NaN`.

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```"##),
        ("none", r##"Remove a chave especificada"##),
        ("set-save-to-help", r##"Muito parecido com `-t` no subcomando `conv`, a maior diferença é que `-t` em `conv` pode ser chamado várias vezes seguidas, gerando saída em vários formatos/arquivos.
Enquanto `-t` no subcomando `set` só pode ser chamado uma vez, gerando saída em um único arquivo.

> Nota: Após a execução da operação de conversão do subcomando `conv`, todos os arquivos em diferentes formatos não preservarão os comentários.
> Enquanto o subcomando `set` manterá os comentários do **toml**, exceto o **toml**, onde nenhum dos outros formatos manterá os comentários.

- Sem chamar `-t`, chamar `--save` sobrescreverá o próprio arquivo.
- Se já foi chamado `-t` e também foi chamado `--save`, o arquivo modificado será salvo no arquivo apontado por `-t`.
- A menos que o arquivo de origem seja o mesmo que o apontado por `-t`, o comportamento de `-t` no subcomando `set` será o mesmo que o de `-t` no subcomando `conv`. Eles renomearão automaticamente os caminhos de arquivo não específicos.
  - Assumindo que from: **test.yml**, to: **json**, o arquivo de saída se tornará **test.json**, não **json**
  - Se você não precisar dessa funcionalidade, use `-t ./json` ou `-t $PWD/json`, não `-t json`.
"##),
        ("preview", r##"Visualizar o arquivo inteiro após a modificação"##),
        ("set-key", r##"Selecionar uma chave específica com "." como separador"##),
        ("get-opts", r##"Obter o valor da configuração especificada"##),
        ("saot", r##"Array de tabelas com um único par chave-valor (Uso: `--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"O formato é: "a.b.c", nome da sub tabela ou índice do array é conectado por ".".
Quando há apenas um ".", ele exibirá a tabela inteira."##),
        ("set-opts", r##"Criar/Atualizar/Excluir a estrutura do arquivo de configuração"##),
        ("bool-arr-help", r##"示例：
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: 第1个元素是 true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"bool 的数组"##),
        ("table", r##"Tabela padrão, usada para armazenar vários pares K-V"##),
        ("datetime", r##"Tipo de dados para data e hora"##),
        ("table-help", r##"Exemplo 1:
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

Exemplo 2:
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"Array de f64"##),
        ("string", r##"String (tipo de string)"##),
        ("array-help", r##"Defina elementos do array chamando `-a` várias vezes.
Exemplo:
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"表数组(用法：`--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"Tabela interna, equivalente a uma tabela padrão mais compacta"##),
        ("concat-key", r##"Introduzir a chave em várias partes e o resultado será concatenado automaticamente, sem usar "." como separador"##),
    ],
}
}

/// Language ID: pt;
/// Map name: "opt_md";
/// Description: português, latim, Brasil;
pub(super) const fn get_pt_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255mA diferença entre [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m e [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m é que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m não divide a chave automaticamente com [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msplit('.')[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPara [a.b.c], [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m e [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m são iguais.[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mPara casos simples, é mais conveniente usar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSe o nome da chave contém '.', então use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m em vez de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mPor exemplo:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mNessa estrutura, "editor.defaultFormatter" é uma string completa, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mdefaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m não é uma sub-chave de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255meditor[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mQuando você usa [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ele dividirá a chave em:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mMas quando você usa [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ele considerará a chave como:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEste último é a estrutura correta.
[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mArray de i64[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mbool (tipo booleano), somente pode ser [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ou [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: tipo inteiro de 64 bits com sinal, por exemplo: 1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEste parâmetro só é suportado em [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Se adicionarmos [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m no final do comando, o resultado seria:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255mSalva em um formato/arquivo especificado, apenas um pode ser salvo de cada vez[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemplo de uso 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExemplo de uso 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Observação: o formato RON tem um tipo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, que pode ser [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m ou [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Aqui, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m se refere a remover a chave especificada, em vez do tipo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSuponha que tenhamos o seguinte array de tabelas em [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSe quisermos remover a chave [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mname: test[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, podemos executar o seguinte comando:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Observação: uma string vazia e [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m são coisas diferentes.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Chamar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m definirá a chave como uma string vazia, enquanto [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m removerá completamente a chave.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSe quisermos remover a segunda tabela de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mbin[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m (índice 1), podemos executar:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mou usar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255mnúmero de ponto flutuante duplo (padrão IEEE 754-2008), por exemplo: 3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Observação: embora usando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m só seja possível criar um único par K-V por tabela, você pode primeiro especificar o índice do array da tabela e, em seguida, usar outras opções para expandi-lo.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mDefinir um par chave-valor. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z valor[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m é equivalente a [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s valor[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255mSe chamado sem [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, sobrepõe (escreve) o arquivo atual[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255mArray de strings[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSelecione primeiro com a chave especificada e, em seguida, modifique com o valor do tipo de dados especificado.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Quando apenas a chave é fornecida e nenhum valor é fornecido, ele exibe informações detalhadas do tipo de dados e do valor correspondente.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Por exemplo, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, ele exibe a tabela package e suas sub-tabelas em uma tabela e fornece informações mais detalhadas do que 'get'.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExemplos[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExemplo 1:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mComo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m é especificado em vez de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, "2021" é do tipo de string, não de inteiro.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m new value [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExemplo 2:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExemplo 3:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: Este é o primeiro índice*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: os exemplos acima começam com argumentos/subcomandos, omitindo o nome do comando raiz. Na operação real, você precisa digitá-lo manualmente no início.[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Exceto em [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, outros formatos usarão tabelas padrão, não internas.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mConsultar um valor através da chave (key)[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m JSON 1.0 não suporta NaN e Inf
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEle sairá como [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, não [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255mRemove a chave especificada[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mMuito parecido com [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m no subcomando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, a maior diferença é que [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m em [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m pode ser chamado várias vezes seguidas, gerando saída em vários formatos/arquivos.
[48;2;34;34;34m[38;2;255;255;255mEnquanto [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m no subcomando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m só pode ser chamado uma vez, gerando saída em um único arquivo.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Nota: Após a execução da operação de conversão do subcomando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, todos os arquivos em diferentes formatos não preservarão os comentários.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Enquanto o subcomando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m manterá os comentários do [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m, exceto o [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m, onde nenhum dos outros formatos manterá os comentários.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mSem chamar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, chamar [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sobrescreverá o próprio arquivo.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Se já foi chamado [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m e também foi chamado [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, o arquivo modificado será salvo no arquivo apontado por [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m A menos que o arquivo de origem seja o mesmo que o apontado por [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, o comportamento de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m no subcomando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m será o mesmo que o de [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m no subcomando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Eles renomearão automaticamente os caminhos de arquivo não específicos.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Assumindo que from: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, to: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, o arquivo de saída se tornará [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, não [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Se você não precisar dessa funcionalidade, use [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ou [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, não [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255mVisualizar o arquivo inteiro após a modificação[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255mSelecionar uma chave específica com "." como separador[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255mObter o valor da configuração especificada[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255mArray de tabelas com um único par chave-valor (Uso: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mO formato é: "a.b.c", nome da sub tabela ou índice do array é conectado por ".".
[48;2;34;34;34m[38;2;255;255;255mQuando há apenas um ".", ele exibirá a tabela inteira.[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255mCriar/Atualizar/Excluir a estrutura do arquivo de configuração[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: 第1个元素是 true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mbool 的数组[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255mTabela padrão, usada para armazenar vários pares K-V[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255mTipo de dados para data e hora[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255mExemplo 1:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mExemplo 2:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mArray de f64[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString (tipo de string)[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255mDefina elementos do array chamando [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m várias vezes.
[48;2;34;34;34m[38;2;255;255;255mExemplo:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255m表数组(用法：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mTabela interna, equivalente a uma tabela padrão mais compacta[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mIntroduzir a chave em várias partes e o resultado será concatenado automaticamente, sem usar "." como separador[0m"##),
    ],
}
}

/// Language ID: pt;
/// Map name: "parser";
/// Description: português, latim, Brasil;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "Abaixo está a saída stderr. Ao redirecionar a saída do script de conclusão, você deve usar `>/caminho/para/o/arquivo` em vez de `2>/caminho/para/o/arquivo`.");
/// ```
pub(super) const fn get_pt_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"Abaixo está a saída stderr. Ao redirecionar a saída do script de conclusão, você deve usar `>/caminho/para/o/arquivo` em vez de `2>/caminho/para/o/arquivo`."##),
    ],
}
}

/// Language ID: pt;
/// Map name: "theme";
/// Description: português, latim, Brasil;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nome de tema inválido");
/// ```
pub(super) const fn get_pt_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nome de tema inválido"##),
    ],
}
}

/// pt: português, latim, Brasil
pub(super) const fn get_pt_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_pt_map_args),
        ("conv_md", get_pt_map_conv_md),
        ("parser", get_pt_map_parser),
        ("opt_md", get_pt_map_opt_md),
        ("args_md", get_pt_map_args_md),
        ("opt", get_pt_map_opt),
        ("log-core", get_pt_map_log_core),
        ("conv", get_pt_map_conv),
        ("theme", get_pt_map_theme),
    ],
}
}

/// Language ID: ro;
/// Map name: "log-core";
/// Description: română, latină, România;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "depanare");
/// ```
pub(super) const fn get_ro_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"depanare"##),
        ("warn", r##"avertizează"##),
        ("info", r##"informații"##),
        ("error", r##"eroare"##),
        ("trace", r##"urmă"##),
        ("init-logger", r##"Inițializarea loggerului ..."##),
    ],
}
}

/// Language ID: ro;
/// Map name: "theme";
/// Description: română, latină, România;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Nume temă nevalid");
/// ```
pub(super) const fn get_ro_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Nume temă nevalid"##),
    ],
}
}

/// ro: română, latină, România
pub(super) const fn get_ro_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ro_map_log_core),
        ("theme", get_ro_map_theme),
    ],
}
}

/// Language ID: ru;
/// Map name: "log-core";
/// Description: русский, кириллица, Россия;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "отладка");
/// ```
pub(super) const fn get_ru_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"отладка"##),
        ("warn", r##"предупреждать"##),
        ("info", r##"Информация"##),
        ("error", r##"ошибка"##),
        ("trace", r##"трассировка"##),
        ("init-logger", r##"Инициализация регистратора ..."##),
    ],
}
}

/// Language ID: ru;
/// Map name: "theme";
/// Description: русский, кириллица, Россия;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Неверное название темы");
/// ```
pub(super) const fn get_ru_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Неверное название темы"##),
    ],
}
}

/// Language ID: ru;
/// Map name: "args";
/// Description: русский, кириллица, Россия;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Отключить фон для темы (по умолчанию включено, если использовать `--dis-theme-bg`, то будет отключено)");
/// ```
pub(super) const fn get_ru_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"Файл, содержащий пользовательские темы. Пользователь должен указать также имя темы через `--theme`"##),
        ("shell-completion", r##"Создание сценария для автодополнения оболочки"##),
        ("theme-file-help", r##"Поддерживаемый формат файлов - двоичные данные, созданные в результате выполнения dump-операции из библиотеки rust crate `syntect`.
Такие темы в основном представляют собой темы для старых версий Sublime.
Если параметр `--tf` не указан, то будет прочитано значение переменной окружения `$TOMLYRE_THEME_FILE`."##),
        ("from-format", r##"Ручное определение формата исходного файла (обычно автоматически определяется)"##),
        ("disable-theme-background", r##"Отключить фон для темы (по умолчанию включено, если использовать `--dis-theme-bg`, то будет отключено)"##),
        ("theme-uncompressed-file-help", r##"> Несжатые версии имеют более высокий приоритет, чем сжатые.

Если параметр `--tuf` не указан, то будет прочитано значение переменной окружения `$TOMLYRE_THEME_UNCOMPRESSED_FILE`."##),
        ("disable-theme-background-help", r##"Если не указан параметр `--dis-theme-bg`, то он будет прочитан из переменной окружения `$TOMLYRE_THEME_BG`.

Если значение равно `false`, то фон отключен.

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"Файл, содержащий пользовательские темы в негруппированном формате (не сжатый файл)"##),
        ("shell-completion-help", r##"Необходимо вручную указать имя используемой оболочки для генерации скрипта автодополнения.
Например, для powershell на Windows можно использовать: `--sh-comp pwsh`

> Q: Что такое автодополнение в оболочке?

> A: Когда мы вводим команду в командной строке и нажимаем клавишу Tab, запускается скрипт автодополнения, который начинает поиск возможных вариантов автодополнения и отображает их для выбора. Эта функция значительно повышает эффективность и точность ввода команд.

Скрипты автодополнения различных оболочек располагаются в разных местах. Для zsh необходимо поместить скрипт в путь, указанный в массиве `$fpath`.
Вы можете использовать `print -l $fpath`, чтобы получить содержимое этого массива.
Предположим, что он содержит каталог `/usr/local/share/zsh/site-functions`. Тогда вы можете использовать следующую команду, чтобы записать скрипт автодополнения в файл `_tomlyre` в этом каталоге.

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

Чтобы избежать использования прав root, вы можете поместить его в подкаталог пользователя. Например, вы можете использовать следующую команду, чтобы добавить новый элемент в начало массива:
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"Когда данные читаются из stdin, например: `cat a | tomlyre conv -`, система автоматически пытается определить формат файла.
Однако автоматическое определение может завершиться неудачей, в таком случае вы должны указать формат вручную.
Пример использования: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Если использовать параметр `--ts` без дополнительных значений, то будут выведены доступные стили таблиц.
За исключением markdown, который можно сократить как md, остальные стили должны быть записаны полностью.
Пример использования 1:`--ts md`
Пример использования 2: `--table-style ascii`

Примечание: в настоящее время невозможно загрузить коллекцию **стилей таблиц** из файла. Если вам нужна эта функция или что-то еще, пожалуйста, сообщите об этом, создав issue."##),
        ("table-style", r##"Определение стиля таблицы"##),
        ("theme-help", r##"Если использовать только параметр `-t`, то будет выведен список доступных тем.
Если не указан файл темы, то будет использоваться встроенный набор тем.
Если существует тема **ayu-dark**, то можно использовать `-t ayu-dark` для выбора этой темы.
Если название темы содержит пробелы, то ее нужно заключить в двойные или одинарные кавычки.

> Примечание: есть специальная тема `None`, которая обозначает отсутствие подсветки синтаксиса для чистого текста.

Если опция `--theme` не указана, то она будет прочитана из переменной окружения `$TOMLYRE_THEME`.
Тема подсветки синтаксиса в этом справочном сообщении также происходит из этой **переменной окружения**.

> Стандартное справочное сообщение закодировано жестко, поэтому оно загружается очень быстро.  
> Загрузка кодированного текста и загрузка текста без подсветки синтаксиса `None` выполняются с одинаковой скоростью, а динамическая загрузка будет медленнее на два и более порядков.  
> Установка переменной окружения `$TOMLYRE_THEME` может привести к перезагрузке темы, что **сильно** замедлит скорость запуска.  
> Если вы не собираетесь изменять тему подсветки для сообщений о помощи или отключать подсветку синтаксиса, то используйте `-t`, а не изменяйте переменную окружения.

Пример использования переменной окружения:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Пример использования параметров командной строки:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Для вывода содержимого, получаемого при ручном выполнении операций "conv/get/set" и т. д., приоритет имеет тема, определенная посредством `-t`, а затем переменная окружения.
Если не вызывать `-t`, то будет использоваться переменная окружения. Если значение переменной окружения пусто, то будет использоваться тема **Monokai Extended**."##),
        ("theme", r##"Название темы, определяющей подсветку синтаксиса"##),
    ],
}
}

/// Language ID: ru;
/// Map name: "args_md";
/// Description: русский, кириллица, Россия;
pub(super) const fn get_ru_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mФайл, содержащий пользовательские темы. Пользователь должен указать также имя темы через [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mСоздание сценария для автодополнения оболочки[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mПоддерживаемый формат файлов - двоичные данные, созданные в результате выполнения dump-операции из библиотеки rust crate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mТакие темы в основном представляют собой темы для старых версий Sublime.
[48;2;34;34;34m[38;2;255;255;255mЕсли параметр [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m не указан, то будет прочитано значение переменной окружения [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mРучное определение формата исходного файла (обычно автоматически определяется)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mОтключить фон для темы (по умолчанию включено, если использовать [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, то будет отключено)[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Несжатые версии имеют более высокий приоритет, чем сжатые.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mЕсли параметр [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m не указан, то будет прочитано значение переменной окружения [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mЕсли не указан параметр [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, то он будет прочитан из переменной окружения [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mЕсли значение равно [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, то фон отключен.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mФайл, содержащий пользовательские темы в негруппированном формате (не сжатый файл)[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mНеобходимо вручную указать имя используемой оболочки для генерации скрипта автодополнения.
[48;2;34;34;34m[38;2;255;255;255mНапример, для powershell на Windows можно использовать: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q: Что такое автодополнение в оболочке?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: Когда мы вводим команду в командной строке и нажимаем клавишу Tab, запускается скрипт автодополнения, который начинает поиск возможных вариантов автодополнения и отображает их для выбора. Эта функция значительно повышает эффективность и точность ввода команд.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mСкрипты автодополнения различных оболочек располагаются в разных местах. Для zsh необходимо поместить скрипт в путь, указанный в массиве [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mВы можете использовать [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, чтобы получить содержимое этого массива.
[48;2;34;34;34m[38;2;255;255;255mПредположим, что он содержит каталог [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Тогда вы можете использовать следующую команду, чтобы записать скрипт автодополнения в файл [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m в этом каталоге.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mЧтобы избежать использования прав root, вы можете поместить его в подкаталог пользователя. Например, вы можете использовать следующую команду, чтобы добавить новый элемент в начало массива:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mКогда данные читаются из stdin, например: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, система автоматически пытается определить формат файла.
[48;2;34;34;34m[38;2;255;255;255mОднако автоматическое определение может завершиться неудачей, в таком случае вы должны указать формат вручную.
[48;2;34;34;34m[38;2;255;255;255mПример использования: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mЕсли использовать параметр [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m без дополнительных значений, то будут выведены доступные стили таблиц.
[48;2;34;34;34m[38;2;255;255;255mЗа исключением markdown, который можно сократить как md, остальные стили должны быть записаны полностью.
[48;2;34;34;34m[38;2;255;255;255mПример использования 1:[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПример использования 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПримечание: в настоящее время невозможно загрузить коллекцию [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mстилей таблиц[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m из файла. Если вам нужна эта функция или что-то еще, пожалуйста, сообщите об этом, создав issue.[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mОпределение стиля таблицы[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mЕсли использовать только параметр [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, то будет выведен список доступных тем.
[48;2;34;34;34m[38;2;255;255;255mЕсли не указан файл темы, то будет использоваться встроенный набор тем.
[48;2;34;34;34m[38;2;255;255;255mЕсли существует тема [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, то можно использовать [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m для выбора этой темы.
[48;2;34;34;34m[38;2;255;255;255mЕсли название темы содержит пробелы, то ее нужно заключить в двойные или одинарные кавычки.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Примечание: есть специальная тема [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, которая обозначает отсутствие подсветки синтаксиса для чистого текста.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mЕсли опция [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m не указана, то она будет прочитана из переменной окружения [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mТема подсветки синтаксиса в этом справочном сообщении также происходит из этой [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mпеременной окружения[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Стандартное справочное сообщение закодировано жестко, поэтому оно загружается очень быстро.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Загрузка кодированного текста и загрузка текста без подсветки синтаксиса [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m выполняются с одинаковой скоростью, а динамическая загрузка будет медленнее на два и более порядков.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Установка переменной окружения [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m может привести к перезагрузке темы, что [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mсильно[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m замедлит скорость запуска.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Если вы не собираетесь изменять тему подсветки для сообщений о помощи или отключать подсветку синтаксиса, то используйте [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, а не изменяйте переменную окружения.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПример использования переменной окружения:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПример использования параметров командной строки:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mДля вывода содержимого, получаемого при ручном выполнении операций "conv/get/set" и т. д., приоритет имеет тема, определенная посредством [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, а затем переменная окружения.
[48;2;34;34;34m[38;2;255;255;255mЕсли не вызывать [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, то будет использоваться переменная окружения. Если значение переменной окружения пусто, то будет использоваться тема [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mНазвание темы, определяющей подсветку синтаксиса[0m"##),
    ],
}
}

/// Language ID: ru;
/// Map name: "conv";
/// Description: русский, кириллица, Россия;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Конвертация между форматами toml, yaml, json и ron");
/// ```
pub(super) const fn get_ru_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Конвертация между форматами toml, yaml, json и ron"##),
        ("from", r##"Указывает входной файл"##),
        ("conv-to", r##"Указывает выходной формат/файл, можно использовать несколько раз"##),
        ("save", r##"При вызове этого параметра файл, указанный в `--to`, будет сохранен автоматически"##),
        ("conv-to-help", r##"Если в текущем каталоге существует файл **test.json**, вы можете использовать `conv test.json -t yml` для преобразования его в формат yml (yaml).
Также можно задать путь к файлу вручную: `conv test.json -t /tmp/test.yml`

Поддерживаемые форматы зависят от активированных функций этого пакета, они не будут все по умолчанию активированы.

Кроме того, перед использованием вам необходимо знать следующее:
- 1. Он переименовывает не конкретные пути к файлам.
  - 1.1: Файл будет распознан как формат файла, а не путь к файлу, если он содержит только один уровень
    Если вызвать `conv test.toml -t ron`, то имя выходного файла автоматически станет `test.ron`, а не `ron`.
    Если вам это не нужно, используйте `-t ./ron` или `-t $PWD/ron`.
  - 1.2: Ссылка на директорию, а не файл
    Например, `-t /tmp/`, мы можем поместить `test.toml` только в дочерний каталог `/tmp/`, но не находящийся в `/tmp`.
    Если вам это не нужно, используйте полный путь к файлу (не к директории), например `-t /tmp/xx.toml`, а не `-t /tmp/`.
- 2. Если параметр `--save` не вызывается, указанный файл не будет автоматически сохранен.
- 3. При вызове `--save` входной и выходной файлы conv-команды не могут быть одним и тем же файлом.
- 4. В конечном файле не сохраняются комментарии."##),
    ],
}
}

/// Language ID: ru;
/// Map name: "conv_md";
/// Description: русский, кириллица, Россия;
pub(super) const fn get_ru_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mКонвертация между форматами toml, yaml, json и ron[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mУказывает входной файл[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mУказывает выходной формат/файл, можно использовать несколько раз[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mПри вызове этого параметра файл, указанный в [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, будет сохранен автоматически[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mЕсли в текущем каталоге существует файл [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, вы можете использовать [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m для преобразования его в формат yml (yaml).
[48;2;34;34;34m[38;2;255;255;255mТакже можно задать путь к файлу вручную: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПоддерживаемые форматы зависят от активированных функций этого пакета, они не будут все по умолчанию активированы.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mКроме того, перед использованием вам необходимо знать следующее:
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Он переименовывает не конкретные пути к файлам.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Файл будет распознан как формат файла, а не путь к файлу, если он содержит только один уровень
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mЕсли вызвать [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, то имя выходного файла автоматически станет [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, а не [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mЕсли вам это не нужно, используйте [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m или [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: Ссылка на директорию, а не файл
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mНапример, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, мы можем поместить [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m только в дочерний каталог [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, но не находящийся в [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mЕсли вам это не нужно, используйте полный путь к файлу (не к директории), например [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, а не [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. Если параметр [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m не вызывается, указанный файл не будет автоматически сохранен.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. При вызове [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m входной и выходной файлы conv-команды не могут быть одним и тем же файлом.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. В конечном файле не сохраняются комментарии.[0m"##),
    ],
}
}

/// Language ID: ru;
/// Map name: "opt";
/// Description: русский, кириллица, Россия;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "Массив таблиц (использование: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_ru_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"В отличие от `-k`, `--ck` не разбивает ключ на подстроки с помощью `split('.')`.

Для [a.b.c] `-k a.b.c` и `--ck a --ck b --ck c` эквивалентны.
Для простых случаев использование `-k` будет удобнее.

Если имя ключа содержит "." , то в таком случае используйте `--ck` вместо `-k`.
Например:

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

В этой структуре "editor.defaultFormatter" - это полная строка, а "defaultFormatter" не является подключением (sub-key) "editor".

При использовании `-k '[typescript]'.'editor.defaultFormatter'` он разделится на:

- "[typescript]"
- "editor"
- "defaultFormatter"

А при использовании `--ck '[typescript]' --ck editor.defaultFormatter`, он будет интерпретирован как:

- "[typescript]"
- "editor.defaultFormatter"

Последний вариант является корректной структурой. "##),
        ("num-arr", r##"Массив i64"##),
        ("bool", r##"bool (булевый тип), может быть только `true` или `false`"##),
        ("num", r##"i64: 64-битное знаковое целочисленное значение, например: 1145"##),
        ("datetime-help", r##"Пример:
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

Этот параметр доступен только в формате `toml`. Если мы добавим `-t json` в конце команды, то результат будет выглядеть так:

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"Сохранить в указанном формате/файле, одновременно можно сохранить только один формат"##),
        ("get-opts-help", r##"Пример использования 1: `get Cargo.toml -k package.name`
Пример использования 2: `get cfg.yaml -k .`
"##),
        ("none-help", r##"> Примечание: в `ron` есть настоящий тип `Option<T>`, который может быть либо `Some(T)`, либо `None`.
> Здесь `--none` означает удаление указанного ключа, а не настоящий тип `Option<T>`.

Предположим, что у нас есть следующий массив таблиц в `test.yml`:

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

Чтобы удалить **name: test**, мы можем выполнить следующую команду:
```sh
set test.yml -k bin.1.name --none
```

> Примечание: пустая строка и `None` - это не одно и то же.
> Вызов `set test.yml --kv bin.1.name ""` установит значение ключа на пустую строку, а `--none` удалит ключ целиком.

Чтобы удалить вторую таблицу массива `bin` (с индексом 1), мы можем выполнить:
```sh
set test.yml -k bin.1 --none
```

Или использовать `--rm`
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"Двойная точность с плавающей запятой (стандарт IEEE 754-2008), например: 3.14159265359"##),
        ("saot-help", r##"> Примечание: хотя через `--saot` можно создать только одну пару K-V для каждой таблицы, вы можете указать индексы массива таблиц заранее и расширить их с помощью других параметров.

Пример:
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"Установить пару ключ-значение, `--kv x.y.z value` эквивалентно`-k x.y.z -s value`"##),
        ("set-save", r##"Если вызывается без параметра `--to`, то перезаписывает(записывает) текущий файл"##),
        ("array", r##"Массив строк"##),
        ("aot-help", r##"Пример:
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

Сначала выберите ключ, а затем измените его, задав тип данных значения.

> Если вы предоставляете только ключ, а не значение, то будут перечислены соответствующие типы данных и значения.
> Например, `set Cargo.toml -k package` выведет таблицу package и ее дочерних таблиц и предоставит более подробную информацию, чем `get`.

## Примеры

### Пример 1:
```sh
--ts md set test.toml --key package.edition --str 2021
```

Поскольку указано `--str`, а не `--num`, "2021" является строковым значением, а не целочисленным.

| package | новое значение |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### Пример 2:
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### Пример 3:
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: это первый индекс*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> Примечание: примеры начинаются с аргумента/подкоманды, опуская имя корневой команды для краткости.

"##),
        ("num-arr-help", r##"Пример:
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"Пример:
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> За исключением `toml`, другие форматы будут использовать стандартную таблицу вместо встроенной.

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"Запрос значения по ключу"##),
        ("f64-arr-help", r##"Пример:
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```
```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> Формат JSON 1.0 не поддерживает NaN и Inf.
> Он выводит `null`, а не `NaN`.

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"Удалить указанный ключ"##),
        ("set-save-to-help", r##"Очень похож на `-t` из подкоманды `conv`, но отличие заключается в том, что `-t` из `conv` можно вызывать многократно, чтобы вывести в несколько форматов/файлов. 
Тогда как `-t` из `set` вызывается только один раз и выводит в один файл.

> Примечание: после выполнения операции преобразования в `conv`, все файлы теряют свои комментарии.
> В то время как `set` сохраняет комментарии **только** для формата **toml**, а все остальные форматы **не** сохраняют комментарии.

- Если параметр `-t` не вызывается, то использование `--save` перезаписывает файл.
- Если уже был вызван `-t`, и далее вызывается `--save`, то измененный файл будет сохранен в файл, на который указывает `-t`.
- Если исходный файл и файл, на который указывает `-t`, отличаются, то поведение параметра `-t` из подкоманды `set` будет аналогичным `-t` из подкоманды `conv`. В этом случае у них есть возможность автоматически переименовать файлы, кроме тех, которые являются конкретными путями к файлам.
  - Предположим, что from: **test.yml**, to: **json**, тогда выходной файл станет **test.json**, а не **json**
  - Если вам не нужна эта функция, используйте `-t ./json` или `-t $PWD/json`, а не `-t json`.
"##),
        ("preview", r##"Просмотр полного файла после внесения изменений"##),
        ("set-key", r##"Выбрать указанный ключ (ключ), используя "." в качестве разделителя"##),
        ("get-opts", r##"Получить значение определенной настройки"##),
        ("saot", r##"Таблица с одной парой ключ-значение (использование: `--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"Формат ключа: "a.b.c", где "." используется для указания подтаблицы или индекса массива.
Если при запросе ключа задать только одну точку ".", то будут перечислены все таблицы."##),
        ("set-opts", r##"Создание/обновление/удаление структуры файла настроек"##),
        ("bool-arr-help", r##"Пример:
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: Первый элемент - true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"Массив bool"##),
        ("table", r##"Стандартная таблица (standard table), используется для хранения множества пар K-V"##),
        ("datetime", r##"Дата и время"##),
        ("table-help", r##"Пример 1:
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

Пример 2:
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"Массив f64"##),
        ("string", r##"String (строковый тип)"##),
        ("array-help", r##"Установка элементов массива производится через многократный вызов параметра `-a`.
Пример:
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> Первый вызов устанавливает элемент с индексом 0, второй - элемент с индексом 1 и так далее.

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"Массив таблиц (использование: `--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"Встроенная таблица, эквивалентна более компактной стандартной таблице"##),
        ("concat-key", r##"Ввод key по частям, результат будет автоматически объединен без использования "." в качестве разделителя"##),
    ],
}
}

/// Language ID: ru;
/// Map name: "opt_md";
/// Description: русский, кириллица, Россия;
pub(super) const fn get_ru_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255mВ отличие от [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m не разбивает ключ на подстроки с помощью [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msplit('.')[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mДля [a.b.c] [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m и [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m эквивалентны.
[48;2;34;34;34m[38;2;255;255;255mДля простых случаев использование [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m будет удобнее.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mЕсли имя ключа содержит "." , то в таком случае используйте [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m вместо [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mНапример:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mВ этой структуре "editor.defaultFormatter" - это полная строка, а "defaultFormatter" не является подключением (sub-key) "editor".
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПри использовании [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m он разделится на:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mА при использовании [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, он будет интерпретирован как:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПоследний вариант является корректной структурой. [0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mМассив i64[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mbool (булевый тип), может быть только [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m или [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: 64-битное знаковое целочисленное значение, например: 1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255mПример:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mЭтот параметр доступен только в формате [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Если мы добавим [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m в конце команды, то результат будет выглядеть так:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255mСохранить в указанном формате/файле, одновременно можно сохранить только один формат[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255mПример использования 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПример использования 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Примечание: в [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m есть настоящий тип [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, который может быть либо [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, либо [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Здесь [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m означает удаление указанного ключа, а не настоящий тип [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПредположим, что у нас есть следующий массив таблиц в [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mЧтобы удалить [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mname: test[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, мы можем выполнить следующую команду:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Примечание: пустая строка и [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m - это не одно и то же.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Вызов [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m установит значение ключа на пустую строку, а [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m удалит ключ целиком.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mЧтобы удалить вторую таблицу массива [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mbin[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m (с индексом 1), мы можем выполнить:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mИли использовать [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255mДвойная точность с плавающей запятой (стандарт IEEE 754-2008), например: 3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Примечание: хотя через [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m можно создать только одну пару K-V для каждой таблицы, вы можете указать индексы массива таблиц заранее и расширить их с помощью других параметров.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПример:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255mУстановить пару ключ-значение, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m эквивалентно[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255mЕсли вызывается без параметра [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, то перезаписывает(записывает) текущий файл[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255mМассив строк[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255mПример:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mСначала выберите ключ, а затем измените его, задав тип данных значения.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Если вы предоставляете только ключ, а не значение, то будут перечислены соответствующие типы данных и значения.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Например, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m выведет таблицу package и ее дочерних таблиц и предоставит более подробную информацию, чем [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mПримеры[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mПример 1:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПоскольку указано [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, а не [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, "2021" является строковым значением, а не целочисленным.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m новое значение [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mПример 2:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mПример 3:[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: это первый индекс*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Примечание: примеры начинаются с аргумента/подкоманды, опуская имя корневой команды для краткости.
[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mПример:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255mПример:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m За исключением [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, другие форматы будут использовать стандартную таблицу вместо встроенной.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255mЗапрос значения по ключу[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mПример:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Формат JSON 1.0 не поддерживает NaN и Inf.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Он выводит [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, а не [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255mУдалить указанный ключ[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mОчень похож на [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m из подкоманды [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, но отличие заключается в том, что [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m из [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m можно вызывать многократно, чтобы вывести в несколько форматов/файлов. 
[48;2;34;34;34m[38;2;255;255;255mТогда как [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m из [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m вызывается только один раз и выводит в один файл.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Примечание: после выполнения операции преобразования в [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, все файлы теряют свои комментарии.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m В то время как [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m сохраняет комментарии [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mтолько[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m для формата [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m, а все остальные форматы [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mне[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m сохраняют комментарии.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mЕсли параметр [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m не вызывается, то использование [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m перезаписывает файл.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Если уже был вызван [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, и далее вызывается [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, то измененный файл будет сохранен в файл, на который указывает [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Если исходный файл и файл, на который указывает [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, отличаются, то поведение параметра [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m из подкоманды [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m будет аналогичным [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m из подкоманды [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. В этом случае у них есть возможность автоматически переименовать файлы, кроме тех, которые являются конкретными путями к файлам.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Предположим, что from: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, to: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, тогда выходной файл станет [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, а не [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m Если вам не нужна эта функция, используйте [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m или [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, а не [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255mПросмотр полного файла после внесения изменений[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255mВыбрать указанный ключ (ключ), используя "." в качестве разделителя[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255mПолучить значение определенной настройки[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255mТаблица с одной парой ключ-значение (использование: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255mФормат ключа: "a.b.c", где "." используется для указания подтаблицы или индекса массива.
[48;2;34;34;34m[38;2;255;255;255mЕсли при запросе ключа задать только одну точку ".", то будут перечислены все таблицы.[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255mСоздание/обновление/удаление структуры файла настроек[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255mПример:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: Первый элемент - true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mМассив bool[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255mСтандартная таблица (standard table), используется для хранения множества пар K-V[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255mДата и время[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255mПример 1:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mПример 2:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mМассив f64[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString (строковый тип)[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255mУстановка элементов массива производится через многократный вызов параметра [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mПример:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Первый вызов устанавливает элемент с индексом 0, второй - элемент с индексом 1 и так далее.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255mМассив таблиц (использование: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255mВстроенная таблица, эквивалентна более компактной стандартной таблице[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255mВвод key по частям, результат будет автоматически объединен без использования "." в качестве разделителя[0m"##),
    ],
}
}

/// Language ID: ru;
/// Map name: "parser";
/// Description: русский, кириллица, Россия;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "Ниже приведен вывод stderr. При перенаправлении вывода для скрипта автодополнения вы должны использовать `>/путь/к/файлу`, а не `2>/путь/к/файлу`.");
/// ```
pub(super) const fn get_ru_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"Ниже приведен вывод stderr. При перенаправлении вывода для скрипта автодополнения вы должны использовать `>/путь/к/файлу`, а не `2>/путь/к/файлу`."##),
    ],
}
}

/// ru: русский, кириллица, Россия
pub(super) const fn get_ru_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_ru_map_args),
        ("conv_md", get_ru_map_conv_md),
        ("parser", get_ru_map_parser),
        ("opt_md", get_ru_map_opt_md),
        ("args_md", get_ru_map_args_md),
        ("opt", get_ru_map_opt),
        ("log-core", get_ru_map_log_core),
        ("conv", get_ru_map_conv),
        ("theme", get_ru_map_theme),
    ],
}
}

/// Language ID: sd;
/// Map name: "log-core";
/// Description: سنڌي, عربي, پاڪستان;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ڊيبگ");
/// ```
pub(super) const fn get_sd_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ڊيبگ"##),
        ("warn", r##"ڊي warn اريندڙ"##),
        ("info", r##"info اڻ"##),
        ("error", r##"غلطي"##),
        ("trace", r##"نشان"##),
        ("init-logger", r##"لوگر شروع ڪرڻ ..."##),
    ],
}
}

/// Language ID: sd;
/// Map name: "theme";
/// Description: سنڌي, عربي, پاڪستان;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "غلط موضوع جو نالو");
/// ```
pub(super) const fn get_sd_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"غلط موضوع جو نالو"##),
    ],
}
}

/// sd: سنڌي, عربي, پاڪستان
pub(super) const fn get_sd_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_sd_map_log_core),
        ("theme", get_sd_map_theme),
    ],
}
}

/// Language ID: si;
/// Map name: "log-core";
/// Description: සිංහල, සිංහල, ශ්‍රී ලංකාව;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "න\u{dd2}දොස\u{dca} ක\u{dd2}ර\u{dd3}ම");
/// ```
pub(super) const fn get_si_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"නිදොස් කිරීම"##),
        ("warn", r##"අනතුරු ඇඟවීම"##),
        ("info", r##"තොරතුරු"##),
        ("error", r##"දෝෂයකි"##),
        ("trace", r##"හෝඩුවාව"##),
        ("init-logger", r##"ලොගර් ආරම්භ කිරීම ..."##),
    ],
}
}

/// Language ID: si;
/// Map name: "theme";
/// Description: සිංහල, සිංහල, ශ්‍රී ලංකාව;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "වලංග\u{dd4} නොවන තේම\u{dcf} නම");
/// ```
pub(super) const fn get_si_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"වලංගු නොවන තේමා නම"##),
    ],
}
}

/// si: සිංහල, සිංහල, ශ්‍රී ලංකාව
pub(super) const fn get_si_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_si_map_log_core),
        ("theme", get_si_map_theme),
    ],
}
}

/// Language ID: sk;
/// Map name: "log-core";
/// Description: slovenčina, latinka, Slovensko;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ladenie");
/// ```
pub(super) const fn get_sk_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ladenie"##),
        ("warn", r##"varovať"##),
        ("info", r##"Informácie"##),
        ("error", r##"chyba"##),
        ("trace", r##"stopa"##),
        ("init-logger", r##"Inicializácia Loggeru ..."##),
    ],
}
}

/// Language ID: sk;
/// Map name: "theme";
/// Description: slovenčina, latinka, Slovensko;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Neplatný názov témy");
/// ```
pub(super) const fn get_sk_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Neplatný názov témy"##),
    ],
}
}

/// sk: slovenčina, latinka, Slovensko
pub(super) const fn get_sk_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_sk_map_log_core),
        ("theme", get_sk_map_theme),
    ],
}
}

/// Language ID: sl;
/// Map name: "log-core";
/// Description: slovenščina, latinica, Slovenija;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "odpravljanje napak");
/// ```
pub(super) const fn get_sl_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"odpravljanje napak"##),
        ("warn", r##"Opozori"##),
        ("info", r##"informacije"##),
        ("error", r##"Napaka"##),
        ("trace", r##"Trace"##),
        ("init-logger", r##"Inicializacija zapisovanja ..."##),
    ],
}
}

/// Language ID: sl;
/// Map name: "theme";
/// Description: slovenščina, latinica, Slovenija;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Neveljavno ime teme");
/// ```
pub(super) const fn get_sl_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Neveljavno ime teme"##),
    ],
}
}

/// sl: slovenščina, latinica, Slovenija
pub(super) const fn get_sl_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_sl_map_log_core),
        ("theme", get_sl_map_theme),
    ],
}
}

/// Language ID: sm;
/// Map name: "log-core";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_sm_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"lapatai"##),
        ("info", r##"faʻamatalaga"##),
        ("error", r##"sese"##),
        ("trace", r##"trace"##),
        ("init-logger", r##"Amataga i le Logger ..."##),
    ],
}
}

/// Language ID: sm;
/// Map name: "theme";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Le aoga le igoa autu");
/// ```
pub(super) const fn get_sm_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Le aoga le igoa autu"##),
    ],
}
}

/// sm: sm-Latn-WS
pub(super) const fn get_sm_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_sm_map_log_core),
        ("theme", get_sm_map_theme),
    ],
}
}

/// Language ID: sn;
/// Map name: "log-core";
/// Description: chiShona, Latn, Zimbabwe;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_sn_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"Inyambira"##),
        ("info", r##"info"##),
        ("error", r##"kukanganisa"##),
        ("trace", r##"trace"##),
        ("init-logger", r##"Kutanga logger ..."##),
    ],
}
}

/// Language ID: sn;
/// Map name: "theme";
/// Description: chiShona, Latn, Zimbabwe;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Izita risingabvumirwe zita");
/// ```
pub(super) const fn get_sn_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Izita risingabvumirwe zita"##),
    ],
}
}

/// sn: chiShona, Latn, Zimbabwe
pub(super) const fn get_sn_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_sn_map_log_core),
        ("theme", get_sn_map_theme),
    ],
}
}

/// Language ID: so;
/// Map name: "log-core";
/// Description: Soomaali, Laatiin, Soomaaliya;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Bax");
/// ```
pub(super) const fn get_so_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Bax"##),
        ("warn", r##"Dig"##),
        ("info", r##"Macluumaadka"##),
        ("error", r##"qaladka"##),
        ("trace", r##"raad raac"##),
        ("init-logger", r##"Bilowga Logger ..."##),
    ],
}
}

/// Language ID: so;
/// Map name: "theme";
/// Description: Soomaali, Laatiin, Soomaaliya;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Magaca mawduuca khaldan");
/// ```
pub(super) const fn get_so_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Magaca mawduuca khaldan"##),
    ],
}
}

/// Language ID: so;
/// Map name: "args";
/// Description: Soomaali, Laatiin, Soomaaliya;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Aan u diiwaan gelin magaca muhimka ah (asal ahaan loo isticmaalayo; haddii `--dis-theme-bg` laga soo saaray, marka la diiwaan geliyo).");
/// ```
pub(super) const fn get_so_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"faylka cradka ee xogta la shaqeeyay, waxaad sidoo kale u baahan tahay inaad ku taliso magaca muhimka."##),
        ("shell-completion", r##"Laga bilaabo shell-ga ah oo ku shaqeeya xaqiijinta"##),
        ("theme-file-help", r##"Qaababka faylasha ee la gudbiyay wuxuu ah in uu ka kooban yahay xogta dhabta ah ee bin kale la shaqeeyay.  
Dhammaadka ah, midigtaas waa mid ka mid ah main menu-ka **sublime**.
Haddii aadan adeegsan `--tf`, waxaa loo akhriyaa macluumaadkan `$TOMLYRE_THEME_FILE` ee milieu gaar ah."##),
        ("from-format", r##"Qiimaha dhawrka soo dirka ah ee faylka asaliga ah ( waxaa loo baahan yahay in loo adeegsado qaabka sii soconaya)"##),
        ("disable-theme-background", r##"Aan u diiwaan gelin magaca muhimka ah (asal ahaan loo isticmaalayo; haddii `--dis-theme-bg` laga soo saaray, marka la diiwaan geliyo)."##),
        ("theme-uncompressed-file-help", r##"> Faah-faahintii sare ee la samayay faylasha aanay compress geliyay, sideedana xayiraaya sida macluumaadkan lagu bilawdaami karo.
Haddii maqlayo `-tuf`, waxaa loo akhriyaa macluumaadkan `$TOMLYRE_THEME_UNCOMPRESSED_FILE` ee milieu gaar ah."##),
        ("disable-theme-background-help", r##"Haddii maqlayo `--dis-theme-bg`, waxaa loo akhriyaa macluumaadkan `$TOMLYRE_THEME_BG` ee milieu gaar ah.

Haddii qiimaheeda ka yar yahay midigtaas, wuxuu ku diiwaan gashaa

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"Waxaad isticmaali kartaa faylka shaqo u qaadan ee la shaqeeyay, oo kaliya haddii aanay compress geliyay."##),
        ("shell-completion-help", r##"Waxaad u baahan tahay inaad soo jeediyaan magaca shell-ga si aad ugu shaqeyso xaqiijinta.
Tusaale sida, powershell ee Windows -ku, waxaad adeegsan kartaa: `--sh-comp pwsh`

> Q: Maxaa la bilaabayo xaqiijinta shell?

> A: marka aanu dhiganno amarada websaydhka, haddii waa Hubkiisa, haddii aan hoggaaminnoo qorshaha kooxaha, ama haddii aan mas'uuliyad tirobad leh si fiican u diyaariyeen habka guuleysiga iyo saxda awooda sare weyn ee hawlgalada. Waxaa suuragal ah in ay ka baxaan saameynada amarnaanta command-lineka uma aragtidaba.

Calaamadka xaqiijinta hal-abuuraha ee shell-ga ah waxaa la keydiyey meelaha kala duwan. Haddii aanu yeelano zsh, waxaad u baahan tahay inaad ku xirto dhamaan jidka `$fpath`. Waxaad adeegsataa `print -l $fpath` si aad u heshid xogtaas.
Haddii jidka ku jira `/usr/local/share/zsh/site-functions` directory, tusaale sidaas dartaa ka isticmaalo sawirada dhanka hoos ee lagu soo saarayo in ay diiwaan galiyaan faylka `_tomlyre`

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```
Si loo talagalay in la adeegsan karo aqoon gaar ah, waxaad ka fiican tahay inaad sameeso hoyga shaqada.

```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"Marka aad ka akhrisan kartid dakhliga stdin-ka, sidaas darteed haddii aadan soo dirin isticmaalaha ama formatka faylka sax ah.
Sida loogu baahan yahay: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Marka aad adeegsato `--ts` kaliya, waxaa loo qoray liiska midigta ah ee qaabka asaliga ah.
Isbedelka miisaanka markdown fadlan qaabeel md-ka sidaas darteed waxaa loo baahan yahay magac buuxaan.
Sida looga baahan yahay:
kaliya1: `--ts md`
kaliya2: `--table-style ascii`

Fikirka: Haddii aad u baahan tahay faah-faahin dheeraad ah oo kugu caawiyey ama waxaad rabtaa wax khatar ah oo kale, fadlan dib u eeg arrimaha sare. "##),
        ("table-style", r##"Soo jeedinta qaabka miisaanka"##),
        ("theme-help", r##"Marka ay ka isticmaalaan paramada `-t` oo kaliya, waxaa loo qoray liiska ardayda hadda jira ee midigta ah.
Haddii magacaagana aan lagu talin faylka ah, waxaad adeegsataa collection-ka asaliga ah. Haddii liiska jira xulasho **ayu-dark**, marka aad doonto `-t ayu-dark`, waxaad adeegsataa midigtaas.
Haddii magaca luuqada ku jiro meelaha, fadlan adeegsada dabin ama naagtan.

> Fikirkii: Midigta aan la mid ahayn `None`, wuxuu dhacmeysaa habka illao dhammaystiran ee qoraalka sax ah

Marka aadan adeegsan opshinka `--theme`, waxaa loo akhriyaa macluumaadkani `$TOMLYRE_THEME` ee milieu gaar ah.
Magac muhimka ee **syntax-highlighting**-ka ee macluumaadkan ayaa kalaba la xayeysiinaysa in lagu samaysto **milieu-goobta**.

> Macluumaadkan aasaaska ah wuxuu ah in uu noqdo khariidada dhabta ah, sidaas darteed waxaa loogu talagalay.  
> Xiriirka macluumaadkan iyo `None`-ka ee syntax-highlighting-ka waa midka qaarkood in ay tahay.
> Marka aad isu diiwaangashan kartid magaca miisaanka ah ee `$TOMLYRE_THEME`, waxaad qabtaa qarsoodiga daarana waxaa lagu soo saari karaa midigta si uu u kala faa'idi doono.

Ficilka adeegsada milieu-goobta:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Ficilka adeegsada xallinteeda:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Haddii aadan adeegsan `-t` oo kaliya, waxaa loo akhriyaa macluumaadkan milieu-goobta ah. Haddii milieu-gaalka aan waxba ka jirin, waxaad adeegsataa **Monokai Extended**.

Tusaaleeyaha ka hor adeegsiga `conv/get/set` iyo calaamado kale oo la sameeyey, magac muhimka ee `-t` wuxuu awood u leh > magaca milieu-gaalka.  
Marka aan la adeegsan `-t`, waxaan akhriyaa milieu-gaalkan; haddii milieu-gaalku aanay mid ahayn, waxaad adeegsataa **Monokai Extended**."##),
        ("theme", r##"Magaca muhimka ee syntax-highlighting-ka"##),
    ],
}
}

/// Language ID: so;
/// Map name: "args_md";
/// Description: Soomaali, Laatiin, Soomaaliya;
pub(super) const fn get_so_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mfaylka cradka ee xogta la shaqeeyay, waxaad sidoo kale u baahan tahay inaad ku taliso magaca muhimka.[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mLaga bilaabo shell-ga ah oo ku shaqeeya xaqiijinta[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mQaababka faylasha ee la gudbiyay wuxuu ah in uu ka kooban yahay xogta dhabta ah ee bin kale la shaqeeyay.[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDhammaadka ah, midigtaas waa mid ka mid ah main menu-ka [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114msublime[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255mHaddii aadan adeegsan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, waxaa loo akhriyaa macluumaadkan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ee milieu gaar ah.[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mQiimaha dhawrka soo dirka ah ee faylka asaliga ah ( waxaa loo baahan yahay in loo adeegsado qaabka sii soconaya)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mAan u diiwaan gelin magaca muhimka ah (asal ahaan loo isticmaalayo; haddii [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m laga soo saaray, marka la diiwaan geliyo).[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Faah-faahintii sare ee la samayay faylasha aanay compress geliyay, sideedana xayiraaya sida macluumaadkan lagu bilawdaami karo.
[48;2;34;34;34m[38;2;102;217;239mHaddii maqlayo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, waxaa loo akhriyaa macluumaadkan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m ee milieu gaar ah.[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255mHaddii maqlayo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, waxaa loo akhriyaa macluumaadkan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ee milieu gaar ah.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mHaddii qiimaheeda ka yar yahay midigtaas, wuxuu ku diiwaan gashaa
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255mWaxaad isticmaali kartaa faylka shaqo u qaadan ee la shaqeeyay, oo kaliya haddii aanay compress geliyay.[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mWaxaad u baahan tahay inaad soo jeediyaan magaca shell-ga si aad ugu shaqeyso xaqiijinta.
[48;2;34;34;34m[38;2;255;255;255mTusaale sida, powershell ee Windows -ku, waxaad adeegsan kartaa: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q: Maxaa la bilaabayo xaqiijinta shell?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: marka aanu dhiganno amarada websaydhka, haddii waa Hubkiisa, haddii aan hoggaaminnoo qorshaha kooxaha, ama haddii aan mas'uuliyad tirobad leh si fiican u diyaariyeen habka guuleysiga iyo saxda awooda sare weyn ee hawlgalada. Waxaa suuragal ah in ay ka baxaan saameynada amarnaanta command-lineka uma aragtidaba.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mCalaamadka xaqiijinta hal-abuuraha ee shell-ga ah waxaa la keydiyey meelaha kala duwan. Haddii aanu yeelano zsh, waxaad u baahan tahay inaad ku xirto dhamaan jidka [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m. Waxaad adeegsataa [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m si aad u heshid xogtaas.
[48;2;34;34;34m[38;2;255;255;255mHaddii jidka ku jira [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m directory, tusaale sidaas dartaa ka isticmaalo sawirada dhanka hoos ee lagu soo saarayo in ay diiwaan galiyaan faylka [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mSi loo talagalay in la adeegsan karo aqoon gaar ah, waxaad ka fiican tahay inaad sameeso hoyga shaqada.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mMarka aad ka akhrisan kartid dakhliga stdin-ka, sidaas darteed haddii aadan soo dirin isticmaalaha ama formatka faylka sax ah.
[48;2;34;34;34m[38;2;255;255;255mSida loogu baahan yahay: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mMarka aad adeegsato [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m kaliya, waxaa loo qoray liiska midigta ah ee qaabka asaliga ah.
[48;2;34;34;34m[38;2;255;255;255mIsbedelka miisaanka markdown fadlan qaabeel md-ka sidaas darteed waxaa loo baahan yahay magac buuxaan.
[48;2;34;34;34m[38;2;255;255;255mSida looga baahan yahay:
[48;2;34;34;34m[38;2;255;255;255mkaliya1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mkaliya2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mFikirka: Haddii aad u baahan tahay faah-faahin dheeraad ah oo kugu caawiyey ama waxaad rabtaa wax khatar ah oo kale, fadlan dib u eeg arrimaha sare. [0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mSoo jeedinta qaabka miisaanka[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mMarka ay ka isticmaalaan paramada [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m oo kaliya, waxaa loo qoray liiska ardayda hadda jira ee midigta ah.
[48;2;34;34;34m[38;2;255;255;255mHaddii magacaagana aan lagu talin faylka ah, waxaad adeegsataa collection-ka asaliga ah. Haddii liiska jira xulasho [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, marka aad doonto [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, waxaad adeegsataa midigtaas.
[48;2;34;34;34m[38;2;255;255;255mHaddii magaca luuqada ku jiro meelaha, fadlan adeegsada dabin ama naagtan.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Fikirkii: Midigta aan la mid ahayn [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, wuxuu dhacmeysaa habka illao dhammaystiran ee qoraalka sax ah
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mMarka aadan adeegsan opshinka [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, waxaa loo akhriyaa macluumaadkani [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ee milieu gaar ah.
[48;2;34;34;34m[38;2;255;255;255mMagac muhimka ee [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114msyntax-highlighting[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m-ka ee macluumaadkan ayaa kalaba la xayeysiinaysa in lagu samaysto [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mmilieu-goobta[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Macluumaadkan aasaaska ah wuxuu ah in uu noqdo khariidada dhabta ah, sidaas darteed waxaa loogu talagalay.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Xiriirka macluumaadkan iyo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m-ka ee syntax-highlighting-ka waa midka qaarkood in ay tahay.
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Marka aad isu diiwaangashan kartid magaca miisaanka ah ee [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m, waxaad qabtaa qarsoodiga daarana waxaa lagu soo saari karaa midigta si uu u kala faa'idi doono.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mFicilka adeegsada milieu-goobta:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mFicilka adeegsada xallinteeda:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mHaddii aadan adeegsan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m oo kaliya, waxaa loo akhriyaa macluumaadkan milieu-goobta ah. Haddii milieu-gaalka aan waxba ka jirin, waxaad adeegsataa [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTusaaleeyaha ka hor adeegsiga [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv/get/set[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m iyo calaamado kale oo la sameeyey, magac muhimka ee [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m wuxuu awood u leh > magaca milieu-gaalka.[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mMarka aan la adeegsan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, waxaan akhriyaa milieu-gaalkan; haddii milieu-gaalku aanay mid ahayn, waxaad adeegsataa [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mMagaca muhimka ee syntax-highlighting-ka[0m"##),
    ],
}
}

/// Language ID: so;
/// Map name: "conv";
/// Description: Soomaali, Laatiin, Soomaaliya;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "Toml, yamal, json iyo ron waxay u shaqeynayaan foomaatada qoraalka");
/// ```
pub(super) const fn get_so_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"Toml, yamal, json iyo ron waxay u shaqeynayaan foomaatada qoraalka"##),
        ("from", r##"Faylka soo socda la sheeg"##),
        ("conv-to", r##"Foomka/faylka soo saarida ee la shaqeeya, oo ka taageero badan"##),
        ("save", r##"Marka la adeegsan yahay, faylka ayuu ku kaydsan doonaa `--to` wax ka beddelka."##),
        ("conv-to-help", r##"Haddii horey loo heli karo **test.json** faylka hadda jira, marka aad qabto `conv test.json -t yml`, waxaad u shaqeyn kartaa yml (yaml) foomka,
oo aad kaladuwan isaga sugi kartaa faylka: `conv test.json -t /tmp/test.yml`

Foomkaha ayaa la doonayo qaababka ku xeeran ee adeegsigaan, sidaas darteed ma waxaad iloobi kartaa dhammaan dhinaca xogta.
- 1. Waxay samaysaa magac uruurinta faylasha aan lahayn wax kasta oo loo yaqaan, adigoo aan musuqmaasuq ahayn
  - 1.1: Fashilka mid abuurka ah ayaa loo dhiso file format ahaan, maadaama looma baahnid inay yihiin jidhka faylka
    Marka laga reebo `conv test.toml -t ron`, faylka tayo leh wuxuu la badalaa `test.ron`, sidaasna ma wuxuu noqon doono `ron`.
    Haddii aadan u baahan inaad adeegsato foomkan, fadlan adeegsada `-t ./ron`, ama `-t $PWD/ron`.
  - 1.2: Talaabooyinku waxay la xiriira ka mid ah folder ah
    Tusaale, `-t /tmp/`, waa inaanan helin faylka `test.toml` oo aan lagu magacaabin dadka `/tmp/`, laakiinse lama marin doono `/tmp` sida dhanka fog.
    Haddii aadan u baahan inaad adeegsato talaabooyinkaas, fadlan adeegsada file fudud (maahi folder) sida `-t /tmp/xx.toml`, ama `-t /tmp/`.
- 2. Marka aadan adeegsan `--save`, faylka la shaqeeyo qaybinta ma noqon doono.
- 3. Marka aadan adeegsan `--save`, ma haayo faylka soo socda iyo ee lacagta uga yar ee la shaqeeyaa.
- 4. Foomkan la shaqeeyaa kaliya, muhiimada xogta laga saaro mayno."##),
    ],
}
}

/// Language ID: so;
/// Map name: "conv_md";
/// Description: Soomaali, Laatiin, Soomaaliya;
pub(super) const fn get_so_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mToml, yamal, json iyo ron waxay u shaqeynayaan foomaatada qoraalka[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mFaylka soo socda la sheeg[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mFoomka/faylka soo saarida ee la shaqeeya, oo ka taageero badan[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255mMarka la adeegsan yahay, faylka ayuu ku kaydsan doonaa [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m wax ka beddelka.[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mHaddii horey loo heli karo [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m faylka hadda jira, marka aad qabto [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, waxaad u shaqeyn kartaa yml (yaml) foomka,
[48;2;34;34;34m[38;2;255;255;255moo aad kaladuwan isaga sugi kartaa faylka: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mFoomkaha ayaa la doonayo qaababka ku xeeran ee adeegsigaan, sidaas darteed ma waxaad iloobi kartaa dhammaan dhinaca xogta.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Waxay samaysaa magac uruurinta faylasha aan lahayn wax kasta oo loo yaqaan, adigoo aan musuqmaasuq ahayn
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Fashilka mid abuurka ah ayaa loo dhiso file format ahaan, maadaama looma baahnid inay yihiin jidhka faylka
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mMarka laga reebo [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, faylka tayo leh wuxuu la badalaa [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, sidaasna ma wuxuu noqon doono [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mHaddii aadan u baahan inaad adeegsato foomkan, fadlan adeegsada [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ama [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: Talaabooyinku waxay la xiriira ka mid ah folder ah
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mTusaale, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, waa inaanan helin faylka [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m oo aan lagu magacaabin dadka [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, laakiinse lama marin doono [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m sida dhanka fog.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mHaddii aadan u baahan inaad adeegsato talaabooyinkaas, fadlan adeegsada file fudud (maahi folder) sida [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ama [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. Marka aadan adeegsan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, faylka la shaqeeyo qaybinta ma noqon doono.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. Marka aadan adeegsan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ma haayo faylka soo socda iyo ee lacagta uga yar ee la shaqeeyaa.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. Foomkan la shaqeeyaa kaliya, muhiimada xogta laga saaro mayno.[0m"##),
    ],
}
}

/// so: Soomaali, Laatiin, Soomaaliya
pub(super) const fn get_so_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
        (4, 2),
    ],
    entries: &[
        ("conv", get_so_map_conv),
        ("log-core", get_so_map_log_core),
        ("conv_md", get_so_map_conv_md),
        ("theme", get_so_map_theme),
        ("args", get_so_map_args),
        ("args_md", get_so_map_args_md),
    ],
}
}

/// Language ID: sq;
/// Map name: "log-core";
/// Description: shqip, latin, Shqipëri;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debugimi");
/// ```
pub(super) const fn get_sq_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debugimi"##),
        ("warn", r##"Paralajmëroni"##),
        ("info", r##"Informacioni"##),
        ("error", r##"Gabim"##),
        ("trace", r##"Gjurmë"##),
        ("init-logger", r##"Inicializimi i loggerit ..."##),
    ],
}
}

/// Language ID: sq;
/// Map name: "theme";
/// Description: shqip, latin, Shqipëri;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Emri i pavlefshëm i temës");
/// ```
pub(super) const fn get_sq_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Emri i pavlefshëm i temës"##),
    ],
}
}

/// sq: shqip, latin, Shqipëri
pub(super) const fn get_sq_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_sq_map_log_core),
        ("theme", get_sq_map_theme),
    ],
}
}

/// Language ID: sr;
/// Map name: "log-core";
/// Description: српски, ћирилица, Србија;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "уклањање погрешака");
/// ```
pub(super) const fn get_sr_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"уклањање погрешака"##),
        ("warn", r##"упозорите"##),
        ("info", r##"Информације о"##),
        ("error", r##"Грешка"##),
        ("trace", r##"траг"##),
        ("init-logger", r##"Иницијализација логгер ..."##),
    ],
}
}

/// Language ID: sr;
/// Map name: "theme";
/// Description: српски, ћирилица, Србија;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Неважеће име теме");
/// ```
pub(super) const fn get_sr_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Неважеће име теме"##),
    ],
}
}

/// sr: српски, ћирилица, Србија
pub(super) const fn get_sr_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_sr_map_log_core),
        ("theme", get_sr_map_theme),
    ],
}
}

/// Language ID: st;
/// Map name: "log-core";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_st_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"warn"##),
        ("info", r##"Info"##),
        ("error", r##"phoso"##),
        ("trace", r##"trace"##),
        ("init-logger", r##"Ho qala ka mokopu ..."##),
    ],
}
}

/// Language ID: st;
/// Map name: "theme";
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Lebitso la sehlooho la sehlooho");
/// ```
pub(super) const fn get_st_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Lebitso la sehlooho la sehlooho"##),
    ],
}
}

/// st: st-Latn-ZA
pub(super) const fn get_st_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_st_map_log_core),
        ("theme", get_st_map_theme),
    ],
}
}

/// Language ID: su;
/// Map name: "log-core";
/// Description: Basa Sunda, Latin, Indonesia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "debug");
/// ```
pub(super) const fn get_su_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"debug"##),
        ("warn", r##"ngingetkeun"##),
        ("info", r##"info"##),
        ("error", r##"Kasalahan"##),
        ("trace", r##"Renal"##),
        ("init-logger", r##"NAMPANGKEUN"##),
    ],
}
}

/// Language ID: su;
/// Map name: "theme";
/// Description: Basa Sunda, Latin, Indonesia;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ngaran tema henteu leres");
/// ```
pub(super) const fn get_su_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ngaran tema henteu leres"##),
    ],
}
}

/// su: Basa Sunda, Latin, Indonesia
pub(super) const fn get_su_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_su_map_log_core),
        ("theme", get_su_map_theme),
    ],
}
}

/// Language ID: sv;
/// Map name: "log-core";
/// Description: svenska, latinska, Sverige;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "felsökning");
/// ```
pub(super) const fn get_sv_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"felsökning"##),
        ("warn", r##"varna"##),
        ("info", r##"info"##),
        ("error", r##"fel"##),
        ("trace", r##"spår"##),
        ("init-logger", r##"Initialisering av logger ..."##),
    ],
}
}

/// Language ID: sv;
/// Map name: "theme";
/// Description: svenska, latinska, Sverige;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Ogiltigt temamamn");
/// ```
pub(super) const fn get_sv_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Ogiltigt temamamn"##),
    ],
}
}

/// sv: svenska, latinska, Sverige
pub(super) const fn get_sv_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_sv_map_log_core),
        ("theme", get_sv_map_theme),
    ],
}
}

/// Language ID: sw;
/// Map name: "log-core";
/// Description: Kiswahili, Kilatini, Tanzania;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_sw_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"Onyo"##),
        ("info", r##"Maelezo"##),
        ("error", r##"kosa"##),
        ("trace", r##"kufuatilia"##),
        ("init-logger", r##"Kuanzisha Logger ..."##),
    ],
}
}

/// Language ID: sw;
/// Map name: "theme";
/// Description: Kiswahili, Kilatini, Tanzania;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Jina batili la mandhari");
/// ```
pub(super) const fn get_sw_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Jina batili la mandhari"##),
    ],
}
}

/// sw: Kiswahili, Kilatini, Tanzania
pub(super) const fn get_sw_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_sw_map_log_core),
        ("theme", get_sw_map_theme),
    ],
}
}

/// Language ID: ta;
/// Map name: "log-core";
/// Description: தமிழ், தமிழ், இந்தியா;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "பிழைத\u{bcd}திருத\u{bcd}தம\u{bcd}");
/// ```
pub(super) const fn get_ta_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"பிழைத்திருத்தம்"##),
        ("warn", r##"எச்சரிக்கை"##),
        ("info", r##"தகவல்"##),
        ("error", r##"பிழை"##),
        ("trace", r##"சுவடு"##),
        ("init-logger", r##"லாகரைத் தொடங்குதல் ..."##),
    ],
}
}

/// Language ID: ta;
/// Map name: "theme";
/// Description: தமிழ், தமிழ், இந்தியா;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "தவற\u{bbe}ன த\u{bc0}ம\u{bcd} பெயர\u{bcd}");
/// ```
pub(super) const fn get_ta_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"தவறான தீம் பெயர்"##),
    ],
}
}

/// ta: தமிழ், தமிழ், இந்தியா
pub(super) const fn get_ta_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ta_map_log_core),
        ("theme", get_ta_map_theme),
    ],
}
}

/// Language ID: te;
/// Map name: "log-core";
/// Description: తెలుగు, తెలుగు, భారతదేశం;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "డ\u{c40}బగ\u{c4d}");
/// ```
pub(super) const fn get_te_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"డీబగ్"##),
        ("warn", r##"హెచ్చరించండి"##),
        ("info", r##"సమాచారం"##),
        ("error", r##"లోపం"##),
        ("trace", r##"ట్రేస్"##),
        ("init-logger", r##"లాగర్ ప్రారంభించడం ..."##),
    ],
}
}

/// Language ID: te;
/// Map name: "theme";
/// Description: తెలుగు, తెలుగు, భారతదేశం;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "చ\u{c46}ల\u{c4d}లన\u{c3f} థ\u{c40}మ\u{c4d} ప\u{c47}రు");
/// ```
pub(super) const fn get_te_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"చెల్లని థీమ్ పేరు"##),
    ],
}
}

/// te: తెలుగు, తెలుగు, భారతదేశం
pub(super) const fn get_te_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_te_map_log_core),
        ("theme", get_te_map_theme),
    ],
}
}

/// Language ID: tg;
/// Map name: "log-core";
/// Description: тоҷикӣ, Кириллӣ, Тоҷикистон;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_tg_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"Огоҳӣ"##),
        ("info", r##"Маълумот"##),
        ("error", r##"Хатогӣ"##),
        ("trace", r##"Пай"##),
        ("init-logger", r##"Оғози система ..."##),
    ],
}
}

/// Language ID: tg;
/// Map name: "theme";
/// Description: тоҷикӣ, Кириллӣ, Тоҷикистон;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Номи мавзӯи беэътибор");
/// ```
pub(super) const fn get_tg_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Номи мавзӯи беэътибор"##),
    ],
}
}

/// tg: тоҷикӣ, Кириллӣ, Тоҷикистон
pub(super) const fn get_tg_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_tg_map_log_core),
        ("theme", get_tg_map_theme),
    ],
}
}

/// Language ID: th;
/// Map name: "log-core";
/// Description: ไทย, ไทย, ไทย;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "การด\u{e35}บ\u{e31}ก");
/// ```
pub(super) const fn get_th_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"การดีบัก"##),
        ("warn", r##"เตือน"##),
        ("info", r##"ข้อมูล"##),
        ("error", r##"ข้อผิดพลาด"##),
        ("trace", r##"การติดตาม"##),
        ("init-logger", r##"เริ่มต้น Logger ..."##),
    ],
}
}

/// Language ID: th;
/// Map name: "theme";
/// Description: ไทย, ไทย, ไทย;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "ช\u{e37}\u{e48}อธ\u{e35}มไม\u{e48}ถ\u{e39}กต\u{e49}อง");
/// ```
pub(super) const fn get_th_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"ชื่อธีมไม่ถูกต้อง"##),
    ],
}
}

/// th: ไทย, ไทย, ไทย
pub(super) const fn get_th_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_th_map_log_core),
        ("theme", get_th_map_theme),
    ],
}
}

/// Language ID: tr;
/// Map name: "log-core";
/// Description: Türkçe, Latin, Türkiye;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Hata ayıklama");
/// ```
pub(super) const fn get_tr_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Hata ayıklama"##),
        ("warn", r##"Uyar"##),
        ("info", r##"Bilgi"##),
        ("error", r##"Hatası"##),
        ("trace", r##"İzleme"##),
        ("init-logger", r##"Logger'ı başlatma ..."##),
    ],
}
}

/// Language ID: tr;
/// Map name: "theme";
/// Description: Türkçe, Latin, Türkiye;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Geçersiz Tema Adı");
/// ```
pub(super) const fn get_tr_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Geçersiz Tema Adı"##),
    ],
}
}

/// Language ID: tr;
/// Map name: "args";
/// Description: Türkçe, Latin, Türkiye;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "Tema arka planını devre dışı bırakın (varsayılan olarak etkin durumdadır, `--dis-theme-bg` kullanıldığında devre dışı bırakılır)");
/// ```
pub(super) const fn get_tr_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"Özel tema dosyasını belirtin, ayrıca `--theme` ile tema adını belirtmeniz gerekir"##),
        ("shell-completion", r##"Shell için tamamlama betiği oluşturun"##),
        ("theme-file-help", r##"Desteklenen dosya biçimi, rust paketi olan `syntect` tarafından dökülen ikili verilerdir.
Temelde, bu temalar eski bir sublime temasıdır.
`--tf` belirtilmediyse, `$TOMLYRE_THEME_FILE` ortam değişkeninin değeri okunur."##),
        ("from-format", r##"Kaynak dosya biçimini manuel belirtin (varsayılan olarak otomatik tespit edilir)"##),
        ("disable-theme-background", r##"Tema arka planını devre dışı bırakın (varsayılan olarak etkin durumdadır, `--dis-theme-bg` kullanıldığında devre dışı bırakılır)"##),
        ("theme-uncompressed-file-help", r##"> Sıkıştırılmamış sürüm, sıkıştırılmış sürümden daha yüksek önceliğe sahiptir.  

`--tuf` belirtilmediyse, `$TOMLYRE_THEME_UNCOMPRESSED_FILE` ortam değişkeninin değeri okunur."##),
        ("disable-theme-background-help", r##"`--dis-theme-bg` belirtilmediyse, `$TOMLYRE_THEME_BG` ortam değişkeninin değeri okunur.

Eğer bu değer false ise devre dışı bırakılacaktır.
Örneğin:

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"`theme-file`'a benzer, ancak sıkıştırılmamış dump verileri (dosya) gerektirir"##),
        ("shell-completion-help", r##"Tamamlama betiğini oluşturmak için, el ile kabuk adını belirtmeniz gerekir.
Örneğin, Windows'taki PowerShell için şunları kullanabilirsiniz: `--sh-comp pwsh`

> Soru: Nedir shell tamamlama işlevi?

> Yanıt: Komut satırında bir komut yazarken, TAB tuşuna basarsanız, tamamlama betiği tetiklenir ve mümkün olan tamamlama seçeneklerinin aranmasına ve seçeneklerin seçilmesi için sunulmasına başlar. Bu özellik, komut satırı girdilerinin verimliliğini ve doğruluğunu büyük ölçüde artırabilir.

Farklı kabukların tamamlama betiği farklı yerlerde saklanır. zsh için, tamamlama betiğini `$fpath` dizisinin içerdiği yollardan birinde saklamanız gerekir.
`print -l $fpath` komutunu kullanarak bu diziye ait içeriği alabilirsiniz.
Örneğin, `/usr/local/share/zsh/site-functions` dizini içeriyorsa, aşağıdaki komutu kullanarak tamamlama betiğini `_tomlyre` dosyasına yazabilirsiniz.

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

Root izni kullanmaktan kaçınmak için, kullanıcı alt dizininde bunu yapabilirsiniz. Örneğin, aşağıdaki komutu kullanarak bu diziye bir öğe eklersiniz ve ilgili dizini kullanırsınız.

```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"Veriler stdin'den okunduğunda, örneğin: `cat a | tomlyre conv -`, ilk önce dosya biçimini otomatik olarak belirlemeye çalışır.
Ancak, otomatik belirleme başarısız olabilir, bu durumda dosya biçimini manuel olarak belirtmeniz gerekebilir.
Örnek: `cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"Sadece `--ts` parametresi kullanıldığında, dahili stil koleksiyonu listelenir.
Markdown hariç diğer stiller için tam isimler gereklidir.
Kullanım örneği 1: `--ts md`
Kullanım örneği 2: `--table-style ascii`

Not: Şu anda **tablo stili** koleksiyonunu dosyalardan okuyacak şekilde desteklenmiyor. Eğer bu işlevselliğe veya başka ekstra işlevselliklere ihtiyacınız varsa, lütfen geri bildirimde bulunmak için bir konu açın."##),
        ("table-style", r##"Tablo stilini belirtin"##),
        ("theme-help", r##"Yalnızca `-t` parametresiyle kullanıldığında, mevcut temaların bir listesi verilir.
Temalar dosyası belirtilmezse, yerleşik tema koleksiyonu kullanılır.
Varsayalım ki **ayu-dark** adında bir tema var, bu temayı kullanmak için `-t ayu-dark` kullanabilirsiniz.
Tema adı boşluk içeriyorsa, çift veya tek tırnakla çevreleyin.

> Not: `None` adlı özel bir tema vardır, sözdizimi vurgulama olmayan saf metin stili için kullanılır.

`--theme` seçeneği belirtilmediğinde, veriler `$TOMLYRE_THEME` ortam değişkeninden okunur.
Bu yardım bilgisinin **sözdizimi vurgusu** rengi de bu **ortam değişkeninden** gelir.

> Varsayılan yardım bilgisi sert kodlu olduğundan yükleme hızı çok hızlıdır.  
> Sert kodlamalı metnin yüklenmesi ve `None` temasına sahip metnin yüklenmesi aynı miktardadır, ancak dinamik yükleme iki boyut daha yavaş olabilir.  
> `$TOMLYRE_THEME` ortam değişkenini değiştirmeyi planlamıyorsanız, sözdizimi vurgusu devre dışı bırakmadığınız sürece `-t` parametresini kullanın.

Ortam değişkeni kullanarak örnek:
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

Komut satırı parametreleri kullanarak örnek:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

Manuel olarak çalıştırılan "conv/get/set" gibi ilgili işlemler için çıktı verileri, `-t` ile belirtilen tema önceliklidir. Ortam değişkeninin içeriğine bakılır ve boş ise **Monokai Extended** teması varsayılan olarak kullanılır."##),
        ("theme", r##"Sözdizimi vurgusunun kullanılacağı ana temayı belirtin"##),
    ],
}
}

/// Language ID: tr;
/// Map name: "args_md";
/// Description: Türkçe, Latin, Türkiye;
pub(super) const fn get_tr_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255mÖzel tema dosyasını belirtin, ayrıca [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ile tema adını belirtmeniz gerekir[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255mShell için tamamlama betiği oluşturun[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255mDesteklenen dosya biçimi, rust paketi olan [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m tarafından dökülen ikili verilerdir.
[48;2;34;34;34m[38;2;255;255;255mTemelde, bu temalar eski bir sublime temasıdır.
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m belirtilmediyse, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ortam değişkeninin değeri okunur.[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255mKaynak dosya biçimini manuel belirtin (varsayılan olarak otomatik tespit edilir)[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255mTema arka planını devre dışı bırakın (varsayılan olarak etkin durumdadır, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m kullanıldığında devre dışı bırakılır)[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Sıkıştırılmamış sürüm, sıkıştırılmış sürümden daha yüksek önceliğe sahiptir.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m belirtilmediyse, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ortam değişkeninin değeri okunur.[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m belirtilmediyse, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ortam değişkeninin değeri okunur.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mEğer bu değer false ise devre dışı bırakılacaktır.
[48;2;34;34;34m[38;2;255;255;255mÖrneğin:
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m'a benzer, ancak sıkıştırılmamış dump verileri (dosya) gerektirir[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255mTamamlama betiğini oluşturmak için, el ile kabuk adını belirtmeniz gerekir.
[48;2;34;34;34m[38;2;255;255;255mÖrneğin, Windows'taki PowerShell için şunları kullanabilirsiniz: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Soru: Nedir shell tamamlama işlevi?
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Yanıt: Komut satırında bir komut yazarken, TAB tuşuna basarsanız, tamamlama betiği tetiklenir ve mümkün olan tamamlama seçeneklerinin aranmasına ve seçeneklerin seçilmesi için sunulmasına başlar. Bu özellik, komut satırı girdilerinin verimliliğini ve doğruluğunu büyük ölçüde artırabilir.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mFarklı kabukların tamamlama betiği farklı yerlerde saklanır. zsh için, tamamlama betiğini [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m dizisinin içerdiği yollardan birinde saklamanız gerekir.
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m komutunu kullanarak bu diziye ait içeriği alabilirsiniz.
[48;2;34;34;34m[38;2;255;255;255mÖrneğin, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m dizini içeriyorsa, aşağıdaki komutu kullanarak tamamlama betiğini [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m dosyasına yazabilirsiniz.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mRoot izni kullanmaktan kaçınmak için, kullanıcı alt dizininde bunu yapabilirsiniz. Örneğin, aşağıdaki komutu kullanarak bu diziye bir öğe eklersiniz ve ilgili dizini kullanırsınız.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255mVeriler stdin'den okunduğunda, örneğin: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, ilk önce dosya biçimini otomatik olarak belirlemeye çalışır.
[48;2;34;34;34m[38;2;255;255;255mAncak, otomatik belirleme başarısız olabilir, bu durumda dosya biçimini manuel olarak belirtmeniz gerekebilir.
[48;2;34;34;34m[38;2;255;255;255mÖrnek: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255mSadece [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m parametresi kullanıldığında, dahili stil koleksiyonu listelenir.
[48;2;34;34;34m[38;2;255;255;255mMarkdown hariç diğer stiller için tam isimler gereklidir.
[48;2;34;34;34m[38;2;255;255;255mKullanım örneği 1: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mKullanım örneği 2: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mNot: Şu anda [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtablo stili[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m koleksiyonunu dosyalardan okuyacak şekilde desteklenmiyor. Eğer bu işlevselliğe veya başka ekstra işlevselliklere ihtiyacınız varsa, lütfen geri bildirimde bulunmak için bir konu açın.[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255mTablo stilini belirtin[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255mYalnızca [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m parametresiyle kullanıldığında, mevcut temaların bir listesi verilir.
[48;2;34;34;34m[38;2;255;255;255mTemalar dosyası belirtilmezse, yerleşik tema koleksiyonu kullanılır.
[48;2;34;34;34m[38;2;255;255;255mVarsayalım ki [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m adında bir tema var, bu temayı kullanmak için [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m kullanabilirsiniz.
[48;2;34;34;34m[38;2;255;255;255mTema adı boşluk içeriyorsa, çift veya tek tırnakla çevreleyin.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Not: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m adlı özel bir tema vardır, sözdizimi vurgulama olmayan saf metin stili için kullanılır.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m seçeneği belirtilmediğinde, veriler [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ortam değişkeninden okunur.
[48;2;34;34;34m[38;2;255;255;255mBu yardım bilgisinin [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114msözdizimi vurgusu[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m rengi de bu [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mortam değişkeninden[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m gelir.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Varsayılan yardım bilgisi sert kodlu olduğundan yükleme hızı çok hızlıdır.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Sert kodlamalı metnin yüklenmesi ve [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m temasına sahip metnin yüklenmesi aynı miktardadır, ancak dinamik yükleme iki boyut daha yavaş olabilir.[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m ortam değişkenini değiştirmeyi planlamıyorsanız, sözdizimi vurgusu devre dışı bırakmadığınız sürece [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m parametresini kullanın.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mOrtam değişkeni kullanarak örnek:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mKomut satırı parametreleri kullanarak örnek:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mManuel olarak çalıştırılan "conv/get/set" gibi ilgili işlemler için çıktı verileri, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ile belirtilen tema önceliklidir. Ortam değişkeninin içeriğine bakılır ve boş ise [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m teması varsayılan olarak kullanılır.[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255mSözdizimi vurgusunun kullanılacağı ana temayı belirtin[0m"##),
    ],
}
}

/// Language ID: tr;
/// Map name: "conv";
/// Description: Türkçe, Latin, Türkiye;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "toml, yaml, json ve ron gibi biçimler arasında dönüştürme yapın");
/// ```
pub(super) const fn get_tr_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"toml, yaml, json ve ron gibi biçimler arasında dönüştürme yapın"##),
        ("from", r##"Giriş dosyasını belirtin"##),
        ("conv-to", r##"Çıktı formatını/dosyasını belirtin, birden fazla çağrıya izin verilir"##),
        ("save", r##"`--to` seçeneği belirlendiğinde, dönüştürülmüş dosya kaydedilecektir."##),
        ("conv-to-help", r##"Eğer mevcut dizinde **test.json** dosyası varsa, `conv test.json -t yml` komutuyla onu yml (yaml) olarak dönüştürebilirsiniz.
Ayrıca, dosya yolu manuel olarak belirtilebilir: `conv test.json -t /tmp/test.yml`

Desteklenen biçimler, bu paketin hangi özelliklerinin etkinleştirildiğine bağlıdır ve tüm özellikler varsayılan olarak etkinleştirilmez.

Bunun yanı sıra, önceden bilmeniz gereken bazı hususlar vardır:
- 1. Belirli olmayan dosya yolları otomatik olarak yeniden adlandırılır.
  - 1.1: Dosyanın yalnızca bir katmanı biçim olarak belirlendiğinden, dosya yolu olarak değil.
    Örneğin `conv test.toml -t ron` komutunu kullanarak çıktı dosyasını ron biçimine dönüştürdüğünüzde, çıktı dosyası otomatik olarak `test.ron` olarak adlandırılır, `ron` olarak değil.
    Bu özelliğe ihtiyacınız yoksa, `-t ./ron` veya `-t $PWD/ron` kullanın.
  - 1.2: Belirtilen şey bir dizinse, dosya yolunun kendisi değil.
    Örneğin `conv -f toml -t /tmp/` komutunu kullandığınızda, `test.toml` dosyasını `/tmp/` alt dizininde yerleştirmeniz gerektiğini unutmayın, `/tmp` dizinini kendisi üzerine yazamazsınız.
    Bu özellik size gerekli değilse, tam (dizin olmayan) bir dosya yolu belirtin, örneğin `-t /tmp/xx.toml` yerine `-t /tmp/` kullanın.
- 2. `--save` çağrılmadığında, belirtilen dosya otomatik olarak kaydedilmez.
- 3. `--save` çağrıldığında, conv alt komutu giriş ve çıkış için aynı dosyayı kullanamaz.
- 4. Dönüştürülen dosyanın açıklaması saklanmaz."##),
    ],
}
}

/// Language ID: tr;
/// Map name: "conv_md";
/// Description: Türkçe, Latin, Türkiye;
pub(super) const fn get_tr_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255mtoml, yaml, json ve ron gibi biçimler arasında dönüştürme yapın[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255mGiriş dosyasını belirtin[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255mÇıktı formatını/dosyasını belirtin, birden fazla çağrıya izin verilir[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m seçeneği belirlendiğinde, dönüştürülmüş dosya kaydedilecektir.[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255mEğer mevcut dizinde [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m dosyası varsa, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m komutuyla onu yml (yaml) olarak dönüştürebilirsiniz.
[48;2;34;34;34m[38;2;255;255;255mAyrıca, dosya yolu manuel olarak belirtilebilir: [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mDesteklenen biçimler, bu paketin hangi özelliklerinin etkinleştirildiğine bağlıdır ve tüm özellikler varsayılan olarak etkinleştirilmez.
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mBunun yanı sıra, önceden bilmeniz gereken bazı hususlar vardır:
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m Belirli olmayan dosya yolları otomatik olarak yeniden adlandırılır.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: Dosyanın yalnızca bir katmanı biçim olarak belirlendiğinden, dosya yolu olarak değil.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mÖrneğin [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m komutunu kullanarak çıktı dosyasını ron biçimine dönüştürdüğünüzde, çıktı dosyası otomatik olarak [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m olarak adlandırılır, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m olarak değil.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mBu özelliğe ihtiyacınız yoksa, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m veya [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m kullanın.
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: Belirtilen şey bir dizinse, dosya yolunun kendisi değil.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mÖrneğin [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv -f toml -t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m komutunu kullandığınızda, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m dosyasını [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m alt dizininde yerleştirmeniz gerektiğini unutmayın, [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m dizinini kendisi üzerine yazamazsınız.
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255mBu özellik size gerekli değilse, tam (dizin olmayan) bir dosya yolu belirtin, örneğin [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m yerine [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m kullanın.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m çağrılmadığında, belirtilen dosya otomatik olarak kaydedilmez.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m çağrıldığında, conv alt komutu giriş ve çıkış için aynı dosyayı kullanamaz.
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. Dönüştürülen dosyanın açıklaması saklanmaz.[0m"##),
    ],
}
}

/// tr: Türkçe, Latin, Türkiye
pub(super) const fn get_tr_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
        (4, 2),
    ],
    entries: &[
        ("conv", get_tr_map_conv),
        ("log-core", get_tr_map_log_core),
        ("conv_md", get_tr_map_conv_md),
        ("theme", get_tr_map_theme),
        ("args", get_tr_map_args),
        ("args_md", get_tr_map_args_md),
    ],
}
}

/// Language ID: ug;
/// Map name: "log-core";
/// Description: ئۇيغۇرچە, ئەرەب, جۇڭگو;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "manug");
/// ```
pub(super) const fn get_ug_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"manug"##),
        ("warn", r##"ئاگاھلاندۇرۇش"##),
        ("info", r##"informo"##),
        ("error", r##"خاتالىقى"##),
        ("trace", r##"ئىز"##),
        ("init-logger", r##"خاتىرىسىنى باشلاش ..."##),
    ],
}
}

/// Language ID: ug;
/// Map name: "theme";
/// Description: ئۇيغۇرچە, ئەرەب, جۇڭگو;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "ئىناۋەتسىز تېما ئىسمى");
/// ```
pub(super) const fn get_ug_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"ئىناۋەتسىز تېما ئىسمى"##),
    ],
}
}

/// ug: ئۇيغۇرچە, ئەرەب, جۇڭگو
pub(super) const fn get_ug_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ug_map_log_core),
        ("theme", get_ug_map_theme),
    ],
}
}

/// Language ID: uk;
/// Map name: "log-core";
/// Description: українська, кирилиця, Україна;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "налагодження");
/// ```
pub(super) const fn get_uk_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"налагодження"##),
        ("warn", r##"попередити"##),
        ("info", r##"Інформація"##),
        ("error", r##"помилка"##),
        ("trace", r##"слід"##),
        ("init-logger", r##"Ініціалізація журналу ..."##),
    ],
}
}

/// Language ID: uk;
/// Map name: "theme";
/// Description: українська, кирилиця, Україна;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Недійсна назва теми");
/// ```
pub(super) const fn get_uk_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Недійсна назва теми"##),
    ],
}
}

/// uk: українська, кирилиця, Україна
pub(super) const fn get_uk_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_uk_map_log_core),
        ("theme", get_uk_map_theme),
    ],
}
}

/// Language ID: ur;
/// Map name: "log-core";
/// Description: اردو, عربی, پاکستان;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "ڈیبگ");
/// ```
pub(super) const fn get_ur_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"ڈیبگ"##),
        ("warn", r##"انتباہ کریں"##),
        ("info", r##"معلومات"##),
        ("error", r##"غلطی"##),
        ("trace", r##"ٹریس"##),
        ("init-logger", r##"لاگر کو شروع کرنا ..."##),
    ],
}
}

/// Language ID: ur;
/// Map name: "theme";
/// Description: اردو, عربی, پاکستان;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "نالی تھیم کا نام");
/// ```
pub(super) const fn get_ur_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"نالی تھیم کا نام"##),
    ],
}
}

/// ur: اردو, عربی, پاکستان
pub(super) const fn get_ur_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_ur_map_log_core),
        ("theme", get_ur_map_theme),
    ],
}
}

/// Language ID: uz;
/// Map name: "log-core";
/// Description: o‘zbek, lotin, Oʻzbekiston;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "disksing");
/// ```
pub(super) const fn get_uz_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"disksing"##),
        ("warn", r##"ogohlantirish"##),
        ("info", r##"INFO"##),
        ("error", r##"xato"##),
        ("trace", r##"iz"##),
        ("init-logger", r##"Loggeratsiyani boshlash ..."##),
    ],
}
}

/// Language ID: uz;
/// Map name: "theme";
/// Description: o‘zbek, lotin, Oʻzbekiston;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Noto'g'ri mavzu nomi");
/// ```
pub(super) const fn get_uz_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Noto'g'ri mavzu nomi"##),
    ],
}
}

/// uz: o‘zbek, lotin, Oʻzbekiston
pub(super) const fn get_uz_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_uz_map_log_core),
        ("theme", get_uz_map_theme),
    ],
}
}

/// Language ID: vi;
/// Map name: "log-core";
/// Description: Tiếng Việt, Chữ La tinh, Việt Nam;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "gỡ lỗi");
/// ```
pub(super) const fn get_vi_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"gỡ lỗi"##),
        ("warn", r##"cảnh báo"##),
        ("info", r##"thông tin"##),
        ("error", r##"lỗi"##),
        ("trace", r##"dấu vết"##),
        ("init-logger", r##"Khởi tạo logger ..."##),
    ],
}
}

/// Language ID: vi;
/// Map name: "theme";
/// Description: Tiếng Việt, Chữ La tinh, Việt Nam;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Tên chủ đề không hợp lệ");
/// ```
pub(super) const fn get_vi_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Tên chủ đề không hợp lệ"##),
    ],
}
}

/// vi: Tiếng Việt, Chữ La tinh, Việt Nam
pub(super) const fn get_vi_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_vi_map_log_core),
        ("theme", get_vi_map_theme),
    ],
}
}

/// Language ID: xh;
/// Map name: "log-core";
/// Description: IsiXhosa, IsiLatin, EMzantsi Afrika;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "Debug");
/// ```
pub(super) const fn get_xh_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"Debug"##),
        ("warn", r##"Lumkisa"##),
        ("info", r##"Ulwazi"##),
        ("error", r##"Impazamo"##),
        ("trace", r##"Trace"##),
        ("init-logger", r##"Ukuqala ungene ..."##),
    ],
}
}

/// Language ID: xh;
/// Map name: "theme";
/// Description: IsiXhosa, IsiLatin, EMzantsi Afrika;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Igama lomxholo ongasebenziyo");
/// ```
pub(super) const fn get_xh_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Igama lomxholo ongasebenziyo"##),
    ],
}
}

/// xh: IsiXhosa, IsiLatin, EMzantsi Afrika
pub(super) const fn get_xh_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_xh_map_log_core),
        ("theme", get_xh_map_theme),
    ],
}
}

/// Language ID: yi;
/// Map name: "log-core";
/// Description: ייִדיש, העברעיש, וועלט;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "דעבוג");
/// ```
pub(super) const fn get_yi_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"דעבוג"##),
        ("warn", r##"וואָרענען"##),
        ("info", r##"אינפֿאָרמאַציע"##),
        ("error", r##"טעות"##),
        ("trace", r##"שפּור"##),
        ("init-logger", r##"יניטיאַליזינג לאַגער ..."##),
    ],
}
}

/// Language ID: yi;
/// Map name: "theme";
/// Description: ייִדיש, העברעיש, וועלט;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "פא\u{5b7}רקריפ\u{5bc}לט טעמע נא\u{5b8}מען");
/// ```
pub(super) const fn get_yi_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"פאַרקריפּלט טעמע נאָמען"##),
    ],
}
}

/// yi: ייִדיש, העברעיש, וועלט
pub(super) const fn get_yi_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_yi_map_log_core),
        ("theme", get_yi_map_theme),
    ],
}
}

/// Language ID: yo;
/// Map name: "log-core";
/// Description: Èdè Yorùbá, Èdè Látìn, Nàìjíríà;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "wadi");
/// ```
pub(super) const fn get_yo_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"wadi"##),
        ("warn", r##"kirn"##),
        ("info", r##"Alaye"##),
        ("error", r##"aṣiṣe"##),
        ("trace", r##"wa kakiri"##),
        ("init-logger", r##"Logoli ti ipilẹṣẹ ..."##),
    ],
}
}

/// Language ID: yo;
/// Map name: "theme";
/// Description: Èdè Yorùbá, Èdè Látìn, Nàìjíríà;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Orukọ akojọ ti ko wulo");
/// ```
pub(super) const fn get_yo_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Orukọ akojọ ti ko wulo"##),
    ],
}
}

/// yo: Èdè Yorùbá, Èdè Látìn, Nàìjíríà
pub(super) const fn get_yo_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_yo_map_log_core),
        ("theme", get_yo_map_theme),
    ],
}
}

/// Language ID: zh;
/// Map name: "log-core";
/// Description: 简体中文, 中国;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "调试");
/// ```
pub(super) const fn get_zh_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"调试"##),
        ("warn", r##"警告"##),
        ("info", r##"信息"##),
        ("error", r##"错误"##),
        ("trace", r##"追踪"##),
        ("init-logger", r##"正在初始化 logger（日志记录器）..."##),
    ],
}
}

/// Language ID: zh;
/// Map name: "opt";
/// Description: 简体中文, 中国;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "表数组(用法：`--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_zh_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"与 `-k` 的不同之处在于 `--ck` 不会自动 `split('.')`。  
对于 [a.b.c]，`-k a.b.c` 与`--ck a --ck b --ck c`是一样的。  
对于简单情况，用 `-k` 会更简便一点。

如果 key 名包含了 '.'， 那么请使用 `--ck` 而不是 `-k`。
举个例子： 

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

在此结构中，"editor.defaultFormatter" 是一个完整的字符串，`defaultFormatter` 不是 `editor` 的子键（sub-key）。

当您使用 `-k '[typescript]'.'editor.defaultFormatter'` 时，它会将 key 分割成：

- "[typescript]"
- "editor"
- "defaultFormatter"

而使用 `--ck '[typescript]' --ck editor.defaultFormatter`，它会将 key 当作：

- "[typescript]"
- "editor.defaultFormatter"

后者才是正确的结构。"##),
        ("num-arr", r##"i64的数组"##),
        ("bool", r##"bool (布尔类型)， 只能为 `true` 或 `false`"##),
        ("num", r##"i64: 64位有符号整数类型, 例如：1145"##),
        ("datetime-help", r##"示例：
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

此选项仅支持 `toml`, 如果我们在命令末尾添加 `-t json`, 那么它会变成这样子：

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"保存为指定的格式/文件，一次只能保存一个"##),
        ("get-opts-help", r##"用法示例1：`get Cargo.toml -k package.name`
用法示例2：`get cfg.yaml -k .`
"##),
        ("none-help", r##"> 注：ron 拥有真正的 `Option<T>` 类型，值为 `Some(T)` 或 `None`
> 此处的 `--none` 指的是删掉指定的 key，而不是真正的 `Option<T>` 类型。

假设 test.yml 存在以下表数组：

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

我们如果要删掉 **name: test**, 那么可以执行：
```sh
set test.yml -k bin.1.name --none
```

> 注：空字符串与 None 是不一样的
> 调用 `set test.yml --kv bin.1.name ""` 可将其设置为空字符串，而 `--none` 则是删掉整个 key。

我们如果要删掉 bin 的第二张表（索引为 1）, 那就执行: 
```sh
set test.yml -k bin.1 --none
```

或者是使用 `--rm`
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"双精度浮点数 (IEEE 754-2008标准)，例如： 3.14159265359"##),
        ("saot-help", r##"> 注：尽管通过 `--saot` 只能为每张表创建一对 K-V pair，但您可以先指定表数组的索引，再用其他选项来扩充。

示例：
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"设置 k-v pair，`--kv x.y.z value` 相当于`-k x.y.z -s value`"##),
        ("set-save", r##"在没有搭配 `--to` 的情况下，若调用则覆盖(写入)当前文件"##),
        ("array", r##"字符串数组"##),
        ("aot-help", r##"示例：
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

先通过指定的 key 来选择，再通过指定数据类型的 value 来修改。

> 当仅提供 key，未提供 value 时，会列出相应的数据类型和 value。
> 比如 `set Cargo.toml -k package`，会列出 package 及其子表的表格，并提供比 `get` 更详细的信息。

## Examples

### 示例1：
```sh
--ts md set test.toml --key package.edition --str 2021
```

由于指定了 `--str`，而不是 `--num`, 因此 "2021" 是字符串类型，而不是整数类型。

| package | new value |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### 示例2：
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### 示例3：
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: 这是第1个索引*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> 注：以上示例以 参数(argument)/子命令(sub-command) 开头，省略了根命令名称，在实际操作中，您需要手动补充。"##),
        ("num-arr-help", r##"示例：
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"您如果不清楚自己该用标准表，还是内联表，那就用内联表！

示例：
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> 除了 `toml` 外，其他格式将会输出为标准表的格式。

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"通过 key（键）来查询 value（值）"##),
        ("f64-arr-help", r##"示例：
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> json 1.0 不支持 NaN 和 Inf

它会输出 `null`， 而不是 `NaN` 。

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"删掉指定的key"##),
        ("set-save-to-help", r##"与 `conv` 子命令的 `-t` 非常像，最大的区别在于 `conv` 的 `-t` 能多次连续调用，输出到多种格式/多个文件。
而 `set` 子命令的 `-t` 只能调用一次，输出到单个文件。

> 注： `conv` 子命令在执行完转换操作后，所有格式的文件都不会保留注释。
> 而 `set` 子命令会保留 **toml** 的注释，除了 **toml** 外，其他的所有格式 **都不会** 保留注释。

- 在未调用 `-t` 的前提下，调用 `--save` 会覆盖自身。
- 若已调用 `-t` ，并且还调用了 `--save`, 则修改后的文件会保存至 `-t` 所指向的文件。
- 除非源文件与 `-t` 指向的文件相同，否则 `set` 子命令的 `-t` 的行为将会与 `conv` 的 `-t` 部分一致，它们都会对非具体文件路径进行自动更名。
  - 假设 from: **test.yml**, to: **json**, 那么输出的文件会变成 **test.json**, 而不是 **json**
  - 若您无需此特性，则请使用 `-t ./json` 或 `-t $PWD/json`，而不是 `-t json`。"##),
        ("preview", r##"在修改完成后，预览完整的文件"##),
        ("set-key", r##"选择指定的 key（键），以"."作为分隔符"##),
        ("get-opts", r##"获取指定配置的 value"##),
        ("saot", r##"单对 key-value pair 的表数组(用法：`--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"格式为: "a.b.c", 子表名称或数组索引通过 "."（半角的dot符号）来连接。
当只有一个 "." 时，会列出整张表"##),
        ("set-opts", r##"创建/更新/删除 配置文件的结构"##),
        ("bool-arr-help", r##"示例：
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: 第1个元素是 true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"bool 的数组"##),
        ("table", r##"标准表（standard table），用于存储多对 K-V pairs"##),
        ("datetime", r##"日期与时间的类型"##),
        ("table-help", r##"示例1：
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

示例2：
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"f64的数组"##),
        ("string", r##"String（字符串类型）"##),
        ("array-help", r##"通过多次调用 `-a` 来设置数组元素。
示例：
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> 第一次调用时设置索引 0 的元素，第二次设置索引 1 的元素，以此类推。

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"表数组(用法：`--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"内联表，相当于更紧凑的标准表。此外，对于嵌套且不存在的keys，请使用内联表，而不是标准表。"##),
        ("concat-key", r##"分段输入 key, 结果会自动拼接，不使用 "." 作为分隔符"##),
    ],
}
}

/// Language ID: zh;
/// Map name: "opt_md";
/// Description: 简体中文, 中国;
pub(super) const fn get_zh_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255m与 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的不同之处在于 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 不会自动 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msplit('.')[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m对于 [a.b.c]，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 与[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m是一样的。[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m对于简单情况，用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 会更简便一点。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m如果 key 名包含了 '.'， 那么请使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m举个例子： 
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m在此结构中，"editor.defaultFormatter" 是一个完整的字符串，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mdefaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255meditor[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的子键（sub-key）。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m当您使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 时，它会将 key 分割成：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m而使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，它会将 key 当作：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m后者才是正确的结构。[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mi64的数组[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mbool (布尔类型)， 只能为 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 或 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: 64位有符号整数类型, 例如：1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m此选项仅支持 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 如果我们在命令末尾添加 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 那么它会变成这样子：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255m保存为指定的格式/文件，一次只能保存一个[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255m用法示例1：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m用法示例2：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：ron 拥有真正的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 类型，值为 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 或 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 此处的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 指的是删掉指定的 key，而不是真正的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 类型。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m假设 test.yml 存在以下表数组：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m我们如果要删掉 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mname: test[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 那么可以执行：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：空字符串与 None 是不一样的
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 调用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 可将其设置为空字符串，而 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 则是删掉整个 key。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m我们如果要删掉 bin 的第二张表（索引为 1）, 那就执行: 
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m或者是使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255m双精度浮点数 (IEEE 754-2008标准)，例如： 3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：尽管通过 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 只能为每张表创建一对 K-V pair，但您可以先指定表数组的索引，再用其他选项来扩充。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255m设置 k-v pair，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 相当于[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255m在没有搭配 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的情况下，若调用则覆盖(写入)当前文件[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255m字符串数组[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m先通过指定的 key 来选择，再通过指定数据类型的 value 来修改。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 当仅提供 key，未提供 value 时，会列出相应的数据类型和 value。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 比如 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m，会列出 package 及其子表的表格，并提供比 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 更详细的信息。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExamples[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m示例1：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m由于指定了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 因此 "2021" 是字符串类型，而不是整数类型。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m new value [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m示例2：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m示例3：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: 这是第1个索引*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：以上示例以 参数(argument)/子命令(sub-command) 开头，省略了根命令名称，在实际操作中，您需要手动补充。[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255m您如果不清楚自己该用标准表，还是内联表，那就用内联表！
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 除了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 外，其他格式将会输出为标准表的格式。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255m通过 key（键）来查询 value（值）[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m json 1.0 不支持 NaN 和 Inf
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m它会输出 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255m删掉指定的key[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255m与 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 子命令的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 非常像，最大的区别在于 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 能多次连续调用，输出到多种格式/多个文件。
[48;2;34;34;34m[38;2;255;255;255m而 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 子命令的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 只能调用一次，输出到单个文件。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注： [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 子命令在执行完转换操作后，所有格式的文件都不会保留注释。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 而 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 子命令会保留 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 的注释，除了 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 外，其他的所有格式 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m都不会[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 保留注释。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m在未调用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的前提下，调用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 会覆盖自身。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 若已调用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ，并且还调用了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 则修改后的文件会保存至 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 所指向的文件。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 除非源文件与 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 指向的文件相同，否则 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 子命令的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的行为将会与 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 部分一致，它们都会对非具体文件路径进行自动更名。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 假设 from: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, to: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 那么输出的文件会变成 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 而不是 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 若您无需此特性，则请使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 或 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。[0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255m在修改完成后，预览完整的文件[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255m选择指定的 key（键），以"."作为分隔符[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255m获取指定配置的 value[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255m单对 key-value pair 的表数组(用法：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255m格式为: "a.b.c", 子表名称或数组索引通过 "."（半角的dot符号）来连接。
[48;2;34;34;34m[38;2;255;255;255m当只有一个 "." 时，会列出整张表[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255m创建/更新/删除 配置文件的结构[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: 第1个元素是 true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mbool 的数组[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255m标准表（standard table），用于存储多对 K-V pairs[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255m日期与时间的类型[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例1：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m示例2：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mf64的数组[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString（字符串类型）[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255m通过多次调用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 来设置数组元素。
[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 第一次调用时设置索引 0 的元素，第二次设置索引 1 的元素，以此类推。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255m表数组(用法：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255m内联表，相当于更紧凑的标准表。此外，对于嵌套且不存在的keys，请使用内联表，而不是标准表。[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255m分段输入 key, 结果会自动拼接，不使用 "." 作为分隔符[0m"##),
    ],
}
}

/// Language ID: zh;
/// Map name: "parser";
/// Description: 简体中文, 中国;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "下面是 stderr 输出，在重定向输出补全脚本时，您应该使用 `>/path/to/file` 而不是 `2>/path/to/file`");
/// ```
pub(super) const fn get_zh_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"下面是 stderr 输出，在重定向输出补全脚本时，您应该使用 `>/path/to/file` 而不是 `2>/path/to/file`"##),
    ],
}
}

/// Language ID: zh;
/// Map name: "theme";
/// Description: 简体中文, 中国;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "无效的主题名称");
/// ```
pub(super) const fn get_zh_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"无效的主题名称"##),
    ],
}
}

/// Language ID: zh;
/// Map name: "conv";
/// Description: 简体中文, 中国;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "在 toml、yaml、json 和 ron 等格式之间进行转换");
/// ```
pub(super) const fn get_zh_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"在 toml、yaml、json 和 ron 等格式之间进行转换"##),
        ("from", r##"指定输入的文件"##),
        ("conv-to", r##"指定输出的格式/文件，支持多次调用"##),
        ("save", r##"当调用此选项后，`--to` 所指定的文件将被保存"##),
        ("conv-to-help", r##"如果当前目录下存在 **test.json** 文件，那么您可以调用 `conv test.json -t yml` 将其转换为 yml (yaml),
也可以手动指定文件路径：`conv test.json -t /tmp/test.yml`

支持的格式取决于此软件包启用的功能，并非所有功能都会默认启用。

此外，在使用前，您必须要了解的事情如下：
- 1. 它会对非具体文件路径进行自动更名。
  - 1.1: 只有一个层级的文件会被断定为文件格式，而不是文件路径
    当调用 `conv test.toml -t ron`, 输出的文件会自动变成 `test.ron`, 而不是 `ron`。
    若您无需此特性，则请使用 `-t ./ron`, 或者是 `-t $PWD/ron`。
  - 1.2: 指向的是目录而不是文件
    比如 `-t /tmp/`，我们只能将 `test.toml` 放在 `/tmp/` 的子级，而不是覆盖 `/tmp` 自身。
    若您无需此特性，则请使用完整的（非目录）文件路径，例如 `-t /tmp/xx.toml`，而不是 `-t /tmp/`。
- 2. 当未调用 `--save` 时，指定的文件不会自动保存。
- 3. 当调用 `--save` 时，conv 子命令的输入与输出不能为相同的文件。
- 4. 转换后的文件不会保留注释。"##),
    ],
}
}

/// Language ID: zh;
/// Map name: "conv_md";
/// Description: 简体中文, 中国;
pub(super) const fn get_zh_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255m在 toml、yaml、json 和 ron 等格式之间进行转换[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255m指定输入的文件[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255m指定输出的格式/文件，支持多次调用[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255m当调用此选项后，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 所指定的文件将被保存[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255m如果当前目录下存在 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 文件，那么您可以调用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 将其转换为 yml (yaml),
[48;2;34;34;34m[38;2;255;255;255m也可以手动指定文件路径：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m支持的格式取决于此软件包启用的功能，并非所有功能都会默认启用。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m此外，在使用前，您必须要了解的事情如下：
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m 它会对非具体文件路径进行自动更名。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: 只有一个层级的文件会被断定为文件格式，而不是文件路径
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m当调用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 输出的文件会自动变成 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m若您无需此特性，则请使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 或者是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: 指向的是目录而不是文件
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m比如 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，我们只能将 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 放在 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的子级，而不是覆盖 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 自身。
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m若您无需此特性，则请使用完整的（非目录）文件路径，例如 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. 当未调用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 时，指定的文件不会自动保存。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. 当调用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 时，conv 子命令的输入与输出不能为相同的文件。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. 转换后的文件不会保留注释。[0m"##),
    ],
}
}

/// Language ID: zh;
/// Map name: "args";
/// Description: 简体中文, 中国;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "禁用主题背景（默认为启用状态，若调用`--dis-theme-bg` 则禁用）");
/// ```
pub(super) const fn get_zh_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"自定义主题集合文件，您还需要搭配 `--theme` 指定主题名称"##),
        ("shell-completion", r##"为 shell 生成补全脚本"##),
        ("theme-file-help", r##"支持的文件格式为 rust crate `syntect` 所 dump 出来的二进制数据。  
本质上，这些主题是旧版的 sublime 的主题。
若未指定 `--tf`， 则读取环境变量 `$TOMLYRE_THEME_FILE` 的值。"##),
        ("from-format", r##"手动指定来源的文件格式（默认会自动判断）"##),
        ("disable-theme-background", r##"禁用主题背景（默认为启用状态，若调用`--dis-theme-bg` 则禁用）"##),
        ("theme-uncompressed-file-help", r##"> 未压缩的版本要比压缩版本的优先级更高。  

若未指定 `--tuf`， 则读取环境变量 `$TOMLYRE_THEME_UNCOMPRESSED_FILE` 的值。"##),
        ("disable-theme-background-help", r##"若未指定 `--dis-theme-bg`， 则读取环境变量 `$TOMLYRE_THEME_BG` 的值。

若该值为 false 则禁用

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"类似于`theme-file`, 但要求未压缩的 dump 数据(文件)"##),
        ("shell-completion-help", r##"您需要手动指定 shell 的名称来生成补全脚本。
例如，对于 Windows 上的 powershell, 您可以使用：`--sh-comp pwsh`

> Q: 什么是 shell 的补全功能？

> A: 当我们在命令行上输入命令时，如果按下Tab键，补全脚本就会被触发，开始搜索可能的补全选项，并将这些选项呈现出来以供选择。此功能可以极大地提高命令行输入的效率和准确性。

不同 shell 的补全脚本所存放的位置是不一样的。对于 zsh，您需要将其存放于 `$fpath` 数组所包含的路径。
您可以使用 `print -l $fpath` 来获取该数组的内容。
假设里面包含了 `/usr/local/share/zsh/site-functions` 目录， 那么您可以使用以下命令来将补全脚本写入到该目录下的 `_tomlyre` 文件。

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

为了避免使用 root 权限，您可以将其放在用户子目录下。比如说，您可以使用以下命令向该数组头部插入一个元素，再使用相关的目录。
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"当从 stdin 读取数据时，例如： `cat a | tomlyre conv -`，它首先会尝试自动判断文件格式。
然而，自动判断有可能会失败，这时候就需要手动指定格式了。
示例：`cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"当仅使用 `--ts` 参数时，会列出内置的样式集合。
除了 markdown 可以缩写为 md 外, 其他的样式都要用完整的名称。
用法示例1：`--ts md` 
用法示例2：`--table-style ascii`

注：目前不支持从文件中读取 **表格样式** 的集合，如果您需要此功能或其他额外的功能，欢迎提交 issue 进行反馈。"##),
        ("table-style", r##"指定表格的样式"##),
        ("theme-help", r##"当仅使用 `-t` 参数时，会列出当前主题的列表。
若未指定主题文件，则使用内置的主题集合。
假设存在一个主题为 **ayu-dark**, 则您可以使用 `-t ayu-dark` 来使用该主题。
若主题名称包含空格，则请使用双引号或单引号将其包裹起来。

> 注：有一个特殊主题为 `None`，代表没有语法高亮的纯文本样式。

当未指定 `--theme` 选项时，将会从环境变量 `$TOMLYRE_THEME` 中读取数据。
此帮助信息的 **语法高亮** 主题也来自于该 **环境变量**。

> 默认的帮助信息是硬编码的，加载速度非常快。  
> 加载硬编码的文本与加载 `None` 主题的文本的速度是同一个数量级的，而动态加载则会变慢两个数量级及以上。  
> 设置 `$TOMLYRE_THEME` 环境变量可能会导致它重新加载主题，这将 **严重** 拖慢启动速度。  
> 除非您打算修改帮助信息的主题，或者是禁用语法高亮，否则请使用 `-t` 参数，而不是修改环境变量。  

使用环境变量的例子：
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

使用命令行参数的例子:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

对于手动执行 "conv/get/set" 等相关操作所输出的内容，`-t` 指定的主题的优先级 > 环境变量。
若未调用 `-t`，则读取环境变量；若环境变量为空，则使用 **Monokai Extended**。"##),
        ("theme", r##"指定语法高亮的主题名称"##),
    ],
}
}

/// Language ID: zh;
/// Map name: "args_md";
/// Description: 简体中文, 中国;
pub(super) const fn get_zh_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255m自定义主题集合文件，您还需要搭配 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 指定主题名称[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255m为 shell 生成补全脚本[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255m支持的文件格式为 rust crate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 所 dump 出来的二进制数据。[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m本质上，这些主题是旧版的 sublime 的主题。
[48;2;34;34;34m[38;2;255;255;255m若未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 则读取环境变量 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的值。[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255m手动指定来源的文件格式（默认会自动判断）[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255m禁用主题背景（默认为启用状态，若调用[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 则禁用）[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 未压缩的版本要比压缩版本的优先级更高。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m若未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 则读取环境变量 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的值。[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255m若未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 则读取环境变量 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的值。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m若该值为 false 则禁用
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255m类似于[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 但要求未压缩的 dump 数据(文件)[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255m您需要手动指定 shell 的名称来生成补全脚本。
[48;2;34;34;34m[38;2;255;255;255m例如，对于 Windows 上的 powershell, 您可以使用：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q: 什么是 shell 的补全功能？
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: 当我们在命令行上输入命令时，如果按下Tab键，补全脚本就会被触发，开始搜索可能的补全选项，并将这些选项呈现出来以供选择。此功能可以极大地提高命令行输入的效率和准确性。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m不同 shell 的补全脚本所存放的位置是不一样的。对于 zsh，您需要将其存放于 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 数组所包含的路径。
[48;2;34;34;34m[38;2;255;255;255m您可以使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 来获取该数组的内容。
[48;2;34;34;34m[38;2;255;255;255m假设里面包含了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 目录， 那么您可以使用以下命令来将补全脚本写入到该目录下的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 文件。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m为了避免使用 root 权限，您可以将其放在用户子目录下。比如说，您可以使用以下命令向该数组头部插入一个元素，再使用相关的目录。
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255m当从 stdin 读取数据时，例如： [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，它首先会尝试自动判断文件格式。
[48;2;34;34;34m[38;2;255;255;255m然而，自动判断有可能会失败，这时候就需要手动指定格式了。
[48;2;34;34;34m[38;2;255;255;255m示例：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255m当仅使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 参数时，会列出内置的样式集合。
[48;2;34;34;34m[38;2;255;255;255m除了 markdown 可以缩写为 md 外, 其他的样式都要用完整的名称。
[48;2;34;34;34m[38;2;255;255;255m用法示例1：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 
[48;2;34;34;34m[38;2;255;255;255m用法示例2：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m注：目前不支持从文件中读取 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m表格样式[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 的集合，如果您需要此功能或其他额外的功能，欢迎提交 issue 进行反馈。[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255m指定表格的样式[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255m当仅使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 参数时，会列出当前主题的列表。
[48;2;34;34;34m[38;2;255;255;255m若未指定主题文件，则使用内置的主题集合。
[48;2;34;34;34m[38;2;255;255;255m假设存在一个主题为 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 则您可以使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 来使用该主题。
[48;2;34;34;34m[38;2;255;255;255m若主题名称包含空格，则请使用双引号或单引号将其包裹起来。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：有一个特殊主题为 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m，代表没有语法高亮的纯文本样式。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m当未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 选项时，将会从环境变量 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 中读取数据。
[48;2;34;34;34m[38;2;255;255;255m此帮助信息的 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m语法高亮[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 主题也来自于该 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m环境变量[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 默认的帮助信息是硬编码的，加载速度非常快。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 加载硬编码的文本与加载 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 主题的文本的速度是同一个数量级的，而动态加载则会变慢两个数量级及以上。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 设置 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 环境变量可能会导致它重新加载主题，这将 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m严重[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 拖慢启动速度。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 除非您打算修改帮助信息的主题，或者是禁用语法高亮，否则请使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 参数，而不是修改环境变量。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m使用环境变量的例子：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m使用命令行参数的例子:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m对于手动执行 "conv/get/set" 等相关操作所输出的内容，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 指定的主题的优先级 > 环境变量。
[48;2;34;34;34m[38;2;255;255;255m若未调用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，则读取环境变量；若环境变量为空，则使用 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m。[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255m指定语法高亮的主题名称[0m"##),
    ],
}
}

/// zh: 简体中文, 中国
pub(super) const fn get_zh_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_zh_map_args),
        ("conv_md", get_zh_map_conv_md),
        ("parser", get_zh_map_parser),
        ("opt_md", get_zh_map_opt_md),
        ("args_md", get_zh_map_args_md),
        ("opt", get_zh_map_opt),
        ("log-core", get_zh_map_log_core),
        ("conv", get_zh_map_conv),
        ("theme", get_zh_map_theme),
    ],
}
}

/// Language ID: zh-Hant;
/// Map name: "log-core";
/// Description: 正體中文, 中國台灣;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "除錯");
/// ```
pub(super) const fn get_zh_hant_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"除錯"##),
        ("warn", r##"警告"##),
        ("info", r##"資訊"##),
        ("error", r##"錯誤"##),
        ("trace", r##"追蹤"##),
        ("init-logger", r##"正在初始化 logger（日誌記錄器）..."##),
    ],
}
}

/// Language ID: zh-Hant;
/// Map name: "parser";
/// Description: 正體中文, 中國台灣;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "下面是 stderr 輸出，在重定向輸出補全指令碼時，您應該使用 `>/path/to/file` 而不是 `2>/path/to/file`");
/// ```
pub(super) const fn get_zh_hant_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"下面是 stderr 輸出，在重定向輸出補全指令碼時，您應該使用 `>/path/to/file` 而不是 `2>/path/to/file`"##),
    ],
}
}

/// Language ID: zh-Hant;
/// Map name: "opt";
/// Description: 正體中文, 中國台灣;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "表陣列(用法：`--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_zh_hant_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"與 `-k` 的不同之處在於 `--ck` 不會自動 `split('.')`。  
對於 [a.b.c]，`-k a.b.c` 與`--ck a --ck b --ck c`是一樣的。  
對於簡單情況，用 `-k` 會更簡便一點。

如果 key 名包含了 '.'， 那麼請使用 `--ck` 而不是 `-k`。
舉個例子： 

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

在此結構中，"editor.defaultFormatter" 是一個完整的字串，`defaultFormatter` 不是 `editor` 的子鍵（sub-key）。

當您使用 `-k '[typescript]'.'editor.defaultFormatter'` 時，它會將 key 分割成：

- "[typescript]"
- "editor"
- "defaultFormatter"

而使用 `--ck '[typescript]' --ck editor.defaultFormatter`，它會將 key 當作：

- "[typescript]"
- "editor.defaultFormatter"

後者才是正確的結構。"##),
        ("num-arr", r##"i64的陣列"##),
        ("bool", r##"bool (布林型別)， 只能為 `true` 或 `false`"##),
        ("num", r##"i64: 64位有符號整數型別, 例如：1145"##),
        ("datetime-help", r##"示例：
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

此選項僅支援 `toml`, 如果我們在命令末尾新增 `-t json`, 那麼它會變成這樣子：

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"儲存為指定的格式/檔案，一次只能儲存一個"##),
        ("get-opts-help", r##"用法示例1：`get Cargo.toml -k package.name`
用法示例2：`get cfg.yaml -k .`
"##),
        ("none-help", r##"> 注：ron 擁有真正的 `Option<T>` 型別，值為 `Some(T)` 或 `None`
> 此處的 `--none` 指的是刪掉指定的 key，而不是真正的 `Option<T>` 型別。

假設 test.yml 存在以下表陣列：

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

我們如果要刪掉 **name: test**, 那麼可以執行：
```sh
set test.yml -k bin.1.name --none
```

> 注：空字串與 None 是不一樣的
> 呼叫 `set test.yml --kv bin.1.name ""` 可將其設定為空字串，而 `--none` 則是刪掉整個 key。

我們如果要刪掉 bin 的第二張表（索引為 1）, 那就執行: 
```sh
set test.yml -k bin.1 --none
```

或者是使用 `--rm`
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"雙精度浮點數 (IEEE 754-2008標準)，例如： 3.14159265359"##),
        ("saot-help", r##"> 注：儘管透過 `--saot` 只能為每張表建立一對 K-V pair，但您可以先指定表陣列的索引，再用其他選項來擴充。

示例：
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"設定 k-v pair，`--kv x.y.z value` 相當於`-k x.y.z -s value`"##),
        ("set-save", r##"在沒有搭配 `--to` 的情況下，若呼叫則覆蓋(寫入)當前檔案"##),
        ("array", r##"字串陣列"##),
        ("aot-help", r##"示例：
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

先透過指定的 key 來選擇，再透過指定資料型別的 value 來修改。

> 當僅提供 key，未提供 value 時，會列出相應的資料型別和 value。
> 比如 `set Cargo.toml -k package`，會列出 package 及其子表的表格，並提供比 `get` 更詳細的資訊。

## Examples

### 示例1：
```sh
--ts md set test.toml --key package.edition --str 2021
```

由於指定了 `--str`，而不是 `--num`, 因此 "2021" 是字串型別，而不是整數型別。

| package | new value |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### 示例2：
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### 示例3：
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: 這是第1個索引*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> 注：以上示例以 引數(argument)/子命令(sub-command) 開頭，省略了根命令名稱，在實際操作中，您需要手動補充。"##),
        ("num-arr-help", r##"示例：
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"您如果不清楚自己該用標準表，還是內聯表，那就用內聯表！

示例：
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> 除了 `toml` 外，其他格式將會輸出為標準表的格式。

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"透過 key（鍵）來查詢 value（值）"##),
        ("f64-arr-help", r##"示例：
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> json 1.0 不支援 NaN 和 Inf

它會輸出 `null`， 而不是 `NaN` 。

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"刪掉指定的key"##),
        ("set-save-to-help", r##"與 `conv` 子命令的 `-t` 非常像，最大的區別在於 `conv` 的 `-t` 能多次連續呼叫，輸出到多種格式/多個檔案。
而 `set` 子命令的 `-t` 只能呼叫一次，輸出到單個檔案。

> 注： `conv` 子命令在執行完轉換操作後，所有格式的檔案都不會保留註釋。
> 而 `set` 子命令會保留 **toml** 的註釋，除了 **toml** 外，其他的所有格式 **都不會** 保留註釋。

- 在未呼叫 `-t` 的前提下，呼叫 `--save` 會覆蓋自身。
- 若已呼叫 `-t` ，並且還呼叫了 `--save`, 則修改後的檔案會儲存至 `-t` 所指向的檔案。
- 除非原始檔與 `-t` 指向的檔案相同，否則 `set` 子命令的 `-t` 的行為將會與 `conv` 的 `-t` 部分一致，它們都會對非具體檔案路徑進行自動更名。
  - 假設 from: **test.yml**, to: **json**, 那麼輸出的檔案會變成 **test.json**, 而不是 **json**
  - 若您無需此特性，則請使用 `-t ./json` 或 `-t $PWD/json`，而不是 `-t json`。"##),
        ("preview", r##"在修改完成後，預覽完整的檔案"##),
        ("set-key", r##"選擇指定的 key（鍵），以"."作為分隔符"##),
        ("get-opts", r##"獲取指定配置的 value"##),
        ("saot", r##"單對 key-value pair 的表陣列(用法：`--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"格式為: "a.b.c", 子表名稱或陣列索引透過 "."（半形的dot符號）來連線。
當只有一個 "." 時，會列出整張表"##),
        ("set-opts", r##"建立/更新/刪除 配置檔案的結構"##),
        ("bool-arr-help", r##"示例：
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: 第1個元素是 true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"bool 的陣列"##),
        ("table", r##"標準表（standard table），用於儲存多對 K-V pairs"##),
        ("datetime", r##"日期與時間的型別"##),
        ("table-help", r##"示例1：
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

示例2：
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"f64的陣列"##),
        ("string", r##"String（字串型別）"##),
        ("array-help", r##"透過多次呼叫 `-a` 來設定陣列元素。
示例：
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> 第一次呼叫時設定索引 0 的元素，第二次設定索引 1 的元素，以此類推。

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"表陣列(用法：`--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"內聯表，相當於更緊湊的標準表。此外，對於巢狀且不存在的keys，請使用內聯表，而不是標準表。"##),
        ("concat-key", r##"分段輸入 key, 結果會自動拼接，不使用 "." 作為分隔符"##),
    ],
}
}

/// Language ID: zh-Hant;
/// Map name: "opt_md";
/// Description: 正體中文, 中國台灣;
pub(super) const fn get_zh_hant_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255m與 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的不同之處在於 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 不會自動 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msplit('.')[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m對於 [a.b.c]，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 與[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m是一樣的。[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m對於簡單情況，用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 會更簡便一點。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m如果 key 名包含了 '.'， 那麼請使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m舉個例子： 
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m在此結構中，"editor.defaultFormatter" 是一個完整的字串，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mdefaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255meditor[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的子鍵（sub-key）。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m當您使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 時，它會將 key 分割成：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m而使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，它會將 key 當作：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m後者才是正確的結構。[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mi64的陣列[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mbool (布林型別)， 只能為 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 或 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: 64位有符號整數型別, 例如：1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m此選項僅支援 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 如果我們在命令末尾新增 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 那麼它會變成這樣子：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255m儲存為指定的格式/檔案，一次只能儲存一個[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255m用法示例1：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m用法示例2：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：ron 擁有真正的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 型別，值為 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 或 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 此處的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 指的是刪掉指定的 key，而不是真正的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 型別。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m假設 test.yml 存在以下表陣列：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m我們如果要刪掉 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mname: test[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 那麼可以執行：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：空字串與 None 是不一樣的
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 呼叫 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 可將其設定為空字串，而 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 則是刪掉整個 key。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m我們如果要刪掉 bin 的第二張表（索引為 1）, 那就執行: 
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m或者是使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255m雙精度浮點數 (IEEE 754-2008標準)，例如： 3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：儘管透過 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 只能為每張表建立一對 K-V pair，但您可以先指定表陣列的索引，再用其他選項來擴充。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255m設定 k-v pair，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 相當於[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255m在沒有搭配 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的情況下，若呼叫則覆蓋(寫入)當前檔案[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255m字串陣列[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m先透過指定的 key 來選擇，再透過指定資料型別的 value 來修改。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 當僅提供 key，未提供 value 時，會列出相應的資料型別和 value。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 比如 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m，會列出 package 及其子表的表格，並提供比 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 更詳細的資訊。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExamples[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m示例1：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m由於指定了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 因此 "2021" 是字串型別，而不是整數型別。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m new value [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m示例2：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m示例3：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: 這是第1個索引*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：以上示例以 引數(argument)/子命令(sub-command) 開頭，省略了根命令名稱，在實際操作中，您需要手動補充。[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255m您如果不清楚自己該用標準表，還是內聯表，那就用內聯表！
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 除了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 外，其他格式將會輸出為標準表的格式。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255m透過 key（鍵）來查詢 value（值）[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m json 1.0 不支援 NaN 和 Inf
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m它會輸出 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255m刪掉指定的key[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255m與 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 子命令的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 非常像，最大的區別在於 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 能多次連續呼叫，輸出到多種格式/多個檔案。
[48;2;34;34;34m[38;2;255;255;255m而 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 子命令的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 只能呼叫一次，輸出到單個檔案。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注： [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 子命令在執行完轉換操作後，所有格式的檔案都不會保留註釋。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 而 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 子命令會保留 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 的註釋，除了 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 外，其他的所有格式 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m都不會[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 保留註釋。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m在未呼叫 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的前提下，呼叫 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 會覆蓋自身。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 若已呼叫 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ，並且還呼叫了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 則修改後的檔案會儲存至 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 所指向的檔案。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 除非原始檔與 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 指向的檔案相同，否則 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 子命令的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的行為將會與 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 部分一致，它們都會對非具體檔案路徑進行自動更名。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 假設 from: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, to: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 那麼輸出的檔案會變成 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 而不是 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 若您無需此特性，則請使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 或 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。[0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255m在修改完成後，預覽完整的檔案[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255m選擇指定的 key（鍵），以"."作為分隔符[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255m獲取指定配置的 value[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255m單對 key-value pair 的表陣列(用法：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255m格式為: "a.b.c", 子表名稱或陣列索引透過 "."（半形的dot符號）來連線。
[48;2;34;34;34m[38;2;255;255;255m當只有一個 "." 時，會列出整張表[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255m建立/更新/刪除 配置檔案的結構[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: 第1個元素是 true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mbool 的陣列[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255m標準表（standard table），用於儲存多對 K-V pairs[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255m日期與時間的型別[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例1：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m示例2：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mf64的陣列[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString（字串型別）[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255m透過多次呼叫 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 來設定陣列元素。
[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 第一次呼叫時設定索引 0 的元素，第二次設定索引 1 的元素，以此類推。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255m表陣列(用法：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255m內聯表，相當於更緊湊的標準表。此外，對於巢狀且不存在的keys，請使用內聯表，而不是標準表。[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255m分段輸入 key, 結果會自動拼接，不使用 "." 作為分隔符[0m"##),
    ],
}
}

/// Language ID: zh-Hant;
/// Map name: "theme";
/// Description: 正體中文, 中國台灣;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "無效的主題名稱");
/// ```
pub(super) const fn get_zh_hant_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"無效的主題名稱"##),
    ],
}
}

/// Language ID: zh-Hant;
/// Map name: "args";
/// Description: 正體中文, 中國台灣;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "禁用主題背景（預設為啟用狀態，若呼叫`--dis-theme-bg` 則禁用）");
/// ```
pub(super) const fn get_zh_hant_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"自定義主題集合檔案，您還需要搭配 `--theme` 指定主題名稱"##),
        ("shell-completion", r##"為 shell 生成補全指令碼"##),
        ("theme-file-help", r##"支援的檔案格式為 rust crate `syntect` 所 dump 出來的二進位制資料。  
本質上，這些主題是舊版的 sublime 的主題。
若未指定 `--tf`， 則讀取環境變數 `$TOMLYRE_THEME_FILE` 的值。"##),
        ("from-format", r##"手動指定來源的檔案格式（預設會自動判斷）"##),
        ("disable-theme-background", r##"禁用主題背景（預設為啟用狀態，若呼叫`--dis-theme-bg` 則禁用）"##),
        ("theme-uncompressed-file-help", r##"> 未壓縮的版本要比壓縮版本的優先順序更高。  

若未指定 `--tuf`， 則讀取環境變數 `$TOMLYRE_THEME_UNCOMPRESSED_FILE` 的值。"##),
        ("disable-theme-background-help", r##"若未指定 `--dis-theme-bg`， 則讀取環境變數 `$TOMLYRE_THEME_BG` 的值。

若該值為 false 則禁用

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"類似於`theme-file`, 但要求未壓縮的 dump 資料(檔案)"##),
        ("shell-completion-help", r##"您需要手動指定 shell 的名稱來生成補全指令碼。
例如，對於 Windows 上的 powershell, 您可以使用：`--sh-comp pwsh`

> Q: 什麼是 shell 的補全功能？

> A: 當我們在命令列上輸入命令時，如果按下Tab鍵，補全指令碼就會被觸發，開始搜尋可能的補全選項，並將這些選項呈現出來以供選擇。此功能可以極大地提高命令列輸入的效率和準確性。

不同 shell 的補全指令碼所存放的位置是不一樣的。對於 zsh，您需要將其存放於 `$fpath` 陣列所包含的路徑。
您可以使用 `print -l $fpath` 來獲取該陣列的內容。
假設裡面包含了 `/usr/local/share/zsh/site-functions` 目錄， 那麼您可以使用以下命令來將補全指令碼寫入到該目錄下的 `_tomlyre` 檔案。

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

為了避免使用 root 許可權，您可以將其放在使用者子目錄下。比如說，您可以使用以下命令向該陣列頭部插入一個元素，再使用相關的目錄。
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"當從 stdin 讀取資料時，例如： `cat a | tomlyre conv -`，它首先會嘗試自動判斷檔案格式。
然而，自動判斷有可能會失敗，這時候就需要手動指定格式了。
示例：`cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"當僅使用 `--ts` 引數時，會列出內建的樣式集合。
除了 markdown 可以縮寫為 md 外, 其他的樣式都要用完整的名稱。
用法示例1：`--ts md` 
用法示例2：`--table-style ascii`

注：目前不支援從檔案中讀取 **表格樣式** 的集合，如果您需要此功能或其他額外的功能，歡迎提交 issue 進行反饋。"##),
        ("table-style", r##"指定表格的樣式"##),
        ("theme-help", r##"當僅使用 `-t` 引數時，會列出當前主題的列表。
若未指定主題檔案，則使用內建的主題集合。
假設存在一個主題為 **ayu-dark**, 則您可以使用 `-t ayu-dark` 來使用該主題。
若主題名稱包含空格，則請使用雙引號或單引號將其包裹起來。

> 注：有一個特殊主題為 `None`，代表沒有語法高亮的純文字樣式。

當未指定 `--theme` 選項時，將會從環境變數 `$TOMLYRE_THEME` 中讀取資料。
此幫助資訊的 **語法高亮** 主題也來自於該 **環境變數**。

> 預設的幫助資訊是硬編碼的，載入速度非常快。  
> 載入硬編碼的文字與載入 `None` 主題的文字的速度是同一個數量級的，而動態載入則會變慢兩個數量級及以上。  
> 設定 `$TOMLYRE_THEME` 環境變數可能會導致它重新載入主題，這將 **嚴重** 拖慢啟動速度。  
> 除非您打算修改幫助資訊的主題，或者是禁用語法高亮，否則請使用 `-t` 引數，而不是修改環境變數。  

使用環境變數的例子：
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

使用命令列引數的例子:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

對於手動執行 "conv/get/set" 等相關操作所輸出的內容，`-t` 指定的主題的優先順序 > 環境變數。
若未呼叫 `-t`，則讀取環境變數；若環境變數為空，則使用 **Monokai Extended**。"##),
        ("theme", r##"指定語法高亮的主題名稱"##),
    ],
}
}

/// Language ID: zh-Hant;
/// Map name: "args_md";
/// Description: 正體中文, 中國台灣;
pub(super) const fn get_zh_hant_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255m自定義主題集合檔案，您還需要搭配 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 指定主題名稱[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255m為 shell 生成補全指令碼[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255m支援的檔案格式為 rust crate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 所 dump 出來的二進位制資料。[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m本質上，這些主題是舊版的 sublime 的主題。
[48;2;34;34;34m[38;2;255;255;255m若未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 則讀取環境變數 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的值。[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255m手動指定來源的檔案格式（預設會自動判斷）[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255m禁用主題背景（預設為啟用狀態，若呼叫[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 則禁用）[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 未壓縮的版本要比壓縮版本的優先順序更高。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m若未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 則讀取環境變數 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的值。[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255m若未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 則讀取環境變數 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的值。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m若該值為 false 則禁用
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255m類似於[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 但要求未壓縮的 dump 資料(檔案)[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255m您需要手動指定 shell 的名稱來生成補全指令碼。
[48;2;34;34;34m[38;2;255;255;255m例如，對於 Windows 上的 powershell, 您可以使用：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q: 什麼是 shell 的補全功能？
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: 當我們在命令列上輸入命令時，如果按下Tab鍵，補全指令碼就會被觸發，開始搜尋可能的補全選項，並將這些選項呈現出來以供選擇。此功能可以極大地提高命令列輸入的效率和準確性。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m不同 shell 的補全指令碼所存放的位置是不一樣的。對於 zsh，您需要將其存放於 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 陣列所包含的路徑。
[48;2;34;34;34m[38;2;255;255;255m您可以使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 來獲取該陣列的內容。
[48;2;34;34;34m[38;2;255;255;255m假設裡面包含了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 目錄， 那麼您可以使用以下命令來將補全指令碼寫入到該目錄下的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 檔案。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m為了避免使用 root 許可權，您可以將其放在使用者子目錄下。比如說，您可以使用以下命令向該陣列頭部插入一個元素，再使用相關的目錄。
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255m當從 stdin 讀取資料時，例如： [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，它首先會嘗試自動判斷檔案格式。
[48;2;34;34;34m[38;2;255;255;255m然而，自動判斷有可能會失敗，這時候就需要手動指定格式了。
[48;2;34;34;34m[38;2;255;255;255m示例：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255m當僅使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 引數時，會列出內建的樣式集合。
[48;2;34;34;34m[38;2;255;255;255m除了 markdown 可以縮寫為 md 外, 其他的樣式都要用完整的名稱。
[48;2;34;34;34m[38;2;255;255;255m用法示例1：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 
[48;2;34;34;34m[38;2;255;255;255m用法示例2：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m注：目前不支援從檔案中讀取 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m表格樣式[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 的集合，如果您需要此功能或其他額外的功能，歡迎提交 issue 進行反饋。[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255m指定表格的樣式[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255m當僅使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 引數時，會列出當前主題的列表。
[48;2;34;34;34m[38;2;255;255;255m若未指定主題檔案，則使用內建的主題集合。
[48;2;34;34;34m[38;2;255;255;255m假設存在一個主題為 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 則您可以使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 來使用該主題。
[48;2;34;34;34m[38;2;255;255;255m若主題名稱包含空格，則請使用雙引號或單引號將其包裹起來。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：有一個特殊主題為 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m，代表沒有語法高亮的純文字樣式。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m當未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 選項時，將會從環境變數 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 中讀取資料。
[48;2;34;34;34m[38;2;255;255;255m此幫助資訊的 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m語法高亮[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 主題也來自於該 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m環境變數[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 預設的幫助資訊是硬編碼的，載入速度非常快。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 載入硬編碼的文字與載入 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 主題的文字的速度是同一個數量級的，而動態載入則會變慢兩個數量級及以上。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 設定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 環境變數可能會導致它重新載入主題，這將 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m嚴重[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 拖慢啟動速度。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 除非您打算修改幫助資訊的主題，或者是禁用語法高亮，否則請使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 引數，而不是修改環境變數。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m使用環境變數的例子：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m使用命令列引數的例子:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m對於手動執行 "conv/get/set" 等相關操作所輸出的內容，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 指定的主題的優先順序 > 環境變數。
[48;2;34;34;34m[38;2;255;255;255m若未呼叫 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，則讀取環境變數；若環境變數為空，則使用 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m。[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255m指定語法高亮的主題名稱[0m"##),
    ],
}
}

/// Language ID: zh-Hant;
/// Map name: "conv";
/// Description: 正體中文, 中國台灣;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "在 toml、yaml、json 和 ron 等格式之間進行轉換");
/// ```
pub(super) const fn get_zh_hant_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"在 toml、yaml、json 和 ron 等格式之間進行轉換"##),
        ("from", r##"指定輸入的檔案"##),
        ("conv-to", r##"指定輸出的格式/檔案，支援多次呼叫"##),
        ("save", r##"當呼叫此選項後，`--to` 所指定的檔案將被儲存"##),
        ("conv-to-help", r##"如果當前目錄下存在 **test.json** 檔案，那麼您可以呼叫 `conv test.json -t yml` 將其轉換為 yml (yaml),
也可以手動指定檔案路徑：`conv test.json -t /tmp/test.yml`

支援的格式取決於此軟體包啟用的功能，並非所有功能都會預設啟用。

此外，在使用前，您必須要了解的事情如下：
- 1. 它會對非具體檔案路徑進行自動更名。
  - 1.1: 只有一個層級的檔案會被斷定為檔案格式，而不是檔案路徑
    當呼叫 `conv test.toml -t ron`, 輸出的檔案會自動變成 `test.ron`, 而不是 `ron`。
    若您無需此特性，則請使用 `-t ./ron`, 或者是 `-t $PWD/ron`。
  - 1.2: 指向的是目錄而不是檔案
    比如 `-t /tmp/`，我們只能將 `test.toml` 放在 `/tmp/` 的子級，而不是覆蓋 `/tmp` 自身。
    若您無需此特性，則請使用完整的（非目錄）檔案路徑，例如 `-t /tmp/xx.toml`，而不是 `-t /tmp/`。
- 2. 當未呼叫 `--save` 時，指定的檔案不會自動儲存。
- 3. 當呼叫 `--save` 時，conv 子命令的輸入與輸出不能為相同的檔案。
- 4. 轉換後的檔案不會保留註釋。"##),
    ],
}
}

/// Language ID: zh-Hant;
/// Map name: "conv_md";
/// Description: 正體中文, 中國台灣;
pub(super) const fn get_zh_hant_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255m在 toml、yaml、json 和 ron 等格式之間進行轉換[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255m指定輸入的檔案[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255m指定輸出的格式/檔案，支援多次呼叫[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255m當呼叫此選項後，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 所指定的檔案將被儲存[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255m如果當前目錄下存在 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 檔案，那麼您可以呼叫 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 將其轉換為 yml (yaml),
[48;2;34;34;34m[38;2;255;255;255m也可以手動指定檔案路徑：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m支援的格式取決於此軟體包啟用的功能，並非所有功能都會預設啟用。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m此外，在使用前，您必須要了解的事情如下：
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m 它會對非具體檔案路徑進行自動更名。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: 只有一個層級的檔案會被斷定為檔案格式，而不是檔案路徑
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m當呼叫 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 輸出的檔案會自動變成 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m若您無需此特性，則請使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 或者是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: 指向的是目錄而不是檔案
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m比如 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，我們只能將 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 放在 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的子級，而不是覆蓋 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 自身。
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m若您無需此特性，則請使用完整的（非目錄）檔案路徑，例如 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. 當未呼叫 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 時，指定的檔案不會自動儲存。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. 當呼叫 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 時，conv 子命令的輸入與輸出不能為相同的檔案。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. 轉換後的檔案不會保留註釋。[0m"##),
    ],
}
}

/// zh-Hant: 正體中文, 中國台灣
pub(super) const fn get_zh_hant_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_zh_hant_map_args),
        ("conv_md", get_zh_hant_map_conv_md),
        ("parser", get_zh_hant_map_parser),
        ("opt_md", get_zh_hant_map_opt_md),
        ("args_md", get_zh_hant_map_args_md),
        ("opt", get_zh_hant_map_opt),
        ("log-core", get_zh_hant_map_log_core),
        ("conv", get_zh_hant_map_conv),
        ("theme", get_zh_hant_map_theme),
    ],
}
}

/// Language ID: zh-Hant-HK;
/// Map name: "log-core";
/// Description: 繁體中文, 中國香港;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "調試");
/// ```
pub(super) const fn get_zh_hant_hk_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"調試"##),
        ("warn", r##"警告"##),
        ("info", r##"信息"##),
        ("error", r##"錯誤"##),
        ("trace", r##"追蹤"##),
        ("init-logger", r##"正在初始化 logger（日誌記錄器）..."##),
    ],
}
}

/// Language ID: zh-Hant-HK;
/// Map name: "parser";
/// Description: 繁體中文, 中國香港;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("parser", "sh-completion::about-stderr");
///
/// assert_eq!(msg, "下面是 stderr 輸出，在重定向輸出補全腳本時，您應該使用 `>/path/to/file` 而不是 `2>/path/to/file`");
/// ```
pub(super) const fn get_zh_hant_hk_map_parser() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("sh-completion::about-stderr", r##"下面是 stderr 輸出，在重定向輸出補全腳本時，您應該使用 `>/path/to/file` 而不是 `2>/path/to/file`"##),
    ],
}
}

/// Language ID: zh-Hant-HK;
/// Map name: "opt";
/// Description: 繁體中文, 中國香港;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("opt", "aot");
///
/// assert_eq!(msg, "表數組(用法：`--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)");
/// ```
pub(super) const fn get_zh_hant_hk_map_opt() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"與 `-k` 的不同之處在於 `--ck` 不會自動 `split('.')`。  
對於 [a.b.c]，`-k a.b.c` 與`--ck a --ck b --ck c`是一樣的。  
對於簡單情況，用 `-k` 會更簡便一點。

如果 key 名包含了 '.'， 那麼請使用 `--ck` 而不是 `-k`。
舉個例子： 

```json
"[typescript]": {
    "editor.defaultFormatter": "rome.rome"
}
```

在此結構中，"editor.defaultFormatter" 是一個完整的字符串，`defaultFormatter` 不是 `editor` 的子鍵（sub-key）。

當您使用 `-k '[typescript]'.'editor.defaultFormatter'` 時，它會將 key 分割成：

- "[typescript]"
- "editor"
- "defaultFormatter"

而使用 `--ck '[typescript]' --ck editor.defaultFormatter`，它會將 key 當作：

- "[typescript]"
- "editor.defaultFormatter"

後者才是正確的結構。"##),
        ("num-arr", r##"i64的數組"##),
        ("bool", r##"bool (布爾類型)， 只能為 `true` 或 `false`"##),
        ("num", r##"i64: 64位有符號整數類型, 例如：1145"##),
        ("datetime-help", r##"示例：
```sh
set test.toml -k time --datetime '2023-04-16 09:14:44Z'
```

```toml
time = 2023-04-16T09:14:44Z
```

此選項僅支持 `toml`, 如果我們在命令末尾添加 `-t json`, 那麼它會變成這樣子：

```json
"time": {
    "$__toml_private_datetime": "2023-04-16T09:14:44Z"
}
```"##),
        ("set-save-to", r##"保存為指定的格式/文件，一次只能保存一個"##),
        ("get-opts-help", r##"用法示例1：`get Cargo.toml -k package.name`
用法示例2：`get cfg.yaml -k .`
"##),
        ("none-help", r##"> 注：ron 擁有真正的 `Option<T>` 類型，值為 `Some(T)` 或 `None`
> 此處的 `--none` 指的是刪掉指定的 key，而不是真正的 `Option<T>` 類型。

假設 test.yml 存在以下表數組：

```yaml
bin:
- doc: false
  name: tomlyre
  path: src/main.rs
- doc: true
  name: test
  path: src/test.rs
```

我們如果要刪掉 **name: test**, 那麼可以執行：
```sh
set test.yml -k bin.1.name --none
```

> 注：空字符串與 None 是不一樣的
> 調用 `set test.yml --kv bin.1.name ""` 可將其設置為空字符串，而 `--none` 則是刪掉整個 key。

我們如果要刪掉 bin 的第二張表（索引為 1）, 那就執行: 
```sh
set test.yml -k bin.1 --none
```

或者是使用 `--rm`
```sh
set test.yml -k bin.1 --rm
```"##),
        ("f64", r##"雙精度浮點數 (IEEE 754-2008標準)，例如： 3.14159265359"##),
        ("saot-help", r##"> 注：儘管通過 `--saot` 只能為每張表創建一對 K-V pair，但您可以先指定表數組的索引，再用其他選項來擴充。

示例：
```sh
set test.toml -k mount --saot src "/sdcard/Download" --saot src "/data/media/0" --save --pre
```

```toml
[[mount]]
src = "/sdcard/Download"

[[mount]]
src = "/data/media/0"
```"##),
        ("set-kv", r##"設置 k-v pair，`--kv x.y.z value` 相當於`-k x.y.z -s value`"##),
        ("set-save", r##"在沒有搭配 `--to` 的情況下，若調用則覆蓋(寫入)當前文件"##),
        ("array", r##"字符串數組"##),
        ("aot-help", r##"示例：
```sh
set test.toml -k package --aot 'name, glossa, version, 0.0.1-alpha.6' --aot 'name, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84'
```

```toml
[[package]]
name = "glossa"
version = "0.0.1-alpha.6"

[[package]]
name = "wasm-bindgen"
"ver sion" = "0.2  .84"
version = "0.2.84"
```"##),
        ("set-opts-help", r##"# set

先通過指定的 key 來選擇，再通過指定數據類型的 value 來修改。

> 當僅提供 key，未提供 value 時，會列出相應的數據類型和 value。
> 比如 `set Cargo.toml -k package`，會列出 package 及其子表的表格，並提供比 `get` 更詳細的信息。

## Examples

### 示例1：
```sh
--ts md set test.toml --key package.edition --str 2021
```

由於指定了 `--str`，而不是 `--num`, 因此 "2021" 是字符串類型，而不是整數類型。

| package | new value |
|---------|-----------|
| edition | "2021"    |


```toml
[package]
edition = "2021"
```

### 示例2：
```sh
set ~/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml
```

```yaml
remote.SSH.remotePlatform:
  test:
    k1: v1
```

### 示例3：
```sh
set test.yml -k main -a hello -a wonderful -a world --pre --to ron --save
```

```ron
"main": [
  /*[0]: 這是第1個索引*/ "hello",
  /*[1]*/ "wonderful",
  /*[2]*/ "world",
]
```

> 注：以上示例以 參數(argument)/子命令(sub-command) 開頭，省略了根命令名稱，在實際操作中，您需要手動補充。"##),
        ("num-arr-help", r##"示例：
```sh
set test.json -k hello --na -1 --na -45 --na 14
```

```json
"hello": [
    -1,
    -45,
    14
]
```"##),
        ("inline-table-help", r##"您如果不清楚自己該用標準表，還是內聯表，那就用內聯表！

示例：
```sh
set test.toml -k container --itb arch riscv64 --itb os fedora --itb codename rawhide
```

```toml
container = { arch = "riscv64", os = "fedora", codename = "rawhide" }
```

> 除了 `toml` 外，其他格式將會輸出為標準表的格式。

```json
"container": {
    "arch": "riscv64",
    "codename": "rawhide",
    "os": "fedora"
}
```"##),
        ("key", r##"通過 key（鍵）來查詢 value（值）"##),
        ("f64-arr-help", r##"示例：
```sh
set test.yml -k hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf
```

```yaml
hello:
- 3.14
- 0.45
- .nan
- .inf
```

> json 1.0 不支持 NaN 和 Inf

它會輸出 `null`， 而不是 `NaN` 。

```sh
set test.yml -k zz --fa NaN --fa Inf --to json --pre
```

```json
"zz": [
    null,
    null
]
```
"##),
        ("none", r##"刪掉指定的key"##),
        ("set-save-to-help", r##"與 `conv` 子命令的 `-t` 非常像，最大的區別在於 `conv` 的 `-t` 能多次連續調用，輸出到多種格式/多個文件。
而 `set` 子命令的 `-t` 只能調用一次，輸出到單個文件。

> 注： `conv` 子命令在執行完轉換操作後，所有格式的文件都不會保留註釋。
> 而 `set` 子命令會保留 **toml** 的註釋，除了 **toml** 外，其他的所有格式 **都不會** 保留註釋。

- 在未調用 `-t` 的前提下，調用 `--save` 會覆蓋自身。
- 若已調用 `-t` ，並且還調用了 `--save`, 則修改後的文件會保存至 `-t` 所指向的文件。
- 除非源文件與 `-t` 指向的文件相同，否則 `set` 子命令的 `-t` 的行為將會與 `conv` 的 `-t` 部分一致，它們都會對非具體文件路徑進行自動更名。
  - 假設 from: **test.yml**, to: **json**, 那麼輸出的文件會變成 **test.json**, 而不是 **json**
  - 若您無需此特性，則請使用 `-t ./json` 或 `-t $PWD/json`，而不是 `-t json`。"##),
        ("preview", r##"在修改完成後，預覽完整的文件"##),
        ("set-key", r##"選擇指定的 key（鍵），以"."作為分隔符"##),
        ("get-opts", r##"獲取指定配置的 value"##),
        ("saot", r##"單對 key-value pair 的表數組(用法：`--saot k1 v1 --saot k2 v2...`)"##),
        ("key-help", r##"格式為: "a.b.c", 子表名稱或數組索引通過 "."（半角的dot符號）來連接。
當只有一個 "." 時，會列出整張表"##),
        ("set-opts", r##"創建/更新/刪除 配置文件的結構"##),
        ("bool-arr-help", r##"示例：
```sh
set test.ron -k hello --ba true --ba false --ba true
```

```ron
"hello": [
    // [0]: 第1個元素是 true
    true,
    /*[1]*/ false,
    /*[2]*/ true,
]
```"##),
        ("bool-arr", r##"bool 的數組"##),
        ("table", r##"標準表（standard table），用於存儲多對 K-V pairs"##),
        ("datetime", r##"日期與時間的類型"##),
        ("table-help", r##"示例1：
```sh
set test.toml -k mount.1.test --map name sd --map os android
```

```toml
[[mount]]
src = "/data/media/0"

[mount.test]
name = "sd"
os = "android"
```

示例2：
```sh
set test.yml -k container --tb arch riscv64 --tb os debian --tb codename trixie
```

```yaml
container:
  arch: riscv64
  codename: trixie
  os: debian
```"##),
        ("f64-arr", r##"f64的數組"##),
        ("string", r##"String（字符串類型）"##),
        ("array-help", r##"通過多次調用 `-a` 來設置數組元素。
示例：
```sh
set test.toml -k test.array -a hello -a wonderful -a world
```

> 第一次調用時設置索引 0 的元素，第二次設置索引 1 的元素，以此類推。

```toml
[test]
array = ["hello", "wonderful", "world"]
```"##),
        ("aot", r##"表數組(用法：`--aot 'k1, v1, k2, v2' --aot 'k1, v1'`)"##),
        ("inline-table", r##"內聯表，相當於更緊湊的標準表。此外，對於嵌套且不存在的keys，請使用內聯表，而不是標準表。"##),
        ("concat-key", r##"分段輸入 key, 結果會自動拼接，不使用 "." 作為分隔符"##),
    ],
}
}

/// Language ID: zh-Hant-HK;
/// Map name: "opt_md";
/// Description: 繁體中文, 中國香港;
pub(super) const fn get_zh_hant_hk_map_opt_md() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (7, 9),
        (4, 36),
        (0, 24),
        (8, 2),
        (0, 9),
        (18, 3),
        (6, 32),
        (0, 0),
    ],
    entries: &[
        ("concat-key-help", r##"[48;2;34;34;34m[38;2;255;255;255m與 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的不同之處在於 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 不會自動 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msplit('.')[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m對於 [a.b.c]，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k a.b.c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 與[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck a --ck b --ck c[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m是一樣的。[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m對於簡單情況，用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 會更簡便一點。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m如果 key 名包含了 '.'， 那麼請使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m舉個例子： 
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m[typescript][48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31meditor.defaultFormatter[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrome.rome[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m在此結構中，"editor.defaultFormatter" 是一個完整的字符串，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mdefaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255meditor[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的子鍵（sub-key）。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m當您使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k '[typescript]'.'editor.defaultFormatter'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 時，它會將 key 分割成：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m而使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ck '[typescript]' --ck editor.defaultFormatter[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，它會將 key 當作：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[typescript]"
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m "editor.defaultFormatter"
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m後者才是正確的結構。[0m"##),
        ("num-arr", r##"[48;2;34;34;34m[38;2;255;255;255mi64的數組[0m"##),
        ("bool", r##"[48;2;34;34;34m[38;2;255;255;255mbool (布爾類型)， 只能為 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtrue[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 或 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mfalse[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("num", r##"[48;2;34;34;34m[38;2;255;255;255mi64: 64位有符號整數類型, 例如：1145[0m"##),
        ("datetime-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m time --datetime [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116m2023-04-16 09:14:44Z[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mtime[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m此選項僅支持 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 如果我們在命令末尾添加 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 那麼它會變成這樣子：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mtime[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31m$__toml_private_datetime[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2023-04-16T09:14:44Z[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-save-to", r##"[48;2;34;34;34m[38;2;255;255;255m保存為指定的格式/文件，一次只能保存一個[0m"##),
        ("get-opts-help", r##"[48;2;34;34;34m[38;2;255;255;255m用法示例1：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget Cargo.toml -k package.name[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m用法示例2：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget cfg.yaml -k .[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：ron 擁有真正的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 類型，值為 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mSome(T)[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 或 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 此處的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 指的是刪掉指定的 key，而不是真正的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mOption<T>[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 類型。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m假設 test.yml 存在以下表數組：
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mbin[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/main.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mdoc[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtest[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mpath[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116msrc/test.rs[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m我們如果要刪掉 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mname: test[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 那麼可以執行：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1.name --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：空字符串與 None 是不一樣的
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 調用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset test.yml --kv bin.1.name ""[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 可將其設置為空字符串，而 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--none[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 則是刪掉整個 key。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m我們如果要刪掉 bin 的第二張表（索引為 1）, 那就執行: 
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --none[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m或者是使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--rm[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m bin.1 --rm[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64", r##"[48;2;34;34;34m[38;2;255;255;255m雙精度浮點數 (IEEE 754-2008標準)，例如： 3.14159265359[0m"##),
        ("saot-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：儘管通過 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 只能為每張表創建一對 K-V pair，但您可以先指定表數組的索引，再用其他選項來擴充。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --saot src [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m --save --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/sdcard/Download[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-kv", r##"[48;2;34;34;34m[38;2;255;255;255m設置 k-v pair，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--kv x.y.z value[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 相當於[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-k x.y.z -s value[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("set-save", r##"[48;2;34;34;34m[38;2;255;255;255m在沒有搭配 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的情況下，若調用則覆蓋(寫入)當前文件[0m"##),
        ("array", r##"[48;2;34;34;34m[38;2;255;255;255m字符串數組[0m"##),
        ("aot-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m package --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, glossa, version, 0.0.1-alpha.6[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m --aot [48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;230;219;116mname, wasm-bindgen, ver sion, 0.2  .84,  version, 0.2.84[48;2;34;34;34m[38;2;255;255;255m'[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mglossa[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.0.1-alpha.6[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwasm-bindgen[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mver sion[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2  .84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mversion[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m0.2.84[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("set-opts-help", r##"[48;2;34;34;34m[38;2;253;151;31m#[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mset[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m先通過指定的 key 來選擇，再通過指定數據類型的 value 來修改。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 當僅提供 key，未提供 value 時，會列出相應的數據類型和 value。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 比如 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset Cargo.toml -k package[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m，會列出 package 及其子表的表格，並提供比 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mget[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 更詳細的信息。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m##[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31mExamples[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m示例1：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m--ts[48;2;34;34;34m[38;2;255;255;255m md set test.toml[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mkey[48;2;34;34;34m[38;2;255;255;255m package.edition[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mstr[48;2;34;34;34m[38;2;255;255;255m 2021[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m由於指定了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--str[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--num[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 因此 "2021" 是字符串類型，而不是整數類型。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m package [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m new value [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m---------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m-----------[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m edition [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m "2021"    [48;2;34;34;34m[38;2;255;255;255m|[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mpackage[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114medition[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m2021[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m示例2：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.config/Code/User/settings.json --ck remote.SSH.remotePlatform --ck test --map k1 v1 --to /tmp/code.yaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mremote.SSH.remotePlatform[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mtest[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114mk1[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mv1[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;253;151;31m###[48;2;34;34;34m[38;2;253;151;31m [48;2;34;34;34m[38;2;253;151;31m示例3：[48;2;34;34;34m[38;2;253;151;31m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m main [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world --pre --to ron --save[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"main[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[0]: 這是第1個索引*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"wonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"world[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：以上示例以 參數(argument)/子命令(sub-command) 開頭，省略了根命令名稱，在實際操作中，您需要手動補充。[0m"##),
        ("num-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.json [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --na -1 --na -45 --na 14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m1[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;249;38;114m-[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("inline-table-help", r##"[48;2;34;34;34m[38;2;255;255;255m您如果不清楚自己該用標準表，還是內聯表，那就用內聯表！
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --itb arch riscv64 --itb os fedora --itb codename rawhide[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 除了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtoml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 外，其他格式將會輸出為標準表的格式。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mcontainer[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m{[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31march[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mcodename[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mrawhide[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;253;151;31mos[48;2;34;34;34m[38;2;253;151;31m"[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mfedora[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m}[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("key", r##"[48;2;34;34;34m[38;2;255;255;255m通過 key（鍵）來查詢 value（值）[0m"##),
        ("f64-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --fa 3.14 --fa 45e-2 --fa NaN --fa Inf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mhello[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m3[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m14[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m0[48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255m45[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255mnan[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255m.[48;2;34;34;34m[38;2;190;132;255minf[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m json 1.0 不支持 NaN 和 Inf
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m它會輸出 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mnull[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNaN[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m zz --fa NaN --fa Inf --to json --pre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mjson[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mzz[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mnull[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[0m"##),
        ("none", r##"[48;2;34;34;34m[38;2;255;255;255m刪掉指定的key[0m"##),
        ("set-save-to-help", r##"[48;2;34;34;34m[38;2;255;255;255m與 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 子命令的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 非常像，最大的區別在於 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 能多次連續調用，輸出到多種格式/多個文件。
[48;2;34;34;34m[38;2;255;255;255m而 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 子命令的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 只能調用一次，輸出到單個文件。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注： [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 子命令在執行完轉換操作後，所有格式的文件都不會保留註釋。
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 而 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 子命令會保留 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 的註釋，除了 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtoml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 外，其他的所有格式 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m都不會[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 保留註釋。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m在未調用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的前提下，調用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 會覆蓋自身。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 若已調用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m ，並且還調用了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 則修改後的文件會保存至 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 所指向的文件。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 除非源文件與 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 指向的文件相同，否則 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mset[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 子命令的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的行為將會與 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 部分一致，它們都會對非具體文件路徑進行自動更名。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 假設 from: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.yml[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, to: [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 那麼輸出的文件會變成 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 而不是 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mjson[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 若您無需此特性，則請使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 或 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t json[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。[0m"##),
        ("preview", r##"[48;2;34;34;34m[38;2;255;255;255m在修改完成後，預覽完整的文件[0m"##),
        ("set-key", r##"[48;2;34;34;34m[38;2;255;255;255m選擇指定的 key（鍵），以"."作為分隔符[0m"##),
        ("get-opts", r##"[48;2;34;34;34m[38;2;255;255;255m獲取指定配置的 value[0m"##),
        ("saot", r##"[48;2;34;34;34m[38;2;255;255;255m單對 key-value pair 的表數組(用法：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--saot k1 v1 --saot k2 v2...[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("key-help", r##"[48;2;34;34;34m[38;2;255;255;255m格式為: "a.b.c", 子表名稱或數組索引通過 "."（半角的dot符號）來連接。
[48;2;34;34;34m[38;2;255;255;255m當只有一個 "." 時，會列出整張表[0m"##),
        ("set-opts", r##"[48;2;34;34;34m[38;2;255;255;255m創建/更新/刪除 配置文件的結構[0m"##),
        ("bool-arr-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.ron [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m hello --ba true --ba false --ba true[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mron[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;230;219;116m"hello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m: [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m//[48;2;34;34;34m[38;2;117;113;94m [0]: 第1個元素是 true
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[1]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mfalse[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;117;113;94m/*[48;2;34;34;34m[38;2;117;113;94m[2]*/[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;190;132;255mtrue[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("bool-arr", r##"[48;2;34;34;34m[38;2;255;255;255mbool 的數組[0m"##),
        ("table", r##"[48;2;34;34;34m[38;2;255;255;255m標準表（standard table），用於存儲多對 K-V pairs[0m"##),
        ("datetime", r##"[48;2;34;34;34m[38;2;255;255;255m日期與時間的類型[0m"##),
        ("table-help", r##"[48;2;34;34;34m[38;2;255;255;255m示例1：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m mount.1.test --map name sd --map os android[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m]][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114msrc[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116m/data/media/0[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mmount[48;2;34;34;34m[38;2;255;255;255m.[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mname[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116msd[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mandroid[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m示例2：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.yml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m container --tb arch riscv64 --tb os debian --tb codename trixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255myaml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114mcontainer[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114march[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mriscv64[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mcodename[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mtrixie[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;249;38;114mos[48;2;34;34;34m[38;2;255;255;255m:[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116mdebian[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("f64-arr", r##"[48;2;34;34;34m[38;2;255;255;255mf64的數組[0m"##),
        ("string", r##"[48;2;34;34;34m[38;2;255;255;255mString（字符串類型）[0m"##),
        ("array-help", r##"[48;2;34;34;34m[38;2;255;255;255m通過多次調用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-a[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 來設置數組元素。
[48;2;34;34;34m[38;2;255;255;255m示例：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239mset[48;2;34;34;34m[38;2;255;255;255m test.toml [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31mk[48;2;34;34;34m[38;2;255;255;255m test.array [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m hello [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m wonderful [48;2;34;34;34m[38;2;253;151;31m-[48;2;34;34;34m[38;2;253;151;31ma[48;2;34;34;34m[38;2;255;255;255m world[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 第一次調用時設置索引 0 的元素，第二次設置索引 1 的元素，以此類推。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mtoml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;166;226;46mtest[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;249;38;114marray[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m=[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m[[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mhello[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mwonderful[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m,[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;230;219;116mworld[48;2;34;34;34m[38;2;230;219;116m"[48;2;34;34;34m[38;2;255;255;255m][48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("aot", r##"[48;2;34;34;34m[38;2;255;255;255m表數組(用法：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--aot 'k1, v1, k2, v2' --aot 'k1, v1'[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m)[0m"##),
        ("inline-table", r##"[48;2;34;34;34m[38;2;255;255;255m內聯表，相當於更緊湊的標準表。此外，對於嵌套且不存在的keys，請使用內聯表，而不是標準表。[0m"##),
        ("concat-key", r##"[48;2;34;34;34m[38;2;255;255;255m分段輸入 key, 結果會自動拼接，不使用 "." 作為分隔符[0m"##),
    ],
}
}

/// Language ID: zh-Hant-HK;
/// Map name: "theme";
/// Description: 繁體中文, 中國香港;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "無效的主題名稱");
/// ```
pub(super) const fn get_zh_hant_hk_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"無效的主題名稱"##),
    ],
}
}

/// Language ID: zh-Hant-HK;
/// Map name: "args";
/// Description: 繁體中文, 中國香港;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("args", "disable-theme-background");
///
/// assert_eq!(msg, "禁用主題背景（默認為啓用狀態，若調用`--dis-theme-bg` 則禁用）");
/// ```
pub(super) const fn get_zh_hant_hk_map_args() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"自定義主題集合文件，您還需要搭配 `--theme` 指定主題名稱"##),
        ("shell-completion", r##"為 shell 生成補全腳本"##),
        ("theme-file-help", r##"支持的文件格式為 rust crate `syntect` 所 dump 出來的二進制數據。  
本質上，這些主題是舊版的 sublime 的主題。
若未指定 `--tf`， 則讀取環境變量 `$TOMLYRE_THEME_FILE` 的值。"##),
        ("from-format", r##"手動指定來源的文件格式（默認會自動判斷）"##),
        ("disable-theme-background", r##"禁用主題背景（默認為啓用狀態，若調用`--dis-theme-bg` 則禁用）"##),
        ("theme-uncompressed-file-help", r##"> 未壓縮的版本要比壓縮版本的優先級更高。  

若未指定 `--tuf`， 則讀取環境變量 `$TOMLYRE_THEME_UNCOMPRESSED_FILE` 的值。"##),
        ("disable-theme-background-help", r##"若未指定 `--dis-theme-bg`， 則讀取環境變量 `$TOMLYRE_THEME_BG` 的值。

若該值為 false 則禁用

```sh
env TOMLYRE_THEME_BG=false TOMLYRE_THEME="ayu-dark" tomlyre --help
```"##),
        ("theme-uncompressed-file", r##"類似於`theme-file`, 但要求未壓縮的 dump 數據(文件)"##),
        ("shell-completion-help", r##"您需要手動指定 shell 的名稱來生成補全腳本。
例如，對於 Windows 上的 powershell, 您可以使用：`--sh-comp pwsh`

> Q: 什麼是 shell 的補全功能？

> A: 當我們在命令行上輸入命令時，如果按下Tab鍵，補全腳本就會被觸發，開始搜索可能的補全選項，並將這些選項呈現出來以供選擇。此功能可以極大地提高命令行輸入的效率和準確性。

不同 shell 的補全腳本所存放的位置是不一樣的。對於 zsh，您需要將其存放於 `$fpath` 數組所包含的路徑。
您可以使用 `print -l $fpath` 來獲取該數組的內容。
假設裏面包含了 `/usr/local/share/zsh/site-functions` 目錄， 那麼您可以使用以下命令來將補全腳本寫入到該目錄下的 `_tomlyre` 文件。

```sh
tomlyre --sh-comp zsh | doas tee /usr/local/share/zsh/site-functions/_tomlyre
```

為了避免使用 root 權限，您可以將其放在用户子目錄下。比如説，您可以使用以下命令向該數組頭部插入一個元素，再使用相關的目錄。
```zsh
fpath=(~/.local/share/zsh/completions $fpath)
```"##),
        ("from-format-help", r##"當從 stdin 讀取數據時，例如： `cat a | tomlyre conv -`，它首先會嘗試自動判斷文件格式。
然而，自動判斷有可能會失敗，這時候就需要手動指定格式了。
示例：`cat test.bson | tomlyre --src-fmt bson get -`"##),
        ("table-style-help", r##"當僅使用 `--ts` 參數時，會列出內置的樣式集合。
除了 markdown 可以縮寫為 md 外, 其他的樣式都要用完整的名稱。
用法示例1：`--ts md` 
用法示例2：`--table-style ascii`

注：目前不支持從文件中讀取 **表格樣式** 的集合，如果您需要此功能或其他額外的功能，歡迎提交 issue 進行反饋。"##),
        ("table-style", r##"指定表格的樣式"##),
        ("theme-help", r##"當僅使用 `-t` 參數時，會列出當前主題的列表。
若未指定主題文件，則使用內置的主題集合。
假設存在一個主題為 **ayu-dark**, 則您可以使用 `-t ayu-dark` 來使用該主題。
若主題名稱包含空格，則請使用雙引號或單引號將其包裹起來。

> 注：有一個特殊主題為 `None`，代表沒有語法高亮的純文本樣式。

當未指定 `--theme` 選項時，將會從環境變量 `$TOMLYRE_THEME` 中讀取數據。
此幫助信息的 **語法高亮** 主題也來自於該 **環境變量**。

> 默認的幫助信息是硬編碼的，加載速度非常快。  
> 加載硬編碼的文本與加載 `None` 主題的文本的速度是同一個數量級的，而動態加載則會變慢兩個數量級及以上。  
> 設置 `$TOMLYRE_THEME` 環境變量可能會導致它重新加載主題，這將 **嚴重** 拖慢啓動速度。  
> 除非您打算修改幫助信息的主題，或者是禁用語法高亮，否則請使用 `-t` 參數，而不是修改環境變量。  

使用環境變量的例子：
```sh
TOMLYRE_THEME="Monokai Extended" tomlyre --help
```

使用命令行參數的例子:
```sh
tomlyre -t "Monokai Extended" get test.toml
```

對於手動執行 "conv/get/set" 等相關操作所輸出的內容，`-t` 指定的主題的優先級 > 環境變量。
若未調用 `-t`，則讀取環境變量；若環境變量為空，則使用 **Monokai Extended**。"##),
        ("theme", r##"指定語法高亮的主題名稱"##),
    ],
}
}

/// Language ID: zh-Hant-HK;
/// Map name: "args_md";
/// Description: 繁體中文, 中國香港;
pub(super) const fn get_zh_hant_hk_map_args_md() -> L10nMap {
    ::phf::Map {
    key: 351906021642186605,
    disps: &[
        (3, 9),
        (4, 1),
        (0, 0),
    ],
    entries: &[
        ("theme-file", r##"[48;2;34;34;34m[38;2;255;255;255m自定義主題集合文件，您還需要搭配 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 指定主題名稱[0m"##),
        ("shell-completion", r##"[48;2;34;34;34m[38;2;255;255;255m為 shell 生成補全腳本[0m"##),
        ("theme-file-help", r##"[48;2;34;34;34m[38;2;255;255;255m支持的文件格式為 rust crate [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255msyntect[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 所 dump 出來的二進制數據。[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m本質上，這些主題是舊版的 sublime 的主題。
[48;2;34;34;34m[38;2;255;255;255m若未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 則讀取環境變量 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的值。[0m"##),
        ("from-format", r##"[48;2;34;34;34m[38;2;255;255;255m手動指定來源的文件格式（默認會自動判斷）[0m"##),
        ("disable-theme-background", r##"[48;2;34;34;34m[38;2;255;255;255m禁用主題背景（默認為啓用狀態，若調用[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 則禁用）[0m"##),
        ("theme-uncompressed-file-help", r##"[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 未壓縮的版本要比壓縮版本的優先級更高。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m若未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--tuf[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 則讀取環境變量 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_UNCOMPRESSED_FILE[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的值。[0m"##),
        ("disable-theme-background-help", r##"[48;2;34;34;34m[38;2;255;255;255m若未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--dis-theme-bg[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m， 則讀取環境變量 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME_BG[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的值。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m若該值為 false 則禁用
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255menv[48;2;34;34;34m[38;2;255;255;255m TOMLYRE_THEME_BG=false TOMLYRE_THEME=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mayu-dark[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m tomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("theme-uncompressed-file", r##"[48;2;34;34;34m[38;2;255;255;255m類似於[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtheme-file[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 但要求未壓縮的 dump 數據(文件)[0m"##),
        ("shell-completion-help", r##"[48;2;34;34;34m[38;2;255;255;255m您需要手動指定 shell 的名稱來生成補全腳本。
[48;2;34;34;34m[38;2;255;255;255m例如，對於 Windows 上的 powershell, 您可以使用：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--sh-comp pwsh[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m Q: 什麼是 shell 的補全功能？
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m A: 當我們在命令行上輸入命令時，如果按下Tab鍵，補全腳本就會被觸發，開始搜索可能的補全選項，並將這些選項呈現出來以供選擇。此功能可以極大地提高命令行輸入的效率和準確性。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m不同 shell 的補全腳本所存放的位置是不一樣的。對於 zsh，您需要將其存放於 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 數組所包含的路徑。
[48;2;34;34;34m[38;2;255;255;255m您可以使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mprint -l $fpath[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 來獲取該數組的內容。
[48;2;34;34;34m[38;2;255;255;255m假設裏面包含了 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/usr/local/share/zsh/site-functions[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 目錄， 那麼您可以使用以下命令來將補全腳本寫入到該目錄下的 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m_tomlyre[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 文件。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31msh-comp[48;2;34;34;34m[38;2;255;255;255m zsh[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;249;38;114m|[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mdoas[48;2;34;34;34m[38;2;255;255;255m tee /usr/local/share/zsh/site-functions/_tomlyre[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m為了避免使用 root 權限，您可以將其放在用户子目錄下。比如説，您可以使用以下命令向該數組頭部插入一個元素，再使用相關的目錄。
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255mzsh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m([48;2;34;34;34m[38;2;255;255;255m~[48;2;34;34;34m[38;2;255;255;255m/.local/share/zsh/completions [48;2;34;34;34m[38;2;255;255;255m$[48;2;34;34;34m[38;2;255;255;255mfpath[48;2;34;34;34m[38;2;255;255;255m)[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[0m"##),
        ("from-format-help", r##"[48;2;34;34;34m[38;2;255;255;255m當從 stdin 讀取數據時，例如： [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat a | tomlyre conv -[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，它首先會嘗試自動判斷文件格式。
[48;2;34;34;34m[38;2;255;255;255m然而，自動判斷有可能會失敗，這時候就需要手動指定格式了。
[48;2;34;34;34m[38;2;255;255;255m示例：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mcat test.bson | tomlyre --src-fmt bson get -[48;2;34;34;34m[38;2;0;255;255m`[0m"##),
        ("table-style-help", r##"[48;2;34;34;34m[38;2;255;255;255m當僅使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 參數時，會列出內置的樣式集合。
[48;2;34;34;34m[38;2;255;255;255m除了 markdown 可以縮寫為 md 外, 其他的樣式都要用完整的名稱。
[48;2;34;34;34m[38;2;255;255;255m用法示例1：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--ts md[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 
[48;2;34;34;34m[38;2;255;255;255m用法示例2：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--table-style ascii[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m注：目前不支持從文件中讀取 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m表格樣式[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 的集合，如果您需要此功能或其他額外的功能，歡迎提交 issue 進行反饋。[0m"##),
        ("table-style", r##"[48;2;34;34;34m[38;2;255;255;255m指定表格的樣式[0m"##),
        ("theme-help", r##"[48;2;34;34;34m[38;2;255;255;255m當僅使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 參數時，會列出當前主題的列表。
[48;2;34;34;34m[38;2;255;255;255m若未指定主題文件，則使用內置的主題集合。
[48;2;34;34;34m[38;2;255;255;255m假設存在一個主題為 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mayu-dark[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m, 則您可以使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ayu-dark[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 來使用該主題。
[48;2;34;34;34m[38;2;255;255;255m若主題名稱包含空格，則請使用雙引號或單引號將其包裹起來。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 注：有一個特殊主題為 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m，代表沒有語法高亮的純文本樣式。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m當未指定 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--theme[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 選項時，將會從環境變量 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 中讀取數據。
[48;2;34;34;34m[38;2;255;255;255m此幫助信息的 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m語法高亮[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 主題也來自於該 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m環境變量[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 默認的幫助信息是硬編碼的，加載速度非常快。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 加載硬編碼的文本與加載 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mNone[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 主題的文本的速度是同一個數量級的，而動態加載則會變慢兩個數量級及以上。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 設置 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m$TOMLYRE_THEME[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 環境變量可能會導致它重新加載主題，這將 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114m嚴重[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;102;217;239m 拖慢啓動速度。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;102;217;239m>[48;2;34;34;34m[38;2;102;217;239m 除非您打算修改幫助信息的主題，或者是禁用語法高亮，否則請使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;102;217;239m 參數，而不是修改環境變量。[48;2;34;34;34m[38;2;102;217;239m  [48;2;34;34;34m[38;2;102;217;239m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m使用環境變量的例子：
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mTOMLYRE_THEME[48;2;34;34;34m[38;2;249;38;114m=[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m --[48;2;34;34;34m[38;2;253;151;31mhelp[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m使用命令行參數的例子:
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;190;132;255msh[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255mtomlyre[48;2;34;34;34m[38;2;253;151;31m -[48;2;34;34;34m[38;2;253;151;31mt[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;230;219;116mMonokai Extended[48;2;34;34;34m[38;2;255;255;255m"[48;2;34;34;34m[38;2;255;255;255m get test.toml[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m```[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m對於手動執行 "conv/get/set" 等相關操作所輸出的內容，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 指定的主題的優先級 > 環境變量。
[48;2;34;34;34m[38;2;255;255;255m若未調用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，則讀取環境變量；若環境變量為空，則使用 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mMonokai Extended[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m。[0m"##),
        ("theme", r##"[48;2;34;34;34m[38;2;255;255;255m指定語法高亮的主題名稱[0m"##),
    ],
}
}

/// Language ID: zh-Hant-HK;
/// Map name: "conv";
/// Description: 繁體中文, 中國香港;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("conv", "conv-opts");
///
/// assert_eq!(msg, "在 toml、yaml、json 和 ron 等格式之間進行轉換");
/// ```
pub(super) const fn get_zh_hant_hk_map_conv() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"在 toml、yaml、json 和 ron 等格式之間進行轉換"##),
        ("from", r##"指定輸入的文件"##),
        ("conv-to", r##"指定輸出的格式/文件，支持多次調用"##),
        ("save", r##"當調用此選項後，`--to` 所指定的文件將被保存"##),
        ("conv-to-help", r##"如果當前目錄下存在 **test.json** 文件，那麼您可以調用 `conv test.json -t yml` 將其轉換為 yml (yaml),
也可以手動指定文件路徑：`conv test.json -t /tmp/test.yml`

支持的格式取決於此軟件包啓用的功能，並非所有功能都會默認啓用。

此外，在使用前，您必須要了解的事情如下：
- 1. 它會對非具體文件路徑進行自動更名。
  - 1.1: 只有一個層級的文件會被斷定為文件格式，而不是文件路徑
    當調用 `conv test.toml -t ron`, 輸出的文件會自動變成 `test.ron`, 而不是 `ron`。
    若您無需此特性，則請使用 `-t ./ron`, 或者是 `-t $PWD/ron`。
  - 1.2: 指向的是目錄而不是文件
    比如 `-t /tmp/`，我們只能將 `test.toml` 放在 `/tmp/` 的子級，而不是覆蓋 `/tmp` 自身。
    若您無需此特性，則請使用完整的（非目錄）文件路徑，例如 `-t /tmp/xx.toml`，而不是 `-t /tmp/`。
- 2. 當未調用 `--save` 時，指定的文件不會自動保存。
- 3. 當調用 `--save` 時，conv 子命令的輸入與輸出不能為相同的文件。
- 4. 轉換後的文件不會保留註釋。"##),
    ],
}
}

/// Language ID: zh-Hant-HK;
/// Map name: "conv_md";
/// Description: 繁體中文, 中國香港;
pub(super) const fn get_zh_hant_hk_map_conv_md() -> L10nMap {
    ::phf::Map {
    key: 14108922650502679131,
    disps: &[
        (3, 0),
    ],
    entries: &[
        ("conv-opts", r##"[48;2;34;34;34m[38;2;255;255;255m在 toml、yaml、json 和 ron 等格式之間進行轉換[0m"##),
        ("from", r##"[48;2;34;34;34m[38;2;255;255;255m指定輸入的文件[0m"##),
        ("conv-to", r##"[48;2;34;34;34m[38;2;255;255;255m指定輸出的格式/文件，支持多次調用[0m"##),
        ("save", r##"[48;2;34;34;34m[38;2;255;255;255m當調用此選項後，[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--to[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 所指定的文件將被保存[0m"##),
        ("conv-to-help", r##"[48;2;34;34;34m[38;2;255;255;255m如果當前目錄下存在 [48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;249;38;114mtest.json[48;2;34;34;34m[38;2;249;38;114m**[48;2;34;34;34m[38;2;255;255;255m 文件，那麼您可以調用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 將其轉換為 yml (yaml),
[48;2;34;34;34m[38;2;255;255;255m也可以手動指定文件路徑：[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.json -t /tmp/test.yml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m支持的格式取決於此軟件包啓用的功能，並非所有功能都會默認啓用。
[48;2;34;34;34m[38;2;255;255;255m
[48;2;34;34;34m[38;2;255;255;255m此外，在使用前，您必須要了解的事情如下：
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m [48;2;34;34;34m[38;2;255;255;255m1[48;2;34;34;34m[38;2;119;119;119m.[48;2;34;34;34m[38;2;255;255;255m 它會對非具體文件路徑進行自動更名。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.1: 只有一個層級的文件會被斷定為文件格式，而不是文件路徑
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m當調用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mconv test.toml -t ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 輸出的文件會自動變成 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m若您無需此特性，則請使用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t ./ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m, 或者是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t $PWD/ron[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;255;255;255m  [48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 1.2: 指向的是目錄而不是文件
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m比如 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，我們只能將 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255mtest.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 放在 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 的子級，而不是覆蓋 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m/tmp[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 自身。
[48;2;34;34;34m[38;2;255;255;255m    [48;2;34;34;34m[38;2;255;255;255m若您無需此特性，則請使用完整的（非目錄）文件路徑，例如 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/xx.toml[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m，而不是 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m-t /tmp/[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 2. 當未調用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 時，指定的文件不會自動保存。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 3. 當調用 [48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;0;255;255m--save[48;2;34;34;34m[38;2;0;255;255m`[48;2;34;34;34m[38;2;255;255;255m 時，conv 子命令的輸入與輸出不能為相同的文件。
[48;2;34;34;34m[38;2;119;119;119m-[48;2;34;34;34m[38;2;255;255;255m 4. 轉換後的文件不會保留註釋。[0m"##),
    ],
}
}

/// zh-Hant-HK: 繁體中文, 中國香港
pub(super) const fn get_zh_hant_hk_map() -> SubLocaleMap {
    ::phf::Map {
    key: 7485420634051515786,
    disps: &[
        (3, 2),
        (1, 0),
    ],
    entries: &[
        ("args", get_zh_hant_hk_map_args),
        ("conv_md", get_zh_hant_hk_map_conv_md),
        ("parser", get_zh_hant_hk_map_parser),
        ("opt_md", get_zh_hant_hk_map_opt_md),
        ("args_md", get_zh_hant_hk_map_args_md),
        ("opt", get_zh_hant_hk_map_opt),
        ("log-core", get_zh_hant_hk_map_log_core),
        ("conv", get_zh_hant_hk_map_conv),
        ("theme", get_zh_hant_hk_map_theme),
    ],
}
}

/// Language ID: zu;
/// Map name: "log-core";
/// Description: isiZulu, isi-Latin, iNingizimu Afrika;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("log-core", "debug");
///
/// assert_eq!(msg, "iphutha");
/// ```
pub(super) const fn get_zu_map_log_core() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 5),
        (0, 0),
    ],
    entries: &[
        ("debug", r##"iphutha"##),
        ("warn", r##"xwayisa"##),
        ("info", r##"imininingwane"##),
        ("error", r##"Iphutha"##),
        ("trace", r##"Landelela"##),
        ("init-logger", r##"Ukuqalisa i-logger ..."##),
    ],
}
}

/// Language ID: zu;
/// Map name: "theme";
/// Description: isiZulu, isi-Latin, iNingizimu Afrika;
///
/// # Example
///
/// ```no_run
/// let msg = loader.get_or_default("theme", "invalid-name");
///
/// assert_eq!(msg, "Igama le-Theme elingalungile");
/// ```
pub(super) const fn get_zu_map_theme() -> L10nMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (0, 0),
    ],
    entries: &[
        ("invalid-name", r##"Igama le-Theme elingalungile"##),
    ],
}
}

/// zu: isiZulu, isi-Latin, iNingizimu Afrika
pub(super) const fn get_zu_map() -> SubLocaleMap {
    ::phf::Map {
    key: 12913932095322966823,
    disps: &[
        (1, 0),
    ],
    entries: &[
        ("log-core", get_zu_map_log_core),
        ("theme", get_zu_map_theme),
    ],
}
}

/// # Example
///
/// ```no_run
/// let map = locale_map();
///
/// for k in map.keys() {
///     println!("{k}")
/// }
///
/// map.get("en").map(|v| dbg!(v()));
/// ```
pub(super) const fn locale_map() -> LocaleMap {
    ::phf::Map {
    key: 15467950696543387533,
    disps: &[
        (0, 5),
        (33, 70),
        (0, 94),
        (0, 0),
        (5, 7),
        (0, 17),
        (1, 19),
        (0, 105),
        (0, 56),
        (0, 37),
        (43, 61),
        (0, 0),
        (3, 14),
        (63, 87),
        (0, 10),
        (7, 7),
        (0, 7),
        (0, 1),
        (10, 18),
        (1, 54),
        (0, 3),
        (16, 54),
    ],
    entries: &[
        ("mk", get_mk_map),
        ("fr", get_fr_map),
        ("si", get_si_map),
        ("ro", get_ro_map),
        ("ko", get_ko_map),
        ("ga", get_ga_map),
        ("fa", get_fa_map),
        ("lv", get_lv_map),
        ("pt", get_pt_map),
        ("tg", get_tg_map),
        ("co", get_co_map),
        ("mr", get_mr_map),
        ("ku", get_ku_map),
        ("nl", get_nl_map),
        ("hu", get_hu_map),
        ("lo", get_lo_map),
        ("yi", get_yi_map),
        ("kk", get_kk_map),
        ("ne", get_ne_map),
        ("sr", get_sr_map),
        ("no", get_no_map),
        ("he", get_he_map),
        ("ka", get_ka_map),
        ("be", get_be_map),
        ("ml", get_ml_map),
        ("sl", get_sl_map),
        ("sv", get_sv_map),
        ("kn", get_kn_map),
        ("af", get_af_map),
        ("vi", get_vi_map),
        ("jw", get_jw_map),
        ("sn", get_sn_map),
        ("fi", get_fi_map),
        ("ky", get_ky_map),
        ("fy", get_fy_map),
        ("uk", get_uk_map),
        ("bg", get_bg_map),
        ("lt", get_lt_map),
        ("am", get_am_map),
        ("cy", get_cy_map),
        ("or", get_or_map),
        ("ht", get_ht_map),
        ("sk", get_sk_map),
        ("sw", get_sw_map),
        ("zh-Hant-HK", get_zh_hant_hk_map),
        ("cs", get_cs_map),
        ("yo", get_yo_map),
        ("el", get_el_map),
        ("mi", get_mi_map),
        ("az", get_az_map),
        ("pl", get_pl_map),
        ("ja", get_ja_map),
        ("mg", get_mg_map),
        ("eo", get_eo_map),
        ("zh", get_zh_map),
        ("ny", get_ny_map),
        ("de", get_de_map),
        ("zu", get_zu_map),
        ("eu", get_eu_map),
        ("zh-Hant", get_zh_hant_map),
        ("ceb", get_ceb_map),
        ("xh", get_xh_map),
        ("ru", get_ru_map),
        ("ha", get_ha_map),
        ("my", get_my_map),
        ("te", get_te_map),
        ("bn", get_bn_map),
        ("hr", get_hr_map),
        ("tr", get_tr_map),
        ("bs", get_bs_map),
        ("ca", get_ca_map),
        ("hi", get_hi_map),
        ("gu", get_gu_map),
        ("sm", get_sm_map),
        ("lb", get_lb_map),
        ("th", get_th_map),
        ("ar", get_ar_map),
        ("mt", get_mt_map),
        ("sq", get_sq_map),
        ("gd", get_gd_map),
        ("so", get_so_map),
        ("is", get_is_map),
        ("km", get_km_map),
        ("ig", get_ig_map),
        ("ms", get_ms_map),
        ("da", get_da_map),
        ("es", get_es_map),
        ("sd", get_sd_map),
        ("ps", get_ps_map),
        ("ta", get_ta_map),
        ("en", get_en_map),
        ("et", get_et_map),
        ("ur", get_ur_map),
        ("uz", get_uz_map),
        ("it", get_it_map),
        ("iw", get_iw_map),
        ("ug", get_ug_map),
        ("en-GB", get_en_gb_map),
        ("id", get_id_map),
        ("mn", get_mn_map),
        ("pa", get_pa_map),
        ("hy", get_hy_map),
        ("gl", get_gl_map),
        ("st", get_st_map),
        ("su", get_su_map),
        ("haw", get_haw_map),
        ("la", get_la_map),
        ("fil", get_fil_map),
    ],
}
}

/// # Example
///
/// ```no_run
/// let loader = glossa::MapLoader::new(locale_hashmap());
///
/// dbg!(&loader);
/// ```
pub(super) fn locale_hashmap() -> LocaleHashMap {
    use lang_id_consts::*;

    HashMap::from_iter([
        (unsafe { get_af() }, get_af_map()),
        (unsafe { get_am() }, get_am_map()),
        (unsafe { get_ar() }, get_ar_map()),
        (unsafe { get_az() }, get_az_map()),
        (unsafe { get_be() }, get_be_map()),
        (unsafe { get_bg() }, get_bg_map()),
        (unsafe { get_bn() }, get_bn_map()),
        (unsafe { get_bs() }, get_bs_map()),
        (unsafe { get_ca() }, get_ca_map()),
        (unsafe { get_ceb() }, get_ceb_map()),
        (unsafe { get_co() }, get_co_map()),
        (unsafe { get_cs() }, get_cs_map()),
        (unsafe { get_cy() }, get_cy_map()),
        (unsafe { get_da() }, get_da_map()),
        (unsafe { get_de() }, get_de_map()),
        (unsafe { get_el() }, get_el_map()),
        (unsafe { get_en() }, get_en_map()),
        (unsafe { get_en_gb() }, get_en_gb_map()),
        (unsafe { get_eo() }, get_eo_map()),
        (unsafe { get_es() }, get_es_map()),
        (unsafe { get_et() }, get_et_map()),
        (unsafe { get_eu() }, get_eu_map()),
        (unsafe { get_fa() }, get_fa_map()),
        (unsafe { get_fi() }, get_fi_map()),
        (unsafe { get_fil() }, get_fil_map()),
        (unsafe { get_fr() }, get_fr_map()),
        (unsafe { get_fy() }, get_fy_map()),
        (unsafe { get_ga() }, get_ga_map()),
        (unsafe { get_gd() }, get_gd_map()),
        (unsafe { get_gl() }, get_gl_map()),
        (unsafe { get_gu() }, get_gu_map()),
        (unsafe { get_ha() }, get_ha_map()),
        (unsafe { get_haw() }, get_haw_map()),
        (unsafe { get_he() }, get_he_map()),
        (unsafe { get_hi() }, get_hi_map()),
        (unsafe { get_hr() }, get_hr_map()),
        (unsafe { get_ht() }, get_ht_map()),
        (unsafe { get_hu() }, get_hu_map()),
        (unsafe { get_hy() }, get_hy_map()),
        (unsafe { get_id() }, get_id_map()),
        (unsafe { get_ig() }, get_ig_map()),
        (unsafe { get_is() }, get_is_map()),
        (unsafe { get_it() }, get_it_map()),
        (unsafe { get_iw() }, get_iw_map()),
        (unsafe { get_ja() }, get_ja_map()),
        (unsafe { get_jw() }, get_jw_map()),
        (unsafe { get_ka() }, get_ka_map()),
        (unsafe { get_kk() }, get_kk_map()),
        (unsafe { get_km() }, get_km_map()),
        (unsafe { get_kn() }, get_kn_map()),
        (unsafe { get_ko() }, get_ko_map()),
        (unsafe { get_ku() }, get_ku_map()),
        (unsafe { get_ky() }, get_ky_map()),
        (unsafe { get_la() }, get_la_map()),
        (unsafe { get_lb() }, get_lb_map()),
        (unsafe { get_lo() }, get_lo_map()),
        (unsafe { get_lt() }, get_lt_map()),
        (unsafe { get_lv() }, get_lv_map()),
        (unsafe { get_mg() }, get_mg_map()),
        (unsafe { get_mi() }, get_mi_map()),
        (unsafe { get_mk() }, get_mk_map()),
        (unsafe { get_ml() }, get_ml_map()),
        (unsafe { get_mn() }, get_mn_map()),
        (unsafe { get_mr() }, get_mr_map()),
        (unsafe { get_ms() }, get_ms_map()),
        (unsafe { get_mt() }, get_mt_map()),
        (unsafe { get_my() }, get_my_map()),
        (unsafe { get_ne() }, get_ne_map()),
        (unsafe { get_nl() }, get_nl_map()),
        (unsafe { get_no() }, get_no_map()),
        (unsafe { get_ny() }, get_ny_map()),
        (unsafe { get_or() }, get_or_map()),
        (unsafe { get_pa() }, get_pa_map()),
        (unsafe { get_pl() }, get_pl_map()),
        (unsafe { get_ps() }, get_ps_map()),
        (unsafe { get_pt() }, get_pt_map()),
        (unsafe { get_ro() }, get_ro_map()),
        (unsafe { get_ru() }, get_ru_map()),
        (unsafe { get_sd() }, get_sd_map()),
        (unsafe { get_si() }, get_si_map()),
        (unsafe { get_sk() }, get_sk_map()),
        (unsafe { get_sl() }, get_sl_map()),
        (unsafe { get_sm() }, get_sm_map()),
        (unsafe { get_sn() }, get_sn_map()),
        (unsafe { get_so() }, get_so_map()),
        (unsafe { get_sq() }, get_sq_map()),
        (unsafe { get_sr() }, get_sr_map()),
        (unsafe { get_st() }, get_st_map()),
        (unsafe { get_su() }, get_su_map()),
        (unsafe { get_sv() }, get_sv_map()),
        (unsafe { get_sw() }, get_sw_map()),
        (unsafe { get_ta() }, get_ta_map()),
        (unsafe { get_te() }, get_te_map()),
        (unsafe { get_tg() }, get_tg_map()),
        (unsafe { get_th() }, get_th_map()),
        (unsafe { get_tr() }, get_tr_map()),
        (unsafe { get_ug() }, get_ug_map()),
        (unsafe { get_uk() }, get_uk_map()),
        (unsafe { get_ur() }, get_ur_map()),
        (unsafe { get_uz() }, get_uz_map()),
        (unsafe { get_vi() }, get_vi_map()),
        (unsafe { get_xh() }, get_xh_map()),
        (unsafe { get_yi() }, get_yi_map()),
        (unsafe { get_yo() }, get_yo_map()),
        (unsafe { get_zh() }, get_zh_map()),
        (unsafe { get_zh_hant() }, get_zh_hant_map()),
        (unsafe { get_zh_hant_hk() }, get_zh_hant_hk_map()),
        (unsafe { get_zu() }, get_zu_map()),
    ])
}
